@@ -15,14 +15,13 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef _CAMEL_FOLDER_SUMMARY_H
 #define _CAMEL_FOLDER_SUMMARY_H
 
-
 #include <stdio.h>
 #include <time.h>
 #include <camel/camel-mime-parser.h>
@@ -50,10 +49,10 @@ typedef struct _CamelFolderMetaSummary CamelFolderMetaSummary;
    describe the content structure of the message (if it has any) */
 struct _CamelMessageContentInfo {
 	struct _CamelMessageContentInfo *next;
-	
+
 	struct _CamelMessageContentInfo *childs;
 	struct _CamelMessageContentInfo *parent;
-	
+
 	CamelContentType *type;
 	char *id;
 	char *description;
@@ -73,23 +72,24 @@ typedef enum _CamelMessageFlags {
 	CAMEL_MESSAGE_CACHED = 1<<6, /* used                 TNY_HEADER_FLAG_CACHED      */
 	CAMEL_MESSAGE_PARTIAL = 1<<7, /* used                TNY_HEADER_FLAG_PARTIAL     */
 	CAMEL_MESSAGE_EXPUNGED = 1<<8, /* used               TNY_HEADER_FLAG_EXPUNGED    */
-	CAMEL_MESSAGE_HIGH_PRIORITY = 1<<9|1<<10, /* used    TNY_HEADER_FLAG_HIGH_PRIORITY    */
+	CAMEL_MESSAGE_HIGH_PRIORITY = 0<<9|1<<10, /* used    TNY_HEADER_FLAG_HIGH_PRIORITY    */
 	CAMEL_MESSAGE_NORMAL_PRIORITY = 0<<9|0<<10, /* used  TNY_HEADER_FLAG_NORMAL_PRIORITY    */
-	CAMEL_MESSAGE_LOW_PRIORITY = 0<<9|1<<10, /* used     TNY_HEADER_FLAG_LOW_PRIORITY    */
-	CAMEL_MESSAGE_SUSPENDED_PRIORITY = 1<<9|0<<10, /*    TNY_HEADER_FLAG_SUSPENDED_PRIORITY    */
+	CAMEL_MESSAGE_LOW_PRIORITY = 1<<9|0<<10, /* used     TNY_HEADER_FLAG_LOW_PRIORITY    */
+	CAMEL_MESSAGE_SUSPENDED = 1<<11, /*                  TNY_HEADER_FLAG_SUSPENDED    */
 
 	/* internally used */
-	CAMEL_MESSAGE_FOLDER_FLAGGED = 1<<12, /* internally used */
 	CAMEL_MESSAGE_INFO_NEEDS_FREE = 1<<13,/* internally used */
 	CAMEL_MESSAGE_INFO_UNUSED = 1<<14, /* internally used but available */
 	CAMEL_MESSAGE_FREED = 1<<15,  /* internally used */
 	CAMEL_MESSAGE_USER = 1<<16,  /* free slot */
-	CAMEL_MESSAGE_SECURE = 1<<17, /* free slot */
+	CAMEL_MESSAGE_FOLDER_FLAGGED = 1<<17 /* internally used */
 
 } CamelMessageFlags;
 
+#define CAMEL_MESSAGE_PRIORITY_MASK (1<<9|1<<10)
+
 /* Changes to system flags will NOT trigger a folder changed event */
-#define CAMEL_MESSAGE_SYSTEM_MASK (0xffff << 12)
+#define CAMEL_MESSAGE_SYSTEM_MASK (0x1fff << 12)
 
 typedef struct _CamelFlag {
 	struct _CamelFlag *next;
@@ -149,8 +149,7 @@ struct _CamelMessageInfo {
 /* Otherwise they can do their own thing entirely */
 
 /*                                          on x86_32 */
-struct _CamelMessageInfoBase 
-{
+struct _CamelMessageInfoBase {
 	CamelFolderSummary *summary;       /* 4 bytes */
 	guint32 refcount;                  /* 4 bytes */
 	char *uid;                         /* 4 bytes */
@@ -161,6 +160,9 @@ struct _CamelMessageInfoBase
 
 	/* tree of content description - NULL if it is not available */
 	CamelMessageContentInfo *content;  /* 4 bytes */
+#ifdef NON_TINYMAIL_FEATURES
+	struct _camel_header_param *headers;
+#endif
 	CamelSummaryMessageID message_id;  /* 8 bytes */
 
 	guint32 flags;                     /* 4 bytes */
@@ -174,10 +176,9 @@ struct _CamelMessageInfoBase
 
 
 typedef enum _CamelFolderSummaryFlags {
-	CAMEL_SUMMARY_DIRTY = 1<<0,
+	CAMEL_SUMMARY_DIRTY = 1<<0
 } CamelFolderSummaryFlags;
 
-
 struct _CamelFolderSummary {
 	CamelObject parent;
 
@@ -208,8 +209,9 @@ struct _CamelFolderSummary {
 
 	GMappedFile *file;
 	unsigned char *filepos;
-	GMutex *dump_lock, *hash_lock;
-	gboolean in_reload;
+	GMutex *hash_lock;
+	GStaticRecMutex *dump_lock;
+	gboolean in_reload, had_expunges;
 	gint idx;
 
 	void (*set_extra_flags_func) (CamelFolder *folder, CamelMessageInfoBase *mi);
@@ -294,8 +296,8 @@ void camel_folder_summary_touch(CamelFolderSummary *summary);
 void camel_folder_summary_add(CamelFolderSummary *summary, CamelMessageInfo *info);
 
 /* build/add raw summary items */
-CamelMessageInfo *camel_folder_summary_add_from_header(CamelFolderSummary *summary, struct _camel_header_raw *headers);
-CamelMessageInfo *camel_folder_summary_add_from_parser(CamelFolderSummary *summary, CamelMimeParser *parser);
+CamelMessageInfo *camel_folder_summary_add_from_header(CamelFolderSummary *summary, struct _camel_header_raw *headers, const gchar *uid);
+CamelMessageInfo *camel_folder_summary_add_from_parser(CamelFolderSummary *summary, CamelMimeParser *parser, const gchar *uid);
 CamelMessageInfo *camel_folder_summary_add_from_message(CamelFolderSummary *summary, CamelMimeMessage *message);
 
 /* Just build raw summary items */
@@ -394,8 +396,9 @@ void camel_folder_summary_kill_hash (CamelFolderSummary *summary);
 void camel_message_info_clear_normal_flags (CamelMessageInfo *min);
 int camel_folder_summary_get_index_for (CamelFolderSummary *s, const char *uid);
 
+#if 0
 void camel_folder_summary_move_up (CamelFolderSummary *s);
-
+#endif
 
 G_END_DECLS
 
