@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
 
@@ -43,6 +43,7 @@
 #include "camel-session.h"
 #include "camel-store.h"
 #include "camel-vtrash-folder.h"
+#include "camel-multipart.h"
 
 #define d(x)
 #define w(x)
@@ -117,23 +118,78 @@ static gboolean        folder_changed        (CamelObject *object,
 
 static int get_local_size (CamelFolder *folder);
 
-static void 
+
+static char *
+fetch_structure (CamelFolder *folder, const char *uid, CamelException *ex)
+{
+	camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, "No support for BODYSTRUCTURE");
+	return NULL;
+}
+
+char *
+camel_folder_fetch_structure (CamelFolder *folder, const char *uid, CamelException *ex)
+{
+	return CF_CLASS (folder)->fetch_structure (folder, uid, ex);
+}
+
+static char *
+get_cache_filename (CamelFolder *folder, const char *uid, const char *spec, CamelFolderPartState *state)
+{
+	*state = CAMEL_FOLDER_PART_STATE_NOT_CACHED;
+	return NULL;
+}
+
+char *
+camel_folder_get_cache_filename (CamelFolder *folder, const char *uid, const char *spec, CamelFolderPartState *state)
+{
+	return CF_CLASS (folder)->get_cache_filename (folder, uid, spec, state);
+}
+
+static char * 
+fetch (CamelFolder *folder, const char *uid, const char *spec, gboolean *binary, CamelException *ex)
+{
+	if (binary)
+		*binary = TRUE;
+	camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, "No support for BODYSTRUCTURE");
+	return g_strdup ("/dev/null");
+}
+
+static char * 
+convert (CamelFolder *folder, const char *uid, const char *spec, const char *cto, CamelException *ex)
+{
+	camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, "No support for CONVERT");
+	return g_strdup ("/dev/null");
+}
+
+char* 
+camel_folder_convert (CamelFolder *folder, const char *uid, const char *spec, const char *convert_to, CamelException *ex)
+{
+	return CF_CLASS (folder)->convert (folder, uid, spec, convert_to, ex);
+}
+
+char * 
+camel_folder_fetch (CamelFolder *folder, const char *uid, const char *spec, gboolean *binary, CamelException *ex)
+{
+	return CF_CLASS (folder)->fetch (folder, uid, spec, binary, ex);
+}
+
+static void
 delete_attachments (CamelFolder *folder, const char *uid)
 {
 }
 
-static void 
+static void
 rewrite_cache (CamelFolder *folder, const char *uid, CamelMimeMessage *msg)
 {
 }
 
-static void 
+static void
 folder_set_push_email (CamelFolder *folder, gboolean setting)
 {
 	return;
 }
 
-void 
+void
 camel_folder_set_push_email (CamelFolder *folder, gboolean setting)
 {
 	if (!folder)
@@ -150,6 +206,10 @@ camel_folder_class_init (CamelFolderClass *camel_folder_class)
 	parent_class = camel_type_get_global_classfuncs (camel_object_get_type ());
 
 	/* virtual method definition */
+	camel_folder_class->convert = convert;
+	camel_folder_class->get_cache_filename = get_cache_filename;
+	camel_folder_class->fetch_structure = fetch_structure;
+	camel_folder_class->fetch = fetch;
 	camel_folder_class->get_local_size = get_local_size;
 	camel_folder_class->set_push_email = folder_set_push_email;
 	camel_folder_class->sync = folder_sync;
@@ -230,10 +290,10 @@ camel_folder_finalize (CamelObject *object)
 	}
 
 	camel_folder_change_info_free(p->changed_frozen);
-	
+
 	g_static_rec_mutex_free(&p->lock);
 	g_static_mutex_free(&p->change_lock);
-	
+
 	g_free(p);
 }
 
@@ -306,10 +366,10 @@ camel_folder_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 	g_return_if_fail (CAMEL_IS_FOLDER (folder));
 
 	CAMEL_FOLDER_REC_LOCK(folder, lock);
-	
+
 	if (!(folder->folder_flags & CAMEL_FOLDER_HAS_BEEN_DELETED))
 		CF_CLASS (folder)->sync (folder, expunge, ex);
-	
+
 	CAMEL_FOLDER_REC_UNLOCK(folder, lock);
 }
 
@@ -336,20 +396,24 @@ camel_folder_refresh_info (CamelFolder *folder, CamelException *ex)
 	CF_CLASS (folder)->refresh_info (folder, ex);
 }
 
-void 
+void
 camel_folder_delete_attachments (CamelFolder *folder, const char *uid)
 {
 	g_return_if_fail (CAMEL_IS_FOLDER (folder));
 
-	return CF_CLASS (folder)->delete_attachments (folder, uid);
+	CF_CLASS (folder)->delete_attachments (folder, uid);
+
+	return;
 }
 
-void 
+void
 camel_folder_rewrite_cache (CamelFolder *folder, const char *uid, CamelMimeMessage *msg)
 {
 	g_return_if_fail (CAMEL_IS_FOLDER (folder));
 
-	return CF_CLASS (folder)->rewrite_cache (folder, uid, msg);
+	CF_CLASS (folder)->rewrite_cache (folder, uid, msg);
+
+	return;
 }
 
 static int
@@ -360,15 +424,13 @@ folder_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 	guint32 tag;
 	int unread = -1, deleted = 0, count = -1;
 
-	for (i=0; i < args->argc; i++) 
-	{
-	  CamelArgGet *arg = &args->argv[i];
+	for (i = 0; i < args->argc; i++) {
+		CamelArgGet *arg = &args->argv[i];
 
-	  tag = arg->tag;
+		tag = arg->tag;
 
-	  switch (tag & CAMEL_ARG_TAG) 
-	  {
-		/* CamelObject args */
+		switch (tag & CAMEL_ARG_TAG) {
+			/* CamelObject args */
 		case CAMEL_OBJECT_ARG_DESCRIPTION:
 			if (folder->description == NULL)
 				folder->description = g_strdup_printf("%s", folder->full_name);
@@ -394,7 +456,7 @@ folder_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 		case CAMEL_FOLDER_ARG_UNREAD:
 		case CAMEL_FOLDER_ARG_DELETED:
 
-			/* This is so we can get the values atomically, and also 
+			/* This is so we can get the values atomically, and also
 			 * so we can calculate them only once */
 
 			if (unread == -1) {
@@ -404,14 +466,12 @@ folder_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 				/* TODO: Locking? */
 				unread = 0;
 				count = camel_folder_summary_count (folder->summary);
-				for (j=0; j < count; j++) 
-				{
+				for (j = 0; j < count; j++) {
 					info = camel_folder_summary_index (folder->summary, j);
-					if (info) 
-					{
+					if (info) {
 						guint32 flags = camel_message_info_flags (info);
 
-						/* TNY Observation: We assume that 
+						/* TNY Observation: We assume that
 						 * deleted messages are seen too */
 
 						if (flags & CAMEL_MESSAGE_DELETED)
@@ -459,9 +519,9 @@ folder_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 			break;
 		default:
 			continue;
-	  }
+		}
 
-	  arg->tag = (tag & CAMEL_ARG_TYPE) | CAMEL_ARG_IGNORE;
+		arg->tag = (tag & CAMEL_ARG_TYPE) | CAMEL_ARG_IGNORE;
 	}
 
 	return parent_class->getv(object, ex, args);
@@ -582,12 +642,12 @@ void
 camel_folder_expunge (CamelFolder *folder, CamelException *ex)
 {
 	g_return_if_fail (CAMEL_IS_FOLDER (folder));
-	
+
 	CAMEL_FOLDER_REC_LOCK(folder, lock);
-	
+
 	if (!(folder->folder_flags & CAMEL_FOLDER_HAS_BEEN_DELETED))
 		CF_CLASS (folder)->expunge (folder, ex);
-	
+
 	CAMEL_FOLDER_REC_UNLOCK(folder, lock);
 }
 
@@ -634,7 +694,7 @@ camel_folder_get_unread_message_count (CamelFolder *folder)
 	int count = -1;
 
 	g_return_val_if_fail (CAMEL_IS_FOLDER (folder), -1);
-	
+
 	camel_object_get(folder, NULL, CAMEL_FOLDER_UNREAD, &count, 0);
 
 	return count;
@@ -680,10 +740,10 @@ append_message (CamelFolder *folder, CamelMimeMessage *message,
 	camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID,
 			      _("Unsupported operation: append message: for %s"),
 			      camel_type_to_name (CAMEL_OBJECT_GET_TYPE (folder)));
-	
+
 	w(g_warning ("CamelFolder::append_message not implemented for `%s'",
 		     camel_type_to_name (CAMEL_OBJECT_GET_TYPE (folder))));
-	
+
 	return;
 
 }
@@ -945,7 +1005,7 @@ camel_folder_get_message_user_tag (CamelFolder *folder, const char *uid,  const
 {
 	const char *ret;
 
-	g_return_val_if_fail (CAMEL_IS_FOLDER (folder), 0);
+	g_return_val_if_fail (CAMEL_IS_FOLDER (folder), NULL);
 
 	/* FIXME: should duplicate string */
 	ret = CF_CLASS (folder)->get_message_user_tag (folder, uid, name);
@@ -1037,7 +1097,7 @@ free_message_info (CamelFolder *folder, CamelMessageInfo *info)
  * camel_folder_free_message_info:
  * @folder: a #CamelFolder object
  * @info: a #CamelMessageInfo
- * 
+ *
  * Free (unref) a #CamelMessageInfo, previously obtained with
  * #camel_folder_get_message_info.
  **/
@@ -1063,7 +1123,7 @@ ref_message_info (CamelFolder *folder, CamelMessageInfo *info)
  * camel_folder_ref_message_info:
  * @folder: a #CamelFolder object
  * @info: a #CamelMessageInfo
- * 
+ *
  * DEPRECATED: Use #camel_message_info_ref directly.
  *
  * Ref a #CamelMessageInfo, previously obtained with
@@ -1104,7 +1164,7 @@ get_message (CamelFolder *folder, const char *uid, CamelFolderReceiveType type,
 {
 	w(g_warning ("CamelFolder::get_message not implemented for `%s'",
 		     camel_type_to_name (CAMEL_OBJECT_GET_TYPE (folder))));
-	
+
 	return NULL;
 }
 
@@ -1155,15 +1215,15 @@ get_uids(CamelFolder *folder)
 	g_ptr_array_set_size(array, count);
 	for (i = 0, j = 0; i < count; i++) {
 		CamelMessageInfo *info = camel_folder_summary_index(folder->summary, i);
-		
+
 		if (info) {
 			array->pdata[j++] = g_strdup (camel_message_info_uid (info));
 			camel_message_info_free(info);
 		}
 	}
-	
+
 	g_ptr_array_set_size (array, j);
-	
+
 	return array;
 }
 
@@ -1297,10 +1357,10 @@ search_by_expression (CamelFolder *folder, const char *expression,
 	camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID,
 			      _("Unsupported operation: search by expression: for %s"),
 			      camel_type_to_name (CAMEL_OBJECT_GET_TYPE (folder)));
-	
+
 	w(g_warning ("CamelFolder::search_by_expression not implemented for "
 		     "`%s'", camel_type_to_name (CAMEL_OBJECT_GET_TYPE (folder))));
-	
+
 	return NULL;
 }
 
@@ -1338,10 +1398,10 @@ search_by_uids(CamelFolder *folder, const char *exp, GPtrArray *uids, CamelExcep
 	camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID,
 			      _("Unsupported operation: search by UIDs: for %s"),
 			      camel_type_to_name (CAMEL_OBJECT_GET_TYPE (folder)));
-	
+
 	w(g_warning ("CamelFolder::search_by_expression not implemented for "
 		     "`%s'", camel_type_to_name (CAMEL_OBJECT_GET_TYPE (folder))));
-	
+
 	return NULL;
 }
 
@@ -1388,11 +1448,11 @@ search_free (CamelFolder *folder, GPtrArray *result)
  * camel_folder_search_free:
  * @folder: a #CamelFolder object
  * @result: search results to free
- * 
+ *
  * Free the result of a search as gotten by #camel_folder_search or
  * #camel_folder_search_by_uids.
  **/
-void 
+void
 camel_folder_search_free (CamelFolder *folder, GPtrArray *result)
 {
 	g_return_if_fail (CAMEL_IS_FOLDER (folder));
@@ -1409,10 +1469,10 @@ transfer_message_to (CamelFolder *source, const char *uid, CamelFolder *dest,
 {
 	CamelMimeMessage *msg;
 	CamelMessageInfo *minfo, *info;
-	
+
 	/* Default implementation. */
-	
-	/* TNY: Partial message retrieval exception (always transfer the full 
+
+	/* TNY: Partial message retrieval exception (always transfer the full
 		message, not just the body part) */
 	msg = camel_folder_get_message(source, uid, CAMEL_FOLDER_RECEIVE_FULL, -1, ex);
 	if (!msg)
@@ -1425,16 +1485,16 @@ transfer_message_to (CamelFolder *source, const char *uid, CamelFolder *dest,
 		camel_folder_free_message_info(source, minfo);
 	} else
 		info = camel_message_info_new_from_header(NULL, ((CamelMimePart *)msg)->headers);
-	
+
 	/* we don't want to retain the deleted flag */
 	camel_message_info_set_flags(info, CAMEL_MESSAGE_DELETED, 0);
-	
+
 	camel_folder_append_message (dest, msg, info, transferred_uid, ex);
 	camel_object_unref (msg);
-	
+
 	if (delete_original && !camel_exception_is_set (ex))
 		camel_folder_set_message_flags (source, uid, CAMEL_MESSAGE_DELETED|CAMEL_MESSAGE_SEEN, ~0);
-	
+
 	camel_message_info_free (info);
 }
 
@@ -1454,7 +1514,7 @@ transfer_messages_to (CamelFolder *source, GPtrArray *uids, CamelFolder *dest, G
 	if (ex == NULL)
 		ex = &local;
 
-	camel_operation_start(NULL, delete_originals ? "Moving messages" : "Copying messages");
+	camel_operation_start(NULL, delete_originals ? _("Moving messages") : _("Copying messages"));
 
 	if (uids->len > 1) {
 		camel_folder_freeze(dest);
@@ -1465,7 +1525,7 @@ transfer_messages_to (CamelFolder *source, GPtrArray *uids, CamelFolder *dest, G
 		if (transferred_uids)
 			ret_uid = (char **)&((*transferred_uids)->pdata[i]);
 		transfer_message_to (source, uids->pdata[i], dest, ret_uid, delete_originals, ex);
-		camel_operation_progress(NULL, i , uids->len);
+		camel_operation_progress(NULL, i, uids->len);
 	}
 	if (uids->len > 1) {
 		camel_folder_thaw(dest);
@@ -1500,12 +1560,12 @@ camel_folder_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 	g_return_if_fail (CAMEL_IS_FOLDER (source));
 	g_return_if_fail (CAMEL_IS_FOLDER (dest));
 	g_return_if_fail (uids != NULL);
-	
+
 	if (source == dest || uids->len == 0) {
 		/* source and destination folders are the same, or no work to do, do nothing. */
 		return;
 	}
-	
+
 	if (source->parent_store == dest->parent_store) {
 		/* If either folder is a vtrash, we need to use the
 		 * vtrash transfer method.
@@ -1536,15 +1596,15 @@ void
 camel_folder_delete (CamelFolder *folder)
 {
 	g_return_if_fail (CAMEL_IS_FOLDER (folder));
-	
+
 	CAMEL_FOLDER_REC_LOCK (folder, lock);
 	if (folder->folder_flags & CAMEL_FOLDER_HAS_BEEN_DELETED) {
 		CAMEL_FOLDER_REC_UNLOCK (folder, lock);
 		return;
 	}
-	
+
 	folder->folder_flags |= CAMEL_FOLDER_HAS_BEEN_DELETED;
-	
+
 	CF_CLASS (folder)->delete (folder);
 
 	CAMEL_FOLDER_REC_UNLOCK (folder, lock);
@@ -1571,7 +1631,7 @@ folder_rename (CamelFolder *folder, const char *new)
  * camel_folder_rename:
  * @folder: a #CamelFolder object
  * @new: new name for the folder
- * 
+ *
  * Mark an active folder object as renamed.
  *
  * NOTE: This is an internal function used by camel stores, no locking
@@ -1639,7 +1699,7 @@ thaw (CamelFolder * folder)
 		info = folder->priv->changed_frozen;
 		folder->priv->changed_frozen = camel_folder_change_info_new();
 	}
-	
+
 	CAMEL_FOLDER_UNLOCK(folder, change_lock);
 
 	if (info) {
@@ -1709,12 +1769,12 @@ filter_filter(CamelSession *session, CamelSessionThreadMsg *tmsg)
 	CamelJunkPlugin *csp = ((CamelService *)m->folder->parent_store)->session->junk_plugin;
 
 	if (m->junk) {
-		camel_operation_start (NULL, "Learning junk");
+		camel_operation_start (NULL, _("Learning junk"));
 
 		for (i = 0; i < m->junk->len; i ++) {
 			/* TNY TODO: Partial message retrieval exception */
 			CamelMimeMessage *msg = camel_folder_get_message(m->folder, m->junk->pdata[i], CAMEL_FOLDER_RECEIVE_FULL, -1, NULL);
-			
+
 			camel_operation_progress(NULL, i, m->junk->len);
 
 			if (msg) {
@@ -1726,11 +1786,11 @@ filter_filter(CamelSession *session, CamelSessionThreadMsg *tmsg)
 	}
 
 	if (m->notjunk) {
-		camel_operation_start (NULL, "Learning non-junk");
+		camel_operation_start (NULL, _("Learning non-junk"));
 		for (i = 0; i < m->notjunk->len; i ++) {
 			/* TNY TODO: Partial message retrieval exception */
 			CamelMimeMessage *msg = camel_folder_get_message(m->folder, m->notjunk->pdata[i], CAMEL_FOLDER_RECEIVE_FULL, -1, NULL);
-			
+
 			camel_operation_progress(NULL, i, m->notjunk->len);
 
 			if (msg) {
@@ -1745,7 +1805,7 @@ filter_filter(CamelSession *session, CamelSessionThreadMsg *tmsg)
 		camel_junk_plugin_commit_reports (csp);
 
 	if (m->driver && m->recents) {
-		camel_operation_start(NULL, "Filtering new message(s)");
+		camel_operation_start(NULL, _("Filtering new message(s)"));
 
 		source_url = camel_service_get_url((CamelService *)m->folder->parent_store);
 		uri = camel_url_new(source_url, NULL);
@@ -1918,7 +1978,7 @@ camel_folder_free_deep (CamelFolder *folder, GPtrArray *array)
 
 /**
  * camel_folder_change_info_new:
- * 
+ *
  * Create a new folder change info structure.
  *
  * Change info structures are not MT-SAFE and must be
@@ -1951,18 +2011,18 @@ camel_folder_change_info_new(void)
  * camel_folder_change_info_add_source:
  * @info: a #CamelFolderChangeInfo
  * @uid: a uid
- * 
+ *
  * Add a source uid for generating a changeset.
  **/
 void
 camel_folder_change_info_add_source(CamelFolderChangeInfo *info, const char *uid)
 {
 	struct _CamelFolderChangeInfoPrivate *p;
-	
+
 	g_assert(info != NULL);
-	
+
 	p = info->priv;
-	
+
 	if (p->uid_source == NULL)
 		p->uid_source = g_hash_table_new(g_str_hash, g_str_equal);
 
@@ -1975,7 +2035,7 @@ camel_folder_change_info_add_source(CamelFolderChangeInfo *info, const char *uid
  * camel_folder_change_info_add_source_list:
  * @info: a #CamelFolderChangeInfo
  * @list: a list of uids
- * 
+ *
  * Add a list of source uid's for generating a changeset.
  **/
 void
@@ -1983,10 +2043,10 @@ camel_folder_change_info_add_source_list(CamelFolderChangeInfo *info, const GPtr
 {
 	struct _CamelFolderChangeInfoPrivate *p;
 	int i;
-	
+
 	g_assert(info != NULL);
 	g_assert(list != NULL);
-	
+
 	p = info->priv;
 
 	if (p->uid_source == NULL)
@@ -2005,7 +2065,7 @@ camel_folder_change_info_add_source_list(CamelFolderChangeInfo *info, const GPtr
  * camel_folder_change_info_add_update:
  * @info: a #CamelFolderChangeInfo
  * @uid: a uid
- * 
+ *
  * Add a uid from the updated list, used to generate a changeset diff.
  **/
 void
@@ -2014,17 +2074,17 @@ camel_folder_change_info_add_update(CamelFolderChangeInfo *info, const char *uid
 	struct _CamelFolderChangeInfoPrivate *p;
 	char *key;
 	int value;
-	
+
 	g_assert(info != NULL);
-	
+
 	p = info->priv;
-	
+
 	if (p->uid_source == NULL) {
 		camel_folder_change_info_add_uid(info, uid);
 		return;
 	}
 
-	if (g_hash_table_lookup_extended(p->uid_source, uid, (void **)&key, (void **)&value)) {
+	if (g_hash_table_lookup_extended(p->uid_source, uid, (gpointer) &key, (gpointer) &value)) {
 		g_hash_table_remove(p->uid_source, key);
 	} else {
 		camel_folder_change_info_add_uid(info, uid);
@@ -2036,17 +2096,17 @@ camel_folder_change_info_add_update(CamelFolderChangeInfo *info, const char *uid
  * camel_folder_change_info_add_update_list:
  * @info: a #CamelFolderChangeInfo
  * @list: a list of uids
- * 
+ *
  * Add a list of uid's from the updated list.
  **/
 void
 camel_folder_change_info_add_update_list(CamelFolderChangeInfo *info, const GPtrArray *list)
 {
 	int i;
-	
+
 	g_assert(info != NULL);
 	g_assert(list != NULL);
-	
+
 	for (i=0;i<list->len;i++)
 		camel_folder_change_info_add_update(info, list->pdata[i]);
 }
@@ -2058,7 +2118,7 @@ change_info_remove(char *key, void *value, CamelFolderChangeInfo *info)
 	GPtrArray *olduids;
 	char *olduid;
 
-	if (g_hash_table_lookup_extended(p->uid_stored, key, (void **)&olduid, (void **)&olduids)) {
+	if (g_hash_table_lookup_extended(p->uid_stored, key, (gpointer) &olduid, (gpointer) &olduids)) {
 		/* if it was added/changed them removed, then remove it */
 		if (olduids != info->uid_removed) {
 			g_ptr_array_remove_fast(olduids, olduid);
@@ -2077,7 +2137,7 @@ change_info_remove(char *key, void *value, CamelFolderChangeInfo *info)
 /**
  * camel_folder_change_info_build_diff:
  * @info: a #CamelFolderChangeInfo
- * 
+ *
  * Compare the source uid set to the updated uid set and generate the
  * differences into the added and removed lists.
  **/
@@ -2085,11 +2145,11 @@ void
 camel_folder_change_info_build_diff(CamelFolderChangeInfo *info)
 {
 	struct _CamelFolderChangeInfoPrivate *p;
-	
+
 	g_assert(info != NULL);
-	
+
 	p = info->priv;
-	
+
 	if (p->uid_source) {
 		g_hash_table_foreach(p->uid_source, (GHFunc)change_info_remove, info);
 		g_hash_table_destroy(p->uid_source);
@@ -2103,10 +2163,10 @@ change_info_recent_uid(CamelFolderChangeInfo *info, const char *uid)
 	struct _CamelFolderChangeInfoPrivate *p;
 	GPtrArray *olduids;
 	char *olduid;
-	
+
 	p = info->priv;
 
-	/* always add to recent, but dont let anyone else know */	
+	/* always add to recent, but dont let anyone else know */
 	if (!g_hash_table_lookup_extended(p->uid_stored, uid, (void **)&olduid, (void **)&olduids)) {
 		olduid = e_mempool_strdup(p->uid_pool, uid);
 	}
@@ -2119,10 +2179,10 @@ change_info_filter_uid(CamelFolderChangeInfo *info, const char *uid)
 	struct _CamelFolderChangeInfoPrivate *p;
 	GPtrArray *olduids;
 	char *olduid;
-	
+
 	p = info->priv;
 
-	/* always add to filter, but dont let anyone else know */	
+	/* always add to filter, but dont let anyone else know */
 	if (!g_hash_table_lookup_extended(p->uid_stored, uid, (void **)&olduid, (void **)&olduids)) {
 		olduid = e_mempool_strdup(p->uid_pool, uid);
 	}
@@ -2143,7 +2203,7 @@ change_info_cat(CamelFolderChangeInfo *info, GPtrArray *source, void (*add)(Came
  * camel_folder_change_info_cat:
  * @info: a #CamelFolderChangeInfo to append to
  * @src: a #CamelFolderChangeInfo to append from
- * 
+ *
  * Concatenate one change info onto antoher.  Can be used to copy them
  * too.
  **/
@@ -2152,7 +2212,7 @@ camel_folder_change_info_cat(CamelFolderChangeInfo *info, CamelFolderChangeInfo
 {
 	g_assert(info != NULL);
 	g_assert(source != NULL);
-	
+
 	change_info_cat(info, source->uid_added, camel_folder_change_info_add_uid);
 	change_info_cat(info, source->uid_removed, camel_folder_change_info_remove_uid);
 	change_info_cat(info, source->uid_changed, camel_folder_change_info_change_uid);
@@ -2164,7 +2224,7 @@ camel_folder_change_info_cat(CamelFolderChangeInfo *info, CamelFolderChangeInfo
  * camel_folder_change_info_add_uid:
  * @info: a #CamelFolderChangeInfo
  * @uid: a uid
- * 
+ *
  * Add a new uid to the changeinfo.
  **/
 void
@@ -2175,10 +2235,10 @@ camel_folder_change_info_add_uid(CamelFolderChangeInfo *info, const char *uid)
 	char *olduid;
 
 	g_assert(info != NULL);
-	
+
 	p = info->priv;
-	
-	if (g_hash_table_lookup_extended(p->uid_stored, uid, (void **)&olduid, (void **)&olduids)) {
+
+	if (g_hash_table_lookup_extended(p->uid_stored, uid, (gpointer) &olduid, (gpointer) &olduids)) {
 		/* if it was removed then added, promote it to a changed */
 		/* if it was changed then added, leave as changed */
 		if (olduids == info->uid_removed) {
@@ -2199,7 +2259,7 @@ camel_folder_change_info_add_uid(CamelFolderChangeInfo *info, const char *uid)
  * camel_folder_change_info_remove_uid:
  * @info: a #CamelFolderChangeInfo
  * @uid: a uid
- * 
+ *
  * Add a uid to the removed uid list.
  **/
 void
@@ -2208,12 +2268,12 @@ camel_folder_change_info_remove_uid(CamelFolderChangeInfo *info, const char *uid
 	struct _CamelFolderChangeInfoPrivate *p;
 	GPtrArray *olduids;
 	char *olduid;
-	
+
 	g_assert(info != NULL);
-	
+
 	p = info->priv;
-	
-	if (g_hash_table_lookup_extended(p->uid_stored, uid, (void **)&olduid, (void **)&olduids)) {
+
+	if (g_hash_table_lookup_extended(p->uid_stored, uid, (gpointer) &olduid, (gpointer) &olduids)) {
 		/* if it was added/changed them removed, then remove it */
 		if (olduids != info->uid_removed) {
 			g_ptr_array_remove_fast(olduids, olduid);
@@ -2233,7 +2293,7 @@ camel_folder_change_info_remove_uid(CamelFolderChangeInfo *info, const char *uid
  * camel_folder_change_info_change_uid:
  * @info: a #CamelFolderChangeInfo
  * @uid: a uid
- * 
+ *
  * Add a uid to the changed uid list.
  **/
 void
@@ -2242,12 +2302,12 @@ camel_folder_change_info_change_uid(CamelFolderChangeInfo *info, const char *uid
 	struct _CamelFolderChangeInfoPrivate *p;
 	GPtrArray *olduids;
 	char *olduid;
-	
+
 	g_assert(info != NULL);
-	
+
 	p = info->priv;
-	
-	if (g_hash_table_lookup_extended(p->uid_stored, uid, (void **)&olduid, (void **)&olduids)) {
+
+	if (g_hash_table_lookup_extended(p->uid_stored, uid, (gpointer) &olduid, (gpointer) &olduids)) {
 		/* if we have it already, leave it as that */
 		return;
 	}
@@ -2271,7 +2331,7 @@ void
 camel_folder_change_info_recent_uid(CamelFolderChangeInfo *info, const char *uid)
 {
 	g_assert(info != NULL);
-	
+
 	change_info_recent_uid(info, uid);
 	change_info_filter_uid(info, uid);
 }
@@ -2289,7 +2349,7 @@ gboolean
 camel_folder_change_info_changed(CamelFolderChangeInfo *info)
 {
 	g_assert(info != NULL);
-	
+
 	return (info->uid_added->len || info->uid_removed->len || info->uid_changed->len || info->uid_recent->len);
 }
 
@@ -2297,7 +2357,7 @@ camel_folder_change_info_changed(CamelFolderChangeInfo *info)
 /**
  * camel_folder_change_info_clear:
  * @info: a #CamelFolderChangeInfo
- * 
+ *
  * Empty out the change info; called after changes have been
  * processed.
  **/
@@ -2305,11 +2365,11 @@ void
 camel_folder_change_info_clear(CamelFolderChangeInfo *info)
 {
 	struct _CamelFolderChangeInfoPrivate *p;
-	
+
 	g_assert(info != NULL);
 	info->push_email_event = FALSE;
 	p = info->priv;
-	
+
 	g_ptr_array_set_size(info->uid_added, 0);
 	g_ptr_array_set_size(info->uid_removed, 0);
 	g_ptr_array_set_size(info->uid_changed, 0);
@@ -2328,7 +2388,7 @@ camel_folder_change_info_clear(CamelFolderChangeInfo *info)
 /**
  * camel_folder_change_info_free:
  * @info: a #CamelFolderChangeInfo
- * 
+ *
  * Free memory associated with the folder change info lists.
  **/
 void
@@ -2337,9 +2397,9 @@ camel_folder_change_info_free(CamelFolderChangeInfo *info)
 	struct _CamelFolderChangeInfoPrivate *p;
 
 	g_assert(info != NULL);
-	
+
 	p = info->priv;
-	
+
 	if (p->uid_source)
 		g_hash_table_destroy(p->uid_source);
 
