@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
 
@@ -160,13 +160,13 @@ pair_alloc(void)
 	static guint next_id = 1;
 	CamelHookPair *pair;
 
-	pair = g_slice_new(CamelHookPair);
+	pair = g_slice_new (CamelHookPair);
 
-	g_static_mutex_lock(&mutex);
+	g_static_mutex_lock (&mutex);
 	pair->id = next_id++;
 	if (next_id == 0)
 		next_id = 1;
-	g_static_mutex_unlock(&mutex);
+	g_static_mutex_unlock (&mutex);
 
 	return pair;
 }
@@ -314,8 +314,8 @@ cobject_meta_get(CamelObject *obj, const char * name)
 	CamelObjectMeta *meta;
 	char *res = NULL;
 
-	g_return_val_if_fail(CAMEL_IS_OBJECT (obj), 0);
-	g_return_val_if_fail(name != NULL, 0);
+	g_return_val_if_fail(CAMEL_IS_OBJECT (obj), NULL);
+	g_return_val_if_fail(name != NULL, NULL);
 
 	pair = co_metadata_pair(obj, FALSE);
 	if (pair) {
@@ -329,7 +329,7 @@ cobject_meta_get(CamelObject *obj, const char * name)
 		}
 		camel_object_unget_hooks(obj);
 	}
-	
+
 	return res;
 }
 
@@ -413,7 +413,7 @@ cobject_state_read(CamelObject *obj, FILE *fp)
 
 	for (i=0;i<count;i++) {
 		char *name = NULL, *value = NULL;
-			
+
 		if (camel_file_util_decode_string(fp, &name) == 0
 		    && camel_file_util_decode_string(fp, &value) == 0) {
 			camel_object_meta_set(obj, name, value);
@@ -437,13 +437,13 @@ cobject_state_read(CamelObject *obj, FILE *fp)
 		}
 
 		count = MIN(count, CAMEL_ARGV_MAX);
-		
+
 		/* we batch up the properties and set them in one go */
 		argv = g_try_malloc(sizeof(CamelArgV) -
 			((CAMEL_ARGV_MAX - count) * sizeof(CamelArg)));
 		if (argv == NULL)
 			return -1;
-		
+
 		argv->argc = 0;
 		for (i=0;i<count;i++) {
 			if (camel_file_util_decode_uint32(fp, &argv->argv[argv->argc].tag) == -1)
@@ -455,7 +455,7 @@ cobject_state_read(CamelObject *obj, FILE *fp)
 			switch(argv->argv[argv->argc].tag & CAMEL_ARG_TYPE) {
 			case CAMEL_ARG_INT:
 			case CAMEL_ARG_BOO:
-				if (camel_file_util_decode_uint32(fp, (guint32*) &argv->argv[argv->argc].ca_int) == -1)
+				if (camel_file_util_decode_uint32(fp, (guint32 *) &argv->argv[argv->argc].ca_int) == -1)
 					goto cleanup;
 				break;
 			case CAMEL_ARG_STR:
@@ -681,7 +681,7 @@ camel_object_get_type(void)
 							   0, sizeof(CamelInterface),
 							   cinterface_class_init, cinterface_class_finalise,
 							   cinterface_init, NULL);
-							   
+
 	}
 
 	return camel_object_type;
@@ -749,7 +749,7 @@ co_type_register(CamelType parent, const char * name,
 		return NULL;
 	}
 
-	klass = g_slice_alloc (klass_size);
+	klass = g_slice_alloc0 (klass_size);
 	klass->klass_size = klass_size;
 	klass->object_size = object_size;
 	klass->lock = g_mutex_new();
@@ -842,9 +842,7 @@ camel_object_new(CamelType type)
 	camel_object_init(o, type, type);
 
 	if (o->flags & CAMEL_OBJECT_REF_DEBUG)
-	{
 		printf ("An object of the type that you are debugging got created\n");
-	}
 
 	d(printf("%p: new %s()\n", o, o->klass->name));
 
@@ -859,11 +857,8 @@ camel_object_ref(void *vo)
 	if (o->ref_count > 0)
 		g_return_if_fail(CAMEL_IS_OBJECT(o));
 
-
 	if (o->flags & CAMEL_OBJECT_REF_DEBUG)
-	{
 		printf ("An object of the type that you are debugging got referenced\n");
-	}
 
 	REF_LOCK();
 
@@ -881,12 +876,11 @@ camel_object_unref(void *vo)
 	CamelHookList *hooks = NULL;
 
 	g_return_if_fail(CAMEL_IS_OBJECT(o));
+
 	klass = o->klass;
 
 	if (o->flags & CAMEL_OBJECT_REF_DEBUG)
-	{
 		printf ("An object of the type that you are debugging got unreferenced\n");
-	}
 
 	if (o->hooks)
 		hooks = camel_object_get_hooks(o);
@@ -906,9 +900,7 @@ camel_object_unref(void *vo)
 	}
 
 	if (o->flags & CAMEL_OBJECT_REF_DEBUG)
-	{
 		printf ("An object of the type that you are debugging is about to be finalized\n");
-	}
 
 	o->flags |= CAMEL_OBJECT_DESTROY;
 
@@ -1023,7 +1015,7 @@ camel_object_cast(CamelObject *o, CamelType ctype)
 		k = k->parent;
 	}
 
-	g_warning("Object %p (class '%s') doesn't have '%s' in its hierarchy", o, o->klass->name, ctype->name);
+	g_warning("Object %p (class '%s') doesn't have '%s' in its hierarchy", (void *) o, o->klass->name, ctype->name);
 
 	return NULL;
 }
@@ -1203,7 +1195,6 @@ camel_object_get_hooks(CamelObject *o)
 {
 	static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
 	CamelHookList *hooks;
-	CamelHookList *retval = NULL;
 
 	g_static_rec_mutex_lock (&hooks_lock);
 
@@ -1222,14 +1213,12 @@ camel_object_get_hooks(CamelObject *o)
 		}
 		pthread_mutex_unlock(&lock);
 	}
-	
-	g_static_rec_mutex_lock(&(o->hooks->lock));
+
+	g_static_rec_mutex_lock(&o->hooks->lock);
 
 	g_static_rec_mutex_unlock (&hooks_lock);
 
-	retval = o->hooks;
-
-	return retval;
+	return o->hooks;
 }
 
 unsigned int
@@ -1301,10 +1290,9 @@ camel_object_remove_event(void *vo, unsigned int id)
 
 	g_return_if_fail (CAMEL_IS_OBJECT (obj));
 
-	if (id == 0) 
+	if (id == 0)
 		return;
 
-
 	g_static_rec_mutex_lock (&hooks_lock);
 
 	if (obj->hooks == NULL) {
@@ -1394,7 +1382,7 @@ camel_object_unhook_event(void *vo, const char * name, CamelObjectEventHookFunc
 	camel_object_unget_hooks(obj);
 
 	g_warning("camel_object_unhook_event: cannot find hook/data pair %p/%p in an instance of `%s' attached to `%s'",
-		  func, data, obj->klass->name, name);
+		  (void *) func, data, obj->klass->name, name);
 	g_static_rec_mutex_unlock (&hooks_lock);
 }
 
@@ -1426,7 +1414,7 @@ camel_object_trigger_event(void *vo, const char * name, void *event_data)
 	pair = co_find_pair_ptr(obj->klass, interface_name);
 	if (pair) {
 		GPtrArray *interfaces = pair->data;
-		
+
 		for (i=0;i<interfaces->len;i++) {
 			hook = co_find_pair(interfaces->pdata[i], name);
 			if (hook)
@@ -1458,7 +1446,7 @@ trigger_interface:
 	/* lock the object for hook emission */
 	camel_object_ref(obj);
 	hooks = camel_object_get_hooks(obj);
-	
+
 	if (hooks->list) {
 		/* first, copy the items in the list, and say we're in an event */
 		hooks->depth++;
@@ -1589,7 +1577,7 @@ void *camel_object_get_ptr(void *vo, CamelException *ex, int tag)
 	g_return_val_if_fail(CAMEL_IS_OBJECT(o), NULL);
 	g_return_val_if_fail((tag & CAMEL_ARG_TYPE) == CAMEL_ARG_OBJ
 			     || (tag & CAMEL_ARG_TYPE) == CAMEL_ARG_STR
-			     || (tag & CAMEL_ARG_TYPE) == CAMEL_ARG_PTR, 0);
+			     || (tag & CAMEL_ARG_TYPE) == CAMEL_ARG_PTR, NULL);
 
 	/* woefully inefficient, *shrug */
 	args.argc = 1;
@@ -1721,11 +1709,11 @@ co_metadata_free(CamelObject *obj, CamelObjectMeta *meta)
 
 /**
  * camel_object_meta_get:
- * @vo: 
- * @name: 
- * 
+ * @vo:
+ * @name:
+ *
  * Get a meta-data on an object.
- * 
+ *
  * Return value: NULL if the meta-data is not set.
  **/
 char *
@@ -1733,18 +1721,18 @@ camel_object_meta_get(void *vo, const char * name)
 {
 	CamelObject *obj = vo;
 
-	g_return_val_if_fail(CAMEL_IS_OBJECT (obj), 0);
-	g_return_val_if_fail(name != NULL, 0);
+	g_return_val_if_fail(CAMEL_IS_OBJECT (obj), NULL);
+	g_return_val_if_fail(name != NULL, NULL);
 
 	return obj->klass->meta_get(obj, name);
 }
 
 /**
  * camel_object_meta_set:
- * @vo: 
+ * @vo:
  * @name: Name of meta-data.  Should be prefixed with class of setter.
  * @value: Value to set.  If NULL, then the meta-data is removed.
- * 
+ *
  * Set a meta-data item on an object.  If the object supports persistent
  * data, then the meta-data will be persistent across sessions.
  *
@@ -1773,10 +1761,10 @@ camel_object_meta_set(void *vo, const char * name, const char *value)
 
 /**
  * camel_object_state_read:
- * @vo: 
- * 
+ * @vo:
+ *
  * Read persistent object state from object_set(CAMEL_OBJECT_STATE_FILE).
- * 
+ *
  * Return value: -1 on error.
  **/
 int camel_object_state_read(void *vo)
@@ -1808,10 +1796,10 @@ int camel_object_state_read(void *vo)
 
 /**
  * camel_object_state_write:
- * @vo: 
- * 
+ * @vo:
+ *
  * Write persistent state to the file as set by object_set(CAMEL_OBJECT_STATE_FILE).
- * 
+ *
  * Return value: -1 on error.
  **/
 int camel_object_state_write(void *vo)
@@ -1827,7 +1815,7 @@ int camel_object_state_write(void *vo)
 
 	savename = camel_file_util_savename(file);
 	dirname = g_path_get_dirname(savename);
-	e_util_mkdir_hier(dirname, 0777);
+	g_mkdir_with_parents(dirname, 0777);
 	g_free(dirname);
 	fp = g_fopen(savename, "wb");
 	if (fp != NULL) {
@@ -1859,7 +1847,7 @@ void camel_object_free(void *vo, guint32 tag, void *value)
 
 	   Add a 'const' bit to the arg type field,
 	   specifying that the object should not be freed.
-	   
+
 	   And, add free handlers for any pointer objects which are
 	   not const.  The free handlers would be hookpairs off of the
 	   class.
@@ -1891,7 +1879,7 @@ object_class_dump_tree_rec(CamelType root, int depth)
 			CamelHookPair *pair = root->hooks;
 
 			while (pair) {
-				printf("%s  event '%s' prep %p\n", p, pair->name, pair->func.prep);
+				printf("%s  event '%s' prep %p\n", p, pair->name, (void *) pair->func.prep);
 				pair = pair->next;
 			}
 		}
@@ -1913,16 +1901,16 @@ camel_object_class_dump_tree(CamelType root)
 
 /**
  * camel_object_bag_new:
- * @hash: 
- * @equal: 
- * @keycopy: 
- * @keyfree: 
- * 
+ * @hash:
+ * @equal:
+ * @keycopy:
+ * @keyfree:
+ *
  * Allocate a new object bag.  Object bag's are key'd hash tables of
  * camel-objects which can be updated atomically using transaction
  * semantics.
- * 
- * Return value: 
+ *
+ * Return value:
  **/
 CamelObjectBag *
 camel_object_bag_new(GHashFunc hash, GEqualFunc equal, CamelCopyFunc keycopy, GFreeFunc keyfree)
@@ -1957,7 +1945,7 @@ camel_object_bag_destroy(CamelObjectBag *bag)
 	g_hash_table_foreach(bag->object_table, (GHFunc)save_object, objects);
 	for (i=0;i<objects->len;i++)
 		camel_object_bag_remove(bag, objects->pdata[i]);
-	
+
 	g_ptr_array_free(objects, TRUE);
 	g_hash_table_destroy(bag->object_table);
 	g_hash_table_destroy(bag->key_table);
@@ -1997,10 +1985,10 @@ co_bag_unreserve(CamelObjectBag *bag, const void *key)
 
 /**
  * camel_object_bag_add:
- * @bag: 
- * @key: 
- * @vo: 
- * 
+ * @bag:
+ * @key:
+ * @vo:
+ *
  * Add an object @vo to the object bag @bag.  The @key MUST have
  * previously been reserved using camel_object_bag_reserve().
  **/
@@ -2040,19 +2028,19 @@ camel_object_bag_add(CamelObjectBag *bag, const void *key, void *vo)
 	g_hash_table_insert(bag->key_table, vo, k);
 
 	co_bag_unreserve(bag, key);
-	
+
 	REF_UNLOCK();
 	camel_object_unget_hooks(o);
 }
 
 /**
  * camel_object_bag_get:
- * @bag: 
- * @key: 
- * 
+ * @bag:
+ * @key:
+ *
  * Lookup an object by @key.  If the key is currently reserved, then
  * wait until the key has been committed before continuing.
- * 
+ *
  * Return value: NULL if the object corresponding to @key is not
  * in the bag.  Otherwise a ref'd object pointer which the caller owns
  * the ref to.
@@ -2101,21 +2089,21 @@ camel_object_bag_get(CamelObjectBag *bag, const void *key)
 			co_bag_unreserve(bag, key);
 		}
 	}
-	
+
 	REF_UNLOCK();
-	
+
 	return o;
 }
 
 /**
  * camel_object_bag_peek:
- * @bag: 
- * @key: 
- * 
+ * @bag:
+ * @key:
+ *
  * Lookup the object @key in @bag, ignoring any reservations.  If it
  * isn't committed, then it isn't considered.  This should only be
  * used where reliable transactional-based state is not required.
- * 
+ *
  * Unlike other 'peek' operations, the object is still reffed if
  * found.
  *
@@ -2142,9 +2130,9 @@ camel_object_bag_peek(CamelObjectBag *bag, const void *key)
 
 /**
  * camel_object_bag_reserve:
- * @bag: 
- * @key: 
- * 
+ * @bag:
+ * @key:
+ *
  * Reserve a key in the object bag.  If the key is already reserved in
  * another thread, then wait until the reservation has been committed.
  *
@@ -2155,8 +2143,8 @@ camel_object_bag_peek(CamelObjectBag *bag, const void *key)
  *
  * You may reserve multiple keys from the same thread, but they should
  * always be reserved in the same order, to avoid deadlocks.
- * 
- * Return value: 
+ *
+ * Return value:
  **/
 void *
 camel_object_bag_reserve(CamelObjectBag *bag, const void *key)
@@ -2209,7 +2197,7 @@ camel_object_bag_reserve(CamelObjectBag *bag, const void *key)
 			bag->reserved = res;
 		}
 	}
-	
+
 	REF_UNLOCK();
 
 	return o;
@@ -2217,9 +2205,9 @@ camel_object_bag_reserve(CamelObjectBag *bag, const void *key)
 
 /**
  * camel_object_bag_abort:
- * @bag: 
- * @key: 
- * 
+ * @bag:
+ * @key:
+ *
  * Abort a key reservation.
  **/
 void
@@ -2234,10 +2222,10 @@ camel_object_bag_abort(CamelObjectBag *bag, const void *key)
 
 /**
  * camel_object_bag_rekey:
- * @bag: 
- * @o: 
- * @newkey: 
- * 
+ * @bag:
+ * @o:
+ * @newkey:
+ *
  * Re-key an object, atomically.  The key for object @o is set to
  * @newkey, in an atomic manner.
  *
@@ -2332,7 +2320,7 @@ camel_object_bag_remove(CamelObjectBag *inbag, void *vo)
 	REF_LOCK();
 
 	camel_object_bag_remove_unlocked(inbag, o, hooks);
-		
+
 	REF_UNLOCK();
 	camel_object_unget_hooks(o);
 }
@@ -2367,7 +2355,7 @@ int camel_iterator_length(void *it)
 {
 	g_assert(it);
 	g_return_val_if_fail(((CamelIterator *)it)->klass->length != NULL, 0);
-	
+
 	return ((CamelIterator *)it)->klass->length(it);
 }
 
