@@ -20,10 +20,10 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
- 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -73,12 +73,13 @@ camel_session_init (CamelSession *session)
 	session->online = TRUE;
 	session->network_state = TRUE;
 	session->priv = g_malloc0(sizeof(*session->priv));
-	
+
 	session->priv->lock = g_mutex_new();
 	session->priv->thread_lock = g_mutex_new();
 	session->priv->thread_id = 1;
 	session->priv->thread_active = g_hash_table_new(NULL, NULL);
 	session->priv->thread_pool = NULL;
+	session->priv->junk_headers = NULL;
 }
 
 static void
@@ -86,7 +87,7 @@ camel_session_finalise (CamelObject *o)
 {
 	CamelSession *session = (CamelSession *)o;
 	GThreadPool *thread_pool = session->priv->thread_pool;
-	
+
 	g_hash_table_destroy(session->priv->thread_active);
 
 	if (thread_pool != NULL) {
@@ -96,10 +97,13 @@ camel_session_finalise (CamelObject *o)
 	}
 
 	g_free(session->storage_path);
-	
+
 	g_mutex_free(session->priv->lock);
 	g_mutex_free(session->priv->thread_lock);
-	
+	if (session->priv->junk_headers) {
+		g_hash_table_remove_all (session->priv->junk_headers);
+		g_hash_table_destroy (session->priv->junk_headers);
+	}
 	g_free(session->priv);
 }
 
@@ -109,7 +113,7 @@ camel_session_class_init (CamelSessionClass *camel_session_class)
 	/* virtual method definition */
 	camel_session_class->get_service = get_service;
 	camel_session_class->get_storage_path = get_storage_path;
-	
+
 	camel_session_class->thread_msg_new = session_thread_msg_new;
 	camel_session_class->thread_msg_free = session_thread_msg_free;
 	camel_session_class->thread_queue = session_thread_queue;
@@ -160,7 +164,7 @@ get_service (CamelSession *session, const char *url_string,
 	CamelProvider *provider;
 	CamelService *service;
 	CamelException internal_ex;
-	
+
 	url = camel_url_new (url_string, ex);
 	if (!url)
 		return NULL;
@@ -174,7 +178,7 @@ get_service (CamelSession *session, const char *url_string,
 				      url->protocol);
 		provider = NULL;
 	}
-	
+
 	if (!provider) {
 		camel_url_free (url);
 		return NULL;
@@ -185,7 +189,7 @@ get_service (CamelSession *session, const char *url_string,
 	 */
 	if (url->path && !CAMEL_PROVIDER_ALLOWS (provider, CAMEL_URL_PART_PATH))
 		camel_url_set_path (url, NULL);
-	
+
 	/* Now look up the service in the provider's cache */
 	service = camel_object_bag_reserve(provider->service_cache[type], url);
 	if (service == NULL) {
@@ -287,14 +291,14 @@ get_storage_path (CamelSession *session, CamelService *service, CamelException *
 	path = g_strdup_printf ("%s/%s", session->storage_path, p);
 	g_free (p);
 
-#ifdef G_OS_WIN32 
+#ifdef G_OS_WIN32
 	if (g_access (path, F_OK) == 0)
 #else
 	if (access (path, F_OK) == 0)
 #endif
 		return path;
 
-	if (e_util_mkdir_hier (path, S_IRWXU) == -1) {
+	if (g_mkdir_with_parents (path, S_IRWXU) == -1) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Could not create directory %s:\n%s"),
 				      path, g_strerror (errno));
@@ -352,14 +356,14 @@ camel_session_get_storage_path (CamelSession *session, CamelService *service,
  * @prompt is a question to ask the user (if the application doesn't
  * already have the answer cached). If #CAMEL_SESSION_PASSWORD_SECRET
  * is set, the user's input will not be echoed back.
- * This is a human-readable password question, possibly containing an untranslated error 
- * message directly from the server. This should be ignored by most implementations, 
+ * This is a human-readable password question, possibly containing an untranslated error
+ * message directly from the server. This should be ignored by most implementations,
  * so NULL is an acceptable value.
  *
  * If #CAMEL_SESSION_PASSWORD_STATIC is set, it means the password returned
  * will be stored statically by the caller automatically, for the current
  * session.
- * 
+ *
  * The authenticator should set @ex to #CAMEL_EXCEPTION_USER_CANCEL if
  * the user did not provide the information. The caller must #g_free
  * the information returned when it is done with it.
@@ -375,7 +379,7 @@ camel_session_get_password (CamelSession *session, CamelService *service,
 	g_return_val_if_fail (CAMEL_IS_SESSION (session), NULL);
 	g_return_val_if_fail (prompt != NULL, NULL);
 	g_return_val_if_fail (item != NULL, NULL);
-	
+
 	return CS_CLASS (session)->get_password (session, service, domain, prompt, item, flags, ex);
 }
 
@@ -432,10 +436,18 @@ camel_session_alert_user (CamelSession *session, CamelSessionAlertType type,
 	return CS_CLASS (session)->alert_user (session, type, ex, cancel, service);
 }
 
+gboolean
+camel_session_lookup_addressbook (CamelSession *session, const char *name)
+{
+	g_return_val_if_fail (CAMEL_IS_SESSION (session), FALSE);
+	g_return_val_if_fail (name != NULL, FALSE);
+	return CS_CLASS (session)->lookup_addressbook (session, name);
+}
+
 /**
  * camel_session_alert_user_with_id:
  *
- * Like camel_session_alert_user() but constructs and uses a CamelException 
+ * Like camel_session_alert_user() but constructs and uses a CamelException
  * with the error ID and the specified message.
 
  */
@@ -443,12 +455,15 @@ gboolean
 camel_session_alert_user_with_id (CamelSession *session, CamelSessionAlertType type,
 			  ExceptionId id, const gchar* message, gboolean cancel, CamelService *service)
 {
+	CamelException ex = CAMEL_EXCEPTION_INITIALISER;
+	gboolean result;
+
 	g_return_val_if_fail (message, FALSE);
 
-	CamelException ex = CAMEL_EXCEPTION_INITIALISER;
 	camel_exception_set (&ex, id, message);
-	gboolean result = camel_session_alert_user (session, type, &ex, cancel, service);
+	result = camel_session_alert_user (session, type, &ex, cancel, service);
 	camel_exception_clear (&ex);
+
 	return result;
 }
 
@@ -457,17 +472,60 @@ camel_session_alert_user_with_id (CamelSession *session, CamelSessionAlertType t
  *
  * Like camel_session_alert_user_with_id() but uses a
  * generic error ID.
- * camel_session_alert_user()or camel_session_alert_user_with_id() should be 
+ * camel_session_alert_user()or camel_session_alert_user_with_id() should be
  * used where possible, to provide a precise error ID.
  */
 gboolean
 camel_session_alert_user_generic (CamelSession *session, CamelSessionAlertType type,
 			  const gchar* message, gboolean cancel, CamelService *service)
 {
-	return camel_session_alert_user_with_id (session, type, 
+	return camel_session_alert_user_with_id (session, type,
 		CAMEL_EXCEPTION_SYSTEM, message, cancel, service);
 }
 
+/**
+ * camel_session_build_password_prompt:
+ * @type: account type (e.g. "IMAP")
+ * @user: user name for the account
+ * @host: host name for the account
+ *
+ * Constructs a localized password prompt from @type, @user and @host,
+ * suitable for passing to camel_session_get_password().  The resulting
+ * string contains markup tags.  Use g_free() to free it.
+ *
+ * Returns: a newly-allocated password prompt string
+ **/
+char *
+camel_session_build_password_prompt (const char *type,
+                                     const char *user,
+                                     const char *host)
+{
+	char *user_markup;
+	char *host_markup;
+	char *prompt;
+
+	g_return_val_if_fail (type != NULL, NULL);
+	g_return_val_if_fail (user != NULL, NULL);
+	g_return_val_if_fail (host != NULL, NULL);
+
+	/* Add bold tags to the "user" and "host" strings.  We use
+	 * separate strings here to avoid putting markup tags in the
+	 * translatable string below. */
+	user_markup = g_markup_printf_escaped ("<b>%s</b>", user);
+	host_markup = g_markup_printf_escaped ("<b>%s</b>", host);
+
+	/* Translators: The first argument is the account type
+	 * (e.g. "IMAP"), the second is the user name, and the
+	 * third is the host name. */
+	prompt = g_strdup_printf (
+		_("Please enter the %s password for %s on host %s."),
+		type, user_markup, host_markup);
+
+	g_free (user_markup);
+	g_free (host_markup);
+
+	return prompt;
+}
 
 /**
  * camel_session_is_online:
@@ -552,7 +610,7 @@ static void session_thread_msg_free(CamelSession *session, CamelSessionThreadMsg
 	CAMEL_SESSION_UNLOCK(session, thread_lock);
 
 	d(printf("free msg, ops->free = %p\n", msg->ops->free));
-	
+
 	if (msg->ops->free)
 		msg->ops->free(session, msg);
 	if (msg->op)
@@ -621,7 +679,7 @@ static void session_thread_status(CamelSession *session, CamelSessionThreadMsg *
  * @session: a #CamelSession object
  * @ops: thread operations
  * @size: number of bytes
- * 
+ *
  * Create a new thread message, using ops as the receive/reply/free
  * ops, of @size bytes.
  *
@@ -636,7 +694,7 @@ camel_session_thread_msg_new(CamelSession *session, CamelSessionThreadOps *ops,
 	g_assert(CAMEL_IS_SESSION(session));
 	g_assert(ops != NULL);
 	g_assert(size >= sizeof(CamelSessionThreadMsg));
-		 
+
 	return CS_CLASS (session)->thread_msg_new(session, ops, size);
 }
 
@@ -644,7 +702,7 @@ camel_session_thread_msg_new(CamelSession *session, CamelSessionThreadOps *ops,
  * camel_session_thread_msg_free:
  * @session: a #CamelSession object
  * @msg: a #CamelSessionThreadMsg
- * 
+ *
  * Free a @msg.  Note that the message must have been allocated using
  * msg_new, and must nto have been submitted to any queue function.
  **/
@@ -663,11 +721,11 @@ camel_session_thread_msg_free(CamelSession *session, CamelSessionThreadMsg *msg)
  * @session: a #CamelSession object
  * @msg: a #CamelSessionThreadMsg
  * @flags: queue type flags, currently 0.
- * 
+ *
  * Queue a thread message in another thread for processing.
  * The operation should be (but needn't) run in a queued manner
  * with other operations queued in this manner.
- * 
+ *
  * Returns the id of the operation queued
  **/
 int
@@ -683,14 +741,14 @@ camel_session_thread_queue(CamelSession *session, CamelSessionThreadMsg *msg, in
  * camel_session_thread_wait:
  * @session: a #CamelSession object
  * @id: id of the operation to wait on
- * 
+ *
  * Wait on an operation to complete (by id).
  **/
 void
 camel_session_thread_wait(CamelSession *session, int id)
 {
 	g_assert(CAMEL_IS_SESSION(session));
-	
+
 	if (id == -1)
 		return;
 
@@ -700,7 +758,7 @@ camel_session_thread_wait(CamelSession *session, int id)
 /**
  * camel_session_check_junk:
  * @session: a #CamelSession object
- * 
+ *
  * Do we have to check incoming messages to be junk?
  *
  * Returns whether or not we are checking incoming messages for junk
@@ -717,7 +775,7 @@ camel_session_check_junk (CamelSession *session)
  * camel_session_set_check_junk:
  * @session: a #CamelSession object
  * @check_junk: state
- * 
+ *
  * Set check_junk flag, if set, incoming mail will be checked for being junk.
  **/
 void
@@ -732,7 +790,7 @@ gboolean
 camel_session_get_network_state (CamelSession *session)
 {
 	g_return_val_if_fail (CAMEL_IS_SESSION(session), FALSE);
-	
+
 	return session->network_state;
 }
 
@@ -740,6 +798,29 @@ void
 camel_session_set_network_state (CamelSession *session, gboolean network_state)
 {
 	g_return_if_fail (CAMEL_IS_SESSION(session));
-	
+
 	session->network_state = network_state;
 }
+
+void
+camel_session_set_junk_headers (CamelSession *session, const char **headers, const char **values, int len)
+{
+	int i;
+
+	if (session->priv->junk_headers) {
+		g_hash_table_remove_all (session->priv->junk_headers);
+		g_hash_table_destroy (session->priv->junk_headers);
+	}
+
+	session->priv->junk_headers = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+
+	for (i=0; i<len; i++) {
+		g_hash_table_insert (session->priv->junk_headers, g_strdup (headers[i]), g_strdup(values[i]));
+	}
+}
+
+const GHashTable *
+camel_session_get_junk_headers (CamelSession *session)
+{
+	return session->priv->junk_headers;
+}
