@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -84,11 +84,12 @@ static int stream_setsockopt (CamelTcpStream *stream, const CamelSockOptData *da
 static struct sockaddr *stream_get_local_address (CamelTcpStream *stream, socklen_t *len);
 static struct sockaddr *stream_get_remote_address (CamelTcpStream *stream, socklen_t *len);
 static ssize_t stream_read_nb (CamelTcpStream *stream, char *buffer, size_t n);
-static ssize_t stream_read_idle (CamelStream *stream, char *buffer, size_t n);
+static int stream_gettimeout (CamelTcpStream *stream);
+
 
 struct _CamelTcpStreamSSLPrivate {
 	PRFileDesc *sockfd;
-	
+
 	struct _CamelSession *session;
 	char *expected_host;
 	gboolean ssl_mode;
@@ -104,17 +105,15 @@ camel_tcp_stream_ssl_class_init (CamelTcpStreamSSLClass *camel_tcp_stream_ssl_cl
 		CAMEL_TCP_STREAM_CLASS (camel_tcp_stream_ssl_class);
 	CamelStreamClass *camel_stream_class =
 		CAMEL_STREAM_CLASS (camel_tcp_stream_ssl_class);
-	
+
 	parent_class = CAMEL_TCP_STREAM_CLASS (camel_type_get_global_classfuncs (camel_tcp_stream_get_type ()));
-	
-	/* virtual method overload */
-	camel_stream_class->read_idle = stream_read_idle;
 
 	camel_stream_class->read = stream_read;
 	camel_stream_class->write = stream_write;
 	camel_stream_class->flush = stream_flush;
 	camel_stream_class->close = stream_close;
 
+	camel_tcp_stream_class->gettimeout = stream_gettimeout;
 	camel_tcp_stream_class->read_nb = stream_read_nb;
 	camel_tcp_stream_class->connect = stream_connect;
 	camel_tcp_stream_class->getsockopt = stream_getsockopt;
@@ -123,11 +122,17 @@ camel_tcp_stream_ssl_class_init (CamelTcpStreamSSLClass *camel_tcp_stream_ssl_cl
 	camel_tcp_stream_class->get_remote_address = stream_get_remote_address;
 }
 
+static int
+stream_gettimeout (CamelTcpStream *stream)
+{
+	return 330;
+}
+
 static void
 camel_tcp_stream_ssl_init (gpointer object, gpointer klass)
 {
 	CamelTcpStreamSSL *stream = CAMEL_TCP_STREAM_SSL (object);
-	
+
 	stream->priv = g_new0 (struct _CamelTcpStreamSSLPrivate, 1);
 }
 
@@ -135,15 +140,17 @@ static void
 camel_tcp_stream_ssl_finalize (CamelObject *object)
 {
 	CamelTcpStreamSSL *stream = CAMEL_TCP_STREAM_SSL (object);
-	
-	if (stream->priv->sockfd != NULL)
+
+	if (stream->priv->sockfd != NULL) {
+		PR_Shutdown (stream->priv->sockfd, PR_SHUTDOWN_BOTH);
 		PR_Close (stream->priv->sockfd);
+	}
 
 	if (stream->priv->session)
 		camel_object_unref(stream->priv->session);
 
 	g_free (stream->priv->expected_host);
-	
+
 	g_free (stream->priv);
 }
 
@@ -152,7 +159,7 @@ CamelType
 camel_tcp_stream_ssl_get_type (void)
 {
 	static CamelType type = CAMEL_INVALID_TYPE;
-	
+
 	if (type == CAMEL_INVALID_TYPE) {
 		type = camel_type_register (camel_tcp_stream_get_type (),
 					    "CamelTcpStreamSSL",
@@ -163,7 +170,7 @@ camel_tcp_stream_ssl_get_type (void)
 					    (CamelObjectInitFunc) camel_tcp_stream_ssl_init,
 					    (CamelObjectFinalizeFunc) camel_tcp_stream_ssl_finalize);
 	}
-	
+
 	return type;
 }
 
@@ -224,9 +231,9 @@ camel_tcp_stream_ssl_new_raw (CamelService *service, const char *expected_host,
 	CamelTcpStreamSSL *stream;
 
 	g_assert(CAMEL_IS_SESSION(service->session));
-	
+
 	stream = CAMEL_TCP_STREAM_SSL (camel_object_new (camel_tcp_stream_ssl_get_type ()));
-	
+
 	stream->priv->session = service->session;
 	camel_object_ref(service->session);
 	stream->priv->expected_host = g_strdup (expected_host);
@@ -325,26 +332,21 @@ camel_tcp_stream_ssl_enable_ssl (CamelTcpStreamSSL *ssl)
 	PR_SetSocketOption (ssl->priv->sockfd, &sockopts);
 
 	g_return_val_if_fail (CAMEL_IS_TCP_STREAM_SSL (ssl), -1);
-	
-	if (ssl->priv->sockfd && !ssl->priv->ssl_mode) 
-	{
 
-		if (!(fd = enable_ssl (ssl, NULL))) 
-		{
+	if (ssl->priv->sockfd && !ssl->priv->ssl_mode) {
+		if (!(fd = enable_ssl (ssl, NULL))) {
 			set_errno (PR_GetError ());
 			return -1;
 		}
 
 		ssl->priv->sockfd = fd;
 
-		if (SSL_ResetHandshake (fd, FALSE) == SECFailure) 
-		{
+		if (SSL_ResetHandshake (fd, FALSE) == SECFailure) {
 			set_errno (PR_GetError ());
 			return -1;
 		}
 
-		if (SSL_ForceHandshake (fd) == SECFailure) 
-		{
+		if (SSL_ForceHandshake (fd) == SECFailure) {
 			set_errno (PR_GetError ());
 			return -1;
 		}
@@ -380,7 +382,7 @@ stream_read_nb (CamelTcpStream *stream, char *buffer, size_t n)
 
 	pollfds[0].fd = tcp_stream_ssl->priv->sockfd;
 	pollfds[0].in_flags = PR_POLL_READ;
-	
+
 	do {
 		PRInt32 res;
 
@@ -398,18 +400,18 @@ stream_read_nb (CamelTcpStream *stream, char *buffer, size_t n)
 			errno = EIO;
 #endif
 		} else {
-			 do { 
+			 do {
 				nread = -1;
 				if (PR_Available (tcp_stream_ssl->priv->sockfd) != 0)
 					nread = PR_Read (tcp_stream_ssl->priv->sockfd, buffer, n);
 				if (nread == -1)
 					set_errno (PR_GetError ());
-			 } while (0 && (nread == -1 && PR_GetError () == PR_PENDING_INTERRUPT_ERROR)); 
+			 } while (0 && (nread == -1 && PR_GetError () == PR_PENDING_INTERRUPT_ERROR));
 		}
 	} while (0 && (nread == -1 && (PR_GetError () == PR_PENDING_INTERRUPT_ERROR ||
 				 PR_GetError () == PR_IO_PENDING_ERROR ||
 				 PR_GetError () == PR_WOULD_BLOCK_ERROR)));
-	
+
 	/* restore O_NONBLOCK options */
 	sockopts.option = PR_SockOpt_Nonblocking;
 	sockopts.value.non_blocking = nonblock;
@@ -424,20 +426,19 @@ stream_read (CamelStream *stream, char *buffer, size_t n)
 	CamelTcpStreamSSL *tcp_stream_ssl = CAMEL_TCP_STREAM_SSL (stream);
 	PRFileDesc *cancel_fd;
 	ssize_t nread;
-	
+
 	if (camel_operation_cancel_check (NULL)) {
 		errno = EINTR;
 		return -1;
 	}
-	
+
 	cancel_fd = camel_operation_cancel_prfd (NULL);
 	if (cancel_fd == NULL) {
-
 		PRSocketOptionData sockopts;
 		PRPollDesc pollfds[1];
 		gboolean nonblock;
 		int error;
-		
+
 		/* get O_NONBLOCK options */
 		sockopts.option = PR_SockOpt_Nonblocking;
 		PR_GetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
@@ -475,7 +476,7 @@ stream_read (CamelStream *stream, char *buffer, size_t n)
 		} while (nread == -1 && (PR_GetError () == PR_PENDING_INTERRUPT_ERROR ||
 					 PR_GetError () == PR_IO_PENDING_ERROR ||
 					 PR_GetError () == PR_WOULD_BLOCK_ERROR));
-		
+
 		/* restore O_NONBLOCK options */
 		error = errno;
 		sockopts.option = PR_SockOpt_Nonblocking;
@@ -488,7 +489,7 @@ stream_read (CamelStream *stream, char *buffer, size_t n)
 		PRPollDesc pollfds[2];
 		gboolean nonblock;
 		int error;
-		
+
 		/* get O_NONBLOCK options */
 		sockopts.option = PR_SockOpt_Nonblocking;
 		PR_GetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
@@ -501,7 +502,7 @@ stream_read (CamelStream *stream, char *buffer, size_t n)
 		pollfds[0].in_flags = PR_POLL_READ;
 		pollfds[1].fd = cancel_fd;
 		pollfds[1].in_flags = PR_POLL_READ;
-		
+
 		do {
 			PRInt32 res;
 
@@ -532,7 +533,7 @@ stream_read (CamelStream *stream, char *buffer, size_t n)
 		} while (nread == -1 && (PR_GetError () == PR_PENDING_INTERRUPT_ERROR ||
 					 PR_GetError () == PR_IO_PENDING_ERROR ||
 					 PR_GetError () == PR_WOULD_BLOCK_ERROR));
-		
+
 		/* restore O_NONBLOCK options */
 	failed:
 		error = errno;
@@ -541,137 +542,7 @@ stream_read (CamelStream *stream, char *buffer, size_t n)
 		PR_SetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
 		errno = error;
 	}
-	
-	return nread;
-}
 
-
-
-static ssize_t 
-stream_read_idle (CamelStream *stream, char *buffer, size_t n)
-{
-	CamelTcpStreamSSL *tcp_stream_ssl = CAMEL_TCP_STREAM_SSL (stream);
-	PRFileDesc *cancel_fd;
-	ssize_t nread;
-	
-	if (camel_operation_cancel_check (NULL)) {
-		errno = EINTR;
-		return -1;
-	}
-	
-	cancel_fd = camel_operation_cancel_prfd (NULL);
-	if (cancel_fd == NULL) {
-
-		PRSocketOptionData sockopts;
-		PRPollDesc pollfds[1];
-		gboolean nonblock;
-		int error;
-		
-		/* get O_NONBLOCK options */
-		sockopts.option = PR_SockOpt_Nonblocking;
-		PR_GetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
-		sockopts.option = PR_SockOpt_Nonblocking;
-		nonblock = sockopts.value.non_blocking;
-		sockopts.value.non_blocking = TRUE;
-		PR_SetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
-
-		pollfds[0].fd = tcp_stream_ssl->priv->sockfd;
-		pollfds[0].in_flags = PR_POLL_READ;
-
-		do {
-			PRInt32 res;
-
-			pollfds[0].out_flags = 0;
-			nread = -1;
-
-			res = PR_Poll(pollfds, 1, PR_TicksPerSecond () * IDLE_READ_TIMEOUT);
-
-			if (res == -1)
-				set_errno(PR_GetError());
-			else if (res == 0) {
-#ifdef ETIMEDOUT
-				errno = ETIMEDOUT;
-#else
-				errno = EIO;
-#endif
-			} else {
-				do {
-					nread = PR_Read (tcp_stream_ssl->priv->sockfd, buffer, n);
-					if (nread == -1)
-						set_errno (PR_GetError ());
-				} while (nread == -1 && PR_GetError () == PR_PENDING_INTERRUPT_ERROR);
-			}
-		} while (nread == -1 && (PR_GetError () == PR_PENDING_INTERRUPT_ERROR ||
-					 PR_GetError () == PR_IO_PENDING_ERROR ||
-					 PR_GetError () == PR_WOULD_BLOCK_ERROR));
-		
-		/* restore O_NONBLOCK options */
-		error = errno;
-		sockopts.option = PR_SockOpt_Nonblocking;
-		sockopts.value.non_blocking = nonblock;
-		PR_SetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
-		errno = error;
-
-
-	} else {
-		PRSocketOptionData sockopts;
-		PRPollDesc pollfds[2];
-		gboolean nonblock;
-		int error;
-		
-		/* get O_NONBLOCK options */
-		sockopts.option = PR_SockOpt_Nonblocking;
-		PR_GetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
-		sockopts.option = PR_SockOpt_Nonblocking;
-		nonblock = sockopts.value.non_blocking;
-		sockopts.value.non_blocking = TRUE;
-		PR_SetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
-
-		pollfds[0].fd = tcp_stream_ssl->priv->sockfd;
-		pollfds[0].in_flags = PR_POLL_READ;
-		pollfds[1].fd = cancel_fd;
-		pollfds[1].in_flags = PR_POLL_READ;
-		
-		do {
-			PRInt32 res;
-
-			pollfds[0].out_flags = 0;
-			pollfds[1].out_flags = 0;
-			nread = -1;
-
-			res = PR_Poll(pollfds, 2, PR_TicksPerSecond () * IDLE_READ_TIMEOUT);
-
-			if (res == -1)
-				set_errno(PR_GetError());
-			else if (res == 0) {
-#ifdef ETIMEDOUT
-				errno = ETIMEDOUT;
-#else
-				errno = EIO;
-#endif
-			} else if (pollfds[1].out_flags == PR_POLL_READ) {
-				errno = EINTR;
-				goto failed;
-			} else {
-				do {
-					nread = PR_Read (tcp_stream_ssl->priv->sockfd, buffer, n);
-					if (nread == -1)
-						set_errno (PR_GetError ());
-				} while (nread == -1 && PR_GetError () == PR_PENDING_INTERRUPT_ERROR);
-			}
-		} while (nread == -1 && (PR_GetError () == PR_PENDING_INTERRUPT_ERROR ||
-					 PR_GetError () == PR_IO_PENDING_ERROR ||
-					 PR_GetError () == PR_WOULD_BLOCK_ERROR));
-		
-		/* restore O_NONBLOCK options */
-	failed:
-		error = errno;
-		sockopts.option = PR_SockOpt_Nonblocking;
-		sockopts.value.non_blocking = nonblock;
-		PR_SetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
-		errno = error;
-	}
-	
 	return nread;
 }
 
@@ -682,20 +553,19 @@ stream_write (CamelStream *stream, const char *buffer, size_t n)
 	CamelTcpStreamSSL *tcp_stream_ssl = CAMEL_TCP_STREAM_SSL (stream);
 	ssize_t w, written = 0;
 	PRFileDesc *cancel_fd;
-	
+
 	if (camel_operation_cancel_check (NULL)) {
 		errno = EINTR;
 		return -1;
 	}
-	
+
 	cancel_fd = camel_operation_cancel_prfd (NULL);
 	if (cancel_fd == NULL) {
-
 		PRSocketOptionData sockopts;
 		PRPollDesc pollfds[1];
 		gboolean nonblock;
 		int error;
-		
+
 		/* get O_NONBLOCK options */
 		sockopts.option = PR_SockOpt_Nonblocking;
 		PR_GetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
@@ -703,7 +573,7 @@ stream_write (CamelStream *stream, const char *buffer, size_t n)
 		nonblock = sockopts.value.non_blocking;
 		sockopts.value.non_blocking = TRUE;
 		PR_SetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
-		
+
 		pollfds[0].fd = tcp_stream_ssl->priv->sockfd;
 		pollfds[0].in_flags = PR_POLL_WRITE;
 
@@ -730,7 +600,7 @@ stream_write (CamelStream *stream, const char *buffer, size_t n)
 					if (w == -1)
 						set_errno (PR_GetError ());
 				} while (w == -1 && PR_GetError () == PR_PENDING_INTERRUPT_ERROR);
-				
+
 				if (w == -1) {
 					if (PR_GetError () == PR_IO_PENDING_ERROR ||
 					    PR_GetError () == PR_WOULD_BLOCK_ERROR)
@@ -739,7 +609,7 @@ stream_write (CamelStream *stream, const char *buffer, size_t n)
 					written += w;
 			}
 		} while (w != -1 && written < n);
-		
+
 		/* restore O_NONBLOCK options */
 		error = errno;
 		sockopts.option = PR_SockOpt_Nonblocking;
@@ -752,7 +622,7 @@ stream_write (CamelStream *stream, const char *buffer, size_t n)
 		PRPollDesc pollfds[2];
 		gboolean nonblock;
 		int error;
-		
+
 		/* get O_NONBLOCK options */
 		sockopts.option = PR_SockOpt_Nonblocking;
 		PR_GetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
@@ -760,12 +630,12 @@ stream_write (CamelStream *stream, const char *buffer, size_t n)
 		nonblock = sockopts.value.non_blocking;
 		sockopts.value.non_blocking = TRUE;
 		PR_SetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
-		
+
 		pollfds[0].fd = tcp_stream_ssl->priv->sockfd;
 		pollfds[0].in_flags = PR_POLL_WRITE;
 		pollfds[1].fd = cancel_fd;
 		pollfds[1].in_flags = PR_POLL_READ;
-		
+
 		do {
 			PRInt32 res;
 
@@ -792,7 +662,7 @@ stream_write (CamelStream *stream, const char *buffer, size_t n)
 					if (w == -1)
 						set_errno (PR_GetError ());
 				} while (w == -1 && PR_GetError () == PR_PENDING_INTERRUPT_ERROR);
-				
+
 				if (w == -1) {
 					if (PR_GetError () == PR_IO_PENDING_ERROR ||
 					    PR_GetError () == PR_WOULD_BLOCK_ERROR)
@@ -801,7 +671,7 @@ stream_write (CamelStream *stream, const char *buffer, size_t n)
 					written += w;
 			}
 		} while (w != -1 && written < n);
-		
+
 		/* restore O_NONBLOCK options */
 		error = errno;
 		sockopts.option = PR_SockOpt_Nonblocking;
@@ -809,10 +679,10 @@ stream_write (CamelStream *stream, const char *buffer, size_t n)
 		PR_SetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
 		errno = error;
 	}
-	
+
 	if (w == -1)
 		return -1;
-	
+
 	return written;
 }
 
@@ -831,11 +701,12 @@ stream_close (CamelStream *stream)
 		return -1;
 	}
 
+	PR_Shutdown (((CamelTcpStreamSSL *)stream)->priv->sockfd, PR_SHUTDOWN_BOTH);
 	if (PR_Close (((CamelTcpStreamSSL *)stream)->priv->sockfd) == PR_FAILURE)
 		return -1;
-	
+
 	((CamelTcpStreamSSL *)stream)->priv->sockfd = NULL;
-	
+
 	return 0;
 }
 
@@ -845,15 +716,15 @@ static SECStatus
 ssl_get_client_auth (void *data, PRFileDesc *sockfd,
 		     struct CERTDistNamesStr *caNames,
 		     struct CERTCertificateStr **pRetCert,
-		     struct SECKEYPrivateKeyStr **pRetKey) 
+		     struct SECKEYPrivateKeyStr **pRetKey)
 {
 	SECStatus status = SECFailure;
 	SECKEYPrivateKey *privkey;
 	CERTCertificate *cert;
 	void *proto_win;
-	
+
 	proto_win = SSL_RevealPinArg (sockfd);
-	
+
 	if ((char *) data) {
 		cert = PK11_FindCertFromNickname ((char *) data, proto_win);
 		if (cert) {
@@ -868,44 +739,44 @@ ssl_get_client_auth (void *data, PRFileDesc *sockfd,
 		/* no nickname given, automatically find the right cert */
 		CERTCertNicknames *names;
 		int i;
-		
-		names = CERT_GetCertNicknames (CERT_GetDefaultCertDB (), 
+
+		names = CERT_GetCertNicknames (CERT_GetDefaultCertDB (),
 					       SEC_CERT_NICKNAMES_USER,
 					       proto_win);
-		
+
 		if (names != NULL) {
 			for (i = 0; i < names->numnicknames; i++) {
-				cert = PK11_FindCertFromNickname (names->nicknames[i], 
+				cert = PK11_FindCertFromNickname (names->nicknames[i],
 								  proto_win);
 				if (!cert)
 					continue;
-				
+
 				/* Only check unexpired certs */
 				if (CERT_CheckCertValidTimes (cert, PR_Now (), PR_FALSE) != secCertTimeValid) {
 					CERT_DestroyCertificate (cert);
 					continue;
 				}
-				
+
 				status = NSS_CmpCertChainWCANames (cert, caNames);
 				if (status == SECSuccess) {
 					privkey = PK11_FindKeyByAnyCert (cert, proto_win);
 					if (privkey)
 						break;
-					
+
 					status = SECFailure;
 					break;
 				}
-				
+
 				CERT_FreeNicknames (names);
 			}
 		}
 	}
-	
+
 	if (status == SECSuccess) {
 		*pRetCert = cert;
 		*pRetKey  = privkey;
 	}
-	
+
 	return status;
 }
 #endif
@@ -919,34 +790,34 @@ ssl_auth_cert (void *data, PRFileDesc *sockfd, PRBool checksig, PRBool is_server
 	SECStatus status;
 	void *pinarg;
 	char *host;
-	
+
 	cert = SSL_PeerCertificate (sockfd);
 	pinarg = SSL_RevealPinArg (sockfd);
 	status = CERT_VerifyCertNow ((CERTCertDBHandle *)data, cert,
 				     checksig, certUsageSSLClient, pinarg);
-	
+
 	if (status != SECSuccess)
 		return SECFailure;
-	
+
 	/* Certificate is OK.  Since this is the client side of an SSL
 	 * connection, we need to verify that the name field in the cert
 	 * matches the desired hostname.  This is our defense against
 	 * man-in-the-middle attacks.
 	 */
-	
+
 	/* SSL_RevealURL returns a hostname, not a URL. */
 	host = SSL_RevealURL (sockfd);
-	
+
 	if (host && *host) {
 		status = CERT_VerifyCertName (cert, host);
 	} else {
 		PR_SetError (SSL_ERROR_BAD_CERT_DOMAIN, 0);
 		status = SECFailure;
 	}
-	
+
 	if (host)
 		PR_Free (host);
-	
+
 	return secStatus;
 }
 #endif
@@ -958,92 +829,72 @@ void camel_certdb_nss_cert_set(CamelCertDB *certdb, CamelCert *ccert, CERTCertif
 static char *
 cert_fingerprint(CERTCertificate *cert)
 {
-	unsigned char md5sum[16], fingerprint[50], *f;
-	int i;
-	const char tohex[16] = "0123456789abcdef";
+	unsigned char fp[16];
+	SECItem fpitem;
+	char *fpstr;
+	char *c;
 
-	md5_get_digest ((const gchar *) cert->derCert.data, cert->derCert.len, md5sum);
-	for (i=0,f = fingerprint; i<16; i++) {
-		unsigned int c = md5sum[i];
+	PK11_HashBuf (SEC_OID_MD5, fp, cert->derCert.data, cert->derCert.len);
+	fpitem.data = fp;
+	fpitem.len = sizeof (fp);
+	fpstr = CERT_Hexify (&fpitem, 1);
 
-		*f++ = tohex[(c >> 4) & 0xf];
-		*f++ = tohex[c & 0xf];
-#ifndef G_OS_WIN32
-		*f++ = ':';
-#else
-		/* The fingerprint is used as a file name, can't have
-		 * colons in file names. Use underscore instead.
-		 */
-		*f++ = '_';
+	for (c = fpstr; *c != 0; c++) {
+#ifdef G_OS_WIN32
+		if (*c == ':')
+			*c = '_';
 #endif
+		*c = g_ascii_tolower (*c);
 	}
 
-	fingerprint[47] = 0;
-
-	return g_strdup((gchar*) fingerprint);
+	return fpstr;
 }
 
 /* lookup a cert uses fingerprint to index an on-disk file */
 CamelCert *
 camel_certdb_nss_cert_get(CamelCertDB *certdb, CERTCertificate *cert)
 {
-	char *fingerprint, *path;
+	char *fingerprint;
 	CamelCert *ccert;
-	struct stat st;
-	size_t nread;
-	ssize_t n;
-	int fd;
-	
+
 	fingerprint = cert_fingerprint (cert);
 	ccert = camel_certdb_get_cert (certdb, fingerprint);
 	if (ccert == NULL) {
 		g_free (fingerprint);
 		return ccert;
 	}
-	
+
 	if (ccert->rawcert == NULL) {
-#ifndef G_OS_WIN32
-		path = g_strdup_printf ("%s/.camel_certs/%s", getenv ("HOME"), fingerprint);
-#else
-		path = g_build_filename (g_get_home_dir (), ".camel_certs", fingerprint, NULL);
-#endif
-		if (g_stat (path, &st) == -1
-		    || (fd = g_open (path, O_RDONLY | O_BINARY, 0)) == -1) {
-			g_warning ("could not load cert %s: %s", path, strerror (errno));
-			g_free (fingerprint);
-			g_free (path);
-			camel_cert_set_trust (certdb, ccert, CAMEL_CERT_TRUST_UNKNOWN);
+		GByteArray *array;
+		gchar *filename;
+		gchar *contents;
+		gsize length;
+		GError *error = NULL;
+
+		filename = g_build_filename (
+			g_get_home_dir (), ".camel_certs", fingerprint, NULL);
+		g_file_get_contents (filename, &contents, &length, &error);
+		if (error != NULL) {
+			g_warning (
+				"Could not load cert %s: %s",
+				filename, error->message);
+			g_error_free (error);
+
+			camel_cert_set_trust (
+				certdb, ccert, CAMEL_CERT_TRUST_UNKNOWN);
 			camel_certdb_touch (certdb);
-			
-			return ccert;
-		}
-		g_free(path);
-		
-		ccert->rawcert = g_byte_array_new ();
-		g_byte_array_set_size (ccert->rawcert, st.st_size);
-		
-		nread = 0;
-		do {
-			do {
-				n = read (fd, ccert->rawcert->data + nread, st.st_size - nread);
-			} while (n == -1 && errno == EINTR);
-			
-			if (n > 0)
-				nread += n;
-		} while (nread < st.st_size && n != -1);
-		
-		close (fd);
-		
-		if (nread != st.st_size) {
-			/* g_warning ("cert size read truncated %s: %u != %ld", path, nread, st.st_size); */
-			g_byte_array_free(ccert->rawcert, TRUE);
-			ccert->rawcert = NULL;
-			g_free(fingerprint);
-			camel_cert_set_trust(certdb, ccert, CAMEL_CERT_TRUST_UNKNOWN);
-			camel_certdb_touch(certdb);
+			g_free (fingerprint);
+			g_free (filename);
 
 			return ccert;
 		}
+		g_free (filename);
+
+		array = g_byte_array_sized_new (length);
+		g_byte_array_append (array, (guint8 *) contents, length);
+		g_free (contents);
+
+		ccert->rawcert = array;
 	}
 
 	g_free(fingerprint);
@@ -1089,15 +940,15 @@ camel_certdb_nss_cert_set(CamelCertDB *certdb, CamelCert *ccert, CERTCertificate
 	char *dir, *path, *fingerprint;
 	CamelStream *stream;
 	struct stat st;
-	
+
 	fingerprint = ccert->fingerprint;
-	
+
 	if (ccert->rawcert == NULL)
 		ccert->rawcert = g_byte_array_new ();
-	
+
 	g_byte_array_set_size (ccert->rawcert, cert->derCert.len);
 	memcpy (ccert->rawcert->data, cert->derCert.data, cert->derCert.len);
-	
+
 #ifndef G_OS_WIN32
 	dir = g_strdup_printf ("%s/.camel_certs", getenv ("HOME"));
 #else
@@ -1108,10 +959,10 @@ camel_certdb_nss_cert_set(CamelCertDB *certdb, CamelCert *ccert, CERTCertificate
 		g_free (dir);
 		return;
 	}
-	
+
 	path = g_strdup_printf ("%s/%s", dir, fingerprint);
 	g_free (dir);
-	
+
 	stream = camel_stream_fs_new_with_name (path, O_WRONLY | O_CREAT | O_TRUNC, 0600);
 	if (stream != NULL) {
 		if (camel_stream_write (stream, (const char *) ccert->rawcert->data, ccert->rawcert->len) == -1) {
@@ -1123,7 +974,7 @@ camel_certdb_nss_cert_set(CamelCertDB *certdb, CamelCert *ccert, CERTCertificate
 	} else {
 		g_warning ("Could not save cert: %s: %s", path, strerror (errno));
 	}
-	
+
 	g_free (path);
 }
 
@@ -1172,7 +1023,7 @@ ssl_bad_cert (void *data, PRFileDesc *sockfd)
 
 	ssl = data;
 	priv = ssl->priv;
-	
+
 	cert = SSL_PeerCertificate (sockfd);
 	if (cert == NULL)
 		return SECFailure;
@@ -1197,10 +1048,10 @@ ssl_bad_cert (void *data, PRFileDesc *sockfd)
 		g_free(fingerprint);
 
 		/* construct our user prompt */
-		prompt = g_strdup_printf (_("SSL Certificate check for %s:\n\n%s\n\nDo you wish to accept?"),
+		prompt = g_strdup_printf (_("SSL Certificate check for %s:\n\n%s\n"),
 					  ssl->priv->expected_host, cert_str);
 		g_free (cert_str);
-	
+
 		/* query the user to find out if we want to accept this certificate */
 		accept = camel_session_alert_user_with_id (ssl->priv->session, CAMEL_SESSION_ALERT_WARNING, CAMEL_EXCEPTION_SERVICE_CERTIFICATE, prompt, TRUE, priv->service);
 		g_free(prompt);
@@ -1261,7 +1112,7 @@ ssl_bad_cert (void *data, PRFileDesc *sockfd)
 					cert->trust = (CERTCertTrust*)PORT_ArenaZAlloc(cert->arena, sizeof(CERTCertTrust));
 					CERT_DecodeTrustString(cert->trust, "P");
 				}
-		
+
 				certs[0] = &cert->derCert;
 				/*CERT_ImportCerts (cert->dbhandle, certUsageSSLServer, 1, certs, NULL, TRUE, FALSE, nick);*/
 				CERT_ImportCerts(cert->dbhandle, certUsageUserCertImport, 1, certs, NULL, TRUE, FALSE, nick);
@@ -1311,7 +1162,7 @@ ssl_bad_cert (void *data, PRFileDesc *sockfd)
 			}
 
 			break;
-			
+
 		case SEC_ERROR_EXPIRED_CERTIFICATE:
 			printf("expired\n");
 
@@ -1363,17 +1214,20 @@ static PRFileDesc *
 enable_ssl (CamelTcpStreamSSL *ssl, PRFileDesc *fd)
 {
 	PRFileDesc *ssl_fd;
-	
+
 	ssl_fd = SSL_ImportFD (NULL, fd ? fd : ssl->priv->sockfd);
 	if (!ssl_fd)
 		return NULL;
-	
+
 	SSL_OptionSet (ssl_fd, SSL_SECURITY, PR_TRUE);
 
-	if (ssl->priv->flags & CAMEL_TCP_STREAM_SSL_ENABLE_SSL2)
+	if (ssl->priv->flags & CAMEL_TCP_STREAM_SSL_ENABLE_SSL2) {
 		SSL_OptionSet (ssl_fd, SSL_ENABLE_SSL2, PR_TRUE);
-	else
+		SSL_OptionSet (ssl_fd, SSL_V2_COMPATIBLE_HELLO, PR_TRUE);
+	} else {
 		SSL_OptionSet (ssl_fd, SSL_ENABLE_SSL2, PR_FALSE);
+		SSL_OptionSet (ssl_fd, SSL_V2_COMPATIBLE_HELLO, PR_FALSE);
+	}
 
 	if (ssl->priv->flags & CAMEL_TCP_STREAM_SSL_ENABLE_SSL3)
 		SSL_OptionSet (ssl_fd, SSL_ENABLE_SSL3, PR_TRUE);
@@ -1386,13 +1240,13 @@ enable_ssl (CamelTcpStreamSSL *ssl, PRFileDesc *fd)
 		SSL_OptionSet (ssl_fd, SSL_ENABLE_TLS, PR_FALSE);
 
 	SSL_SetURL (ssl_fd, ssl->priv->expected_host);
-	
+
 	/*SSL_GetClientAuthDataHook (sslSocket, ssl_get_client_auth, (void *) certNickname);*/
 	/*SSL_AuthCertificateHook (ssl_fd, ssl_auth_cert, (void *) CERT_GetDefaultCertDB ());*/
 	SSL_BadCertHook (ssl_fd, ssl_bad_cert, ssl);
-	
+
 	ssl->priv->ssl_mode = TRUE;
-	
+
 	return ssl_fd;
 }
 
@@ -1447,28 +1301,29 @@ socket_connect(CamelTcpStream *stream, struct addrinfo *host)
 		errno = EINVAL;
 		return -1;
 	}
-	
+
 	fd = PR_OpenTCPSocket(netaddr.raw.family);
 	if (fd == NULL) {
 		set_errno (PR_GetError ());
 		return -1;
 	}
-	
+
 	if (ssl->priv->ssl_mode) {
 		PRFileDesc *ssl_fd;
-		
+
 		ssl_fd = enable_ssl (ssl, fd);
 		if (ssl_fd == NULL) {
 			int errnosave;
-			
+
 			set_errno (PR_GetError ());
 			errnosave = errno;
+			PR_Shutdown (fd, PR_SHUTDOWN_BOTH);
 			PR_Close (fd);
 			errno = errnosave;
-			
+
 			return -1;
 		}
-		
+
 		fd = ssl_fd;
 	}
 
@@ -1476,7 +1331,7 @@ socket_connect(CamelTcpStream *stream, struct addrinfo *host)
 
 	if (PR_Connect (fd, &netaddr, cancel_fd?0:(CONNECT_TIMEOUT*1000)) == PR_FAILURE) {
 		int errnosave;
-		
+
 		set_errno (PR_GetError ());
 		if (PR_GetError () == PR_IN_PROGRESS_ERROR ||
 		    (cancel_fd && (PR_GetError () == PR_CONNECT_TIMEOUT_ERROR ||
@@ -1488,7 +1343,7 @@ socket_connect(CamelTcpStream *stream, struct addrinfo *host)
 			poll[0].in_flags = PR_POLL_WRITE | PR_POLL_EXCEPT;
 			poll[1].fd = cancel_fd;
 			poll[1].in_flags = PR_POLL_READ;
-			
+
 			do {
 				poll[0].out_flags = 0;
 				poll[1].out_flags = 0;
@@ -1497,9 +1352,9 @@ socket_connect(CamelTcpStream *stream, struct addrinfo *host)
 					set_errno (PR_GetError ());
 					goto exception;
 				}
-				
+
 				if (poll[1].out_flags == PR_POLL_READ) {
-					errno = EINTR;
+					set_errno (PR_GetError ());
 					goto exception;
 				}
 
@@ -1514,16 +1369,17 @@ socket_connect(CamelTcpStream *stream, struct addrinfo *host)
 		} else {
 		exception:
 			errnosave = errno;
+			PR_Shutdown (fd, PR_SHUTDOWN_BOTH);
 			PR_Close (fd);
 			ssl->priv->sockfd = NULL;
 			errno = errnosave;
-			
+
 			return -1;
 		}
-		
+
 		errno = 0;
 	}
-	
+
 	ssl->priv->sockfd = fd;
 
 	return 0;
@@ -1545,15 +1401,15 @@ static int
 stream_getsockopt (CamelTcpStream *stream, CamelSockOptData *data)
 {
 	PRSocketOptionData sodata;
-	
+
 	memset ((void *) &sodata, 0, sizeof (sodata));
 	memcpy ((void *) &sodata, (void *) data, sizeof (CamelSockOptData));
-	
+
 	if (PR_GetSocketOption (((CamelTcpStreamSSL *)stream)->priv->sockfd, &sodata) == PR_FAILURE)
 		return -1;
-	
+
 	memcpy ((void *) data, (void *) &sodata, sizeof (CamelSockOptData));
-	
+
 	return 0;
 }
 
@@ -1561,13 +1417,13 @@ static int
 stream_setsockopt (CamelTcpStream *stream, const CamelSockOptData *data)
 {
 	PRSocketOptionData sodata;
-	
+
 	memset ((void *) &sodata, 0, sizeof (sodata));
 	memcpy ((void *) &sodata, (void *) data, sizeof (CamelSockOptData));
-	
+
 	if (PR_SetSocketOption (((CamelTcpStreamSSL *)stream)->priv->sockfd, &sodata) == PR_FAILURE)
 		return -1;
-	
+
 	return 0;
 }
 
@@ -1597,7 +1453,7 @@ sockaddr_from_praddr(PRNetAddr *addr, socklen_t *len)
 		sin->sin6_scope_id = addr->ipv6.scope_id;
 		*len = sizeof(*sin);
 
-		return (struct sockaddr *)sin;		
+		return (struct sockaddr *)sin;
 	}
 #endif
 
@@ -1609,7 +1465,7 @@ stream_get_local_address(CamelTcpStream *stream, socklen_t *len)
 {
 	PRFileDesc *sockfd = CAMEL_TCP_STREAM_SSL (stream)->priv->sockfd;
 	PRNetAddr addr;
-	
+
 	if (PR_GetSockName(sockfd, &addr) != PR_SUCCESS)
 		return NULL;
 
@@ -1621,7 +1477,7 @@ stream_get_remote_address (CamelTcpStream *stream, socklen_t *len)
 {
 	PRFileDesc *sockfd = CAMEL_TCP_STREAM_SSL (stream)->priv->sockfd;
 	PRNetAddr addr;
-	
+
 	if (PR_GetPeerName(sockfd, &addr) != PR_SUCCESS)
 		return NULL;
 
