@@ -19,8 +19,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -46,6 +46,10 @@
 #include "camel-imap-store-summary.h"
 #include "camel-imap-store.h"
 #include "camel-imap-utils.h"
+#include "camel-imap-summary.h"
+
+#include "camel-string-utils.h"
+#include "camel-stream-buffer.h"
 
 extern int camel_verbose_debug;
 
@@ -55,14 +59,31 @@ static CamelImapResponse *imap_read_response (CamelImapStore *store,
 					      CamelException *ex);
 static char *imap_read_untagged (CamelImapStore *store, char *line,
 				 CamelException *ex);
+static char *imap_read_untagged_opp (CamelImapStore *store, char *line,
+				 CamelException *ex, int len);
 static char *imap_command_strdup_vprintf (CamelImapStore *store,
 					  const char *fmt, va_list ap);
 static char *imap_command_strdup_printf (CamelImapStore *store,
 					 const char *fmt, ...);
 
-static char * imap_read_untagged_idle (CamelImapStore *store, char *line, CamelException *ex);
+#ifdef NOTUSED
+static int
+uid_compar (const void *va, const void *vb)
+{
+	const char **sa = (const char **)va, **sb = (const char **)vb;
+	unsigned long a, b;
 
+	a = strtoul (*sa, NULL, 10);
+	b = strtoul (*sb, NULL, 10);
+	if (a < b)
+		return -1;
+	else if (a == b)
+		return 0;
+	else
+		return 1;
+}
 
+#endif
 
 /**
  * camel_imap_command:
@@ -91,6 +112,8 @@ static char * imap_read_untagged_idle (CamelImapStore *store, char *line, CamelE
  * be set). Otherwise, a CamelImapResponse describing the server's
  * response, which the caller must free with camel_imap_response_free().
  **/
+#define UID_SET_LIMIT  (768)
+
 CamelImapResponse *
 camel_imap_command (CamelImapStore *store, CamelFolder *folder,
 		    CamelException *ex, const char *fmt, ...)
@@ -106,30 +129,48 @@ camel_imap_command (CamelImapStore *store, CamelFolder *folder,
 		va_end (ap);
 	} else {
 
-		/* camel_object_ref(folder); 
+		char *modseq = NULL;
+
+		/* camel_object_ref(folder);
 		if (store->current_folder && CAMEL_IS_OBJECT (store->current_folder))
 			camel_object_unref(store->current_folder); */
 
 		if (!folder)
 			store->last_folder = store->current_folder;
-		else 
+		else {
+			modseq = camel_imap_folder_get_highestmodseq (CAMEL_IMAP_FOLDER (folder));
 			store->last_folder = NULL;
+		}
 
 		store->current_folder = folder;
 
-		if (store->capabilities & IMAP_CAPABILITY_CONDSTORE) 
-			cmd = imap_command_strdup_printf (store, "SELECT %F (CONDSTORE)", folder->full_name);
-		else 
-			cmd = imap_command_strdup_printf (store, "SELECT %F", folder->full_name);
+		if (modseq && (store->capabilities & IMAP_CAPABILITY_QRESYNC))
+		{
+			CamelImapSummary *imap_summary = CAMEL_IMAP_SUMMARY (folder->summary);
+
+			cmd = imap_command_strdup_printf (store,
+				"SELECT %F (QRESYNC (%d %s 1:*))",
+				folder->full_name,
+				imap_summary->validity, modseq);
+
+		} else if (folder) {
+			if (store->capabilities & IMAP_CAPABILITY_CONDSTORE)
+				cmd = imap_command_strdup_printf (store, "SELECT %F (CONDSTORE)", folder->full_name);
+			else
+				cmd = imap_command_strdup_printf (store, "SELECT %F", folder->full_name);
+		}
+
+		if (modseq)
+			g_free (modseq);
 	}
-	
+
 	if (!imap_command_start (store, folder, cmd, ex)) {
 		g_free (cmd);
 		CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
 		return NULL;
 	}
 	g_free (cmd);
-	
+
 	return imap_read_response (store, ex);
 }
 
@@ -175,7 +216,7 @@ camel_imap_command_start (CamelImapStore *store, CamelFolder *folder,
 	va_list ap;
 	char *cmd;
 	gboolean ok;
-	
+
 	va_start (ap, fmt);
 	cmd = imap_command_strdup_vprintf (store, fmt, ap);
 	va_end (ap);
@@ -204,7 +245,7 @@ imap_command_start (CamelImapStore *store, CamelFolder *folder,
 
 	if (store->ostream == NULL || ((CamelObject *)store->ostream)->ref_count <= 0)
 	{
-		if (store->has_login && !camel_service_connect ((CamelService*)store, ex))
+		if (store->has_login && (store->not_recon || !camel_service_connect ((CamelService*)store, ex)))
 		{
 			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
 				     _("You must be working online to "
@@ -232,14 +273,14 @@ imap_command_start (CamelImapStore *store, CamelFolder *folder,
 	/* g_mutex_unlock (store->stream_lock);*/
 
 
-	/* Also read imap_update_summary and all of the IDLE crap. The fetching 
-	 * message and the LOGOUT exception are for camel_imap_folder_fetch_data's 
-	 * case (I know this is a hack): In those cases, store is not the 
-	 * connection where the IDLE is happening, so there's no need to stop it 
-	 * either! On top of that is stopping the IDLE in case of LOGOUT handled 
+	/* Also read imap_update_summary and all of the IDLE crap. The fetching
+	 * message and the LOGOUT exception are for camel_imap_folder_fetch_data's
+	 * case (I know this is a hack): In those cases, store is not the
+	 * connection where the IDLE is happening, so there's no need to stop it
+	 * either! On top of that is stopping the IDLE in case of LOGOUT handled
 	 * elsewere too */
 
-	if (!store->dontdistridlehack && !fetching_message && strcmp (cmd, "LOGOUT")) 
+	if (!store->dontdistridlehack && !fetching_message && strcmp (cmd, "LOGOUT"))
 	{
 		gboolean wasnull = FALSE;
 		if (!store->current_folder) {
@@ -252,7 +293,7 @@ imap_command_start (CamelImapStore *store, CamelFolder *folder,
 	}
 
 	/* Check for current folder */
-	if (folder && folder != store->current_folder) 
+	if (folder && folder != store->current_folder)
 	{
 		CamelImapResponse *response;
 		CamelException internal_ex = CAMEL_EXCEPTION_INITIALISER;
@@ -274,11 +315,11 @@ imap_command_start (CamelImapStore *store, CamelFolder *folder,
 			return FALSE;
 		}
 	}
-	
+
 	/* Send the command */
 	if (camel_verbose_debug) {
 		const char *mask;
-		
+
 		if (!strncmp ("LOGIN \"", cmd, 7))
 			mask = "LOGIN \"xxx\" xxx";
 		else if (!strncmp ("LOGIN {", cmd, 7))
@@ -287,7 +328,7 @@ imap_command_start (CamelImapStore *store, CamelFolder *folder,
 			mask = "LOGIN xxx xxx";
 		else
 			mask = cmd;
-		
+
 		fprintf (stderr, "sending : %c%.5u %s\r\n", store->tag_prefix, store->command, mask);
 	}
 
@@ -304,10 +345,10 @@ imap_command_start (CamelImapStore *store, CamelFolder *folder,
 	if (resp)
 		g_free (resp);
 
-	full_cmd = g_strdup_printf ("%c%.5u %s\r\n", store->tag_prefix, 
+	full_cmd = g_strdup_printf ("%c%.5u %s\r\n", store->tag_prefix,
 		store->command++, cmd);
 	len = strlen (full_cmd);
-	
+
 	nwritten = camel_stream_write (store->ostream, full_cmd, len);
 
 	imap_debug ("(%d, %d) -> %s\n", len, nwritten, full_cmd);
@@ -316,7 +357,7 @@ imap_command_start (CamelImapStore *store, CamelFolder *folder,
 
 	/* g_mutex_unlock (store->stream_lock); */
 
-	if (nwritten != len) 
+	if (nwritten != len)
 	{
 		CamelException mex = CAMEL_EXCEPTION_INITIALISER;
 
@@ -324,16 +365,17 @@ imap_command_start (CamelImapStore *store, CamelFolder *folder,
 			camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
 					     _("Operation cancelled"));
 		} else
-			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_LOST_CONNECTION,
 					     g_strerror (errno));
-		
 
-		camel_imap_recon (store, &mex);
+
+		camel_imap_recon (store, &mex, FALSE);
 		imap_debug ("Recon in start: %s\n", camel_exception_get_description (&mex));
 
+		camel_exception_clear (&mex);
 		return FALSE;
 	}
-	
+
 	return TRUE;
 }
 
@@ -347,7 +389,7 @@ imap_command_start (CamelImapStore *store, CamelFolder *folder,
  * This method is for sending continuing responses to the IMAP server
  * after camel_imap_command() or camel_imap_command_response() returns
  * a continuation response.
- * 
+ *
  * This function assumes you have an exclusive lock on the imap stream.
  *
  * Return value: as for camel_imap_command(). On failure, the store's
@@ -369,19 +411,20 @@ camel_imap_command_continuation (CamelImapStore *store, const char *cmd,
 			CamelException mex = CAMEL_EXCEPTION_INITIALISER;
 			camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
 					     _("Operation cancelled"));
-			camel_imap_recon (store, &mex);
+			camel_imap_recon (store, &mex, TRUE);
 			imap_debug ("Recon in cont: %s\n", camel_exception_get_description (&mex));
 			CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+			camel_exception_clear (&mex);
 			return NULL;
 		} else
-			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_LOST_CONNECTION,
 					     g_strerror (errno));
 		camel_service_disconnect (CAMEL_SERVICE (store), FALSE, NULL);
 
 		CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
 		return NULL;
 	}
-	
+
 	return imap_read_response (store, ex);
 }
 
@@ -407,12 +450,19 @@ camel_imap_command_response (CamelImapStore *store, char **response,
 {
 	CamelImapResponseType type;
 	char *respbuf;
-	
+	int len = -1;
+
 	if (camel_imap_store_readline (store, &respbuf, ex) < 0) {
 		CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
 		return CAMEL_IMAP_RESPONSE_ERROR;
 	}
 
+	char *ptr = strchr (respbuf, '{');
+	if (ptr) {
+		ptr++;
+		len = strtoul (ptr, NULL, 10);
+	}
+
 	imap_debug ("(.., ..) <- %s\n", respbuf);
 
 	switch (*respbuf) {
@@ -420,7 +470,7 @@ camel_imap_command_response (CamelImapStore *store, char **response,
 		if (!g_ascii_strncasecmp (respbuf, "* BYE", 5)) {
 			/* Connection was lost, no more data to fetch */
 			camel_service_disconnect (CAMEL_SERVICE (store), FALSE, NULL);
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_LOST_CONNECTION,
 					      _("Server unexpectedly disconnected: %s"),
 					      _("Unknown error")); /* g_strerror (104));  FIXME after 1.0 is released */
 			store->connected = FALSE;
@@ -429,7 +479,76 @@ camel_imap_command_response (CamelImapStore *store, char **response,
 			type = CAMEL_IMAP_RESPONSE_ERROR;
 			break;
 		}
-		
+
+		/* Read the rest of the response. */
+		type = CAMEL_IMAP_RESPONSE_UNTAGGED;
+
+		if (len != -1)
+			respbuf = imap_read_untagged_opp (store, respbuf, ex, len);
+		else
+			respbuf = imap_read_untagged (store, respbuf, ex);
+
+		if (!respbuf)
+			type = CAMEL_IMAP_RESPONSE_ERROR;
+		else if (!g_ascii_strncasecmp (respbuf, "* OK [ALERT]", 12)
+			 || !g_ascii_strncasecmp (respbuf, "* NO [ALERT]", 12)
+			 || !g_ascii_strncasecmp (respbuf, "* BAD [ALERT]", 13)) {
+			char *msg;
+
+			/* for imap ALERT codes, account user@host */
+			/* we might get a ']' from a BAD response since we +12, but who cares? */
+			msg = g_strdup_printf(_("Alert from IMAP server %s@%s:\n%s"),
+					      ((CamelService *)store)->url->user, ((CamelService *)store)->url->host, respbuf+12);
+			camel_session_alert_user_generic(((CamelService *)store)->session,
+				CAMEL_SESSION_ALERT_WARNING, msg, FALSE, ((CamelService *)store));
+			g_free(msg);
+		}
+
+		break;
+	case '+':
+		type = CAMEL_IMAP_RESPONSE_CONTINUATION;
+		break;
+	default:
+		type = CAMEL_IMAP_RESPONSE_TAGGED;
+		break;
+	}
+	*response = respbuf;
+
+	if (type == CAMEL_IMAP_RESPONSE_ERROR ||
+	    type == CAMEL_IMAP_RESPONSE_TAGGED)
+		CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
+
+	return type;
+}
+
+
+CamelImapResponseType
+camel_imap_command_response_idle (CamelImapStore *store, char **response,
+			     CamelException *ex)
+{
+	CamelImapResponseType type;
+	char *respbuf;
+
+	if (camel_imap_store_readline_nl (store, &respbuf, ex) < 0)
+		return CAMEL_IMAP_RESPONSE_ERROR;
+
+	imap_debug ("(.., ..) <- %s (IDLE response)\n", respbuf);
+
+	switch (*respbuf) {
+	case '*':
+		if (!g_ascii_strncasecmp (respbuf, "* BYE", 5)) {
+			/* Connection was lost, no more data to fetch */
+			camel_service_disconnect (CAMEL_SERVICE (store), FALSE, NULL);
+			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_LOST_CONNECTION,
+					      _("Server unexpectedly disconnected: %s"),
+					      _("Unknown error")); /* g_strerror (104));  FIXME after 1.0 is released */
+			store->connected = FALSE;
+			g_free (respbuf);
+			respbuf = NULL;
+			type = CAMEL_IMAP_RESPONSE_ERROR;
+			break;
+		}
+
 		/* Read the rest of the response. */
 		type = CAMEL_IMAP_RESPONSE_UNTAGGED;
 		respbuf = imap_read_untagged (store, respbuf, ex);
@@ -444,69 +563,7 @@ camel_imap_command_response (CamelImapStore *store, char **response,
 			/* we might get a ']' from a BAD response since we +12, but who cares? */
 			msg = g_strdup_printf(_("Alert from IMAP server %s@%s:\n%s"),
 					      ((CamelService *)store)->url->user, ((CamelService *)store)->url->host, respbuf+12);
-			camel_session_alert_user_generic(((CamelService *)store)->session, 
-				CAMEL_SESSION_ALERT_WARNING, msg, FALSE, ((CamelService *)store));
-			g_free(msg);
-		}
-		
-		break;
-	case '+':
-		type = CAMEL_IMAP_RESPONSE_CONTINUATION;
-		break;
-	default:
-		type = CAMEL_IMAP_RESPONSE_TAGGED;
-		break;
-	}
-	*response = respbuf;
-
-	if (type == CAMEL_IMAP_RESPONSE_ERROR ||
-	    type == CAMEL_IMAP_RESPONSE_TAGGED) 
-		CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
-
-	return type;
-}
-
-
-CamelImapResponseType
-camel_imap_command_response_idle (CamelImapStore *store, char **response,
-			     CamelException *ex)
-{
-	CamelImapResponseType type;
-	char *respbuf;
-
-	if (camel_imap_store_readline_idle (store, &respbuf, ex) < 0)
-		return CAMEL_IMAP_RESPONSE_ERROR;
-
-	switch (*respbuf) {
-	case '*':
-		if (!g_ascii_strncasecmp (respbuf, "* BYE", 5)) {
-			/* Connection was lost, no more data to fetch */
-			camel_service_disconnect (CAMEL_SERVICE (store), FALSE, NULL);
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
-					      _("Server unexpectedly disconnected: %s"),
-					      _("Unknown error")); /* g_strerror (104));  FIXME after 1.0 is released */
-			store->connected = FALSE;
-			g_free (respbuf);
-			respbuf = NULL;
-			type = CAMEL_IMAP_RESPONSE_ERROR;
-			break;
-		}
-		
-		/* Read the rest of the response. */
-		type = CAMEL_IMAP_RESPONSE_UNTAGGED;
-		respbuf = imap_read_untagged_idle (store, respbuf, ex);
-		if (!respbuf)
-			type = CAMEL_IMAP_RESPONSE_ERROR;
-		else if (!g_ascii_strncasecmp (respbuf, "* OK [ALERT]", 12)
-			 || !g_ascii_strncasecmp (respbuf, "* NO [ALERT]", 12)
-			 || !g_ascii_strncasecmp (respbuf, "* BAD [ALERT]", 13)) {
-			char *msg;
-
-			/* for imap ALERT codes, account user@host */
-			/* we might get a ']' from a BAD response since we +12, but who cares? */
-			msg = g_strdup_printf(_("Alert from IMAP server %s@%s:\n%s"),
-					      ((CamelService *)store)->url->user, ((CamelService *)store)->url->host, respbuf+12);
-			camel_session_alert_user_generic(((CamelService *)store)->session, 
+			camel_session_alert_user_generic(((CamelService *)store)->session,
 				CAMEL_SESSION_ALERT_WARNING, msg, FALSE, ((CamelService *)store));
 			g_free(msg);
 		} else if (!g_ascii_strncasecmp (respbuf, "* BAD Invalid tag",17))
@@ -516,11 +573,18 @@ camel_imap_command_response_idle (CamelImapStore *store, char **response,
 		type = CAMEL_IMAP_RESPONSE_CONTINUATION;
 		break;
 	default:
-		type = CAMEL_IMAP_RESPONSE_TAGGED;
+		if (camel_strstrcase (respbuf, "OK") != NULL ||
+			camel_strstrcase (respbuf, "NO") != NULL ||
+			camel_strstrcase (respbuf, "BAD") != NULL) {
+
+			type = CAMEL_IMAP_RESPONSE_TAGGED;
+
+		} else
+			type = CAMEL_IMAP_RESPONSE_UNTAGGED;
 		break;
 	}
 	*response = respbuf;
-	
+
 	return type;
 }
 
@@ -530,7 +594,7 @@ imap_read_response (CamelImapStore *store, CamelException *ex)
 	CamelImapResponse *response;
 	CamelImapResponseType type;
 	char *respbuf, *p;
-	
+
 	/* Get another lock so that when we reach the tagged
 	 * response and camel_imap_command_response unlocks,
 	 * we're still locked. This lock is owned by response
@@ -544,17 +608,17 @@ imap_read_response (CamelImapStore *store, CamelException *ex)
 		response->folder = store->current_folder;
 		camel_object_ref (CAMEL_OBJECT (response->folder));
 	}
-	
+
 	response->untagged = g_ptr_array_new ();
 	while ((type = camel_imap_command_response (store, &respbuf, ex))
 	       == CAMEL_IMAP_RESPONSE_UNTAGGED)
 		g_ptr_array_add (response->untagged, respbuf);
-	
+
 	if (type == CAMEL_IMAP_RESPONSE_ERROR) {
 		camel_imap_response_free_without_processing (store, response);
 		return NULL;
 	}
-	
+
 	response->status = respbuf;
 
 	/* Check for OK or continuation response. */
@@ -563,24 +627,24 @@ imap_read_response (CamelImapStore *store, CamelException *ex)
 	p = strchr (respbuf, ' ');
 	if (p && !g_ascii_strncasecmp (p, " OK", 3))
 		return response;
-	
+
 	/* We should never get BAD, or anything else but +, OK, or NO
 	 * for that matter.  Well, we could get BAD, treat as NO.
 	 */
 	if (!p || (g_ascii_strncasecmp(p, " NO", 3) != 0 && g_ascii_strncasecmp(p, " BAD", 4)) ) {
 		g_warning ("Unexpected response from IMAP server: %s",
 			   respbuf);
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_PROTOCOL,
 				      _("Unexpected response from IMAP "
 					"server: %s"), respbuf);
 		camel_imap_response_free_without_processing (store, response);
 		return NULL;
 	}
-	
+
 	p += 3;
 	if (!*p++)
 		p = NULL;
-	camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_INVALID,
+	camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_PROTOCOL,
 			      _("IMAP command failed: %s"),
 			      p ? p : _("Unknown error"));
 	camel_imap_response_free_without_processing (store, response);
@@ -592,30 +656,30 @@ imap_read_response (CamelImapStore *store, CamelException *ex)
  * of literals.
  */
 static char *
-imap_read_untagged (CamelImapStore *store, char *line, CamelException *ex)
+imap_read_untagged_opp (CamelImapStore *store, char *line, CamelException *ex, int len)
 {
 	int fulllen, ldigits, nread, n, i, sexp = 0;
 	unsigned int length;
 	GPtrArray *data;
 	GString *str;
 	char *end, *p, *s, *d;
-	
+
 	p = strrchr (line, '{');
 	if (!p)
 		return line;
-	
+
 	data = g_ptr_array_new ();
 	fulllen = 0;
-	
+
 	while (1) {
 		str = g_string_new (line);
 		g_free (line);
 		fulllen += str->len;
 		g_ptr_array_add (data, str);
-		
+
 		if (!(p = strrchr (str->str, '{')) || p[1] == '-')
 			break;
-		
+
 		/* HACK ALERT: We scan the non-literal part of the string, looking for possible s expression braces.
 		   This assumes we're getting s-expressions, which we should be.
 		   This is so if we get a blank line after a literal, in an s-expression, we can keep going, since
@@ -627,35 +691,37 @@ imap_read_untagged (CamelImapStore *store, char *line, CamelException *ex)
 			else if (*s == ')')
 				sexp--;
 		}
-		
+
 		length = strtoul (p + 1, &end, 10);
 		if (*end != '}' || *(end + 1) || end == p + 1 || length >= UINT_MAX - 2)
 			break;
 		ldigits = end - (p + 1);
-		
+
 		/* Read the literal */
 		str = g_string_sized_new (length + 2);
 		str->str[0] = '\n';
 		nread = 0;
-		
+
 		do {
-			if ((n = camel_stream_read (store->istream, str->str + nread + 1, length - nread)) == -1) {
-				if (errno == EINTR) 
+			if ((n = camel_stream_buffer_read_opp (store->istream, str->str + nread + 1, length - nread, len)) == -1) {
+
+				if (errno == EINTR)
 				{
 					CamelException mex = CAMEL_EXCEPTION_INITIALISER;
 					camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
 							     _("Operation cancelled"));
-					camel_imap_recon (store, &mex);
+					camel_imap_recon (store, &mex, TRUE);
 					imap_debug ("Recon in untagged: %s\n", camel_exception_get_description (&mex));
+					camel_exception_clear (&mex);
 				} else {
-					camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+					camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_LOST_CONNECTION,
 							     g_strerror (errno));
 					camel_service_disconnect (CAMEL_SERVICE (store), FALSE, NULL);
 				}
 				g_string_free (str, TRUE);
 				goto lose;
 			}
-			
+
 			nread += n;
 		} while (n > 0 && nread < length);
 
@@ -664,10 +730,11 @@ imap_read_untagged (CamelImapStore *store, char *line, CamelException *ex)
 				CamelException mex = CAMEL_EXCEPTION_INITIALISER;
 				camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
 						     _("Operation cancelled"));
-				camel_imap_recon (store, &mex);
+				camel_imap_recon (store, &mex, TRUE);
 				imap_debug ("Recon in untagged idle: %s\n", camel_exception_get_description (&mex));
+				camel_exception_clear (&mex);
 			} else {
-				camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+				camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_LOST_CONNECTION,
 					     _("Server response ended too soon."));
 				camel_service_disconnect (CAMEL_SERVICE (store), FALSE, NULL);
 			}
@@ -681,7 +748,7 @@ imap_read_untagged (CamelImapStore *store, char *line, CamelException *ex)
 			fwrite(str->str+1, 1, length, stdout);
 			printf("<--\n");
 		}
-		
+
 		/* Fix up the literal, turning CRLFs into LF. Also, if
 		 * we find any embedded NULs, strip them. This is
 		 * dubious, but:
@@ -691,7 +758,7 @@ imap_read_untagged (CamelImapStore *store, char *line, CamelException *ex)
 		 *   - WU-imapd 12.264 (at least) will cheerily pass
 		 *     NULs along if they are embedded in the message
 		 */
-		
+
 		s = d = str->str + 1;
 		end = str->str + 1 + length;
 		while (s < end) {
@@ -707,7 +774,7 @@ imap_read_untagged (CamelImapStore *store, char *line, CamelException *ex)
 		}
 		*d = '\0';
 		str->len = length + 1;
-		
+
 		/* p points to the "{" in the line that starts the
 		 * literal. The length of the CR-less response must be
 		 * less than or equal to the length of the response
@@ -717,7 +784,7 @@ imap_read_untagged (CamelImapStore *store, char *line, CamelException *ex)
 		 * GString's length would be off...
 		 */
 		sprintf (p, "{%0*u}", ldigits, length);
-		
+
 		fulllen += str->len;
 		g_ptr_array_add (data, str);
 
@@ -732,7 +799,7 @@ imap_read_untagged (CamelImapStore *store, char *line, CamelException *ex)
 				g_warning("Server sent empty line after a literal, assuming in error");
 		} while (line[0] == 0 && sexp > 0);
 	}
-	
+
 	/* Now reassemble the data. */
 	p = line = g_malloc (fulllen + 1);
 	for (i = 0; i < data->len; i++) {
@@ -744,7 +811,7 @@ imap_read_untagged (CamelImapStore *store, char *line, CamelException *ex)
 	*p = '\0';
 	g_ptr_array_free (data, TRUE);
 	return line;
-	
+
  lose:
 	for (i = 0; i < data->len; i++)
 		g_string_free (data->pdata[i], TRUE);
@@ -755,30 +822,30 @@ imap_read_untagged (CamelImapStore *store, char *line, CamelException *ex)
 
 
 static char *
-imap_read_untagged_idle (CamelImapStore *store, char *line, CamelException *ex)
+imap_read_untagged (CamelImapStore *store, char *line, CamelException *ex)
 {
 	int fulllen, ldigits, nread, n, i, sexp = 0;
 	unsigned int length;
 	GPtrArray *data;
 	GString *str;
 	char *end, *p, *s, *d;
-	
+
 	p = strrchr (line, '{');
 	if (!p)
 		return line;
-	
+
 	data = g_ptr_array_new ();
 	fulllen = 0;
-	
+
 	while (1) {
 		str = g_string_new (line);
 		g_free (line);
 		fulllen += str->len;
 		g_ptr_array_add (data, str);
-		
+
 		if (!(p = strrchr (str->str, '{')) || p[1] == '-')
 			break;
-		
+
 		/* HACK ALERT: We scan the non-literal part of the string, looking for possible s expression braces.
 		   This assumes we're getting s-expressions, which we should be.
 		   This is so if we get a blank line after a literal, in an s-expression, we can keep going, since
@@ -790,46 +857,49 @@ imap_read_untagged_idle (CamelImapStore *store, char *line, CamelException *ex)
 			else if (*s == ')')
 				sexp--;
 		}
-		
+
 		length = strtoul (p + 1, &end, 10);
 		if (*end != '}' || *(end + 1) || end == p + 1 || length >= UINT_MAX - 2)
 			break;
 		ldigits = end - (p + 1);
-		
+
 		/* Read the literal */
 		str = g_string_sized_new (length + 2);
 		str->str[0] = '\n';
 		nread = 0;
-		
+
 		do {
-			if ((n = camel_stream_read_idle (store->istream, str->str + nread + 1, length - nread)) == -1) {
-				if (errno == EINTR) {
+			if ((n = camel_stream_read (store->istream, str->str + nread + 1, length - nread)) == -1) {
+				if (errno == EINTR)
+				{
 					CamelException mex = CAMEL_EXCEPTION_INITIALISER;
 					camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
 							     _("Operation cancelled"));
-					camel_imap_recon (store, &mex);
-					imap_debug ("Recon in untagged idle: %s\n", camel_exception_get_description (&mex));
+					camel_imap_recon (store, &mex, TRUE);
+					imap_debug ("Recon in untagged: %s\n", camel_exception_get_description (&mex));
+					camel_exception_clear (&mex);
 				} else {
-					camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+					camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_LOST_CONNECTION,
 							     g_strerror (errno));
 					camel_service_disconnect (CAMEL_SERVICE (store), FALSE, NULL);
 				}
 				g_string_free (str, TRUE);
 				goto lose;
 			}
-			
+
 			nread += n;
 		} while (n > 0 && nread < length);
-		
+
 		if (nread < length) {
 			if (errno == EINTR) {
 				CamelException mex = CAMEL_EXCEPTION_INITIALISER;
 				camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
 						     _("Operation cancelled"));
-				camel_imap_recon (store, &mex);
+				camel_imap_recon (store, &mex, TRUE);
 				imap_debug ("Recon in untagged idle: %s\n", camel_exception_get_description (&mex));
-			}  else {
-				camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+				camel_exception_clear (&mex);
+			} else {
+				camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_LOST_CONNECTION,
 					     _("Server response ended too soon."));
 				camel_service_disconnect (CAMEL_SERVICE (store), FALSE, NULL);
 			}
@@ -843,7 +913,7 @@ imap_read_untagged_idle (CamelImapStore *store, char *line, CamelException *ex)
 			fwrite(str->str+1, 1, length, stdout);
 			printf("<--\n");
 		}
-		
+
 		/* Fix up the literal, turning CRLFs into LF. Also, if
 		 * we find any embedded NULs, strip them. This is
 		 * dubious, but:
@@ -853,7 +923,7 @@ imap_read_untagged_idle (CamelImapStore *store, char *line, CamelException *ex)
 		 *   - WU-imapd 12.264 (at least) will cheerily pass
 		 *     NULs along if they are embedded in the message
 		 */
-		
+
 		s = d = str->str + 1;
 		end = str->str + 1 + length;
 		while (s < end) {
@@ -869,7 +939,7 @@ imap_read_untagged_idle (CamelImapStore *store, char *line, CamelException *ex)
 		}
 		*d = '\0';
 		str->len = length + 1;
-		
+
 		/* p points to the "{" in the line that starts the
 		 * literal. The length of the CR-less response must be
 		 * less than or equal to the length of the response
@@ -879,13 +949,13 @@ imap_read_untagged_idle (CamelImapStore *store, char *line, CamelException *ex)
 		 * GString's length would be off...
 		 */
 		sprintf (p, "{%0*u}", ldigits, length);
-		
+
 		fulllen += str->len;
 		g_ptr_array_add (data, str);
 
 		/* Read the next line. */
 		do {
-			if (camel_imap_store_readline_idle (store, &line, ex) < 0)
+			if (camel_imap_store_readline (store, &line, ex) < 0)
 				goto lose;
 
 			/* MAJOR HACK ALERT, gropuwise sometimes sends an extra blank line after literals, check that here
@@ -894,7 +964,7 @@ imap_read_untagged_idle (CamelImapStore *store, char *line, CamelException *ex)
 				g_warning("Server sent empty line after a literal, assuming in error");
 		} while (line[0] == 0 && sexp > 0);
 	}
-	
+
 	/* Now reassemble the data. */
 	p = line = g_malloc (fulllen + 1);
 	for (i = 0; i < data->len; i++) {
@@ -906,7 +976,7 @@ imap_read_untagged_idle (CamelImapStore *store, char *line, CamelException *ex)
 	*p = '\0';
 	g_ptr_array_free (data, TRUE);
 	return line;
-	
+
  lose:
 	for (i = 0; i < data->len; i++)
 		g_string_free (data->pdata[i], TRUE);
@@ -939,12 +1009,15 @@ camel_imap_response_free (CamelImapStore *store, CamelImapResponse *response)
 
 	for (i = 0; i < response->untagged->len; i++) {
 		resp = response->untagged->pdata[i];
-		
+
 		if (response->folder) {
 			/* Check if it's something we need to handle. */
 			number = strtoul (resp + 2, &p, 10);
 			if (!g_ascii_strcasecmp (p, " EXISTS")) {
 				exists = number;
+
+			/* TNY TODO, QRESYNC TODO: add VANISHED HERE */
+
 			} else if (!g_ascii_strcasecmp (p, " EXPUNGE")
 				   || !g_ascii_strcasecmp(p, " XGWMOVE")) {
 				/* XGWMOVE response is the same as an EXPUNGE response */
@@ -957,10 +1030,10 @@ camel_imap_response_free (CamelImapStore *store, CamelImapResponse *response)
 		}
 		g_free (resp);
 	}
-	
+
 	g_ptr_array_free (response->untagged, TRUE);
 	g_free (response->status);
-	
+
 	if (response->folder && !fetching_message) {
 		if (exists > 0 || expunged) {
 			/* Update the summary */
@@ -969,10 +1042,10 @@ camel_imap_response_free (CamelImapStore *store, CamelImapResponse *response)
 			if (expunged)
 				g_array_free (expunged, TRUE);
 		}
-		
+
 		camel_object_unref (CAMEL_OBJECT (response->folder));
 	}
-	
+
 	g_free (response);
 
 	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
@@ -992,7 +1065,7 @@ camel_imap_response_free_without_processing (CamelImapStore *store,
 {
 	if (!response)
 		return;
-	
+
 	if (response->folder) {
 		camel_object_unref (CAMEL_OBJECT (response->folder));
 		response->folder = NULL;
@@ -1025,28 +1098,28 @@ camel_imap_response_extract (CamelImapStore *store,
 	char *resp = NULL;
 
 	len = strlen (type);
-	
+
 	for (i = 0; i < response->untagged->len; i++) {
 		resp = response->untagged->pdata[i];
 		/* Skip "* ", and initial sequence number, if present */
 		strtoul (resp + 2, &resp, 10);
 		if (*resp == ' ')
 			resp = (char *) imap_next_word (resp);
-		
+
 		if (!g_ascii_strncasecmp (resp, type, len))
 			break;
 	}
-	
+
 	if (i < response->untagged->len) {
 		resp = response->untagged->pdata[i];
 		g_ptr_array_remove_index (response->untagged, i);
 	} else {
 		resp = NULL;
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_PROTOCOL,
 				      _("IMAP server response did not contain "
 					"%s information"), type);
 	}
-	
+
 	camel_imap_response_free (store, response);
 	return resp;
 }
@@ -1070,15 +1143,15 @@ camel_imap_response_extract_continuation (CamelImapStore *store,
 					  CamelException *ex)
 {
 	char *status;
-	
+
 	if (response->status && *response->status == '+') {
 		status = response->status;
 		response->status = NULL;
 		camel_imap_response_free (store, response);
 		return status;
 	}
-	
-	camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+
+	camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_PROTOCOL,
 			      _("Unexpected OK response from IMAP server: %s"),
 			      response->status);
 	camel_imap_response_free (store, response);
@@ -1095,7 +1168,7 @@ imap_command_strdup_vprintf (CamelImapStore *store, const char *fmt,
 	int num, len, i, arglen = 0;
 
 	args = g_ptr_array_new ();
-	
+
 	/* Determine the length of the data */
 	len = strlen (fmt);
 	p = start = fmt;
@@ -1103,7 +1176,7 @@ imap_command_strdup_vprintf (CamelImapStore *store, const char *fmt,
 		p = strchr (start, '%');
 		if (!p)
 			break;
-		
+
 		switch (*++p) {
 		case 'd':
 			num = va_arg (ap, int);
@@ -1157,7 +1230,7 @@ imap_command_strdup_vprintf (CamelImapStore *store, const char *fmt,
 			break;
 		}
 	}
-	
+
 	/* Now write out the string */
 	outptr = out = g_malloc (len + 1);
 	p = start = fmt;
@@ -1171,13 +1244,13 @@ imap_command_strdup_vprintf (CamelImapStore *store, const char *fmt,
 			strncpy (outptr, start, p - start);
 			outptr += p - start;
 		}
-		
+
 		switch (*++p) {
 		case 'd':
 			num = GPOINTER_TO_INT (args->pdata[i++]);
 			outptr += sprintf (outptr, "%d", num);
 			break;
-			
+
 		case 's':
 			string = args->pdata[i++];
 			outptr += sprintf (outptr, "%s", string);
@@ -1194,12 +1267,12 @@ imap_command_strdup_vprintf (CamelImapStore *store, const char *fmt,
 					outptr += sprintf (outptr, "{%d+}\r\n%s", len, string);
 				} else {
 					char *quoted = imap_quote_string (string);
-					
+
 					outptr += sprintf (outptr, "%s", quoted);
 					g_free (quoted);
 				}
 			}
-			
+
 			if (*p == 'F' || *p == 'G')
 				g_free (string);
 			break;
@@ -1207,10 +1280,12 @@ imap_command_strdup_vprintf (CamelImapStore *store, const char *fmt,
 			*outptr++ = '%';
 			*outptr++ = *p;
 		}
-		
+
 		start = *p ? p + 1 : p;
 	}
-	
+
+	g_ptr_array_free (args, TRUE);
+
 	return out;
 }
 
@@ -1219,10 +1294,10 @@ imap_command_strdup_printf (CamelImapStore *store, const char *fmt, ...)
 {
 	va_list ap;
 	char *result;
-	
+
 	va_start (ap, fmt);
 	result = imap_command_strdup_vprintf (store, fmt, ap);
 	va_end (ap);
-	
+
 	return result;
 }
