@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -175,7 +175,7 @@ local_finalize(CamelObject * object)
 	g_free(local_folder->priv);
 }
 
-static int 
+static int
 local_get_local_size (CamelFolder *folder)
 {
 	CamelLocalFolder *local_folder = (CamelLocalFolder *) folder;
@@ -226,10 +226,10 @@ camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, con
 	char folder_path[PATH_MAX];
 	struct stat st;
 #endif
-	int forceindex, len;
+	int len;
 	CamelURL *url;
 	CamelLocalStore *ls = (CamelLocalStore *)parent_store;
-	
+
 	folder = (CamelFolder *)lf;
 
 	name = g_path_get_basename(full_name);
@@ -259,7 +259,7 @@ camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, con
 	if (camel_object_state_read(lf) == -1) {
 		/* No metadata - load defaults and persitify */
 		camel_object_set(lf, NULL, CAMEL_LOCAL_FOLDER_INDEX_BODY, TRUE, 0);
-		camel_object_state_write(lf);
+		/* camel_object_state_write(lf); */
 	}
 #ifndef G_OS_WIN32
 	/* follow any symlinks to the mailbox */
@@ -285,9 +285,9 @@ camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, con
 #if 0
 	/* if we have no/invalid index file, force it */
 	forceindex = camel_text_index_check(lf->index_path) == -1;
-#endif 
+#endif
 
-	/* TNY: Ignore indexing (for Tinymail) 
+	/* TNY: Ignore indexing (for Tinymail)
 	foceindex = FALSE; */
 
 #if 0
@@ -315,9 +315,8 @@ camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, con
 
 
 	folder->summary = (CamelFolderSummary *)CLOCALF_CLASS(lf)->create_summary(lf, lf->summary_path, lf->folder_path, lf->index);
-	if (camel_local_summary_load((CamelLocalSummary *)folder->summary, forceindex, NULL) == -1) {
-		/* ? */
-	}
+
+	/* camel_local_summary_load((CamelLocalSummary *)folder->summary, forceindex, NULL); */
 
 	/* We don't need to sync here ..., it can sync later on when it calls refresh info */
 #if 0
@@ -334,7 +333,7 @@ camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, con
 	if ((flags & CAMEL_STORE_FOLDER_CREATE) != 0) {
 		url = camel_url_copy (((CamelService *) parent_store)->url);
 		camel_url_set_fragment (url, full_name);
-	
+
 		fi = camel_folder_info_new ();
 		fi->full_name = g_strdup (full_name);
 		fi->name = g_strdup (name);
@@ -348,7 +347,7 @@ camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, con
 		}
 
 		camel_url_free (url);
-	
+
 		camel_object_trigger_event(CAMEL_OBJECT (parent_store), "folder_created", fi);
 		camel_folder_info_free(fi);
 	}
@@ -423,7 +422,7 @@ local_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 									      ((CamelService *)folder->parent_store)->url->protocol);
 				else
 					/* a full path + protocol */
-					folder->description = g_strdup_printf(_("%s (%s)"), path, 
+					folder->description = g_strdup_printf(_("%s (%s)"), path,
 									      ((CamelService *)folder->parent_store)->url->protocol);
 			}
 			*arg->ca_str = folder->description;
@@ -507,7 +506,7 @@ local_refresh_info(CamelFolder *folder, CamelException *ex)
 	if (camel_local_summary_check((CamelLocalSummary *)folder->summary, lf->changes, ex) == -1)
 		return;
 
-	/* TNY TODO: Temporary fix (changeinfo's for local folders?! 
+	/* TNY TODO: Temporary fix (changeinfo's for local folders?!
 	if (camel_folder_change_info_changed(lf->changes)) {
 		camel_object_trigger_event((CamelObject *)folder, "folder_changed", lf->changes);
 		camel_folder_change_info_clear(lf->changes);
@@ -525,7 +524,7 @@ local_sync(CamelFolder *folder, gboolean expunge, CamelException *ex)
 	if (camel_local_folder_lock(lf, CAMEL_LOCK_WRITE, ex) == -1)
 		return;
 
-	camel_object_state_write(lf);
+	/* camel_object_state_write(lf); */
 
 	changes = camel_folder_change_info_new ();
 	/* if sync fails, we'll pass it up on exit through ex */
@@ -553,7 +552,7 @@ static void
 local_delete(CamelFolder *folder)
 {
 	CamelLocalFolder *lf = (CamelLocalFolder *)folder;
-	
+
 	if (lf->index)
 		camel_index_delete(lf->index);
 
@@ -571,6 +570,8 @@ local_rename(CamelFolder *folder, const char *newname)
 
 	/* Sync? */
 
+	camel_object_unref (folder->summary);
+
 	g_free(lf->folder_path);
 	g_free(lf->summary_path);
 	g_free(lf->index_path);
@@ -582,12 +583,16 @@ local_rename(CamelFolder *folder, const char *newname)
 	camel_object_set(lf, NULL, CAMEL_OBJECT_STATE_FILE, statepath, NULL);
 	g_free(statepath);
 
+	parent_class->rename(folder, newname);
+
+	folder->summary = (CamelFolderSummary *)CLOCALF_CLASS(lf)->create_summary(lf, lf->summary_path, lf->folder_path, lf->index);
+
 	/* FIXME: Poke some internals, sigh */
 	camel_folder_summary_set_filename(folder->summary, lf->summary_path);
 	g_free(((CamelLocalSummary *)folder->summary)->folder_path);
 	((CamelLocalSummary *)folder->summary)->folder_path = g_strdup(lf->folder_path);
 
-	parent_class->rename(folder, newname);
+
 }
 
 static GPtrArray *
