@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -68,7 +68,7 @@ CamelType
 camel_local_summary_get_type(void)
 {
 	static CamelType type = CAMEL_INVALID_TYPE;
-	
+
 	if (type == CAMEL_INVALID_TYPE) {
 		type = camel_type_register(camel_folder_summary_get_type(), "CamelLocalSummary",
 					   sizeof (CamelLocalSummary),
@@ -78,7 +78,7 @@ camel_local_summary_get_type(void)
 					   (CamelObjectInitFunc) camel_local_summary_init,
 					   (CamelObjectFinalizeFunc) camel_local_summary_finalise);
 	}
-	
+
 	return type;
 }
 
@@ -86,7 +86,7 @@ static void
 camel_local_summary_class_init(CamelLocalSummaryClass *klass)
 {
 	CamelFolderSummaryClass *sklass = (CamelFolderSummaryClass *) klass;
-	
+
 	camel_local_summary_parent = CAMEL_FOLDER_SUMMARY_CLASS(camel_type_get_global_classfuncs(camel_folder_summary_get_type()));
 
 	sklass->summary_header_load = summary_header_load;
@@ -155,9 +155,6 @@ local_summary_load(CamelLocalSummary *cls, int forceindex, CamelException *ex)
 int
 camel_local_summary_load(CamelLocalSummary *cls, int forceindex, CamelException *ex)
 {
-	struct stat st;
-	CamelFolderSummary *s = (CamelFolderSummary *)cls;
-
 	d(printf("Loading summary ...\n"));
 
  	if (((CamelLocalSummaryClass *)(CAMEL_OBJECT_GET_CLASS(cls)))->load(cls, forceindex, ex) == -1) {
@@ -293,7 +290,7 @@ camel_local_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changei
 		}
 
 		printf("\nMemory used by summary:\n\n");
-		printf("Total of %d messages\n", camel_folder_summary_count(s)); 
+		printf("Total of %d messages\n", camel_folder_summary_count(s));
 		printf("Total: %d bytes (ave %f)\n", stats.citotal + stats.mitotal,
 		       (double)(stats.citotal+stats.mitotal)/(double)camel_folder_summary_count(s));
 		printf("Message Info: %d (ave %f)\n", stats.mitotal, (double)stats.mitotal/(double)stats.micount);
@@ -318,17 +315,17 @@ camel_local_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const Cam
 
 /**
  * camel_local_summary_write_headers:
- * @fd: 
- * @header: 
- * @xevline: 
+ * @fd:
+ * @header:
+ * @xevline:
  * @status:
  * @xstatus:
- * 
+ *
  * Write a bunch of headers to the file @fd.  IF xevline is non NULL, then
  * an X-Evolution header line is created at the end of all of the headers.
  * If @status is non NULL, then a Status header line is also written.
  * The headers written are termianted with a blank line.
- * 
+ *
  * Return value: -1 on error, otherwise the number of bytes written.
  **/
 int
@@ -418,10 +415,10 @@ local_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChangeIn
 
 	ret = camel_folder_summary_save((CamelFolderSummary *)cls, ex);
 	if (ret == -1) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM_IO_WRITE,
 				      _("Could not save summary: %s: %s"),
 				      cls->folder_path, g_strerror (errno));
-		
+
 		g_warning ("Could not save summary for %s: %s", cls->folder_path, strerror (errno));
 	}
 
@@ -438,11 +435,12 @@ local_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMess
 	char *xev;
 
 	d(printf("Adding message to summary\n"));
-	
+
 	mi = (CamelLocalMessageInfo *)camel_folder_summary_add_from_message((CamelFolderSummary *)cls, msg);
 	if (mi) {
 		d(printf("Added, uid = %s\n", mi->uid));
 		if (info) {
+
 #ifdef NON_TINYMAIL_FEATURES
 
 			const CamelTag *tag = camel_message_info_user_tags(info);
@@ -452,15 +450,14 @@ local_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMess
 				camel_message_info_set_user_flag((CamelMessageInfo *)mi, flag->name, TRUE);
 				flag = flag->next;
 			}
-			
+
 			while (tag) {
 				camel_message_info_set_user_tag((CamelMessageInfo *)mi, tag->name, tag->value);
 				tag = tag->next;
 			}
 #endif
-
 			mi->info.flags = camel_message_info_flags(info);
-			/*mi->info.flags |= (camel_message_info_flags(info) & 0xffff);*/
+			/*mi->info.flags |= (camel_message_info_flags(info) & 0x1fff);*/
 			mi->info.size = ((CamelMessageInfoBase*)info)->size;
 		}
 
@@ -469,7 +466,7 @@ local_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMess
 			CamelStreamNull *sn = (CamelStreamNull *)camel_stream_null_new();
 
 			camel_data_wrapper_write_to_stream((CamelDataWrapper *)msg, (CamelStream *)sn);
-			mi->info.size = (sn->written);
+			mi->info.size = sn->written;
 			camel_object_unref((CamelObject *)sn);
 		}
 
@@ -481,7 +478,7 @@ local_summary_add(CamelLocalSummary *cls, CamelMimeMessage *msg, const CamelMess
 			camel_folder_change_info_add_uid(ci, camel_message_info_uid(mi));
 	} else {
 		d(printf("Failed!\n"));
-		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
+		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM_MEMORY,
 				     _("Unable to add message to summary: unknown reason"));
 	}
 	return (CamelMessageInfo *)mi;
@@ -508,15 +505,15 @@ local_summary_encode_x_evolution(CamelLocalSummary *cls, const CamelLocalMessage
 	while (*p && isdigit(*p))
 		p++;
 	if (*p == 0 && sscanf (uidstr, "%u", &uid) == 1) {
-		g_string_printf (out, "%08x-%04x", uid, mi->info.flags & 0xffff);
+		g_string_printf (out, "%08x-%04x", uid, mi->info.flags & 0x1fff);
 	} else {
-		g_string_printf (out, "%s-%04x", uidstr, mi->info.flags & 0xffff);
+		g_string_printf (out, "%s-%04x", uidstr, mi->info.flags & 0x1fff);
 	}
 
 #ifdef NON_TINYMAIL_FEATURES
 	if (flag || tag) {
 		val = g_string_new ("");
-		
+
 		if (flag) {
 			while (flag) {
 				g_string_append (val, flag->name);
@@ -546,7 +543,7 @@ local_summary_encode_x_evolution(CamelLocalSummary *cls, const CamelLocalMessage
 
 	ret = out->str;
 	g_string_free (out, FALSE);
-	
+
 	return ret;
 }
 
@@ -576,7 +573,7 @@ local_summary_decode_x_evolution(CamelLocalSummary *cls, const char *xev, CamelL
 	if (mi == NULL)
 		return 0;
 
-	/* check for additional data */	
+	/* check for additional data */
 	header = strchr(xev, ';');
 	if (header) {
 		params = camel_header_param_list_decode(header+1);
@@ -628,8 +625,8 @@ summary_header_load(CamelFolderSummary *s)
 		return 0;
 
 	/* otherwise load the version number */
-	
-	cls->version = g_ntohl(get_unaligned_u32(s->filepos)); 
+
+	cls->version = g_ntohl(get_unaligned_u32(s->filepos));
 	s->filepos += 4;
 
 	return 0;
