@@ -1,20 +1,20 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* camel-pop3-store.c : class for a pop3 store */
 
-/* 
+/*
  * Authors:
  *   Dan Winship <danw@ximian.com>
  *   Michael Zucchi <notzed@ximian.com>
  *   Philip Van Hoof <pvanhoof@gnome.org>
  *
- * This is CamelPop3Store for camel-lite that implements CamelDiscoStore. Its 
- * implementation is significantly different from Camel's upstream version 
+ * This is CamelPop3Store for camel-lite that implements CamelDiscoStore. Its
+ * implementation is significantly different from Camel's upstream version
  * (being used by Evolution): this version supports offline and online modes.
- * 
+ *
  * Copyright (C) 2000-2002 Ximian, Inc. (www.ximian.com)
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of version 2 of the GNU Lesser General Public 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -24,7 +24,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
 
@@ -90,7 +90,7 @@ static gboolean pop3_connect (CamelService *service, CamelException *ex);
 static gboolean pop3_disconnect (CamelService *service, gboolean clean, CamelException *ex);
 static GList *query_auth_types (CamelService *service, CamelException *ex);
 
-static CamelFolder *get_folder (CamelStore *store, const char *folder_name, 
+static CamelFolder *get_folder (CamelStore *store, const char *folder_name,
 				guint32 flags, CamelException *ex);
 
 static CamelFolder *get_trash  (CamelStore *store, CamelException *ex);
@@ -98,13 +98,22 @@ static CamelFolder *get_trash  (CamelStore *store, CamelException *ex);
 static void pop3_get_folder_status (CamelStore *store, const char *folder_name, int *unseen, int *messages, int *uidnext);
 
 
+static int
+pop3store_get_local_size (CamelStore *store, const gchar *folder_name)
+{
+	CamelPOP3Store *p3store = (CamelPOP3Store *) store;
+	int msize = 0;
+	camel_du (p3store->storage_path , &msize);
+	return msize;
+}
 
-static void 
+static char*
 pop3_delete_cache  (CamelStore *store)
 {
 	CamelPOP3Store *pop3_store = (CamelPOP3Store *) store;
 	gchar *folder_dir = pop3_store->storage_path;
 	camel_rm (folder_dir);
+	return g_strdup (folder_dir);
 }
 
 
@@ -158,7 +167,7 @@ pop3_disconnect_online (CamelService *service, gboolean clean, CamelException *e
 
 	if (clean) {
 		CamelPOP3Command *pc;
-		
+
 		pc = camel_pop3_engine_command_new(store->engine, 0, NULL, NULL, "QUIT\r\n");
 		while (camel_pop3_engine_iterate(store->engine, NULL) > 0)
 			;
@@ -209,7 +218,7 @@ pop3_build_folder_info (CamelPOP3Store *store, const char *folder_name)
 		char *buffer = malloc (tsize), *ptr;
 		guint32 version, a;
 		a = fread (buffer, 1, tsize, f);
-		if (a == tsize) 
+		if (a == tsize)
 		{
 			ptr = buffer;
 			version = g_ntohl(get_unaligned_u32(ptr));
@@ -221,7 +230,7 @@ pop3_build_folder_info (CamelPOP3Store *store, const char *folder_name)
 		}
 		g_free (buffer);
 		fclose (f);
-	} 
+	}
 
 	fi->local_size = (guint) msize;
 
@@ -271,7 +280,7 @@ pop3_get_folder_info_online (CamelStore *store, const char *top, guint32 flags,
 enum {
 	MODE_CLEAR,
 	MODE_SSL,
-	MODE_TLS,
+	MODE_TLS
 };
 
 #ifdef HAVE_SSL
@@ -279,6 +288,98 @@ enum {
 #define STARTTLS_FLAGS (CAMEL_TCP_STREAM_SSL_ENABLE_TLS)
 #endif
 
+static void 
+kill_lists (CamelPOP3Store *pop3_store)
+{
+	if (pop3_store->uids)
+		g_ptr_array_free(pop3_store->uids, TRUE);
+	pop3_store->uids = NULL;
+	if (pop3_store->uids_uid)
+		g_hash_table_destroy(pop3_store->uids_uid);
+	if (pop3_store->uids_id)
+		g_hash_table_destroy(pop3_store->uids_id);
+}
+
+void
+camel_pop3_store_destroy_lists (CamelPOP3Store *pop3_store)
+{
+	g_static_rec_mutex_lock (pop3_store->uidl_lock);
+
+	g_static_rec_mutex_lock (pop3_store->eng_lock);
+
+	if (pop3_store->uids != NULL)
+	{
+		CamelPOP3FolderInfo **fi = (CamelPOP3FolderInfo **) pop3_store->uids->pdata;
+		int i;
+
+		for (i=0;i<pop3_store->uids->len;i++,fi++) {
+			if (fi[0]->cmd) {
+
+				if (pop3_store->engine == NULL) {
+					g_ptr_array_free(pop3_store->uids, TRUE);
+					g_hash_table_destroy(pop3_store->uids_uid);
+					g_free(fi[0]->uid);
+					g_free(fi[0]);
+					g_static_rec_mutex_unlock (pop3_store->eng_lock);
+					g_static_rec_mutex_unlock (pop3_store->uidl_lock);
+					return;
+				}
+
+				while (camel_pop3_engine_iterate(pop3_store->engine, fi[0]->cmd) > 0)
+					;
+				camel_pop3_engine_command_free(pop3_store->engine, fi[0]->cmd);
+				fi[0]->cmd = NULL;
+			}
+
+			g_free(fi[0]->uid);
+			g_free(fi[0]);
+		}
+
+		kill_lists (pop3_store);
+
+		pop3_store->uids = g_ptr_array_new ();
+		pop3_store->uids_uid = g_hash_table_new(g_str_hash, g_str_equal);
+		pop3_store->uids_id = g_hash_table_new(NULL, NULL);
+
+	}
+
+	g_static_rec_mutex_unlock (pop3_store->eng_lock);
+	g_static_rec_mutex_unlock (pop3_store->uidl_lock);
+
+}
+
+static gpointer
+wait_for_login_delay (gpointer user_data)
+{
+	CamelPOP3Store *store = CAMEL_POP3_STORE (user_data);
+	gboolean killed = FALSE;
+	guint login_delay = 300;
+
+	g_static_rec_mutex_lock (store->eng_lock);
+	if (!store->engine)
+		killed = TRUE;
+	else
+		login_delay = store->engine->login_delay;
+	g_static_rec_mutex_unlock (store->eng_lock);
+
+	while (!killed) {
+
+		sleep (login_delay);
+
+		g_static_rec_mutex_lock (store->eng_lock);
+		if (!store->is_refreshing) {
+			CamelException dex = CAMEL_EXCEPTION_INITIALISER;
+			camel_pop3_store_destroy_lists (store);
+			camel_service_disconnect (CAMEL_SERVICE (store), TRUE, &dex);
+			killed = TRUE;
+		}
+		g_static_rec_mutex_unlock (store->eng_lock);
+	}
+
+	camel_object_unref (store);
+	return NULL;
+}
+
 static gboolean
 connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, int must_tls, CamelException *ex)
 {
@@ -292,64 +393,67 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, int
 
 	store->connected = FALSE;
 
-	if (ssl_mode != MODE_CLEAR) 
+	if (ssl_mode != MODE_CLEAR)
 	{
 
 #ifdef HAVE_SSL
 		if (ssl_mode == MODE_TLS)
 			tcp_stream = camel_tcp_stream_ssl_new_raw (service, service->url->host, STARTTLS_FLAGS);
-		else 
+		else
 			tcp_stream = camel_tcp_stream_ssl_new (service, service->url->host, SSL_PORT_FLAGS);
 #else
 
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CONNECT,
 				_("Could not connect to %s: %s"),
 				service->url->host, _("SSL unavailable"));
 
 		return FALSE;
 
 #endif /* HAVE_SSL */
-	} else 
+	} else
 		tcp_stream = camel_tcp_stream_raw_new ();
 
-	if ((ret = camel_tcp_stream_connect ((CamelTcpStream *) tcp_stream, ai)) == -1) 
-	{
+	if (!tcp_stream) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CONNECT,
+				_("Could not connect to %s: %s"),
+				service->url->host, _("Couldn't create socket"));
+		return FALSE;
+	}
+
+	if ((ret = camel_tcp_stream_connect ((CamelTcpStream *) tcp_stream, ai)) == -1) {
 		if (errno == EINTR)
 			camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
 				_("Connection canceled"));
 		else
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CONNECT,
 				_("Could not connect to %s: %s"),
 				service->url->host,
 				g_strerror (errno));
-		
+
 		camel_object_unref (tcp_stream);
-		
+
 		return FALSE;
 	}
 
-	/* parent class connect initialization */
-	/*if (CAMEL_SERVICE_CLASS (parent_class)->connect (service, ex) == FALSE) {
-		camel_object_unref (tcp_stream);
-		return FALSE;
-	}*/
-
 	if (camel_url_get_param (service->url, "disable_extensions"))
 		flags |= CAMEL_POP3_ENGINE_DISABLE_EXTENSIONS;
 
-	if ((delete_days = (gchar *) camel_url_get_param(service->url,"delete_after"))) 
+	if ((delete_days = (gchar *) camel_url_get_param(service->url,"delete_after")))
 		store->delete_after =  atoi(delete_days);
 
-	g_static_rec_mutex_lock (store->eng_lock); /* ! */
+	if (!tcp_stream) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CONNECT,
+				_("Could not connect to %s: %s"),
+				service->url->host, _("Couldn't create socket"));
+		return FALSE;
+	}
 
 	if (!(store->engine = camel_pop3_engine_new (tcp_stream, flags))) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CONNECT,
 			_("Failed to read a valid greeting from POP server %s"),
 			service->url->host);
 		camel_object_unref (tcp_stream);
 
-		g_static_rec_mutex_unlock (store->eng_lock); /* ! */
-
 		return FALSE;
 	}
 	store->engine->store = store;
@@ -359,23 +463,16 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, int
 	g_static_rec_mutex_unlock (store->eng_lock);
 #endif
 
-	if (!must_tls && (ssl_mode != MODE_TLS)) 
-	{
+	if (!must_tls && (ssl_mode != MODE_TLS)) {
 		camel_object_unref (tcp_stream);
 		store->connected = TRUE;
-
-#ifdef HAVE_SSL /* ! */
-		g_static_rec_mutex_unlock (store->eng_lock);
-#endif
-
 		return TRUE;
 	}
 
 #ifdef HAVE_SSL
 
-	if (!(store->engine->capa & CAMEL_POP3_CAP_STLS)) 
-	{
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+	if (!(store->engine->capa & CAMEL_POP3_CAP_STLS)) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
 			_("Failed to connect to POP server %s in secure mode: %s"),
 			service->url->host, _("STLS not supported by server"));
 		goto stls_exception;
@@ -391,9 +488,8 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, int
 	ret = pc->state == CAMEL_POP3_COMMAND_OK;
 	camel_pop3_engine_command_free (store->engine, pc);
 
-	if (ret == FALSE) 
-	{
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+	if (ret == FALSE) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
 			_("Failed to connect to POP server %s in secure mode: %s"),
 			service->url->host, store->engine->line);
 		goto stls_exception;
@@ -403,14 +499,14 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, int
 	ret = camel_tcp_stream_ssl_enable_ssl (CAMEL_TCP_STREAM_SSL (tcp_stream));
 
 	if (ret == -1) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
 				      _("Failed to connect to POP server %s in secure mode: %s"),
 				      service->url->host, _("TLS negotiations failed"));
 		goto stls_exception;
 	}
 
 #else
-	camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+	camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
 		_("Failed to connect to POP server %s in secure mode: %s"),
 		service->url->host, _("TLS is not available in this build"));
 
@@ -425,15 +521,14 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, int
 	camel_pop3_engine_reget_capabilities (store->engine);
 	store->connected = TRUE;
 
-	g_static_rec_mutex_unlock (store->eng_lock); /* ! */
+	camel_object_ref (store);
 
 	return TRUE;
 
  stls_exception:
 
 
-	if (clean_quit) 
-	{
+	if (clean_quit) {
 		/* try to disconnect cleanly */
 		pc = camel_pop3_engine_command_new (store->engine, 0, NULL, NULL, "QUIT\r\n");
 		while (camel_pop3_engine_iterate (store->engine, NULL) > 0)
@@ -446,8 +541,6 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, int
 	store->engine = NULL;
 	store->connected = FALSE;
 
-	g_static_rec_mutex_unlock (store->eng_lock); /* ! */
-
 	return FALSE;
 }
 
@@ -475,6 +568,9 @@ connect_to_server_wrapper (CamelService *service, CamelException *ex)
 	char *serv;
 	const char *port;
 
+	if (!service->url)
+		return FALSE;
+
 	if ((ssl_mode = camel_url_get_param (service->url, "use_ssl"))) {
 		for (i = 0; ssl_options[i].value; i++)
 			if (!strcmp (ssl_options[i].value, ssl_mode))
@@ -489,29 +585,31 @@ connect_to_server_wrapper (CamelService *service, CamelException *ex)
 		port = POP3S_PORT;
 		must_tls = 0;
 	}
-	
+
 	if (service->url->port) {
 		serv = g_alloca (16);
 		sprintf (serv, "%d", service->url->port);
 		port = NULL;
 	}
-	
+
 	memset (&hints, 0, sizeof (hints));
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_family = PF_UNSPEC;
+	if (!service->url)
+		return FALSE;
 	ai = camel_getaddrinfo(service->url->host, serv, &hints, ex);
 	if (ai == NULL && port != NULL && camel_exception_get_id(ex) != CAMEL_EXCEPTION_USER_CANCEL) {
 		camel_exception_clear (ex);
 		ai = camel_getaddrinfo(service->url->host, port, &hints, ex);
 	}
-	
+
 	if (ai == NULL)
 		return FALSE;
-	
+
 	ret = connect_to_server (service, ai, mode, must_tls, ex);
-	
+
 	camel_freeaddrinfo (ai);
-	
+
 	return ret;
 }
 
@@ -542,7 +640,7 @@ query_auth_types (CamelService *service, CamelException *ex)
 
 		pop3_disconnect (service, TRUE, NULL);
 	} else {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CONNECT,
 				      _("Could not connect to POP server %s"),
 				      service->url->host);
 	}
@@ -562,7 +660,7 @@ try_sasl(CamelPOP3Store *store, const char *mech, CamelException *ex)
 
 	sasl = camel_sasl_new("pop3", mech, (CamelService *)store);
 	if (sasl == NULL) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_URL_INVALID,
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
 				      _("Unable to connect to POP server %s: "
 					"No support for requested authentication mechanism."),
 				      CAMEL_SERVICE (store)->url->host);
@@ -604,12 +702,12 @@ try_sasl(CamelPOP3Store *store, const char *mech, CamelException *ex)
 	}
 	camel_object_unref((CamelObject *)sasl);
 	return 0;
-	
+
  ioerror:
 	if (errno == EINTR) {
-		camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL, "Canceled");
+		camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL, _("Canceled"));
 	} else {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
 				      _("Failed to authenticate on POP server %s: %s"),
 				      CAMEL_SERVICE (store)->url->host, g_strerror (errno));
 	}
@@ -624,25 +722,33 @@ pop3_try_authenticate (CamelService *service, gboolean reprompt, const char *err
 	CamelPOP3Store *store = (CamelPOP3Store *)service;
 	CamelPOP3Command *pcu = NULL, *pcp = NULL;
 	int status;
-	
+
 	/* override, testing only */
 	/*printf("Forcing authmech to 'login'\n");
 	service->url->authmech = g_strdup("LOGIN");*/
-	
+
 	if (!service->url->passwd) {
-		char *prompt;
+		char *base_prompt;
+		char *full_prompt;
 		guint32 flags = CAMEL_SESSION_PASSWORD_SECRET;
-		
+
 		if (reprompt)
 			flags |= CAMEL_SESSION_PASSWORD_REPROMPT;
-		
-		prompt = g_strdup_printf (_("%sPlease enter the POP password for %s on host %s"),
-					  errmsg ? errmsg : "",
-					  service->url->user,
-					  service->url->host);
-		service->url->passwd = camel_session_get_password (camel_service_get_session (service), service, NULL,
-								   prompt, "password", flags, ex);
-		g_free (prompt);
+
+		base_prompt = camel_session_build_password_prompt (
+			"POP", service->url->user, service->url->host);
+
+		if (errmsg != NULL)
+			full_prompt = g_strconcat (errmsg, base_prompt, NULL);
+		else
+			full_prompt = g_strdup (base_prompt);
+
+		service->url->passwd = camel_session_get_password (
+			camel_service_get_session (service), service,
+			NULL, full_prompt, "password", flags, ex);
+
+		g_free (base_prompt);
+		g_free (full_prompt);
 		if (!service->url->passwd)
 			return FALSE;
 	}
@@ -672,7 +778,7 @@ pop3_try_authenticate (CamelService *service, gboolean reprompt, const char *err
 			if (!isascii((int)*d)) {
 
 				/* README for Translators: The string APOP should not be translated */
-				camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_URL_INVALID,
+				camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
 						_("Unable to connect to POP server %s:	Invalid APOP ID received. Impersonation attack suspected. Please contact your admin."),
 						CAMEL_SERVICE (store)->url->host);
 
@@ -681,10 +787,7 @@ pop3_try_authenticate (CamelService *service, gboolean reprompt, const char *err
 			d++;
 		}
 
-		g_static_rec_mutex_lock (store->eng_lock);
-
 		if (store->engine == NULL) {
-			g_static_rec_mutex_unlock (store->eng_lock);
 			return FALSE;
 		}
 
@@ -698,16 +801,11 @@ pop3_try_authenticate (CamelService *service, gboolean reprompt, const char *err
 		pcp = camel_pop3_engine_command_new(store->engine, 0, NULL, NULL, "APOP %s %s\r\n",
 						    service->url->user, md5asc);
 
-		g_static_rec_mutex_unlock (store->eng_lock);
-
 	} else {
 		CamelServiceAuthType *auth;
 		GList *l;
 
-		g_static_rec_mutex_lock (store->eng_lock);
-
 		if (store->engine == NULL) {
-			g_static_rec_mutex_unlock (store->eng_lock);
 			return FALSE;
 		}
 
@@ -715,37 +813,30 @@ pop3_try_authenticate (CamelService *service, gboolean reprompt, const char *err
 		while (l) {
 			auth = l->data;
 			if (strcmp(auth->authproto, service->url->authmech) == 0) {
-				g_static_rec_mutex_unlock (store->eng_lock);
 				return try_sasl(store, service->url->authmech, ex) == -1;
 			}
 			l = l->next;
 		}
 
-		g_static_rec_mutex_unlock (store->eng_lock);
-
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_URL_INVALID,
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
 				      _("Unable to connect to POP server %s: "
 					"No support for requested authentication mechanism."),
 				      CAMEL_SERVICE (store)->url->host);
 		return FALSE;
 	}
 
-	g_static_rec_mutex_lock (store->eng_lock);
-
 	if (store->engine == NULL) {
-		g_static_rec_mutex_unlock (store->eng_lock);
 		return FALSE;
 	}
 
 	while ((status = camel_pop3_engine_iterate(store->engine, pcp)) > 0)
 		;
 
-
 	if (status == -1) {
 		if (errno == EINTR) {
-			camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL, "Canceled");
+			camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL, _("Canceled"));
 		} else {
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
 					      _("Unable to connect to POP server %s.\n"
 						"Error sending password: %s"),
 					      CAMEL_SERVICE (store)->url->host,
@@ -765,15 +856,20 @@ pop3_try_authenticate (CamelService *service, gboolean reprompt, const char *err
 				      store->engine->line ? (char *)store->engine->line : _("Unknown error"));
 
 	camel_pop3_engine_command_free (store->engine, pcp);
-	
+
 	if (pcu)
 		camel_pop3_engine_command_free(store->engine, pcu);
 
-	g_static_rec_mutex_unlock (store->eng_lock);
-
 	return status;
 }
 
+static void 
+camel_pop3_store_prepare (CamelStore *store)
+{
+	camel_object_ref (store);
+	g_thread_create (wait_for_login_delay, store, FALSE, NULL);
+}
+
 static gboolean
 pop3_connect (CamelService *service, CamelException *ex)
 {
@@ -782,49 +878,63 @@ pop3_connect (CamelService *service, CamelException *ex)
 	CamelSession *session;
 	char *errbuf = NULL;
 	int status;
+	int mytry;
+	gboolean auth = FALSE;
+
+	if (store->logged_in)
+		return TRUE;
 
 	session = camel_service_get_session (service);
 
 	if (!connect_to_server_wrapper (service, ex))
 		return FALSE;
-	
-	while (1) {
+
+	mytry = 0;
+	while (mytry < 3) {
 		status = pop3_try_authenticate (service, reprompt, errbuf, ex);
 		g_free (errbuf);
 		errbuf = NULL;
-		
+
 		/* we only re-prompt if we failed to authenticate, any other error and we just abort */
 		if (status == 0 && camel_exception_get_id (ex) == CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE) {
-			errbuf = g_strdup_printf ("%s\n\n", camel_exception_get_description (ex));
+			errbuf = g_markup_printf_escaped ("%s\n\n", camel_exception_get_description (ex));
 			g_free (service->url->passwd);
 			service->url->passwd = NULL;
 			reprompt = TRUE;
 			camel_exception_clear (ex);
 			sleep (5); /* For example Cyrus-POPd dislikes hammering */
-		} else
+		} else {
+			auth = TRUE;
 			break;
+		}
+
+		mytry++;
 	}
-	
+
 	g_free (errbuf);
-	
+
+	if (!auth) {
+		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
+			_("Authentication failure"));
+		camel_service_disconnect(service, TRUE, ex);
+		return FALSE;
+	}
+
 	if (status == -1 || camel_exception_is_set(ex)) {
 		camel_service_disconnect(service, TRUE, ex);
 		return FALSE;
 	}
-	
+
 	/* Now that we are in the TRANSACTION state, try regetting the capabilities */
 
-	g_static_rec_mutex_lock (store->eng_lock);
-
 	if (store->engine == NULL) {
-		g_static_rec_mutex_unlock (store->eng_lock);
 		return FALSE;
 	}
 
 	store->engine->state = CAMEL_POP3_ENGINE_TRANSACTION;
 	camel_pop3_engine_reget_capabilities (store->engine);
 
-	g_static_rec_mutex_unlock (store->eng_lock);
+	camel_pop3_store_prepare ((CamelStore *) store);
 
 	return TRUE;
 }
@@ -834,7 +944,8 @@ pop3_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 {
 	CamelPOP3Store *store = CAMEL_POP3_STORE (service);
 
-	g_static_rec_mutex_lock (store->eng_lock);
+	store->logged_in = FALSE;
+
 	if (store->engine == NULL) {
 		g_static_rec_mutex_unlock (store->eng_lock);
 		return TRUE;
@@ -842,7 +953,7 @@ pop3_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 
 	if (clean) {
 		CamelPOP3Command *pc;
-		
+
 		pc = camel_pop3_engine_command_new(store->engine, 0, NULL, NULL, "QUIT\r\n");
 		while (camel_pop3_engine_iterate(store->engine, NULL) > 0)
 			;
@@ -851,7 +962,6 @@ pop3_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 
 	camel_object_unref((CamelObject *)store->engine);
 	store->engine = NULL;
-	g_static_rec_mutex_unlock (store->eng_lock);
 
 	return TRUE;
 }
@@ -893,8 +1003,15 @@ finalize (CamelObject *object)
 		g_free (pop3_store->storage_path);
 	pop3_store->storage_path = NULL;
 
-	g_static_rec_mutex_free (pop3_store->eng_lock);
+	/* g_static_rec_mutex_free (pop3_store->eng_lock); */
+	g_free (pop3_store->eng_lock);
 	pop3_store->eng_lock = NULL;
+
+	g_free (pop3_store->uidl_lock);
+	pop3_store->uidl_lock = NULL;
+
+	kill_lists (pop3_store);
+
 	camel_object_unref (pop3_store->book);
 	pop3_store->book = NULL;
 	return;
@@ -913,7 +1030,7 @@ cmd_stat (CamelPOP3Engine *pe, CamelPOP3Stream *stream, void *data)
 	if (!line)
 		return 0;
 
-	sscanf((char *) line, "+OK %d %d", &info->items, &info->bytes);
+	sscanf((char *) line, "+OK %d %d", (int *) &info->items, (int *) &info->bytes);
 
 	return 1;
 }
@@ -962,7 +1079,7 @@ pop3_get_folder_status (CamelStore *store, const char *folder_name, int *unseen,
 	info->items = -1;
 	info->bytes = -1;
 
-	cmd = camel_pop3_engine_command_new(pop3_store->engine, 0, 
+	cmd = camel_pop3_engine_command_new(pop3_store->engine, 0,
 			cmd_stat, info, "STAT\r\n");
 	while ((i = camel_pop3_engine_iterate(pop3_store->engine, NULL)) > 0);
 
@@ -1040,6 +1157,7 @@ camel_pop3_store_class_init (CamelPOP3StoreClass *camel_pop3_store_class)
 	/* camel_service_class->connect = pop3_connect; */
 	/* camel_service_class->disconnect = pop3_disconnect; */
 
+	camel_store_class->get_local_size = pop3store_get_local_size;
 	camel_store_class->get_folder = get_folder;
 	camel_store_class->get_trash = get_trash;
 	camel_store_class->get_folder_status = pop3_get_folder_status;
@@ -1065,11 +1183,17 @@ camel_pop3_store_init (gpointer object, gpointer klass)
 {
 	CamelPOP3Store *store = (CamelPOP3Store *) object;
 
+	store->uids = g_ptr_array_new ();
+	store->uids_uid = g_hash_table_new(g_str_hash, g_str_equal);
+	store->uids_id = g_hash_table_new(NULL, NULL);
+
 	store->is_refreshing = FALSE;
 	store->immediate_delete_after = FALSE;
 	store->book = camel_pop3_logbook_new (store);
 	store->eng_lock = g_new0 (GStaticRecMutex, 1);
 	g_static_rec_mutex_init (store->eng_lock);
+	store->uidl_lock = g_new0 (GStaticRecMutex, 1);
+	g_static_rec_mutex_init (store->uidl_lock);
 
 	return;
 }
