@@ -1,13 +1,13 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* camel-smtp-transport.c : class for a smtp transport */
 
-/* 
+/*
  * Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
  * Copyright (C) 2000 Ximian, Inc. (www.ximian.com)
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of version 2 of the GNU Lesser General Public 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
 
@@ -102,16 +102,16 @@ camel_smtp_transport_class_init (CamelSmtpTransportClass *camel_smtp_transport_c
 		CAMEL_TRANSPORT_CLASS (camel_smtp_transport_class);
 	CamelServiceClass *camel_service_class =
 		CAMEL_SERVICE_CLASS (camel_smtp_transport_class);
-	
+
 	parent_class = CAMEL_TRANSPORT_CLASS (camel_type_get_global_classfuncs (camel_transport_get_type ()));
-	
+
 	/* virtual method overload */
 	camel_service_class->construct = smtp_construct;
 	camel_service_class->connect = smtp_connect;
 	camel_service_class->disconnect = smtp_disconnect;
 	camel_service_class->query_auth_types = query_auth_types;
 	camel_service_class->get_name = get_name;
-	
+
 	camel_transport_class->send_to = smtp_send_to;
 }
 
@@ -119,7 +119,7 @@ static void
 camel_smtp_transport_init (gpointer object)
 {
 	CamelSmtpTransport *smtp = CAMEL_SMTP_TRANSPORT (object);
-	
+
 	smtp->flags = 0;
 	smtp->connected = FALSE;
 }
@@ -134,7 +134,7 @@ CamelType
 camel_smtp_transport_get_type (void)
 {
 	static CamelType type = CAMEL_INVALID_TYPE;
-	
+
 	if (type == CAMEL_INVALID_TYPE) {
 		type = camel_type_register (CAMEL_TRANSPORT_TYPE,
 					    "CamelSmtpTransport",
@@ -145,7 +145,7 @@ camel_smtp_transport_get_type (void)
 					    (CamelObjectInitFunc) camel_smtp_transport_init,
 					    (CamelObjectFinalizeFunc) camel_smtp_transport_finalize);
 	}
-	
+
 	return type;
 }
 
@@ -208,7 +208,7 @@ smtp_error_string (int error)
 		return _("Start mail input; end with <CRLF>.<CRLF>");
 	case 554:
 		return _("Transaction failed");
-		
+
 	/* AUTH error codes: */
 	case 432:
 		return _("A password transition is needed");
@@ -220,7 +220,7 @@ smtp_error_string (int error)
 		return _("Temporary authentication failure");
 	case 530:
 		return _("Authentication required");
-		
+
 	default:
 		return _("Unknown");
 	}
@@ -229,7 +229,7 @@ smtp_error_string (int error)
 enum {
 	MODE_CLEAR,
 	MODE_SSL,
-	MODE_TLS,
+	MODE_TLS
 };
 
 #ifdef HAVE_SSL
@@ -244,14 +244,14 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, int
 	CamelStream *tcp_stream;
 	char *respbuf = NULL;
 	int ret;
-	
+
 	if (!CAMEL_SERVICE_CLASS (parent_class)->connect (service, ex))
 		return FALSE;
-	
+
 	/* set some smtp transport defaults */
 	transport->flags = 0;
 	transport->authtypes = NULL;
-	
+
 	if (ssl_mode != MODE_CLEAR) {
 #ifdef HAVE_SSL
 		if (ssl_mode == MODE_TLS) {
@@ -263,13 +263,13 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, int
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
 				      _("Could not connect to %s: %s"),
 				      service->url->host, _("SSL unavailable"));
-		
+
 		return FALSE;
 #endif /* HAVE_SSL */
 	} else {
 		tcp_stream = camel_tcp_stream_raw_new ();
 	}
-	
+
 	if ((ret = camel_tcp_stream_connect ((CamelTcpStream *) tcp_stream, ai)) == -1) {
 		if (errno == EINTR)
 			camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
@@ -278,20 +278,20 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, int
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
 					      _("Could not connect to %s: %s"),
 					      service->url->host, g_strerror (errno));
-		
+
 		camel_object_unref (tcp_stream);
-		
+
 		return FALSE;
 	}
-	
+
 	transport->connected = TRUE;
-	
+
 	/* get the localaddr - needed later by smtp_helo */
 	transport->localaddr = camel_tcp_stream_get_local_address (CAMEL_TCP_STREAM (tcp_stream), &transport->localaddrlen);
-	
+
 	transport->ostream = tcp_stream;
 	transport->istream = camel_stream_buffer_new (tcp_stream, CAMEL_STREAM_BUFFER_READ);
-	
+
 	/* Read the greeting, note whether the server is ESMTP or not. */
 	do {
 		/* Check for "220" */
@@ -304,23 +304,27 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, int
 		}
 	} while (*(respbuf+3) == '-'); /* if we got "220-" then loop again */
 	g_free (respbuf);
-	
+
 	/* Try sending EHLO */
 	transport->flags |= CAMEL_SMTP_TRANSPORT_IS_ESMTP;
 	if (!smtp_helo (transport, ex)) {
 		if (!transport->connected)
 			return FALSE;
-		
+
 		/* Fall back to HELO */
 		camel_exception_clear (ex);
 		transport->flags &= ~CAMEL_SMTP_TRANSPORT_IS_ESMTP;
-		if (!smtp_helo (transport, ex) && !transport->connected)
+
+		if (!smtp_helo (transport, ex)) {
+			camel_service_disconnect ((CamelService *) transport, TRUE, NULL);
+
 			return FALSE;
+		}
 	}
-	
+
 	/* clear any EHLO/HELO exception and assume that any SMTP errors encountered were non-fatal */
 	camel_exception_clear (ex);
-	
+
 	if (ssl_mode != MODE_TLS) {
 
 		int i = 0;
@@ -330,16 +334,16 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, int
 		/* we're done */
 		return TRUE;
 	} else {
-	
+
 #ifdef HAVE_SSL
 	if (!(transport->flags & CAMEL_SMTP_TRANSPORT_STARTTLS)) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Failed to connect to SMTP server %s in secure mode: %s"),
 				      service->url->host, _("STARTTLS not supported"));
-		
+
 		goto exception_cleanup;
 	}
-	
+
 	smtp_debug ("-> STARTTLS\r\n");
 	if (camel_stream_write (tcp_stream, "STARTTLS\r\n", 10) == -1) {
 		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
@@ -347,23 +351,23 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, int
 				      g_strerror (errno));
 		goto exception_cleanup;
 	}
-	
+
 	respbuf = NULL;
-	
+
 	do {
 		/* Check for "220 Ready for TLS" */
 		g_free (respbuf);
 		respbuf = camel_stream_buffer_read_line (CAMEL_STREAM_BUFFER (transport->istream));
-		
+
 		smtp_debug ("<- %s\n", respbuf ? respbuf : "(null)");
-		
+
 		if (!respbuf || strncmp (respbuf, "220", 3)) {
 			smtp_set_exception (transport, FALSE, respbuf, _("STARTTLS command failed"), ex);
 			g_free (respbuf);
 			goto exception_cleanup;
 		}
 	} while (*(respbuf+3) == '-'); /* if we got "220-" then loop again */
-	
+
 	/* Okay, now toggle SSL/TLS mode */
 	if (camel_tcp_stream_ssl_enable_ssl (CAMEL_TCP_STREAM_SSL (tcp_stream)) == -1) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
@@ -377,25 +381,28 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, int
 			      service->url->host, _("SSL is not available in this build"));
 	goto exception_cleanup;
 #endif /* HAVE_SSL */
-	
+
 	/* We are supposed to re-EHLO after a successful STARTTLS to
            re-fetch any supported extensions. */
-	if (!smtp_helo (transport, ex) && !transport->connected)
+	if (!smtp_helo (transport, ex)) {
+		camel_service_disconnect ((CamelService *) transport, TRUE, NULL);
+
 		return FALSE;
-	
+	}
+
 	return TRUE;
 
 	}
 
  exception_cleanup:
-	
+
 	camel_object_unref (transport->istream);
 	transport->istream = NULL;
 	camel_object_unref (transport->ostream);
 	transport->ostream = NULL;
-	
+
 	transport->connected = FALSE;
-	
+
 	return FALSE;
 }
 
@@ -437,13 +444,13 @@ connect_to_server_wrapper (CamelService *service, CamelException *ex)
 		port = SMTP_PORT;
 		must_tls = 0;
 	}
-	
+
 	if (service->url->port) {
 		serv = g_alloca (16);
 		sprintf (serv, "%d", service->url->port);
 		port = NULL;
 	}
-	
+
 	memset (&hints, 0, sizeof (hints));
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_family = PF_UNSPEC;
@@ -452,14 +459,14 @@ connect_to_server_wrapper (CamelService *service, CamelException *ex)
 		camel_exception_clear (ex);
 		ai = camel_getaddrinfo(service->url->host, port, &hints, ex);
 	}
-	
+
 	if (ai == NULL)
 		return FALSE;
-	
+
 	ret = connect_to_server (service, ai, mode, must_tls, ex);
-	
+
 	camel_freeaddrinfo (ai);
-	
+
 	return ret;
 }
 
@@ -468,29 +475,29 @@ smtp_connect (CamelService *service, CamelException *ex)
 {
 	CamelSmtpTransport *transport = CAMEL_SMTP_TRANSPORT (service);
 	gboolean has_authtypes;
-	
+
 	/* We (probably) need to check popb4smtp before we connect ... */
 	if (service->url->authmech && !strcmp (service->url->authmech, "POPB4SMTP")) {
 		int truth;
 		GByteArray *chal;
 		CamelSasl *sasl;
-		
+
 		sasl = camel_sasl_new ("smtp", "POPB4SMTP", service);
 		chal = camel_sasl_challenge (sasl, NULL, ex);
 		truth = camel_sasl_authenticated (sasl);
 		if (chal)
 			g_byte_array_free (chal, TRUE);
 		camel_object_unref (sasl);
-		
+
 		if (!truth)
 			return FALSE;
-		
+
 		return connect_to_server_wrapper (service, ex);
 	}
-	
+
 	if (!connect_to_server_wrapper (service, ex))
 		return FALSE;
-	
+
 	/* check to see if AUTH is required, if so...then AUTH ourselves */
 	has_authtypes = transport->authtypes ? g_hash_table_size (transport->authtypes) > 0 : FALSE;
 	if (service->url->authmech && (transport->flags & CAMEL_SMTP_TRANSPORT_IS_ESMTP) && has_authtypes) {
@@ -498,20 +505,20 @@ smtp_connect (CamelService *service, CamelException *ex)
 		CamelServiceAuthType *authtype;
 		gboolean authenticated = FALSE;
 		char *errbuf = NULL;
-		
+
 		if (!g_hash_table_lookup (transport->authtypes, service->url->authmech)) {
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
 					      _("SMTP server %s does not support requested "
 						"authentication type %s."),
 					      service->url->host, service->url->authmech);
 
-			camel_session_alert_user (session, CAMEL_SESSION_ALERT_ERROR, 
+			camel_session_alert_user (session, CAMEL_SESSION_ALERT_ERROR,
 				ex, FALSE, service);
 
 			camel_service_disconnect (service, TRUE, NULL);
 			return FALSE;
 		}
-		
+
 		authtype = camel_sasl_authtype (service->url->authmech);
 		if (!authtype) {
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
@@ -520,7 +527,7 @@ smtp_connect (CamelService *service, CamelException *ex)
 			camel_service_disconnect (service, TRUE, NULL);
 			return FALSE;
 		}
-		
+
 		if (!authtype->need_password) {
 			/* authentication mechanism doesn't need a password,
 			   so if it fails there's nothing we can do */
@@ -530,7 +537,7 @@ smtp_connect (CamelService *service, CamelException *ex)
 				return FALSE;
 			}
 		}
-		
+
 		/* keep trying to login until either we succeed or the user cancels */
 		while (!authenticated) {
 			if (errbuf) {
@@ -539,37 +546,45 @@ smtp_connect (CamelService *service, CamelException *ex)
 				g_free (service->url->passwd);
 				service->url->passwd = NULL;
 			}
-			
+
 			if (!service->url->passwd) {
-				char *prompt;
-				
-				prompt = g_strdup_printf (_("%sPlease enter the SMTP password for %s on host %s"),
-							  errbuf ? errbuf : "", service->url->user,
-							  service->url->host);
-				
-				service->url->passwd = camel_session_get_password (session, service, NULL,
-						prompt, "password", CAMEL_SESSION_PASSWORD_SECRET, ex);
-				
-				g_free (prompt);
+				char *base_prompt;
+				char *full_prompt;
+
+				base_prompt = camel_session_build_password_prompt (
+					"SMTP", service->url->user, service->url->host);
+
+				if (errbuf != NULL)
+					full_prompt = g_strconcat (errbuf, base_prompt, NULL);
+				else
+					full_prompt = g_strdup (base_prompt);
+
+				service->url->passwd = camel_session_get_password (
+					session, service, NULL, full_prompt,
+					"password", CAMEL_SESSION_PASSWORD_SECRET, ex);
+
+				g_free (base_prompt);
+				g_free (full_prompt);
 				g_free (errbuf);
 				errbuf = NULL;
-				
+
 				if (!service->url->passwd) {
 					camel_service_disconnect (service, TRUE, NULL);
 					return FALSE;
 				}
 			}
-			
+
 			authenticated = smtp_auth (transport, authtype->authproto, ex);
 			if (!authenticated) {
-				errbuf = g_strdup_printf (_("Unable to authenticate "
-							    "to SMTP server.\n%s\n\n"),
-							  camel_exception_get_description (ex));
+				errbuf = g_markup_printf_escaped (
+					_("Unable to authenticate "
+					  "to SMTP server.\n%s\n\n"),
+					camel_exception_get_description (ex));
 				camel_exception_clear (ex);
 			}
 		}
 	}
-	
+
 	return TRUE;
 }
 
@@ -583,40 +598,40 @@ static gboolean
 smtp_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 {
 	CamelSmtpTransport *transport = CAMEL_SMTP_TRANSPORT (service);
-	
+
 	/*if (!service->connected)
 	 *	return TRUE;
 	 */
-	
+
 	if (transport->connected && clean) {
 		/* send the QUIT command to the SMTP server */
 		smtp_quit (transport, ex);
 	}
-	
+
 	if (!CAMEL_SERVICE_CLASS (parent_class)->disconnect (service, clean, ex))
 		return FALSE;
-	
+
 	if (transport->authtypes) {
 		g_hash_table_foreach (transport->authtypes, authtypes_free, NULL);
 		g_hash_table_destroy (transport->authtypes);
 		transport->authtypes = NULL;
 	}
-	
+
 	if (transport->istream) {
 		camel_object_unref (transport->istream);
 		transport->istream = NULL;
 	}
-	
+
 	if (transport->ostream) {
 		camel_object_unref (transport->ostream);
 		transport->ostream = NULL;
 	}
-	
+
 	g_free(transport->localaddr);
 	transport->localaddr = NULL;
-	
+
 	transport->connected = FALSE;
-	
+
 	return TRUE;
 }
 
@@ -625,34 +640,34 @@ esmtp_get_authtypes (const unsigned char *buffer)
 {
 	const unsigned char *start, *end;
 	GHashTable *table = NULL;
-	
+
 	/* advance to the first token */
 	start = buffer;
 	while (isspace ((int) *start) || *start == '=')
 		start++;
-	
+
 	if (!*start)
 		return NULL;
-	
+
 	table = g_hash_table_new (g_str_hash, g_str_equal);
-	
+
 	for ( ; *start; ) {
 		char *type;
-		
+
 		/* advance to the end of the token */
 		end = start;
 		while (*end && !isspace ((int) *end))
 			end++;
-		
+
 		type = g_strndup ((gchar*) start, end - start);
 		g_hash_table_insert (table, type, type);
-		
+
 		/* advance to the next token */
 		start = end;
 		while (isspace ((int) *start))
 			start++;
 	}
-	
+
 	return table;
 }
 
@@ -662,23 +677,23 @@ query_auth_types (CamelService *service, CamelException *ex)
 	CamelSmtpTransport *transport = CAMEL_SMTP_TRANSPORT (service);
 	CamelServiceAuthType *authtype;
 	GList *types, *t, *next;
-	
+
 	if (!connect_to_server_wrapper (service, ex))
 		return NULL;
-	
+
 	types = g_list_copy (service->provider->authtypes);
 	for (t = types; t; t = next) {
 		authtype = t->data;
 		next = t->next;
-		
+
 		if (!g_hash_table_lookup (transport->authtypes, authtype->authproto)) {
 			types = g_list_remove_link (types, t);
 			g_list_free_1 (t);
 		}
 	}
-	
+
 	smtp_disconnect (service, TRUE, NULL);
-	
+
 	return types;
 }
 
@@ -703,31 +718,31 @@ smtp_send_to (CamelTransport *transport, CamelMimeMessage *message,
 	gboolean has_8bit_parts;
 	const char *addr;
 	int i, len;
-	
+
 	if (!smtp_transport->connected) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_NOT_CONNECTED,
 				      _("Cannot send message: service not connected."));
 		return FALSE;
 	}
-	
+
 	if (!camel_internet_address_get (CAMEL_INTERNET_ADDRESS (from), 0, NULL, &addr)) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Cannot send message: sender address not valid."));
 		return FALSE;
 	}
-	
+
 	camel_operation_start (NULL, _("Sending message"));
-	
+
 	/* find out if the message has 8bit mime parts */
 	has_8bit_parts = camel_mime_message_has_8bit_parts (message);
-	
+
 	/* rfc1652 (8BITMIME) requires that you notify the ESMTP daemon that
 	   you'll be sending an 8bit mime message at "MAIL FROM:" time. */
 	if (!smtp_mail (smtp_transport, addr, has_8bit_parts, ex)) {
 		camel_operation_end (NULL);
 		return FALSE;
 	}
-	
+
 	len = camel_address_length (recipients);
 	if (len == 0) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
@@ -735,18 +750,18 @@ smtp_send_to (CamelTransport *transport, CamelMimeMessage *message,
 		camel_operation_end (NULL);
 		return FALSE;
 	}
-	
+
 	cia = CAMEL_INTERNET_ADDRESS (recipients);
 	for (i = 0; i < len; i++) {
 		char *enc;
-		
+
 		if (!camel_internet_address_get (cia, i, NULL, &addr)) {
 			camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
 					     _("Cannot send message: one or more invalid recipients"));
 			camel_operation_end (NULL);
 			return FALSE;
 		}
-		
+
 		enc = camel_internet_address_encode_address(NULL, NULL, addr);
 		if (!smtp_rcpt (smtp_transport, enc, ex)) {
 			g_free(enc);
@@ -755,18 +770,18 @@ smtp_send_to (CamelTransport *transport, CamelMimeMessage *message,
 		}
 		g_free(enc);
 	}
-	
+
 	if (!smtp_data (smtp_transport, message, ex)) {
 		camel_operation_end (NULL);
 		return FALSE;
 	}
-	
+
 	/* reset the service for our next transfer session */
 	if (!smtp_rset (smtp_transport, ex))
 		camel_exception_clear (ex);
-	
+
 	camel_operation_end (NULL);
-	
+
 	return TRUE;
 }
 
@@ -774,14 +789,14 @@ static const char *
 smtp_next_token (const char *buf)
 {
 	const unsigned char *token;
-	
+
 	token = (const unsigned char *) buf;
 	while (*token && !isspace ((int) *token))
 		token++;
-	
+
 	while (*token && isspace ((int) *token))
 		token++;
-	
+
 	return (const char *) token;
 }
 
@@ -818,7 +833,7 @@ smtp_decode_status_code (const char *in, size_t len)
 
 	outbuf = (char *) g_malloc (len + 1);
 	outptr = (unsigned char *) outbuf;
-	
+
 	inptr = (unsigned char *) in;
 	inend = inptr + len;
 	while (inptr < inend) {
@@ -831,12 +846,50 @@ smtp_decode_status_code (const char *in, size_t len)
 		} else
 			*outptr++ = *inptr++;
 	}
-	
+
 	*outptr = '\0';
-	
+
 	return outbuf;
 }
 
+/* converts string str to local encoding, thinking it's in utf8.
+   If fails, then converts all character greater than 127 to hex values.
+   Also those under 32, other than \n, \r, \t.
+   Note that the c is signed character, so all characters above 127 have
+   negative value.
+*/
+static void
+convert_to_local (GString *str)
+{
+	char *buf;
+
+	buf = g_locale_from_utf8 (str->str, str->len, NULL, NULL, NULL);
+
+	if (!buf) {
+		int i;
+		gchar c;
+		GString *s = g_string_new_len (str->str, str->len);
+
+		g_string_truncate (str, 0);
+
+		for (i = 0; i < s->len; i++) {
+			c = s->str[i];
+
+			if (c < 32 && c != '\n' && c != '\r' && c != '\t')
+				g_string_append_printf (str, "<%X%X>", (c >> 4) & 0xF, c & 0xF);
+			else
+				g_string_append_c (str, c);
+		}
+
+		g_string_free (s, TRUE);
+	} else {
+		g_string_truncate (str, 0);
+		g_string_append (str, buf);
+
+		g_free (buf);
+	}
+}
+
 static void
 smtp_set_exception (CamelSmtpTransport *transport, gboolean disconnect, const char *respbuf, const char *message, CamelException *ex)
 {
@@ -844,8 +897,8 @@ smtp_set_exception (CamelSmtpTransport *transport, gboolean disconnect, const ch
 	char *buffer = NULL;
 	GString *string;
 	int error;
-	
-	if (!respbuf || !(transport->flags & CAMEL_SMTP_TRANSPORT_ENHANCEDSTATUSCODES)) {
+
+	if (!respbuf) {
 	fake_status_code:
 		error = respbuf ? atoi (respbuf) : 0;
 		camel_exception_setv (ex, error == 0 && errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
@@ -853,13 +906,17 @@ smtp_set_exception (CamelSmtpTransport *transport, gboolean disconnect, const ch
 	} else {
 		string = g_string_new ("");
 		do {
-			token = smtp_next_token (rbuf + 4);
+			if (transport->flags & CAMEL_SMTP_TRANSPORT_ENHANCEDSTATUSCODES)
+				token = smtp_next_token (rbuf + 4);
+			else
+				token = rbuf + 4;
+
 			if (*token == '\0') {
 				g_free (buffer);
 				g_string_free (string, TRUE);
 				goto fake_status_code;
 			}
-			
+
 			g_string_append (string, token);
 			if (*(rbuf + 3) == '-') {
 				g_free (buffer);
@@ -869,21 +926,37 @@ smtp_set_exception (CamelSmtpTransport *transport, gboolean disconnect, const ch
 				g_free (buffer);
 				buffer = NULL;
 			}
-			
+
 			rbuf = buffer;
 		} while (rbuf);
-		
-		buffer = smtp_decode_status_code (string->str, string->len);
-		g_string_free (string, TRUE);
-		if (!buffer)
-			goto fake_status_code;
-		
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      "%s: %s", message, buffer);
-		
-		g_free (buffer);
+
+		convert_to_local (string);
+		if (!(transport->flags & CAMEL_SMTP_TRANSPORT_ENHANCEDSTATUSCODES) && string->len) {
+			string->str = g_strstrip (string->str);
+			string->len = strlen (string->str);
+
+			if (!string->len) {
+				g_string_free (string, TRUE);
+				goto fake_status_code;
+			}
+
+			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+					      "%s: %s", message, string->str);
+
+			g_string_free (string, TRUE);
+		} else {
+			buffer = smtp_decode_status_code (string->str, string->len);
+			g_string_free (string, TRUE);
+			if (!buffer)
+				goto fake_status_code;
+
+			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+					      "%s: %s", message, buffer);
+
+			g_free (buffer);
+		}
 	}
-	
+
 	if (!respbuf) {
 		/* we got disconnected */
 		if (disconnect)
@@ -900,24 +973,24 @@ smtp_helo (CamelSmtpTransport *transport, CamelException *ex)
 	const char *token, *numeric = NULL;
 	struct sockaddr *addr;
 	socklen_t addrlen;
-	
+
 	/* these are flags that we set, so unset them in case we
 	   are being called a second time (ie, after a STARTTLS) */
 	transport->flags &= ~(CAMEL_SMTP_TRANSPORT_8BITMIME |
 			      CAMEL_SMTP_TRANSPORT_ENHANCEDSTATUSCODES |
 			      CAMEL_SMTP_TRANSPORT_STARTTLS);
-	
+
 	if (transport->authtypes) {
 		g_hash_table_foreach (transport->authtypes, authtypes_free, NULL);
 		g_hash_table_destroy (transport->authtypes);
 		transport->authtypes = NULL;
 	}
-	
+
 	camel_operation_start_transient (NULL, _("SMTP Greeting"));
-	
+
 	addr = transport->localaddr;
 	addrlen = transport->localaddrlen;
-	
+
 	if (camel_getnameinfo (addr, addrlen, &name, NULL, NI_NUMERICHOST, NULL) != 0) {
 		name = g_strdup ("localhost.localdomain");
 	} else {
@@ -926,7 +999,7 @@ smtp_helo (CamelSmtpTransport *transport, CamelException *ex)
 		else
 			numeric = "";
 	}
-	
+
 	token = (transport->flags & CAMEL_SMTP_TRANSPORT_IS_ESMTP) ? "EHLO" : "HELO";
 	if (numeric)
 		cmdbuf = g_strdup_printf("%s [%s%s]\r\n", token, numeric, name);
@@ -940,25 +1013,25 @@ smtp_helo (CamelSmtpTransport *transport, CamelException *ex)
 		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
 				      _("HELO command failed: %s"), g_strerror (errno));
 		camel_operation_end (NULL);
-		
+
 		camel_service_disconnect ((CamelService *) transport, FALSE, NULL);
-		
+
 		return FALSE;
 	}
 	g_free (cmdbuf);
-	
+
 	do {
 		/* Check for "250" */
 		g_free (respbuf);
 		respbuf = camel_stream_buffer_read_line (CAMEL_STREAM_BUFFER (transport->istream));
-		
+
 		smtp_debug ("<- %s\n", respbuf ? respbuf : "(null)");
-		
+
 		if (!respbuf || strncmp (respbuf, "250", 3)) {
 			smtp_set_exception (transport, FALSE, respbuf, _("HELO command failed"), ex);
 			camel_operation_end (NULL);
 			g_free (respbuf);
-			
+
 			return FALSE;
 		}
 		
@@ -966,13 +1039,10 @@ smtp_helo (CamelSmtpTransport *transport, CamelException *ex)
 		
 		if (transport->flags & CAMEL_SMTP_TRANSPORT_IS_ESMTP) {
 			if (!strncmp (token, "8BITMIME", 8)) {
-				d(fprintf (stderr, "This server supports 8bit MIME\n"));
 				transport->flags |= CAMEL_SMTP_TRANSPORT_8BITMIME;
 			} else if (!strncmp (token, "ENHANCEDSTATUSCODES", 19)) {
-				d(fprintf (stderr, "This server supports enhanced status codes\n"));
 				transport->flags |= CAMEL_SMTP_TRANSPORT_ENHANCEDSTATUSCODES;
 			} else if (!strncmp (token, "STARTTLS", 8)) {
-				d(fprintf (stderr, "This server supports STARTTLS\n"));
 				transport->flags |= CAMEL_SMTP_TRANSPORT_STARTTLS;
 			} else if (!strncmp (token, "AUTH", 4)) {
 				if (!transport->authtypes || transport->flags & CAMEL_SMTP_TRANSPORT_AUTH_EQUAL) {
@@ -988,29 +1058,29 @@ smtp_helo (CamelSmtpTransport *transport, CamelException *ex)
 					 * format. We want to let the standard way have priority over the
 					 * broken way.
 					 **/
-					
+
 					if (token[4] == '=')
 						transport->flags |= CAMEL_SMTP_TRANSPORT_AUTH_EQUAL;
 					else
 						transport->flags &= ~CAMEL_SMTP_TRANSPORT_AUTH_EQUAL;
-					
+
 					/* parse for supported AUTH types */
 					token += 5;
-					
+
 					if (transport->authtypes) {
 						g_hash_table_foreach (transport->authtypes, authtypes_free, NULL);
 						g_hash_table_destroy (transport->authtypes);
 					}
-					
+
 					transport->authtypes = esmtp_get_authtypes ((const unsigned char *) token);
 				}
 			}
 		}
 	} while (*(respbuf+3) == '-'); /* if we got "250-" then loop again */
 	g_free (respbuf);
-	
+
 	camel_operation_end (NULL);
-	
+
 	return TRUE;
 }
 
@@ -1020,9 +1090,9 @@ smtp_auth (CamelSmtpTransport *transport, const char *mech, CamelException *ex)
 	char *cmdbuf, *respbuf = NULL, *challenge;
 	gboolean auth_challenge = FALSE;
 	CamelSasl *sasl = NULL;
-	
+
 	camel_operation_start_transient (NULL, _("SMTP Authentication"));
-	
+
 	sasl = camel_sasl_new ("smtp", mech, CAMEL_SERVICE (transport));
 	if (!sasl) {
 		camel_operation_end (NULL);
@@ -1030,7 +1100,7 @@ smtp_auth (CamelSmtpTransport *transport, const char *mech, CamelException *ex)
 				      _("Error creating SASL authentication object."));
 		return FALSE;
 	}
-	
+
 	challenge = camel_sasl_challenge_base64 (sasl, NULL, ex);
 	if (challenge) {
 		auth_challenge = TRUE;
@@ -1039,7 +1109,7 @@ smtp_auth (CamelSmtpTransport *transport, const char *mech, CamelException *ex)
 	} else {
 		cmdbuf = g_strdup_printf ("AUTH %s\r\n", mech);
 	}
-	
+
 	smtp_debug ("-> %s\n", cmdbuf);
 	if (camel_stream_write (transport->ostream, cmdbuf, strlen (cmdbuf)) == -1) {
 		g_free (cmdbuf);
@@ -1048,7 +1118,7 @@ smtp_auth (CamelSmtpTransport *transport, const char *mech, CamelException *ex)
 		goto lose;
 	}
 	g_free (cmdbuf);
-	
+
 	respbuf = camel_stream_buffer_read_line (CAMEL_STREAM_BUFFER (transport->istream));
 	smtp_debug ("<- %s\n", respbuf ? respbuf : "(null)");
 
@@ -1058,14 +1128,14 @@ smtp_auth (CamelSmtpTransport *transport, const char *mech, CamelException *ex)
 					      _("AUTH command failed: %s"), g_strerror (errno));
 			goto lose;
 		}
-		
+
 		/* the server challenge/response should follow a 334 code */
 		if (strncmp (respbuf, "334", 3) != 0) {
 			smtp_set_exception (transport, FALSE, respbuf, _("AUTH command failed"), ex);
 			g_free (respbuf);
 			goto lose;
 		}
-		
+
 		if (FALSE) {
 		broken_smtp_server:
 			d(fprintf (stderr, "Your SMTP server's implementation of the %s SASL\n"
@@ -1074,15 +1144,15 @@ smtp_auth (CamelSmtpTransport *transport, const char *mech, CamelException *ex)
 				   "for the first time (specifically Section 4).\n",
 				   mech));
 		}
-		
+
 		/* eat whtspc */
 		for (challenge = respbuf + 4; isspace (*challenge); challenge++);
-		
+
 		challenge = camel_sasl_challenge_base64 (sasl, challenge, ex);
 		g_free (respbuf);
 		if (challenge == NULL)
 			goto break_and_lose;
-		
+
 		/* send our challenge */
 		cmdbuf = g_strdup_printf ("%s\r\n", challenge);
 		g_free (challenge);
@@ -1092,12 +1162,12 @@ smtp_auth (CamelSmtpTransport *transport, const char *mech, CamelException *ex)
 			goto lose;
 		}
 		g_free (cmdbuf);
-		
+
 		/* get the server's response */
 		respbuf = camel_stream_buffer_read_line (CAMEL_STREAM_BUFFER (transport->istream));
 		smtp_debug ("<- %s\n", respbuf ? respbuf : "(null)");
 	}
-	
+
 	/* check that the server says we are authenticated */
 	if (!respbuf || strncmp (respbuf, "235", 3)) {
 		if (respbuf && auth_challenge && !strncmp (respbuf, "334", 3)) {
@@ -1107,28 +1177,29 @@ smtp_auth (CamelSmtpTransport *transport, const char *mech, CamelException *ex)
 		g_free (respbuf);
 		goto lose;
 	}
-	
+
+	g_free (respbuf);
 	camel_object_unref (sasl);
 	camel_operation_end (NULL);
-	
+
 	return TRUE;
-	
+
  break_and_lose:
 	/* Get the server out of "waiting for continuation data" mode. */
 	smtp_debug ("-> *\n");
 	camel_stream_write (transport->ostream, "*\r\n", 3);
 	respbuf = camel_stream_buffer_read_line (CAMEL_STREAM_BUFFER (transport->istream));
 	smtp_debug ("<- %s\n", respbuf ? respbuf : "(null)");
-	
+
  lose:
 	if (!camel_exception_is_set (ex)) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
 				     _("Bad authentication response from server.\n"));
 	}
-	
+
 	camel_object_unref (sasl);
 	camel_operation_end (NULL);
-	
+
 	return FALSE;
 }
 
@@ -1137,33 +1208,33 @@ smtp_mail (CamelSmtpTransport *transport, const char *sender, gboolean has_8bit_
 {
 	/* we gotta tell the smtp server who we are. (our email addy) */
 	char *cmdbuf, *respbuf = NULL;
-	
+
 	if (transport->flags & CAMEL_SMTP_TRANSPORT_8BITMIME && has_8bit_parts)
 		cmdbuf = g_strdup_printf ("MAIL FROM:<%s> BODY=8BITMIME\r\n", sender);
 	else
 		cmdbuf = g_strdup_printf ("MAIL FROM:<%s>\r\n", sender);
-	
+
 	smtp_debug ("-> %s\n", cmdbuf);
-	
+
 	if (camel_stream_write (transport->ostream, cmdbuf, strlen (cmdbuf)) == -1) {
 		g_free (cmdbuf);
 		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
 				      _("MAIL FROM command failed: %s: mail not sent"),
 				      g_strerror (errno));
-		
+
 		camel_service_disconnect ((CamelService *) transport, FALSE, NULL);
-		
+
 		return FALSE;
 	}
 	g_free (cmdbuf);
-	
+
 	do {
 		/* Check for "250 Sender OK..." */
 		g_free (respbuf);
 		respbuf = camel_stream_buffer_read_line (CAMEL_STREAM_BUFFER (transport->istream));
-		
+
 		smtp_debug ("<- %s\n", respbuf ? respbuf : "(null)");
-		
+
 		if (!respbuf || strncmp (respbuf, "250", 3)) {
 			smtp_set_exception (transport, TRUE, respbuf, _("MAIL FROM command failed"), ex);
 			g_free (respbuf);
@@ -1171,7 +1242,7 @@ smtp_mail (CamelSmtpTransport *transport, const char *sender, gboolean has_8bit_
 		}
 	} while (*(respbuf+3) == '-'); /* if we got "250-" then loop again */
 	g_free (respbuf);
-	
+
 	return TRUE;
 }
 
@@ -1181,33 +1252,33 @@ smtp_rcpt (CamelSmtpTransport *transport, const char *recipient, CamelException
 	/* we gotta tell the smtp server who we are going to be sending
 	 * our email to */
 	char *cmdbuf, *respbuf = NULL;
-	
+
 	cmdbuf = g_strdup_printf ("RCPT TO:<%s>\r\n", recipient);
-	
+
 	smtp_debug ("-> %s", cmdbuf);
-	
+
 	if (camel_stream_write (transport->ostream, cmdbuf, strlen (cmdbuf)) == -1) {
 		g_free (cmdbuf);
 		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
 				      _("RCPT TO command failed: %s: mail not sent"),
 				      g_strerror (errno));
-		
+
 		camel_service_disconnect ((CamelService *) transport, FALSE, NULL);
-		
+
 		return FALSE;
 	}
 	g_free (cmdbuf);
-	
+
 	do {
 		/* Check for "250 Recipient OK..." */
 		g_free (respbuf);
 		respbuf = camel_stream_buffer_read_line (CAMEL_STREAM_BUFFER (transport->istream));
-		
+
 		smtp_debug ( "<- %s\n", respbuf ? respbuf : "(null)");
-		
+
 		if (!respbuf || strncmp (respbuf, "250", 3)) {
 			char *message;
-			
+
 			message = g_strdup_printf (_("RCPT TO <%s> failed"), recipient);
 			smtp_set_exception (transport, TRUE, respbuf, message, ex);
 			g_free (message);
@@ -1216,7 +1287,7 @@ smtp_rcpt (CamelSmtpTransport *transport, const char *recipient, CamelException
 		}
 	} while (*(respbuf+3) == '-'); /* if we got "250-" then loop again */
 	g_free (respbuf);
-	
+
 	return TRUE;
 }
 
@@ -1229,37 +1300,37 @@ smtp_data (CamelSmtpTransport *transport, CamelMimeMessage *message, CamelExcept
 	CamelStreamFilter *filtered_stream;
 	CamelMimeFilter *crlffilter;
 	int ret;
-	
+
 	/* If the server doesn't support 8BITMIME, set our required encoding to be 7bit */
 	if (!(transport->flags & CAMEL_SMTP_TRANSPORT_8BITMIME))
 		enctype = CAMEL_BESTENC_7BIT;
-	
+
 	/* FIXME: should we get the best charset too?? */
 	/* Changes the encoding of all mime parts to fit within our required
 	   encoding type and also force any text parts with long lines (longer
 	   than 998 octets) to wrap by QP or base64 encoding them. */
 	camel_mime_message_set_best_encoding (message, CAMEL_BESTENC_GET_ENCODING, enctype);
-	
+
 	cmdbuf = g_strdup ("DATA\r\n");
-	
+
 	smtp_debug ("-> %s\n", cmdbuf);
-	
+
 	if (camel_stream_write (transport->ostream, cmdbuf, strlen (cmdbuf)) == -1) {
 		g_free (cmdbuf);
 		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
 				      _("DATA command failed: %s: mail not sent"),
 				      g_strerror (errno));
-		
+
 		camel_service_disconnect ((CamelService *) transport, FALSE, NULL);
-		
+
 		return FALSE;
 	}
 	g_free (cmdbuf);
-	
+
 	respbuf = camel_stream_buffer_read_line (CAMEL_STREAM_BUFFER (transport->istream));
-	
+
 	smtp_debug ("<- %s\n", respbuf ? respbuf : "(null)");
-	
+
 	if (!respbuf || strncmp (respbuf, "354", 3)) {
 		/* we should have gotten instructions on how to use the DATA command:
 		 * 354 Enter mail, end with "." on a line by itself
@@ -1268,20 +1339,20 @@ smtp_data (CamelSmtpTransport *transport, CamelMimeMessage *message, CamelExcept
 		g_free (respbuf);
 		return FALSE;
 	}
-	
+
 	g_free (respbuf);
 	respbuf = NULL;
-	
+
 	/* setup stream filtering */
 	crlffilter = camel_mime_filter_crlf_new (CAMEL_MIME_FILTER_CRLF_ENCODE, CAMEL_MIME_FILTER_CRLF_MODE_CRLF_DOTS);
 	filtered_stream = camel_stream_filter_new_with_stream (transport->ostream);
 	camel_stream_filter_add (filtered_stream, CAMEL_MIME_FILTER (crlffilter));
 	camel_object_unref (crlffilter);
-	
+
 	/* unlink the bcc headers */
 	savedbcc = NULL;
 	tail = (struct _camel_header_raw *) &savedbcc;
-	
+
 	header = (struct _camel_header_raw *) &CAMEL_MIME_PART (message)->headers;
 	n = header->next;
 	while (n != NULL) {
@@ -1293,52 +1364,52 @@ smtp_data (CamelSmtpTransport *transport, CamelMimeMessage *message, CamelExcept
 		} else {
 			header = n;
 		}
-		
+
 		n = header->next;
 	}
-	
+
 	/* write the message */
 	ret = camel_data_wrapper_write_to_stream (CAMEL_DATA_WRAPPER (message), CAMEL_STREAM (filtered_stream));
-	
+
 	/* restore the bcc headers */
 	header->next = savedbcc;
-	
+
 	if (ret == -1) {
 		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
 				      _("DATA command failed: %s: mail not sent"),
 				      g_strerror (errno));
-		
+
 		camel_object_unref (filtered_stream);
-		
+
 		camel_service_disconnect ((CamelService *) transport, FALSE, NULL);
-		
+
 		return FALSE;
 	}
-	
+
 	camel_stream_flush (CAMEL_STREAM (filtered_stream));
 	camel_object_unref (filtered_stream);
-	
+
 	/* terminate the message body */
-	
+
 	smtp_debug ("-> \\r\\n.\\r\\n\n");
-	
+
 	if (camel_stream_write (transport->ostream, "\r\n.\r\n", 5) == -1) {
 		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
 				      _("DATA command failed: %s: mail not sent"),
 				      g_strerror (errno));
-		
+
 		camel_service_disconnect ((CamelService *) transport, FALSE, NULL);
-		
+
 		return FALSE;
 	}
-	
+
 	do {
 		/* Check for "250 Sender OK..." */
 		g_free (respbuf);
 		respbuf = camel_stream_buffer_read_line (CAMEL_STREAM_BUFFER (transport->istream));
-		
+
 		smtp_debug ("<- %s\n", respbuf ? respbuf : "(null)");
-		
+
 		if (!respbuf || strncmp (respbuf, "250", 3)) {
 			smtp_set_exception (transport, TRUE, respbuf, _("DATA command failed"), ex);
 			g_free (respbuf);
@@ -1346,7 +1417,7 @@ smtp_data (CamelSmtpTransport *transport, CamelMimeMessage *message, CamelExcept
 		}
 	} while (*(respbuf+3) == '-'); /* if we got "250-" then loop again */
 	g_free (respbuf);
-	
+
 	return TRUE;
 }
 
@@ -1355,29 +1426,29 @@ smtp_rset (CamelSmtpTransport *transport, CamelException *ex)
 {
 	/* we are going to reset the smtp server (just to be nice) */
 	char *cmdbuf, *respbuf = NULL;
-	
+
 	cmdbuf = g_strdup ("RSET\r\n");
-	
+
 	smtp_debug ("-> %s", cmdbuf);
-	
+
 	if (camel_stream_write (transport->ostream, cmdbuf, strlen (cmdbuf)) == -1) {
 		g_free (cmdbuf);
 		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
 				      _("RSET command failed: %s"), g_strerror (errno));
-		
+
 		camel_service_disconnect ((CamelService *) transport, FALSE, NULL);
-		
+
 		return FALSE;
 	}
 	g_free (cmdbuf);
-	
+
 	do {
 		/* Check for "250" */
 		g_free (respbuf);
 		respbuf = camel_stream_buffer_read_line (CAMEL_STREAM_BUFFER (transport->istream));
-		
+
 		smtp_debug ("<- %s\n", respbuf ? respbuf : "(null)");
-		
+
 		if (!respbuf || strncmp (respbuf, "250", 3)) {
 			smtp_set_exception (transport, TRUE, respbuf, _("RSET command failed"), ex);
 			g_free (respbuf);
@@ -1385,7 +1456,7 @@ smtp_rset (CamelSmtpTransport *transport, CamelException *ex)
 		}
 	} while (*(respbuf+3) == '-'); /* if we got "250-" then loop again */
 	g_free (respbuf);
-	
+
 	return TRUE;
 }
 
@@ -1394,16 +1465,16 @@ smtp_quit (CamelSmtpTransport *transport, CamelException *ex)
 {
 	/* we are going to reset the smtp server (just to be nice) */
 	char *cmdbuf, *respbuf = NULL;
-	
+
 	cmdbuf = g_strdup ("QUIT\r\n");
-	
+
 	smtp_debug ("-> %s", cmdbuf);
-	
+
 	if (camel_stream_write (transport->ostream, cmdbuf, strlen (cmdbuf)) == -1) {
 		g_free (cmdbuf);
 		camel_exception_setv (ex, errno == EINTR ? CAMEL_EXCEPTION_USER_CANCEL : CAMEL_EXCEPTION_SYSTEM,
 				      _("QUIT command failed: %s"), g_strerror (errno));
-		
+
 		return FALSE;
 	}
 	g_free (cmdbuf);
@@ -1412,12 +1483,12 @@ smtp_quit (CamelSmtpTransport *transport, CamelException *ex)
 	{
 		do {
 			/* Check for "221" */
-			if (respbuf) 
+			if (respbuf)
 				g_free (respbuf);
 			respbuf = camel_stream_buffer_read_line (CAMEL_STREAM_BUFFER (transport->istream));
-			
+
 			smtp_debug ("<- %s\n", respbuf ? respbuf : "(null)");
-			
+
 			if (!respbuf || strncmp (respbuf, "221", 3)) {
 				smtp_set_exception (transport, FALSE, respbuf, _("QUIT command failed"), ex);
 				g_free (respbuf);
