@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with self library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -47,6 +47,8 @@
 #include <tny-camel-shared.h>
 #include <tny-status.h>
 
+#include <tny-camel-default-connection-policy.h>
+
 #include "tny-session-camel-priv.h"
 #include "tny-camel-common-priv.h"
 
@@ -96,6 +98,7 @@ _tny_camel_account_emit_changed (TnyCamelAccount *self)
 
 
 typedef struct {
+	TnyCamelQueueable parent;
 	TnyCamelAccount *self;
 } ReconInfo;
 
@@ -123,11 +126,23 @@ reconnect_thread (gpointer user_data)
 	}
 	apriv->service->reconnecting = FALSE;
 
-	g_object_unref (info->self);
-	g_slice_free (ReconInfo, info);
 	return NULL;
 }
 
+static gboolean 
+did_refresh (gpointer user_data)
+{
+	return FALSE;
+}
+
+static void 
+did_refresh_destroy (gpointer user_data)
+{
+	ReconInfo *info = (ReconInfo *) user_data;
+	g_object_unref (info->self);
+	return;
+}
+
 static gboolean 
 cancelled_refresh (gpointer user_data)
 {
@@ -139,7 +154,6 @@ cancelled_refresh_destroy (gpointer user_data)
 {
 	ReconInfo *info = (ReconInfo *) user_data;
 	g_object_unref (info->self);
-	g_slice_free (ReconInfo, info);
 	return;
 }
 
@@ -229,8 +243,12 @@ _tny_camel_account_refresh (TnyCamelAccount *self, gboolean recon_if)
 				TNY_CAMEL_QUEUE_RECONNECT_ITEM);
 
 			_tny_camel_queue_launch_wflags (aspriv->queue, 
-				reconnect_thread, cancelled_refresh, 
-				cancelled_refresh_destroy, NULL, info,
+				reconnect_thread, 
+				did_refresh, did_refresh_destroy,
+				cancelled_refresh, 
+				cancelled_refresh_destroy, 
+				NULL, 
+				info, sizeof (ReconInfo),
 				TNY_CAMEL_QUEUE_RECONNECT_ITEM,
 				__FUNCTION__);
 		}
@@ -245,7 +263,7 @@ fail:
 static gboolean 
 tny_camel_account_matches_url_string (TnyAccount *self, const gchar *url_string)
 {
-	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->matches_url_string_func (self, url_string);
+	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->matches_url_string(self, url_string);
 }
 
 static gboolean 
@@ -343,7 +361,7 @@ tny_camel_account_matches_url_string_default (TnyAccount *self, const gchar *url
 static TnyAccountType
 tny_camel_account_get_account_type (TnyAccount *self)
 {
-	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_account_type_func (self);
+	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_account_type(self);
 }
 
 static TnyAccountType
@@ -399,7 +417,7 @@ tny_camel_account_get_account_type_default (TnyAccount *self)
 void 
 tny_camel_account_add_option (TnyCamelAccount *self, const gchar *option)
 {
-	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->add_option_func (self, option);
+	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->add_option(self, option);
 }
 
 void 
@@ -427,63 +445,21 @@ tny_camel_account_add_option_default (TnyCamelAccount *self, const gchar *option
 	if (!found)
 	{
 		priv->options = g_list_prepend (priv->options, g_strdup (option));
-		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare_func (self, TRUE, FALSE);
+		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare(self, TRUE, FALSE);
 		_tny_camel_account_emit_changed (self);
 	}
 
 	return;
 }
 
-TnyError
-_tny_camel_account_get_tny_error_code_for_camel_exception_id (CamelException* ex)
-{
-	/* printf ("DEBUG: %s: ex->id=%d, ex->desc=%s\n", __FUNCTION__, ex->id, ex->desc); */
-
-	if (!ex) {
-		g_warning ("%s: The exception was NULL.\n", __FUNCTION__);
-		return TNY_ACCOUNT_ERROR_TRY_CONNECT;
-	}
-
-	/* Try to provide a precise error code, 
-	 * as much as possible: */
-	switch (ex->id) 
-	{
-		case CAMEL_EXCEPTION_SYSTEM_HOST_LOOKUP_FAILED:
-			return TNY_ACCOUNT_ERROR_TRY_CONNECT_HOST_LOOKUP_FAILED;
-		case CAMEL_EXCEPTION_SERVICE_UNAVAILABLE:
-			return TNY_ACCOUNT_ERROR_TRY_CONNECT_SERVICE_UNAVAILABLE;
-		case CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE:
-			return TNY_ACCOUNT_ERROR_TRY_CONNECT_AUTHENTICATION_NOT_SUPPORTED;
-		case CAMEL_EXCEPTION_SERVICE_CERTIFICATE:
-			return TNY_ACCOUNT_ERROR_TRY_CONNECT_CERTIFICATE;
-		case CAMEL_EXCEPTION_USER_CANCEL:
-			/* TODO: This really shouldn't be shown to the user: */
-			return TNY_ACCOUNT_ERROR_TRY_CONNECT_USER_CANCEL;
-		case CAMEL_EXCEPTION_SYSTEM:
-		default:
-			/* A generic exception. 
-			 * We should always try to provide a precise error code rather than this,
-			 * so we can show a more helpful (translated) error message to the user.
-			 */
-			return TNY_ACCOUNT_ERROR_TRY_CONNECT;
-	}
-
-	return TNY_ACCOUNT_ERROR_TRY_CONNECT;
-}
-
 void 
 _tny_camel_account_try_connect (TnyCamelAccount *self, gboolean for_online, GError **err)
 {
 	TnyCamelAccountPriv *priv = TNY_CAMEL_ACCOUNT_GET_PRIVATE (self);
 
-	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare_func (TNY_CAMEL_ACCOUNT (self), for_online, TRUE);
+	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare(TNY_CAMEL_ACCOUNT (self), for_online, TRUE);
 	if (camel_exception_is_set (priv->ex))
-	{
-		g_set_error (err, TNY_ACCOUNT_ERROR, 
-			_tny_camel_account_get_tny_error_code_for_camel_exception_id (priv->ex),
-			camel_exception_get_description (priv->ex));
-	}
-
+		_tny_camel_exception_to_tny_error (priv->ex, err);
 	return;
 }
 
@@ -493,7 +469,7 @@ _tny_camel_account_try_connect (TnyCamelAccount *self, gboolean for_online, GErr
 static void
 tny_camel_account_set_url_string (TnyAccount *self, const gchar *url_string)
 {
-	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_url_string_func (self, url_string);
+	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_url_string(self, url_string);
 }
 
 /* Simplify checks for NULLs: */
@@ -540,7 +516,7 @@ tny_camel_account_set_url_string_default (TnyAccount *self, const gchar *url_str
 	}
 
 	if (changed)
-		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare_func (TNY_CAMEL_ACCOUNT (self), TRUE, TRUE);
+		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare(TNY_CAMEL_ACCOUNT (self), TRUE, TRUE);
 
 	g_static_rec_mutex_unlock (priv->service_lock);
 
@@ -552,7 +528,7 @@ tny_camel_account_set_url_string_default (TnyAccount *self, const gchar *url_str
 static gchar*
 tny_camel_account_get_url_string (TnyAccount *self)
 {
-	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_url_string_func (self);
+	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_url_string(self);
 }
 
 static gchar*
@@ -567,7 +543,7 @@ tny_camel_account_get_url_string_default (TnyAccount *self)
 static void
 tny_camel_account_set_name (TnyAccount *self, const gchar *name)
 {
-	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_name_func (self, name);
+	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_name(self, name);
 }
 
 static void
@@ -591,7 +567,7 @@ tny_camel_account_set_name_default (TnyAccount *self, const gchar *name)
 static const gchar*
 tny_camel_account_get_name (TnyAccount *self)
 {
-	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_name_func (self);
+	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_name(self);
 }
 
 static const gchar*
@@ -674,13 +650,13 @@ _tny_camel_account_stop_camel_operation (TnyCamelAccount *self)
 static void 
 tny_camel_account_start_operation (TnyAccount *self, TnyStatusDomain domain, TnyStatusCode code, TnyStatusCallback status_callback, gpointer status_user_data)
 {
-	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->start_operation_func (self, domain, code, status_callback, status_user_data);
+	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->start_operation(self, domain, code, status_callback, status_user_data);
 }
 
 static void
 tny_camel_account_stop_operation (TnyAccount *self, gboolean *canceled)
 {
-	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->stop_operation_func (self, canceled);
+	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->stop_operation(self, canceled);
 }
 
 
@@ -771,7 +747,7 @@ tny_camel_account_stop_operation_default (TnyAccount *self, gboolean *canceled)
 static TnyConnectionStatus 
 tny_camel_account_get_connection_status (TnyAccount *self)
 {
-	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_connection_status_func (self);
+	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_connection_status(self);
 }
 
 static TnyConnectionStatus 
@@ -802,7 +778,7 @@ tny_camel_account_set_session (TnyCamelAccount *self, TnySessionCamel *session)
 		priv->session = session;
 		_tny_session_camel_register_account (session, self);
 
-		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare_func (self, FALSE, FALSE);
+		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare(self, FALSE, FALSE);
 	}
 
 	g_static_rec_mutex_unlock (priv->service_lock);
@@ -813,7 +789,7 @@ tny_camel_account_set_session (TnyCamelAccount *self, TnySessionCamel *session)
 static void
 tny_camel_account_set_id (TnyAccount *self, const gchar *id)
 {
-	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_id_func (self, id);
+	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_id(self, id);
 }
 
 static void
@@ -837,7 +813,7 @@ tny_camel_account_set_id_default (TnyAccount *self, const gchar *id)
 static void
 tny_camel_account_set_secure_auth_mech (TnyAccount *self, const gchar *mech)
 {
-	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_secure_auth_mech_func (self, mech);
+	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_secure_auth_mech(self, mech);
 }
 
 static void
@@ -870,7 +846,7 @@ tny_camel_account_set_secure_auth_mech_default (TnyAccount *self, const gchar *m
 	}
 
 	if (changed)
-		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare_func (TNY_CAMEL_ACCOUNT (self), TRUE, FALSE);
+		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare(TNY_CAMEL_ACCOUNT (self), TRUE, FALSE);
 
 	g_static_rec_mutex_unlock (priv->service_lock);
 
@@ -882,7 +858,7 @@ tny_camel_account_set_secure_auth_mech_default (TnyAccount *self, const gchar *m
 static void
 tny_camel_account_set_proto (TnyAccount *self, const gchar *proto)
 {
-	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_proto_func (self, proto);
+	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_proto(self, proto);
 }
 
 static void
@@ -915,7 +891,7 @@ tny_camel_account_set_proto_default (TnyAccount *self, const gchar *proto)
 	}
 
 	if (changed)
-		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare_func (TNY_CAMEL_ACCOUNT (self), TRUE, FALSE);
+		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare(TNY_CAMEL_ACCOUNT (self), TRUE, FALSE);
 
 	g_static_rec_mutex_unlock (priv->service_lock);
 
@@ -927,7 +903,7 @@ tny_camel_account_set_proto_default (TnyAccount *self, const gchar *proto)
 static void
 tny_camel_account_set_user (TnyAccount *self, const gchar *user)
 {
-	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_user_func (self, user);
+	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_user(self, user);
 }
 
 static void
@@ -960,7 +936,7 @@ tny_camel_account_set_user_default (TnyAccount *self, const gchar *user)
 	}
 
 	if (changed)
-		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare_func (TNY_CAMEL_ACCOUNT (self), 
+		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare(TNY_CAMEL_ACCOUNT (self), 
 			!priv->in_auth, FALSE);
 
 	g_static_rec_mutex_unlock (priv->service_lock);
@@ -973,7 +949,7 @@ tny_camel_account_set_user_default (TnyAccount *self, const gchar *user)
 static void
 tny_camel_account_set_hostname (TnyAccount *self, const gchar *host)
 {
-	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_hostname_func (self, host);
+	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_hostname(self, host);
 }
 
 static void
@@ -1006,7 +982,7 @@ tny_camel_account_set_hostname_default (TnyAccount *self, const gchar *host)
 	}
 
 	if (changed)
-		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare_func (TNY_CAMEL_ACCOUNT (self), 
+		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare(TNY_CAMEL_ACCOUNT (self), 
 			TRUE, FALSE);
 
 	g_static_rec_mutex_unlock (priv->service_lock);
@@ -1021,7 +997,7 @@ tny_camel_account_set_hostname_default (TnyAccount *self, const gchar *host)
 static void
 tny_camel_account_set_port (TnyAccount *self, guint port)
 {
-	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_port_func (self, port);
+	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_port(self, port);
 }
 
 static void
@@ -1045,7 +1021,7 @@ tny_camel_account_set_port_default (TnyAccount *self, guint port)
 	}
 
 	if (changed)
-		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare_func (TNY_CAMEL_ACCOUNT (self), 
+		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare(TNY_CAMEL_ACCOUNT (self), 
 			TRUE, FALSE);
 
 	g_static_rec_mutex_unlock (priv->service_lock);
@@ -1059,7 +1035,7 @@ tny_camel_account_set_port_default (TnyAccount *self, guint port)
 static void
 tny_camel_account_set_pass_func (TnyAccount *self, TnyGetPassFunc get_pass_func)
 {
-	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_pass_func_func (self, get_pass_func);
+	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_pass_func (self, get_pass_func);
 }
 
 static void
@@ -1082,7 +1058,7 @@ tny_camel_account_set_pass_func_default (TnyAccount *self, TnyGetPassFunc get_pa
 	priv->pass_func_set = TRUE;
 
 	if (changed)
-		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare_func (TNY_CAMEL_ACCOUNT (self), 
+		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare(TNY_CAMEL_ACCOUNT (self), 
 			reconf_if, TRUE);
 
 	if (priv->session)
@@ -1094,9 +1070,9 @@ tny_camel_account_set_pass_func_default (TnyAccount *self, TnyGetPassFunc get_pa
 }
 
 static void
-tny_camel_account_set_forget_pass_func (TnyAccount *self, TnyForgetPassFunc get_forget_pass_func)
+tny_camel_account_set_forget_pass_func(TnyAccount *self, TnyForgetPassFunc get_forget_pass_func)
 {
-	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_forget_pass_func_func (self, get_forget_pass_func);
+	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_forget_pass_func (self, get_forget_pass_func);
 }
 
 static void
@@ -1119,7 +1095,7 @@ tny_camel_account_set_forget_pass_func_default (TnyAccount *self, TnyForgetPassF
 	priv->forget_pass_func_set = TRUE;
 
 	if (changed)
-		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare_func (TNY_CAMEL_ACCOUNT (self), 
+		TNY_CAMEL_ACCOUNT_GET_CLASS (self)->prepare(TNY_CAMEL_ACCOUNT (self), 
 			reconf_if, FALSE);
 
 	g_static_rec_mutex_unlock (priv->service_lock);
@@ -1130,7 +1106,7 @@ tny_camel_account_set_forget_pass_func_default (TnyAccount *self, TnyForgetPassF
 static const gchar*
 tny_camel_account_get_id (TnyAccount *self)
 {
-	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_id_func (self);
+	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_id(self);
 }
 
 static const gchar*
@@ -1147,7 +1123,7 @@ tny_camel_account_get_id_default (TnyAccount *self)
 static const gchar*
 tny_camel_account_get_secure_auth_mech (TnyAccount *self)
 {
-	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_secure_auth_mech_func (self);
+	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_secure_auth_mech(self);
 }
 
 static const gchar*
@@ -1164,7 +1140,7 @@ tny_camel_account_get_secure_auth_mech_default (TnyAccount *self)
 static const gchar*
 tny_camel_account_get_proto (TnyAccount *self)
 {
-	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_proto_func (self);
+	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_proto(self);
 }
 
 static const gchar*
@@ -1181,7 +1157,7 @@ tny_camel_account_get_proto_default (TnyAccount *self)
 static const gchar*
 tny_camel_account_get_user (TnyAccount *self)
 {
-	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_user_func (self);
+	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_user(self);
 }
 
 static const gchar*
@@ -1198,7 +1174,7 @@ tny_camel_account_get_user_default (TnyAccount *self)
 static const gchar*
 tny_camel_account_get_hostname (TnyAccount *self)
 {
-	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_hostname_func (self);
+	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_hostname(self);
 }
 
 static const gchar*
@@ -1216,7 +1192,7 @@ tny_camel_account_get_hostname_default (TnyAccount *self)
 static void 
 tny_camel_account_cancel (TnyAccount *self)
 {
-	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->cancel_func (self);
+	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->cancel(self);
 }
 
 #if 0 /* Not used. */
@@ -1238,6 +1214,8 @@ _tny_camel_account_actual_cancel (TnyCamelAccount *self)
 	g_static_rec_mutex_lock (priv->cancel_lock);
 	if (priv->cancel)
 		camel_operation_cancel (priv->cancel);
+	if (priv->getmsg_cancel)
+		camel_operation_cancel (priv->getmsg_cancel);
 	g_static_rec_mutex_unlock (priv->cancel_lock);
 
 	return;
@@ -1263,9 +1241,16 @@ tny_camel_account_cancel_default (TnyAccount *self)
 	if (TNY_IS_CAMEL_STORE_ACCOUNT (self))
 	{
 		TnyCamelStoreAccountPriv *priv = TNY_CAMEL_STORE_ACCOUNT_GET_PRIVATE (self);
+		TnyCamelAccountPriv *apriv = TNY_CAMEL_ACCOUNT_GET_PRIVATE (self);
 
 		_tny_camel_queue_cancel_remove_items (priv->queue, 
-			TNY_CAMEL_QUEUE_CANCELLABLE_ITEM);
+			TNY_CAMEL_QUEUE_GET_HEADERS_ITEM|TNY_CAMEL_QUEUE_SYNC_ITEM|
+			TNY_CAMEL_QUEUE_REFRESH_ITEM|TNY_CAMEL_QUEUE_CANCELLABLE_ITEM);
+
+		g_static_rec_mutex_lock (apriv->cancel_lock);
+		if (apriv->getmsg_cancel)
+			camel_operation_cancel (apriv->getmsg_cancel);
+		g_static_rec_mutex_unlock (apriv->cancel_lock);
 
 	} else {
 		_tny_camel_account_actual_cancel (TNY_CAMEL_ACCOUNT (self));
@@ -1278,7 +1263,7 @@ tny_camel_account_cancel_default (TnyAccount *self)
 static guint 
 tny_camel_account_get_port (TnyAccount *self)
 {
-	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_port_func (self);
+	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_port(self);
 }
 
 static guint 
@@ -1291,7 +1276,7 @@ tny_camel_account_get_port_default (TnyAccount *self)
 static TnyGetPassFunc
 tny_camel_account_get_pass_func (TnyAccount *self)
 {
-	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_pass_func_func (self);
+	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_pass_func (self);
 }
 
 static TnyGetPassFunc
@@ -1308,7 +1293,7 @@ tny_camel_account_get_pass_func_default (TnyAccount *self)
 static TnyForgetPassFunc
 tny_camel_account_get_forget_pass_func (TnyAccount *self)
 {
-	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_forget_pass_func_func (self);
+	return TNY_CAMEL_ACCOUNT_GET_CLASS (self)->get_forget_pass_func (self);
 }
 
 static TnyForgetPassFunc
@@ -1351,6 +1336,10 @@ tny_camel_account_instance_init (GTypeInstance *instance, gpointer g_class)
 	TnyCamelAccount *self = (TnyCamelAccount *)instance;
 	TnyCamelAccountPriv *priv = TNY_CAMEL_ACCOUNT_GET_PRIVATE (self);
 
+	priv->retry_connect = FALSE;
+	priv->con_strat = tny_camel_default_connection_policy_new ();
+	priv->queue = _tny_camel_queue_new (self);
+	priv->delete_this = NULL;
 	priv->is_ready = FALSE;
 	priv->is_connecting = FALSE;
 	priv->in_auth = FALSE;
@@ -1456,19 +1445,21 @@ _tny_camel_account_set_online (TnyCamelAccount *self, gboolean online, GError **
 	{
 		if (camel_exception_is_set (priv->ex))
 		{
-			g_set_error (err, TNY_ACCOUNT_ERROR, 
-				_tny_camel_account_get_tny_error_code_for_camel_exception_id (priv->ex),
-				camel_exception_get_description (priv->ex));
+			_tny_camel_exception_to_tny_error (priv->ex, err);
+			camel_exception_clear (priv->ex);
 		} else {
-			g_set_error (err, TNY_ACCOUNT_ERROR, 
-				TNY_ACCOUNT_ERROR_TRY_CONNECT,
+			g_set_error (err, TNY_SERVICE_ERROR, 
+				TNY_SERVICE_ERROR_CONNECT,
 				"Account not yet fully configured. "
 				"This problem indicates a bug in the software.");
 		}
 
+
 		return;
 	}
 
+	camel_object_ref (priv->service);
+
 	if (CAMEL_IS_DISCO_STORE (priv->service)) {
 		if (online) {
 			camel_disco_store_set_status (CAMEL_DISCO_STORE (priv->service),
@@ -1487,8 +1478,11 @@ _tny_camel_account_set_online (TnyCamelAccount *self, gboolean online, GError **
 				_tny_camel_store_account_emit_conchg_signal (TNY_CAMEL_STORE_ACCOUNT (self));
 			}
 
-			if (!camel_exception_is_set (&ex))
+			if (!camel_exception_is_set (&ex)) {
+				if (TNY_IS_CAMEL_STORE_ACCOUNT (self))
+					camel_store_restore (CAMEL_STORE (priv->service)); 
 				set_online_happened (priv->session, self, TRUE);
+			} 
 
 			goto done;
 
@@ -1564,18 +1558,18 @@ _tny_camel_account_set_online (TnyCamelAccount *self, gboolean online, GError **
 
 		if (!camel_exception_is_set (&ex))
 			set_online_happened (priv->session, self, FALSE);
-
 	}
 
 done:
 
 	if (camel_exception_is_set (&ex))
 	{
-		g_set_error (err, TNY_ACCOUNT_ERROR, 
-			_tny_camel_account_get_tny_error_code_for_camel_exception_id (&ex),
-			camel_exception_get_description (&ex));
+		_tny_camel_exception_to_tny_error (&ex, err);
+		camel_exception_clear (&ex);
 	}
 
+	camel_object_unref (priv->service);
+
 	return;
 }
 
@@ -1602,7 +1596,7 @@ done:
 void 
 tny_camel_account_set_online (TnyCamelAccount *self, gboolean online, TnyCamelSetOnlineCallback callback, gpointer user_data)
 {
-	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_online_func (self, online, callback, user_data);
+	TNY_CAMEL_ACCOUNT_GET_CLASS (self)->set_online(self, online, callback, user_data);
 }
 
 typedef struct
@@ -1619,7 +1613,7 @@ typedef struct
 
 } OnSetOnlineInfo;
 
-gboolean 
+static gboolean 
 on_set_online_done_idle_func (gpointer data)
 {
 	OnSetOnlineInfo *info = (OnSetOnlineInfo *) data;
@@ -1635,7 +1629,7 @@ on_set_online_done_idle_func (gpointer data)
 }
 
 static void 
-on_set_online_done_destroy_func (gpointer data)
+on_set_online_done_destroy(gpointer data)
 {
 	OnSetOnlineInfo *info = (OnSetOnlineInfo *) data;
 
@@ -1651,83 +1645,12 @@ on_set_online_done_destroy_func (gpointer data)
 		g_cond_broadcast (info->condition);
 		info->had_callback = TRUE;
 		g_mutex_unlock (info->mutex);
-	}
+	} else /* it's a transport account */
+		g_slice_free (OnSetOnlineInfo, data);
 
 	return;
 }
 
-/**
- * When using a #GMainLoop this method will execute a callback using
- * g_idle_add_full.  Note that without a #GMainLoop, the callbacks
- * could happen in a worker thread (depends on who call it) at an
- * unknown moment in time (check your locking in this case).
- */
-static void
-execute_callback (gint depth, 
-		  gint priority,
-		  GSourceFunc idle_func,
-		  gpointer data, 
-		  GDestroyNotify destroy_func)
-{
-	if (depth > 0){
-		g_idle_add_full (priority, idle_func, data, destroy_func);
-	} else {
-		idle_func (data);
-		destroy_func (data);
-	}
-}
-
-typedef struct {
-	TnyCamelSetOnlineCallback callback;
-	gpointer user_data;
-} OnSetOnlineDoneInfo;
-
-static void 
-on_set_online_done (TnySessionCamel *self, TnyCamelAccount *account, GError *err, gpointer user_data)
-{
-	OnSetOnlineInfo *info = g_slice_new (OnSetOnlineInfo);
-	OnSetOnlineDoneInfo *i = (OnSetOnlineDoneInfo *) user_data;
-
-	/* Thread reference */
-	info->account = TNY_CAMEL_ACCOUNT (g_object_ref (account));
-
-	/* We must copy the err because this context will destroy it! */
-
-	info->cancel = FALSE;
-	if (err) {
-		if (!strcmp (err->message, "cancel"))
-			info->cancel = TRUE;
-		else
-			info->err = g_error_copy (err); 
-	} else
-		info->err = NULL;
-
-	info->callback = i->callback;
-	info->user_data = i->user_data;
-
-	g_slice_free (OnSetOnlineDoneInfo, i);
-
-	info->mutex = g_mutex_new ();
-	info->condition = g_cond_new ();
-	info->had_callback = FALSE;
-
-	execute_callback (/* info->depth */ 10, G_PRIORITY_HIGH, 
-		on_set_online_done_idle_func, 
-		info, on_set_online_done_destroy_func);
-
-	/* Wait on the queue for the mainloop callback to be finished */
-	g_mutex_lock (info->mutex);
-	if (!info->had_callback)
-		g_cond_wait (info->condition, info->mutex);
-	g_mutex_unlock (info->mutex);
-
-	g_mutex_free (info->mutex);
-	g_cond_free (info->condition);
-
-	g_slice_free (OnSetOnlineInfo, info);
-
-	return;
-}
 
 void 
 tny_camel_account_set_online_default (TnyCamelAccount *self, gboolean online, TnyCamelSetOnlineCallback callback, gpointer user_data)
@@ -1738,13 +1661,16 @@ tny_camel_account_set_online_default (TnyCamelAccount *self, gboolean online, Tn
 	/* In case we  are a store account, this means that we need to throw the 
 	 * request to go online to the account's queue. */
 
+	if (session == NULL)
+		return;
+
 	if (TNY_IS_CAMEL_STORE_ACCOUNT (self)) {
-		OnSetOnlineDoneInfo *i = g_slice_new0 (OnSetOnlineDoneInfo);
-		i->callback = callback;
-		i->user_data = user_data;
+		priv->retry_connect = TRUE;
+		if (online)
+			camel_session_set_online ((CamelSession *) session, TRUE); 
 		_tny_camel_store_account_queue_going_online (
 			TNY_CAMEL_STORE_ACCOUNT (self), session, online, 
-			on_set_online_done, i);
+			callback, user_data);
 	}
 
 
@@ -1753,8 +1679,25 @@ tny_camel_account_set_online_default (TnyCamelAccount *self, gboolean online, Tn
 	 * moment yet. At the moment of transferring the first message, the
 	 * current implementations will automatically connect themselves. */
 
-	if (TNY_IS_CAMEL_TRANSPORT_ACCOUNT (self))
+	if (TNY_IS_CAMEL_TRANSPORT_ACCOUNT (self)) 
+	{
+		OnSetOnlineInfo *info = g_slice_new0 (OnSetOnlineInfo);
+
 		set_online_happened (session, self, online);
+
+		info->err = NULL;
+		info->callback = callback;
+		info->user_data = user_data;
+		info->mutex = NULL;
+		info->condition = NULL;
+		info->had_callback = FALSE;
+		info->account = TNY_CAMEL_ACCOUNT (g_object_ref (self));
+
+		g_idle_add_full (G_PRIORITY_HIGH, 
+			on_set_online_done_idle_func, 
+			info, on_set_online_done_destroy);
+
+	}
 }
 
 static gboolean
@@ -1765,10 +1708,30 @@ tny_camel_account_is_ready (TnyAccount *self)
 }
 
 
+static TnyConnectionPolicy* 
+tny_camel_account_get_connection_policy (TnyAccount *self)
+{
+	TnyCamelAccountPriv *priv = TNY_CAMEL_ACCOUNT_GET_PRIVATE (self);
+	return TNY_CONNECTION_POLICY (g_object_ref (priv->con_strat));
+}
+
+static void 
+tny_camel_account_set_connection_policy (TnyAccount *self, TnyConnectionPolicy *policy)
+{
+	TnyCamelAccountPriv *priv = TNY_CAMEL_ACCOUNT_GET_PRIVATE (self);
+	g_object_unref (priv->con_strat);
+	priv->con_strat = TNY_CONNECTION_POLICY (g_object_ref (policy));
+	return;
+}
+
+
+
 
 
 typedef struct 
 {
+	TnyCamelQueueable parent;
+
 	TnyCamelAccount *self;
 	TnyCamelGetSupportedSecureAuthCallback callback;
 	TnyStatusCallback status_callback;
@@ -1779,10 +1742,6 @@ typedef struct
 	GError *err;
 	TnySessionCamel *session;
 
-	GCond* condition;
-	gboolean had_callback;
-	GMutex *mutex;
-
 } GetSupportedAuthInfo;
 
 
@@ -1803,6 +1762,22 @@ on_supauth_idle_func (gpointer user_data)
 }
 
 
+static gboolean 
+on_supauth_cancelled_idle_func (gpointer user_data)
+{
+	GetSupportedAuthInfo *info = (GetSupportedAuthInfo *) user_data;
+
+	if (info->callback) {
+		tny_lockable_lock (info->session->priv->ui_lock);
+		info->callback (info->self, TRUE, info->result, info->err, info->user_data);
+		tny_lockable_unlock (info->session->priv->ui_lock);
+	}
+
+	tny_idle_stopper_stop (info->stopper);
+
+	return FALSE; /* Don't call this again. */
+}
+
 static void 
 on_supauth_destroy_func (gpointer user_data)
 {
@@ -1819,19 +1794,18 @@ on_supauth_destroy_func (gpointer user_data)
 	tny_idle_stopper_destroy (info->stopper);
 	info->stopper = NULL;
 
-	if (info->condition) {
-		g_mutex_lock (info->mutex);
-		g_cond_broadcast (info->condition);
-		info->had_callback = TRUE;
-		g_mutex_unlock (info->mutex);
+	if (info->err) {
+		g_error_free (info->err);
+		info->err = NULL;
 	}
+
+	return;
 }
 
 static void 
 on_supauth_destroy_and_kill_func (gpointer user_data)
 {
 	on_supauth_destroy_func (user_data);
-	g_slice_free (GetSupportedAuthInfo, user_data);
 }
 
 /* Starts the operation in the thread: */
@@ -1855,7 +1829,7 @@ tny_camel_account_get_supported_secure_authentication_async_thread (
 		"Get secure authentication methods");
 
 	/* Prepare, so service is set for query_auth_types */
-	TNY_CAMEL_ACCOUNT_GET_CLASS(self)->prepare_func(self, FALSE, TRUE);
+	TNY_CAMEL_ACCOUNT_GET_CLASS(self)->prepare (self, FALSE, TRUE);
 
 	authtypes = camel_service_query_auth_types (priv->service, &ex);
 	
@@ -1880,38 +1854,10 @@ tny_camel_account_get_supported_secure_authentication_async_thread (
 
 	info->err = NULL;
 	if (camel_exception_is_set (&ex))
-	{
-		g_set_error (&info->err, TNY_FOLDER_ERROR, 
-			TNY_FOLDER_ERROR_REFRESH,
-			camel_exception_get_description (&ex));
-	}
+		_tny_camel_exception_to_tny_error (&ex, &info->err);
 
 	g_static_rec_mutex_unlock (priv->service_lock);
 
-	info->mutex = g_mutex_new ();
-	info->condition = g_cond_new ();
-	info->had_callback = FALSE;
-
-	execute_callback (10, G_PRIORITY_HIGH, 
-		on_supauth_idle_func, 
-		info, on_supauth_destroy_func);
-
-	/* Wait on the queue for the mainloop callback to be finished */
-	g_mutex_lock (info->mutex);
-	if (!info->had_callback)
-		g_cond_wait (info->condition, info->mutex);
-	g_mutex_unlock (info->mutex);
-
-	g_mutex_free (info->mutex);
-	g_cond_free (info->condition);
-
-	if (info->err) {
-		g_error_free (info->err);
-		info->err = NULL;
-	}
-
-	g_slice_free (GetSupportedAuthInfo, info);
-
 	tny_status_free(status);
 
 	return NULL;
@@ -1950,7 +1896,7 @@ tny_camel_account_get_supported_secure_authentication (TnyCamelAccount *self, Tn
 	GError *err = NULL;
 
 	if (!_tny_session_check_operation (priv->session, TNY_ACCOUNT (self), &err, 
-			TNY_ACCOUNT_ERROR, TNY_ACCOUNT_ERROR_GET_SUPPORTED_AUTH))
+			TNY_SERVICE_ERROR, TNY_SERVICE_ERROR_UNKNOWN))
 	{
 		if (callback) {
 			callback (self, TRUE /* cancelled */, NULL, err, user_data);
@@ -1988,9 +1934,12 @@ tny_camel_account_get_supported_secure_authentication (TnyCamelAccount *self, Tn
 		TnyCamelStoreAccountPriv *aspriv = TNY_CAMEL_STORE_ACCOUNT_GET_PRIVATE (self);
 		_tny_camel_queue_launch (aspriv->queue, 
 			tny_camel_account_get_supported_secure_authentication_async_thread, 
-			on_supauth_idle_func, on_supauth_destroy_and_kill_func, 
-			&info->cancelled, info, __FUNCTION__);
+			on_supauth_idle_func, on_supauth_destroy_func, 
+			on_supauth_cancelled_idle_func, on_supauth_destroy_and_kill_func, 
+			&info->cancelled, info, sizeof (GetSupportedAuthInfo),
+			__FUNCTION__);
 	} else {
+		/* TnyCamelQueue *temp_queue = _tny_camel_queue_new (self); */
 		g_thread_create (tny_camel_account_get_supported_secure_authentication_async_thread,
 			info, FALSE, NULL);
 	}
@@ -2004,7 +1953,6 @@ tny_camel_account_finalize (GObject *object)
 	TnyCamelAccountPriv *priv = TNY_CAMEL_ACCOUNT_GET_PRIVATE (self);
 	CamelException ex = CAMEL_EXCEPTION_INITIALISER;
 
-
 	if (priv->service && CAMEL_IS_SERVICE (priv->service))
 	{
 		priv->service->connecting = NULL;
@@ -2026,8 +1974,9 @@ tny_camel_account_finalize (GObject *object)
 		camel_operation_unref (priv->cancel);
 	g_static_rec_mutex_unlock (priv->cancel_lock);
 
-	g_static_rec_mutex_free (priv->cancel_lock);
-
+	/* g_static_rec_mutex_free (priv->cancel_lock); */
+	g_free (priv->cancel_lock);
+	priv->cancel_lock = NULL;
 	priv->inuse_spin = FALSE;
 
 	if (priv->options)
@@ -2065,12 +2014,26 @@ tny_camel_account_finalize (GObject *object)
 	if (G_LIKELY (priv->url_string))
 		g_free (priv->url_string);
 
+	if (priv->delete_this && strlen (priv->delete_this) > 0)
+		camel_rm (priv->delete_this);
+	if (priv->delete_this)
+		g_free (priv->delete_this);
+	priv->delete_this = NULL;
+
 	g_static_rec_mutex_unlock (priv->service_lock);
 
+	g_object_unref (priv->queue);
+	g_object_unref (priv->con_strat);
+
 	camel_exception_free (priv->ex);
 
-	g_static_rec_mutex_free (priv->service_lock);
-	g_static_rec_mutex_free (priv->account_lock);
+	/* g_static_rec_mutex_free (priv->service_lock); */
+	g_free (priv->service_lock);
+	priv->service_lock = NULL;
+
+	/* g_static_rec_mutex_free (priv->account_lock); */
+	g_free (priv->account_lock);
+	priv->account_lock = NULL;
 
 	(*parent_class->finalize) (object);
 
@@ -2083,33 +2046,35 @@ tny_account_init (gpointer g, gpointer iface_data)
 {
 	TnyAccountIface *klass = (TnyAccountIface *)g;
 
-	klass->get_port_func = tny_camel_account_get_port;
-	klass->set_port_func = tny_camel_account_set_port;
-	klass->get_hostname_func = tny_camel_account_get_hostname;
-	klass->set_hostname_func = tny_camel_account_set_hostname;
-	klass->get_proto_func = tny_camel_account_get_proto;
-	klass->set_proto_func = tny_camel_account_set_proto;
-	klass->get_secure_auth_mech_func = tny_camel_account_get_secure_auth_mech;
-	klass->set_secure_auth_mech_func = tny_camel_account_set_secure_auth_mech;
-	klass->get_user_func = tny_camel_account_get_user;
-	klass->set_user_func = tny_camel_account_set_user;
-	klass->get_pass_func_func = tny_camel_account_get_pass_func;
-	klass->set_pass_func_func = tny_camel_account_set_pass_func;
-	klass->get_forget_pass_func_func = tny_camel_account_get_forget_pass_func;
-	klass->set_forget_pass_func_func = tny_camel_account_set_forget_pass_func;
-	klass->set_id_func = tny_camel_account_set_id;
-	klass->get_id_func = tny_camel_account_get_id;
-	klass->get_connection_status_func = tny_camel_account_get_connection_status;
-	klass->set_url_string_func = tny_camel_account_set_url_string;
-	klass->get_url_string_func = tny_camel_account_get_url_string;
-	klass->get_name_func = tny_camel_account_get_name;
-	klass->set_name_func = tny_camel_account_set_name;
-	klass->get_account_type_func = tny_camel_account_get_account_type;
-	klass->cancel_func = tny_camel_account_cancel;
-	klass->matches_url_string_func = tny_camel_account_matches_url_string;
-	klass->start_operation_func = tny_camel_account_start_operation;
-	klass->stop_operation_func =  tny_camel_account_stop_operation;
-	klass->is_ready_func = tny_camel_account_is_ready;
+	klass->get_port= tny_camel_account_get_port;
+	klass->set_port= tny_camel_account_set_port;
+	klass->get_hostname= tny_camel_account_get_hostname;
+	klass->set_hostname= tny_camel_account_set_hostname;
+	klass->get_proto= tny_camel_account_get_proto;
+	klass->set_proto= tny_camel_account_set_proto;
+	klass->get_secure_auth_mech= tny_camel_account_get_secure_auth_mech;
+	klass->set_secure_auth_mech= tny_camel_account_set_secure_auth_mech;
+	klass->get_user= tny_camel_account_get_user;
+	klass->set_user= tny_camel_account_set_user;
+	klass->get_pass_func = tny_camel_account_get_pass_func;
+	klass->set_pass_func = tny_camel_account_set_pass_func;
+	klass->get_forget_pass_func = tny_camel_account_get_forget_pass_func;
+	klass->set_forget_pass_func = tny_camel_account_set_forget_pass_func;
+	klass->set_id= tny_camel_account_set_id;
+	klass->get_id= tny_camel_account_get_id;
+	klass->get_connection_status= tny_camel_account_get_connection_status;
+	klass->set_url_string= tny_camel_account_set_url_string;
+	klass->get_url_string= tny_camel_account_get_url_string;
+	klass->get_name= tny_camel_account_get_name;
+	klass->set_name= tny_camel_account_set_name;
+	klass->get_account_type= tny_camel_account_get_account_type;
+	klass->cancel= tny_camel_account_cancel;
+	klass->matches_url_string= tny_camel_account_matches_url_string;
+	klass->start_operation= tny_camel_account_start_operation;
+	klass->stop_operation=  tny_camel_account_stop_operation;
+	klass->is_ready= tny_camel_account_is_ready;
+	klass->set_connection_policy= tny_camel_account_set_connection_policy;
+	klass->get_connection_policy= tny_camel_account_get_connection_policy;
 
 	return;
 }
@@ -2139,41 +2104,41 @@ tny_camel_account_class_init (TnyCamelAccountClass *class)
 		   g_signal_new ("set_online_happened",
 			TNY_TYPE_CAMEL_ACCOUNT,
 			G_SIGNAL_RUN_FIRST,
-			G_STRUCT_OFFSET (TnyCamelAccountClass, set_online_happened_func),
+			G_STRUCT_OFFSET (TnyCamelAccountClass, set_online_happened),
 			NULL, NULL,
 			g_cclosure_marshal_VOID__BOOLEAN, 
 			G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
 	}
 
-	class->get_port_func = tny_camel_account_get_port_default;
-	class->set_port_func = tny_camel_account_set_port_default;
-	class->get_hostname_func = tny_camel_account_get_hostname_default;
-	class->set_hostname_func = tny_camel_account_set_hostname_default;
-	class->get_proto_func = tny_camel_account_get_proto_default;
-	class->set_proto_func = tny_camel_account_set_proto_default;
-	class->get_secure_auth_mech_func = tny_camel_account_get_secure_auth_mech_default;
-	class->set_secure_auth_mech_func = tny_camel_account_set_secure_auth_mech_default;
-	class->get_user_func = tny_camel_account_get_user_default;
-	class->set_user_func = tny_camel_account_set_user_default;
-	class->get_pass_func_func = tny_camel_account_get_pass_func_default;
-	class->set_pass_func_func = tny_camel_account_set_pass_func_default;
-	class->get_forget_pass_func_func = tny_camel_account_get_forget_pass_func_default;
-	class->set_forget_pass_func_func = tny_camel_account_set_forget_pass_func_default;
-	class->set_id_func = tny_camel_account_set_id_default;
-	class->get_id_func = tny_camel_account_get_id_default;
-	class->get_connection_status_func = tny_camel_account_get_connection_status_default;
-	class->set_url_string_func = tny_camel_account_set_url_string_default;
-	class->get_url_string_func = tny_camel_account_get_url_string_default;
-	class->get_name_func = tny_camel_account_get_name_default;
-	class->set_name_func = tny_camel_account_set_name_default;
-	class->get_account_type_func = tny_camel_account_get_account_type_default;
-	class->cancel_func = tny_camel_account_cancel_default;
-	class->matches_url_string_func = tny_camel_account_matches_url_string_default;
-	class->start_operation_func = tny_camel_account_start_operation_default;
-	class->stop_operation_func =  tny_camel_account_stop_operation_default;
+	class->get_port= tny_camel_account_get_port_default;
+	class->set_port= tny_camel_account_set_port_default;
+	class->get_hostname= tny_camel_account_get_hostname_default;
+	class->set_hostname= tny_camel_account_set_hostname_default;
+	class->get_proto= tny_camel_account_get_proto_default;
+	class->set_proto= tny_camel_account_set_proto_default;
+	class->get_secure_auth_mech= tny_camel_account_get_secure_auth_mech_default;
+	class->set_secure_auth_mech= tny_camel_account_set_secure_auth_mech_default;
+	class->get_user= tny_camel_account_get_user_default;
+	class->set_user= tny_camel_account_set_user_default;
+	class->get_pass_func = tny_camel_account_get_pass_func_default;
+	class->set_pass_func = tny_camel_account_set_pass_func_default;
+	class->get_forget_pass_func = tny_camel_account_get_forget_pass_func_default;
+	class->set_forget_pass_func = tny_camel_account_set_forget_pass_func_default;
+	class->set_id= tny_camel_account_set_id_default;
+	class->get_id= tny_camel_account_get_id_default;
+	class->get_connection_status= tny_camel_account_get_connection_status_default;
+	class->set_url_string= tny_camel_account_set_url_string_default;
+	class->get_url_string= tny_camel_account_get_url_string_default;
+	class->get_name= tny_camel_account_get_name_default;
+	class->set_name= tny_camel_account_set_name_default;
+	class->get_account_type= tny_camel_account_get_account_type_default;
+	class->cancel= tny_camel_account_cancel_default;
+	class->matches_url_string= tny_camel_account_matches_url_string_default;
+	class->start_operation= tny_camel_account_start_operation_default;
+	class->stop_operation=  tny_camel_account_stop_operation_default;
 
-	class->add_option_func = tny_camel_account_add_option_default;
-	class->set_online_func = tny_camel_account_set_online_default;
+	class->add_option= tny_camel_account_add_option_default;
+	class->set_online= tny_camel_account_set_online_default;
 
 	object_class->finalize = tny_camel_account_finalize;
 
