@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with self library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <tny-camel-transport-account.h>
@@ -35,6 +35,7 @@ struct _TnyCamelSendQueuePriv
 	GMutex *todo_lock, *sending_lock; 
 	gboolean do_continue, is_running;
 	gboolean observer_attached;
+	gint pending_send_notifies;
 };
 
 #endif
