@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with self library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -24,6 +24,7 @@
 #include <tny-camel-send-queue.h>
 #include <tny-camel-shared.h>
 #include <tny-camel-msg.h>
+#include <tny-device.h>
 #include <tny-simple-list.h>
 #include <tny-folder.h>
 #include <tny-error.h>
@@ -56,6 +57,9 @@ typedef struct {
 	TnyHeader *header;
 	GError *error;
 	gint i, total;
+	GCond *condition;
+	GMutex *mutex;
+	gboolean had_callback;
 } ErrorInfo;
 
 typedef struct {
@@ -67,6 +71,8 @@ typedef struct {
 } ControlInfo;
 
 
+static void emit_queue_control_signals (TnySendQueue *self, guint signal_id);
+
 static TnyFolder*
 get_sentbox (TnySendQueue *self)
 {
@@ -113,7 +119,12 @@ destroy_error_info (gpointer data)
 		g_object_unref (info->self);
 	if (info->error)
 		g_error_free (info->error);
+
 	g_slice_free (ErrorInfo, info);
+/* 	g_mutex_lock (info->mutex); */
+/* 	g_cond_broadcast (info->condition); */
+/* 	info->had_callback = TRUE; */
+/* 	g_mutex_unlock (info->mutex); */
 
 	return;
 }
@@ -133,9 +144,23 @@ emit_error (TnySendQueue *self, TnyHeader *header, TnyMsg *msg, GError *error, i
 		info->header = TNY_HEADER (g_object_ref (header));
 	info->i = i;
 	info->total = total;
+/* 	info->had_callback = FALSE; */
+/* 	info->mutex = g_mutex_new (); */
+/* 	info->condition = g_cond_new (); */
+
 	g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
 		emit_error_on_mainloop, info, destroy_error_info);
 
+/* 	g_mutex_lock (info->mutex); */
+/* 	if (!info->had_callback) */
+/* 		g_cond_wait (info->condition, info->mutex); */
+/* 	g_mutex_unlock (info->mutex); */
+
+/* 	g_mutex_free (info->mutex); */
+/* 	g_cond_free (info->condition); */
+
+/* 	g_slice_free (ErrorInfo, info); */
+
 	return;
 }
 
@@ -151,7 +176,7 @@ emit_control_signals_on_mainloop (gpointer data)
 	if (apriv)
 		tny_lockable_lock (apriv->session->priv->ui_lock);
 	g_signal_emit (info->self, tny_send_queue_signals [info->signal_id], 
-		0, info->header, info->msg, info->i, info->total);
+		       0, info->header, info->msg, info->i, info->total);	
 	if (apriv)
 		tny_lockable_unlock (apriv->session->priv->ui_lock);
 
@@ -211,6 +236,11 @@ on_msg_sent_get_msg (TnyFolder *folder, gboolean cancelled, TnyMsg *msg, GError
 		g_object_unref (header);
 	}
 
+	priv->pending_send_notifies--;
+	if (priv->pending_send_notifies == 0 && priv->thread == NULL) {		
+		emit_queue_control_signals (self, TNY_SEND_QUEUE_STOP);
+	}
+
 	g_object_unref (self);
 }
 
@@ -240,7 +270,9 @@ tny_camel_send_queue_update (TnyFolderObserver *self, TnyFolderChange *change)
 			iter = tny_list_create_iterator (list);
 			while (!tny_iterator_is_done (iter))
 			{
+				TnyCamelSendQueuePriv *priv = TNY_CAMEL_SEND_QUEUE_GET_PRIVATE (self);
 				TnyHeader *cur = TNY_HEADER (tny_iterator_get_current (iter));
+				priv->pending_send_notifies++;
 				tny_folder_get_msg_async (sentbox, cur, 
 					on_msg_sent_get_msg, on_status, 
 					g_object_ref (self));
@@ -258,16 +290,59 @@ tny_camel_send_queue_update (TnyFolderObserver *self, TnyFolderChange *change)
 	return;
 }
 
+
+static gboolean
+emit_queue_control_signals_on_mainloop (gpointer data)
+{
+	ControlInfo *info = data;
+	TnyCamelSendQueuePriv *priv = TNY_CAMEL_SEND_QUEUE_GET_PRIVATE (info->self);
+	TnyCamelAccountPriv *apriv = NULL;
+
+	if ((priv->pending_send_notifies > 0) && (info->signal_id == TNY_SEND_QUEUE_STOP))
+		return FALSE;
+	if (priv && priv->trans_account)
+		apriv = TNY_CAMEL_ACCOUNT_GET_PRIVATE (priv->trans_account);
+	if (apriv)
+		tny_lockable_lock (apriv->session->priv->ui_lock);
+	g_signal_emit (info->self, tny_send_queue_signals [info->signal_id], 0);
+	if (apriv)
+		tny_lockable_unlock (apriv->session->priv->ui_lock);
+
+	return FALSE;
+}
+
+static void
+emit_queue_control_signals (TnySendQueue *self, guint signal_id)
+{
+	ControlInfo *info = g_slice_new0 (ControlInfo);
+
+	info->self = g_object_ref (self);
+	info->signal_id = signal_id;
+
+	g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+			 emit_queue_control_signals_on_mainloop, info, 
+			 destroy_control_info);
+	
+	return;
+}
+
+typedef struct {
+	TnySendQueue *self;
+	TnyDevice *device;
+} MainThreadInfo;
+
 static gpointer
 thread_main (gpointer data)
 {
-	TnySendQueue *self = (TnySendQueue *) data;
+	MainThreadInfo *info = (MainThreadInfo *) data;
+	TnySendQueue *self = info->self;
 	TnyCamelSendQueuePriv *priv = TNY_CAMEL_SEND_QUEUE_GET_PRIVATE (self);
 	TnyFolder *sentbox = NULL, *outbox = NULL;
 	guint i = 0, length = 0;
 	TnyList *list = NULL;
+	TnyDevice *device = info->device;
+	GHashTable *failed_headers = NULL;
 
-	g_object_ref (self); /* My own reference */
 	priv->is_running = TRUE;
 	list = tny_simple_list_new ();
 
@@ -277,6 +352,20 @@ thread_main (gpointer data)
 		sentbox = get_sentbox (self);
 		outbox = tny_send_queue_get_outbox (self);
 
+		if (!outbox || !sentbox) {
+			if (outbox)
+				g_object_unref (outbox);
+			if (sentbox)
+				g_object_unref (sentbox);
+			g_warning ("tny_send_queue_get_outbox and "
+				"tny_send_queue_get_sentbox are not allowed to "
+				"return NULL. This indicates a problem in the "
+				"software.");
+			priv->is_running = FALSE;
+			g_mutex_unlock (priv->todo_lock);
+			goto errorhandler;
+		}
+
 		tny_folder_get_headers (outbox, list, TRUE, &terror);
 
 		if (terror != NULL)
@@ -295,9 +384,9 @@ thread_main (gpointer data)
 
 	g_object_unref (list);
 
-	priv->do_continue = TRUE;
+	failed_headers = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
-	while (length > 0 && priv->do_continue)
+	while ((length - g_hash_table_size (failed_headers)) > 0 && tny_device_is_online (device))
 	{
 		TnyHeader *header = NULL;
 		TnyMsg *msg = NULL;
@@ -309,10 +398,17 @@ thread_main (gpointer data)
 			GError *ferror = NULL;
 			TnyIterator *hdriter = NULL;
 			TnyList *headers = tny_simple_list_new ();
-			GList *to_remove = NULL;
+			GList *to_remove = NULL, *copy;
 			TnyIterator *giter = NULL;
 
-			tny_folder_get_headers (outbox, headers, TRUE, &ferror);
+			if (tny_device_is_online (device))
+				tny_folder_get_headers (outbox, headers, TRUE, &ferror);
+			else {
+				priv->is_running = FALSE;
+				g_mutex_unlock (priv->todo_lock);
+				g_mutex_unlock (priv->sending_lock);
+				goto errorhandler;
+			}
 
 			if (ferror != NULL)
 			{
@@ -332,20 +428,26 @@ thread_main (gpointer data)
 				TnyHeader *curhdr = TNY_HEADER (tny_iterator_get_current (giter));
 				TnyHeaderFlags flags = tny_header_get_flags (curhdr);
 
-				flags &= TNY_HEADER_FLAG_PRIORITY;
-				if (flags & TNY_HEADER_FLAG_SUSPENDED_PRIORITY)
+				if ((flags & TNY_HEADER_FLAG_SUSPENDED) ||
+				    (g_hash_table_lookup_extended (failed_headers, 
+								   tny_header_get_uid (curhdr),
+								   NULL, NULL)))
 					to_remove = g_list_prepend (to_remove, curhdr);
+
 				g_object_unref (curhdr);
 				tny_iterator_next (giter);
 			}
 			g_object_unref (giter);
 
+			copy = to_remove;
+
 			while (to_remove) {
 				tny_list_remove (headers, G_OBJECT (to_remove->data));
 				to_remove = g_list_next (to_remove);
 			}
-			if (to_remove)
-				g_list_free (to_remove);
+
+			if (copy)
+				g_list_free (copy);
 
 			length = tny_list_get_length (headers);
 			priv->total = length;
@@ -367,7 +469,7 @@ thread_main (gpointer data)
 		}
 		g_mutex_unlock (priv->todo_lock);
 
-		if (header && TNY_IS_HEADER (header))
+		if (header && TNY_IS_HEADER (header) && tny_device_is_online (device))
 		{
 			TnyList *hassent = tny_simple_list_new ();
 			GError *err = NULL;
@@ -380,15 +482,22 @@ thread_main (gpointer data)
 
 			if (err == NULL) 
 			{
+				_tny_camel_account_start_camel_operation (TNY_CAMEL_ACCOUNT (priv->trans_account),
+									  NULL, NULL, "Sending message");
+
 				tny_transport_account_send (priv->trans_account, msg, &err);
 
+				_tny_camel_account_stop_camel_operation (TNY_CAMEL_ACCOUNT (priv->trans_account));
+
 				if (err != NULL) {
 					emit_error (self, header, msg, err, i, priv->total);
-					priv->do_continue = FALSE;
+					g_hash_table_insert (failed_headers, 
+							     g_strdup (tny_header_get_uid (header)), NULL);
 				}
-			} else  {
+			} else {
 				emit_error (self, header, msg, err, i, priv->total);
-				priv->do_continue = FALSE;
+				g_hash_table_insert (failed_headers, 
+						     g_strdup (tny_header_get_uid (header)), NULL);
 			}
 
 			g_mutex_lock (priv->todo_lock);
@@ -397,24 +506,19 @@ thread_main (gpointer data)
 				{
 					GError *newerr = NULL;
 					priv->cur_i = i;
+					tny_header_set_flag (header, TNY_HEADER_FLAG_SEEN);
 					tny_folder_transfer_msgs (outbox, hassent, sentbox, TRUE, &newerr);
-					if (newerr != NULL) 
-					{
+					if (newerr != NULL) {
 						emit_error (self, header, msg, newerr, i, priv->total);
-						priv->do_continue = FALSE;
 						g_error_free (newerr);
 					}
 					priv->total--;
 				}
 			}
+			g_mutex_unlock (priv->todo_lock);
 
 			/* Emits msg-sent signal to inform msg has been sent */
-			/* This now happens in on_msg_sent_get_msg! 
-
-			if (priv->do_continue)
-				emit_control (self, header, msg, TNY_SEND_QUEUE_MSG_SENT, i, priv->total); */
-
-			g_mutex_unlock (priv->todo_lock);
+			/* This now happens in on_msg_sent_get_msg! */
 
 			g_object_unref (hassent);
 
@@ -428,29 +532,37 @@ thread_main (gpointer data)
 		} else 
 		{
 			priv->is_running = FALSE;
-			/* Not good, let's just kill this thread */ 
+			/* Not good, or we just went offline, let's just kill this thread */ 
 			length = 0;
 			if (header && G_IS_OBJECT (header))
 				g_object_unref (header);
+			g_mutex_unlock (priv->sending_lock);
+			break;
 		}
 
 		g_mutex_unlock (priv->sending_lock);
 
 	}
 
-	priv->is_running = FALSE;
-
 errorhandler:
 
 	priv->is_running = FALSE;
 
+	if (failed_headers)
+		g_hash_table_destroy (failed_headers);
+
 	g_object_unref (sentbox);
 	g_object_unref (outbox);
-	g_object_unref (self); /* The one added here (My own reference) */
-	g_object_unref (self); /* Thread reference (Before creation reference) */
 
 	priv->thread = NULL;
 
+	g_object_unref (info->device);
+	g_object_unref (info->self);
+	g_slice_free (MainThreadInfo, info);
+
+	/* Emit the queue-stop signal */
+	emit_queue_control_signals (self, TNY_SEND_QUEUE_STOP);
+
 	return NULL;
 }
 
@@ -459,10 +571,25 @@ create_worker (TnySendQueue *self)
 {
 	TnyCamelSendQueuePriv *priv = TNY_CAMEL_SEND_QUEUE_GET_PRIVATE (self);
 
-	if (!priv->is_running)
+	if (!priv->is_running && priv->trans_account && TNY_IS_TRANSPORT_ACCOUNT (priv->trans_account))
 	{
-		g_object_ref (self); /* Before creation reference */
-		priv->thread = g_thread_create (thread_main, self, FALSE, NULL);
+		TnyCamelAccountPriv *apriv = TNY_CAMEL_ACCOUNT_GET_PRIVATE (priv->trans_account);
+		TnySessionCamelPriv *spriv = ((TnySessionCamel *) apriv->session)->priv;
+
+		if (spriv->device && TNY_IS_DEVICE (spriv->device)) {
+			MainThreadInfo *info;
+
+			info = g_slice_new0 (MainThreadInfo);
+			info->self = g_object_ref (self);
+			info->device = g_object_ref (spriv->device);
+
+			emit_queue_control_signals (self, TNY_SEND_QUEUE_START);
+
+			priv->thread = g_thread_create (thread_main, info, FALSE, NULL);
+
+			if (priv->thread == NULL)
+				emit_queue_control_signals (self, TNY_SEND_QUEUE_STOP);
+		}
 	}
 
 	return;
@@ -470,65 +597,64 @@ create_worker (TnySendQueue *self)
 
 
 static void
-tny_camel_send_queue_cancel (TnySendQueue *self, gboolean remove, GError **err)
+tny_camel_send_queue_cancel (TnySendQueue *self, TnySendQueueCancelAction cancel_action, GError **err)
 {
-	TNY_CAMEL_SEND_QUEUE_GET_CLASS (self)->cancel_func (self, remove, err);
+	TNY_CAMEL_SEND_QUEUE_GET_CLASS (self)->cancel(self, cancel_action, err);
 }
 
 static void
-tny_camel_send_queue_cancel_default (TnySendQueue *self, gboolean remove, GError **err)
+tny_camel_send_queue_cancel_default (TnySendQueue *self, TnySendQueueCancelAction cancel_action, GError **err)
 {
 	TnyCamelSendQueuePriv *priv = TNY_CAMEL_SEND_QUEUE_GET_PRIVATE (self);
+	TnyFolder *outbox;
+	TnyList *headers = tny_simple_list_new ();
+	TnyIterator *iter;
 
 	g_mutex_lock (priv->sending_lock);
 
 	priv->is_running = FALSE;
-	priv->do_continue = FALSE;
 
-	if (remove)
+	outbox = tny_send_queue_get_outbox (self);
+
+	tny_folder_get_headers (outbox, headers, TRUE, err);
+	
+	if (err != NULL && *err != NULL)
 	{
-		TnyFolder *outbox;
-		TnyList *headers = tny_simple_list_new ();
-		TnyIterator *iter;
+		g_object_unref (headers);
+		g_object_unref (outbox);
+		g_mutex_unlock (priv->sending_lock);
+		return;
+	}
 
-		outbox = tny_send_queue_get_outbox (self);
-
-		tny_folder_get_headers (outbox, headers, TRUE, err);
+	iter = tny_list_create_iterator (headers);
+	while (!tny_iterator_is_done (iter))
+	{
+		TnyHeader *header = TNY_HEADER (tny_iterator_get_current (iter));
 
+		/* Remove or suspend the message */
+		if (cancel_action == TNY_SEND_QUEUE_CANCEL_ACTION_REMOVE)
+			tny_folder_remove_msg (outbox, header, err);
+		else if (cancel_action == TNY_SEND_QUEUE_CANCEL_ACTION_SUSPEND)
+			tny_header_set_flag (header, TNY_HEADER_FLAG_SUSPENDED);
+		
 		if (err != NULL && *err != NULL)
 		{
+			g_object_unref (header);
+			g_object_unref (iter);
 			g_object_unref (headers);
 			g_object_unref (outbox);
 			g_mutex_unlock (priv->sending_lock);
 			return;
 		}
 
-		iter = tny_list_create_iterator (headers);
-
-		while (!tny_iterator_is_done (iter))
-		{
-			TnyHeader *header = TNY_HEADER (tny_iterator_get_current (iter));
-			tny_folder_remove_msg (outbox, header, err);
-
-			if (err != NULL && *err != NULL)
-			{
-				g_object_unref (header);
-				g_object_unref (iter);
-				g_object_unref (headers);
-				g_object_unref (outbox);
-				g_mutex_unlock (priv->sending_lock);
-				return;
-			}
-
-			g_object_unref (header);
-			tny_iterator_next (iter);
-		}
-		g_object_unref (iter);
-		g_object_unref (headers);
-
-		tny_folder_sync (outbox, TRUE, err);
-		g_object_unref (outbox);
+		g_object_unref (header);
+		tny_iterator_next (iter);
 	}
+	g_object_unref (iter);
+	g_object_unref (headers);
+
+	tny_folder_sync (outbox, TRUE, err);
+	g_object_unref (outbox);
 
 	g_mutex_unlock (priv->sending_lock);
 
@@ -539,7 +665,7 @@ tny_camel_send_queue_cancel_default (TnySendQueue *self, gboolean remove, GError
 static void
 tny_camel_send_queue_add (TnySendQueue *self, TnyMsg *msg, GError **err)
 {
-	TNY_CAMEL_SEND_QUEUE_GET_CLASS (self)->add_func (self, msg, err);
+	TNY_CAMEL_SEND_QUEUE_GET_CLASS (self)->add(self, msg, err);
 }
 
 typedef struct {
@@ -571,7 +697,6 @@ on_added (TnyFolder *folder, gboolean cancelled, GError *err, gpointer user_data
 	priv->total++;
 	if (priv->total >= 1 && !priv->is_running)
 		create_worker (info->self);
-
 	if (info->self)
 		g_object_unref (info->self);
 	if (info->msg)
@@ -599,11 +724,11 @@ tny_camel_send_queue_add_default (TnySendQueue *self, TnyMsg *msg, GError **err)
 
 		if (!outbox || !TNY_IS_FOLDER (outbox))
 		{
-			g_set_error (err, TNY_SEND_QUEUE_ERROR, 
-				TNY_SEND_QUEUE_ERROR_ADD,
-				"Operating can't continue: send queue not ready "
+			g_set_error (err, TNY_SERVICE_ERROR, 
+				TNY_SERVICE_ERROR_ADD_MSG,
+				_("Operating can't continue: send queue not ready "
 				"because it does not have a valid outbox. "
-				"This problem indicates a bug in the software.");
+				"This problem indicates a bug in the software."));
 			g_object_unref (headers);
 			g_mutex_unlock (priv->todo_lock);
 			return;
@@ -637,15 +762,15 @@ tny_camel_send_queue_add_default (TnySendQueue *self, TnyMsg *msg, GError **err)
 }
 
 
-void
-tny_camel_send_queue_add_async (TnyCamelSendQueue *self, TnyMsg *msg, TnySendQueueAddCallback callback, TnyStatusCallback status_callback, gpointer user_data)
+static void
+tny_camel_send_queue_add_async (TnySendQueue *self, TnyMsg *msg, TnySendQueueAddCallback callback, TnyStatusCallback status_callback, gpointer user_data)
 {
-	TNY_CAMEL_SEND_QUEUE_GET_CLASS (self)->add_async_func (self, msg, callback, status_callback, user_data);
+	TNY_CAMEL_SEND_QUEUE_GET_CLASS (self)->add_async(self, msg, callback, status_callback, user_data);
 }
 
 
 static void
-tny_camel_send_queue_add_async_default (TnyCamelSendQueue *self, TnyMsg *msg, TnySendQueueAddCallback callback, TnyStatusCallback status_callback, gpointer user_data)
+tny_camel_send_queue_add_async_default (TnySendQueue *self, TnyMsg *msg, TnySendQueueAddCallback callback, TnyStatusCallback status_callback, gpointer user_data)
 {
 	TnyCamelSendQueuePriv *priv = TNY_CAMEL_SEND_QUEUE_GET_PRIVATE (self);
 	GError *err = NULL;
@@ -662,11 +787,11 @@ tny_camel_send_queue_add_async_default (TnyCamelSendQueue *self, TnyMsg *msg, Tn
 
 		if (!outbox || !TNY_IS_FOLDER (outbox))
 		{
-			g_set_error (&err, TNY_SEND_QUEUE_ERROR, 
-				TNY_SEND_QUEUE_ERROR_ADD,
-				"Operating can't continue: send queue not ready "
+			g_set_error (&err, TNY_SERVICE_ERROR, 
+				TNY_SERVICE_ERROR_ADD_MSG,
+				_("Operating can't continue: send queue not ready "
 				"because it does not have a valid outbox. "
-				"This problem indicates a bug in the software.");
+				"This problem indicates a bug in the software."));
 			g_object_unref (headers);
 			g_mutex_unlock (priv->todo_lock);
 			return;
@@ -724,8 +849,6 @@ create_maildir (TnySendQueue *self, const gchar *name)
 
 	mdstore = camel_session_get_store(session, full_path, &ex);
 
-	/*	mdstore = _tny_camel_account_get_service (TNY_CAMEL_ACCOUNT (priv->trans_account)); */
-
 	if (!camel_exception_is_set (&ex) && mdstore)
 	{
 		CamelFolder *cfolder = NULL;
@@ -735,8 +858,6 @@ create_maildir (TnySendQueue *self, const gchar *name)
 		{
 			CamelFolderInfo *iter;
 
-			/* camel_object_unref (CAMEL_OBJECT (cfolder)); */
-
 			iter = camel_store_get_folder_info (mdstore, name, 
 					CAMEL_STORE_FOLDER_INFO_FAST|CAMEL_STORE_FOLDER_INFO_NO_VIRTUAL,&ex);
 
@@ -789,7 +910,7 @@ create_maildir (TnySendQueue *self, const gchar *name)
 static TnyFolder* 
 tny_camel_send_queue_get_sentbox (TnySendQueue *self)
 {
-	return TNY_CAMEL_SEND_QUEUE_GET_CLASS (self)->get_sentbox_func (self);
+	return TNY_CAMEL_SEND_QUEUE_GET_CLASS (self)->get_sentbox(self);
 }
 
 static TnyFolder* 
@@ -807,7 +928,7 @@ tny_camel_send_queue_get_sentbox_default (TnySendQueue *self)
 static TnyFolder* 
 tny_camel_send_queue_get_outbox (TnySendQueue *self)
 {
-	return TNY_CAMEL_SEND_QUEUE_GET_CLASS (self)->get_outbox_func (self);
+	return TNY_CAMEL_SEND_QUEUE_GET_CLASS (self)->get_outbox(self);
 }
 
 static TnyFolder* 
@@ -912,11 +1033,13 @@ tny_camel_send_queue_set_transport_account (TnyCamelSendQueue *self,
 	g_return_if_fail (TNY_IS_CAMEL_TRANSPORT_ACCOUNT(trans_account));
 
 	priv = TNY_CAMEL_SEND_QUEUE_GET_PRIVATE (self);
-	if (priv->trans_account) {
+
+	if (priv->trans_account) 
+	{
 		apriv = TNY_CAMEL_ACCOUNT_GET_PRIVATE (priv->trans_account);
 		_tny_session_camel_unreg_queue (apriv->session, self);
 
-		g_object_unref (G_OBJECT(priv->trans_account));
+		g_object_unref (priv->trans_account);
 
 		if (priv->signal != -1)
 			g_signal_handler_disconnect (G_OBJECT (priv->trans_account), 
@@ -925,7 +1048,7 @@ tny_camel_send_queue_set_transport_account (TnyCamelSendQueue *self,
 
 	priv->signal = (gint) g_signal_connect (G_OBJECT (trans_account),
 		"set-online-happened", G_CALLBACK (on_setonline_happened), self);
-	priv->trans_account = TNY_TRANSPORT_ACCOUNT (g_object_ref(G_OBJECT(trans_account)));
+	priv->trans_account = TNY_TRANSPORT_ACCOUNT (g_object_ref(trans_account));
 	apriv = TNY_CAMEL_ACCOUNT_GET_PRIVATE (priv->trans_account);
 	_tny_session_camel_reg_queue (apriv->session, self);
 	tny_camel_send_queue_flush (TNY_CAMEL_SEND_QUEUE (self));
@@ -952,7 +1075,7 @@ tny_camel_send_queue_get_transport_account (TnyCamelSendQueue *self)
 	priv = TNY_CAMEL_SEND_QUEUE_GET_PRIVATE (self);
 	if (!priv->trans_account)
 		return NULL;
-	g_object_ref (G_OBJECT(priv->trans_account));
+	g_object_ref (priv->trans_account);
 
 	return TNY_CAMEL_TRANSPORT_ACCOUNT(priv->trans_account);
 }
@@ -977,10 +1100,11 @@ tny_send_queue_init (gpointer g, gpointer iface_data)
 {
 	TnySendQueueIface *klass = (TnySendQueueIface *)g;
 
-	klass->add_func = tny_camel_send_queue_add;
-	klass->get_outbox_func = tny_camel_send_queue_get_outbox;
-	klass->get_sentbox_func = tny_camel_send_queue_get_sentbox;
-	klass->cancel_func = tny_camel_send_queue_cancel;
+	klass->add= tny_camel_send_queue_add;
+	klass->add_async= tny_camel_send_queue_add_async;
+	klass->get_outbox= tny_camel_send_queue_get_outbox;
+	klass->get_sentbox= tny_camel_send_queue_get_sentbox;
+	klass->cancel= tny_camel_send_queue_cancel;
 
 	return;
 }
@@ -993,11 +1117,11 @@ tny_camel_send_queue_class_init (TnyCamelSendQueueClass *class)
 	parent_class = g_type_class_peek_parent (class);
 	object_class = (GObjectClass*) class;
 
-	class->add_func = tny_camel_send_queue_add_default;
-	class->add_async_func = tny_camel_send_queue_add_async_default;
-	class->get_outbox_func = tny_camel_send_queue_get_outbox_default;
-	class->get_sentbox_func = tny_camel_send_queue_get_sentbox_default;
-	class->cancel_func = tny_camel_send_queue_cancel_default;
+	class->add= tny_camel_send_queue_add_default;
+	class->add_async= tny_camel_send_queue_add_async_default;
+	class->get_outbox= tny_camel_send_queue_get_outbox_default;
+	class->get_sentbox= tny_camel_send_queue_get_sentbox_default;
+	class->cancel= tny_camel_send_queue_cancel_default;
 
 	object_class->finalize = tny_camel_send_queue_finalize;
 
@@ -1019,9 +1143,9 @@ tny_camel_send_queue_instance_init (GTypeInstance *instance, gpointer g_class)
 	priv->outbox_cache = NULL;
 	priv->todo_lock = g_mutex_new ();
 	priv->sending_lock = g_mutex_new ();
-	priv->do_continue = FALSE;
 	priv->is_running = FALSE;
 	priv->thread = NULL;
+	priv->pending_send_notifies = 0;
 
 	return;
 }
@@ -1029,7 +1153,7 @@ tny_camel_send_queue_instance_init (GTypeInstance *instance, gpointer g_class)
 static void
 tny_folder_observer_init (TnyFolderObserverIface *klass)
 {
-	klass->update_func = tny_camel_send_queue_update;
+	klass->update= tny_camel_send_queue_update;
 }
 
 /**
