@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with self library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <sys/mman.h>
@@ -70,7 +70,7 @@ struct _TnyGnomeAccountStorePriv
 
 
 static gchar* 
-per_account_get_pass_func (TnyAccount *account, const gchar *prompt, gboolean *cancel)
+per_account_get_pass(TnyAccount *account, const gchar *prompt, gboolean *cancel)
 {
 	TnyPlatformFactory *platfact = tny_gnome_platform_factory_get_instance ();
 	TnyPasswordGetter *pwdgetter;
@@ -86,7 +86,7 @@ per_account_get_pass_func (TnyAccount *account, const gchar *prompt, gboolean *c
 
 
 static void
-per_account_forget_pass_func (TnyAccount *account)
+per_account_forget_pass(TnyAccount *account)
 {
 	TnyPlatformFactory *platfact = tny_gnome_platform_factory_get_instance ();
 	TnyPasswordGetter *pwdgetter;
@@ -99,7 +99,7 @@ per_account_forget_pass_func (TnyAccount *account)
 }
 
 static gboolean
-tny_gnome_account_store_alert (TnyAccountStore *self, TnyAccount *account, TnyAlertType type, gboolean question, const GError *error)
+tny_gnome_account_store_alert (TnyAccountStore *self, TnyAccount *account, TnyAlertType type, gboolean question, GError *error)
 {
 	GtkMessageType gtktype;
 	gboolean retval = FALSE;
@@ -305,9 +305,9 @@ load_accounts (TnyAccountStore *self)
 			g_free (key);
 
 			tny_account_set_forget_pass_func (TNY_ACCOUNT (account),
-				per_account_forget_pass_func);
+				per_account_forget_pass);
 			tny_account_set_pass_func (TNY_ACCOUNT (account),
-				per_account_get_pass_func);
+				per_account_get_pass);
 
 			priv->accounts = g_list_prepend (priv->accounts, account);
 		}
@@ -447,7 +447,7 @@ or
 	gconftool-2 -s /apps/tinymail/accounts/0/url_string -t string url_string
 
 */
-
+#if 0
 static void
 tny_gnome_account_store_add_account (TnyAccountStore *self, TnyAccount *account, const gchar *type)
 {
@@ -483,6 +483,7 @@ tny_gnome_account_store_add_account (TnyAccountStore *self, TnyAccount *account,
 
 	return;
 }
+#endif
 
 
 static TnyDevice*
@@ -592,11 +593,11 @@ tny_account_store_init (gpointer g, gpointer iface_data)
 {
 	TnyAccountStoreIface *klass = (TnyAccountStoreIface *)g;
 
-	klass->get_accounts_func = tny_gnome_account_store_get_accounts;
-	klass->get_cache_dir_func = tny_gnome_account_store_get_cache_dir;
-	klass->get_device_func = tny_gnome_account_store_get_device;
-	klass->alert_func = tny_gnome_account_store_alert;
-	klass->find_account_func = tny_gnome_account_store_find_account;
+	klass->get_accounts= tny_gnome_account_store_get_accounts;
+	klass->get_cache_dir= tny_gnome_account_store_get_cache_dir;
+	klass->get_device= tny_gnome_account_store_get_device;
+	klass->alert= tny_gnome_account_store_alert;
+	klass->find_account= tny_gnome_account_store_find_account;
 
 	return;
 }
