@@ -1,4 +1,4 @@
-/* libtinymail-camel - The Tiny Mail base library for Camel
+/* libtinymail-camel - The Tiny Mail base library for Maemo
  * Copyright (C) 2006-2007 Philip Van Hoof <pvanhoof@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with self library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -25,15 +25,42 @@
 #include <coniciap.h>
 #include <conicconnection.h>
 #include <conicconnectionevent.h>
+#include <string.h>
+#include <tny-error.h>
 #include <gdk/gdk.h> /* For GDK_THREAD_ENTER/LEAVE */
-#include <string.h> /* For strcmp() */
+
+static void stop_loop (TnyMaemoConicDevice *self);
+
+static gboolean tny_maemo_conic_device_is_online (TnyDevice *self);
+
+static GObjectClass *parent_class = NULL;
+
+typedef struct {
+	TnyMaemoConicDevice *self;
+	gchar* iap_id;
+	gpointer user_data;
+	TnyMaemoConicDeviceConnectCallback callback;
+} ConnectInfo;
+
+typedef struct {
+	ConIcConnection *cnx;
+	gboolean is_online;
+	gchar *iap;
+	gboolean forced; /* Whether the is_online value is forced rather than real. */
+	ConnectInfo *connect_slot;
+	/* When non-NULL, we are waiting for the success or failure signal. */
+	GMainLoop *loop;
+	gint signal1;
+} TnyMaemoConicDevicePriv;
 
 
-#ifdef MAEMO_CONIC_DUMMY 
-#include <gtk/gtkmessagedialog.h>
-#endif
-
+#define TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE(o)	\
+	(G_TYPE_INSTANCE_GET_PRIVATE ((o), TNY_TYPE_MAEMO_CONIC_DEVICE, TnyMaemoConicDevicePriv))
 
+typedef struct {
+	GObject *self;
+	gboolean status;
+} EmitStatusInfo;
 
 static gboolean
 dnsmasq_has_resolv (void)
@@ -49,57 +76,17 @@ dnsmasq_has_resolv (void)
 	return TRUE;
 }
 
-#ifdef MAEMO_CONIC_DUMMY
-/* #include "coniciap-private.h"
- * This is not installed, so we predeclare the struct instead. Of course, this 
- * is a hack and could break if the private API changes. It would be better for 
- * libconic to support scratchbox. */
-
-struct _ConIcIap 
-{
-	GObject parent_instance;
-	gboolean dispose_has_run;
-	gchar *id;
-	gchar *name;
-	gchar *bearer;
-};
-
-#define MAEMO_CONIC_DUMMY_IAP_ID_FILENAME "maemo_conic_dummy_id"
-#define MAEMO_CONIC_DUMMY_IAP_ID_NONE "none"
-static gboolean on_dummy_connection_check (gpointer user_data);
-#endif /* MAEMO_CONIC_DUMMY */
-
-static gboolean tny_maemo_conic_device_is_online (TnyDevice *self);
-
-static GObjectClass *parent_class = NULL;
-
-typedef struct {
-	ConIcConnection *cnx;
-	gboolean is_online;
-	gchar *iap;
-	gboolean forced; /* Whether the is_online value is forced rather than real. */
-	/* When non-NULL, we are waiting for the success or failure signal. */
-	GMainLoop *loop;
-#ifdef MAEMO_CONIC_DUMMY
-	gint dummy_env_check_timeout;
-#endif /* MAEMO_CONIC_DUMMY */
-} TnyMaemoConicDevicePriv;
-
-#define TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE(o)	\
-	(G_TYPE_INSTANCE_GET_PRIVATE ((o), TNY_TYPE_MAEMO_CONIC_DEVICE, TnyMaemoConicDevicePriv))
-
-typedef struct {
-	GObject *self;
-	gboolean status;
-} EmitStatusInfo;
-
 static gboolean
 conic_emit_status_idle (gpointer user_data)
 {
-	EmitStatusInfo *info = (EmitStatusInfo *) user_data;
+	EmitStatusInfo *info;
 
-	/* We lock the gdk thread because tinymail wants implementations to do this 
-	 * before emitting _any_ signals.
+	g_return_val_if_fail (user_data, FALSE);
+	
+	info  = (EmitStatusInfo *) user_data;
+
+	/* We lock the gdk thread because tinymail wants implementations to do
+	 * this before emitting signals from within a g_idle_add_full callback.
 	 * See http://www.tinymail.org/trac/tinymail/wiki/HowTnyLockable */
 
 	gdk_threads_enter ();
@@ -113,56 +100,37 @@ conic_emit_status_idle (gpointer user_data)
 static void
 conic_emit_status_destroy (gpointer user_data)
 {
-	EmitStatusInfo *info = (EmitStatusInfo *) user_data;
-	g_object_unref (info->self);
+	EmitStatusInfo *info;
+
+	g_return_if_fail (user_data);
+
+	info = (EmitStatusInfo *) user_data;
+	
+	if (G_IS_OBJECT(info->self))
+		g_object_unref (info->self);
+	
 	g_slice_free (EmitStatusInfo, info);
-	return;
 }
 
 static void 
 conic_emit_status (TnyDevice *self, gboolean status)
 {
-	/* If there is a mainloop (if gtk_main() has been run, for instance),
-	 * then emit the signal via an idle callback, so that it is 
-	 * always emitted in the main context as required by tinymail
-	 * (libconic does not give any guarantee 
-	 * about this - it would be nice if libconic documented that).
-	 * But if there is no mainloop, then just emit it, as tinymail 
-	 * requires when there is no mainloop: */
+	EmitStatusInfo *info;
+	guint time = 1000;
 
-	 /* TODO: We have no way to check for this now, and 
-	  * at this time it's not even clear whether tinymail should/can really 
-	  * demand this. murrayc. 15th Aug. 2007. */
+	g_return_if_fail (TNY_IS_DEVICE(self));
+	
+	/* Emit it in an idle handler: */
+	info = g_slice_new (EmitStatusInfo);
+	
+	info->self = g_object_ref (self);
+	info->status = status;
 
-	if (TRUE) /* TODO: But NULL is not allowed here: g_main_loop_is_running (NULL)) */
-	{
-		/* Emit it in an idle handler: */
-		EmitStatusInfo *info = g_slice_new (EmitStatusInfo);
-		guint time = 5000;
+	if (!dnsmasq_has_resolv())
+		time = 5000;
 
-		info->self = g_object_ref (self);
-		info->status = status;
-
-		if (!dnsmasq_has_resolv())
-			time = 5000;
-
-		g_timeout_add_full (G_PRIORITY_DEFAULT, time, conic_emit_status_idle,
-			info, conic_emit_status_destroy);
-
-	} else {
-		/* Emit it directly: */
-
-		/* We lock the gdk thread because tinymail wants implementations
-		 *  to do this before emitting _any_ signals.
-		 * See http://www.tinymail.org/trac/tinymail/wiki/HowTnyLockable */
-
-		gdk_threads_enter ();
-		g_signal_emit (self, tny_device_signals [TNY_DEVICE_CONNECTION_CHANGED],
-			0, status);
-		gdk_threads_leave ();
-	}
-
-	return;
+	g_timeout_add_full (G_PRIORITY_DEFAULT, time, conic_emit_status_idle,
+		info, conic_emit_status_destroy);
 }
 
 static void 
@@ -173,6 +141,7 @@ tny_maemo_conic_device_reset (TnyDevice *device)
 	gboolean status_before = FALSE;
 
 	g_return_if_fail (TNY_IS_DEVICE(device));
+	
 	self = TNY_MAEMO_CONIC_DEVICE (device);
 	priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (self);
 
@@ -181,41 +150,122 @@ tny_maemo_conic_device_reset (TnyDevice *device)
 
 	if (status_before != tny_maemo_conic_device_is_online (device))
 		conic_emit_status (device, !status_before);
-
-	return;
 }
 
-#ifndef MAEMO_CONIC_DUMMY
+static void
+handle_connect (TnyMaemoConicDevice *self, int con_err, int con_state)
+{
+	TnyMaemoConicDevicePriv *priv;
+
+	g_return_if_fail (TNY_IS_MAEMO_CONIC_DEVICE (self));
+	
+	priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (self);
+
+	if (priv->connect_slot) {
+		GError *err = NULL;
+		gboolean canceled = FALSE;
+		ConnectInfo *info = priv->connect_slot;
+
+		/* Mark it as handled (TODO, this needs a small lock) */
+		priv->connect_slot = NULL;
+
+		switch (con_err) {
+			case CON_IC_CONNECTION_ERROR_NONE:
+				break;
+			case CON_IC_CONNECTION_ERROR_INVALID_IAP:
+				g_set_error (&err, TNY_SERVICE_ERROR, TNY_SERVICE_ERROR_UNKNOWN,
+					"IAP is invalid");
+				break;
+			case CON_IC_CONNECTION_ERROR_CONNECTION_FAILED:
+				g_set_error (&err, TNY_SERVICE_ERROR, TNY_SERVICE_ERROR_UNKNOWN,
+					"Connection failed");
+				break;
+			case CON_IC_CONNECTION_ERROR_USER_CANCELED:
+			default:
+				canceled = TRUE;
+				break;
+		}
+
+		if (info->callback) {
+			/* We lock the gdk thread because tinymail wants implementations to do
+			 * this before invoking callbacks from within a g_idle_add_full callback.
+			 * See http://www.tinymail.org/trac/tinymail/wiki/HowTnyLockable */
+
+			gdk_threads_enter ();
+			info->callback (info->self, info->iap_id, canceled, err, info->user_data);
+			gdk_threads_leave ();
+		}
+
+		if (err)
+			g_error_free (err);
+
+		g_object_unref (info->self);
+		g_free (info->iap_id);
+		g_slice_free (ConnectInfo, info);
+	}
+}
+
+typedef struct {
+	TnyMaemoConicDevice *self;
+	int con_err;
+	int con_state;
+} HandleConnInfo;
+
+static gboolean /* FIXME: this function will always return FALSE, is that correct? */
+handle_con_idle (gpointer data)
+{
+	HandleConnInfo *info;
+
+	g_return_val_if_fail (data, FALSE);
+	
+	info = (HandleConnInfo *) data;
+	handle_connect (info->self, info->con_err, info->con_state);
+
+	return FALSE;
+}
 
 static void 
-stop_loop (TnyMaemoConicDevice *self)
+handle_con_idle_destroy (gpointer data) 
 {
-	TnyMaemoConicDevicePriv *priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (self);
-	if (priv->loop)
-		g_main_loop_quit (priv->loop);
-	return;
+	HandleConnInfo *info;
+	
+	g_return_if_fail (data);
+	
+	info = (HandleConnInfo *) data;
+
+	if (G_IS_OBJECT(info->self))
+		g_object_unref (info->self);
+	
+	g_slice_free (HandleConnInfo, data); 
 }
 
+
 static void
 on_connection_event (ConIcConnection *cnx, ConIcConnectionEvent *event, gpointer user_data)
 {
-	TnyMaemoConicDevice *device = TNY_MAEMO_CONIC_DEVICE (user_data);
-	TnyMaemoConicDevicePriv *priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (device);
+	TnyMaemoConicDevice *device;
+	TnyMaemoConicDevicePriv *priv;
 	gboolean is_online = FALSE;
 	gboolean emit = FALSE;
+	HandleConnInfo *iinfo;
+	int con_err, con_state;
+	
+	/* we don't need cnx in this function */
+	g_return_if_fail (user_data && TNY_IS_MAEMO_CONIC_DEVICE(user_data));
+	g_return_if_fail (event && CON_IC_IS_CONNECTION_EVENT(event));
+	
+	device = TNY_MAEMO_CONIC_DEVICE (user_data);
+	priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (device);
 
 	/* Don't emit nor make any changes in case of forced state */
+
 	if (priv->forced)
 		return;
 
-	g_return_if_fail (CON_IC_IS_CONNECTION(cnx));
+	con_err = con_ic_connection_event_get_error (event);
+	con_state = con_ic_connection_event_get_status (event);
 
-#ifdef DEBUG
-	g_message (__FUNCTION__);
-#endif
-
-	switch (con_ic_connection_event_get_error(event)) 
-	{
+	switch (con_err) {
 		case CON_IC_CONNECTION_ERROR_NONE:
 			break;
 		case CON_IC_CONNECTION_ERROR_INVALID_IAP:
@@ -231,11 +281,9 @@ on_connection_event (ConIcConnection *cnx, ConIcConnectionEvent *event, gpointer
 			g_return_if_reached ();
 	}
 
-	switch (con_ic_connection_event_get_status(event)) 
-	{
+	switch (con_state) {
 		case CON_IC_STATUS_CONNECTED:
-			if (priv->iap)
-				g_free (priv->iap);
+			g_free (priv->iap);
 			priv->iap = g_strdup (con_ic_event_get_iap_id ((ConIcEvent*)(event)));
 			if (!priv->is_online)
 				emit = TRUE;
@@ -245,6 +293,7 @@ on_connection_event (ConIcConnection *cnx, ConIcConnectionEvent *event, gpointer
 			break;
 
 		case CON_IC_STATUS_DISCONNECTED:
+			g_free (priv->iap);
 			priv->iap = NULL;
 			if (priv->is_online)
 				emit = TRUE;
@@ -261,135 +310,88 @@ on_connection_event (ConIcConnection *cnx, ConIcConnectionEvent *event, gpointer
 
 	priv->is_online = is_online;
 
-#ifdef DEBUG
-	g_message ("DEBUG: %s: emitting signal CONNECTION_CHANGED: %s", 
-		   __FUNCTION__, is_online ? "online" : "offline");
-#endif
+	if (priv->connect_slot)	{
+
+		iinfo = g_slice_new (HandleConnInfo);
+		iinfo->self = (TnyMaemoConicDevice *) g_object_ref (device);
+		iinfo->con_err = con_err;
+		iinfo->con_state = con_state;
+
+		g_idle_add_full (G_PRIORITY_HIGH, handle_con_idle, iinfo, 
+				 handle_con_idle_destroy);
+	}
 
 	if (emit)
 		conic_emit_status (TNY_DEVICE (device), is_online);
-
-	return;
 }
-#endif /* MAEMO_CONIC_DUMMY */
-
-#ifdef MAEMO_CONIC_DUMMY
-
-static gchar*
-get_dummy_filename ()
-{
-	gchar *filename = g_build_filename (
-		g_get_home_dir (), 
-		MAEMO_CONIC_DUMMY_IAP_ID_FILENAME,
-		NULL);
-	return filename;
-}
-
-static gboolean 
-dummy_con_ic_connection_connect_by_id (TnyMaemoConicDevice *self, const gchar* iap_id)
-{
-	int response = 0;
-
-	/* Show a dialog, because libconic would show a dialog here,
-	 * and give the user a chance to refuse a new connection, because libconic would allow that too.
-	 * This allows us to see roughly similar behaviour in scratchbox as on the device. */
-	GtkDialog *dialog = GTK_DIALOG (gtk_message_dialog_new( NULL, GTK_DIALOG_MODAL,
-			GTK_MESSAGE_QUESTION, GTK_BUTTONS_OK_CANCEL, 
-			"TnyMaemoConicDevice fake scratchbox implementation:\nThe application requested a connection. Make a fake connection?"));
-
-	response = gtk_dialog_run (dialog);
-	gtk_widget_hide (GTK_WIDGET (dialog));
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-	
-	if (response == GTK_RESPONSE_OK) {
-		GError* error = NULL;
-		/* Make a connection, by setting a name in our dummy text file,
-		 * which will be read later: */
-		gchar *filename = get_dummy_filename ();
-
-		g_file_set_contents (filename, "debug id0", -1, &error);
-		if(error) {
-			g_warning("%s: error from g_file_set_contents(): %s\n", __FUNCTION__, error->message);
-			g_error_free (error);
-			error = NULL;
-		}
-
-		g_free (filename);
-
-		return TRUE;
-	}
-	else
-		return FALSE;
-}
-
-#endif /* MAEMO_CONIC_DUMMY */
 
 
 /**
  * tny_maemo_conic_device_connect:
  * @self: a #TnyDevice object
  * @iap_id: the id of the Internet Access Point (IAP), or NULL for 'any;
+ * @user_requested: whether or not the connection was automatically requested or by an user action
+ * @callback: a #TnyMaemoConicDeviceConnectCallback
+ * @user_data: user data for @callback
  * 
  * Try to connect to a specific IAP, or to any if @iap_id == NULL
  * this calls con_ic_connection_connect(_by_id).
  * This may show a dialog to allow the user to select a connection, or 
- * may otherwise take a significant amount of time. This function blocks until 
- * the connection has either succeeded or failed.
- * 
- * Returns TRUE if a connection was made, FALSE otherwise.
+ * may otherwise take a significant amount of time. 
  **/
-gboolean
-tny_maemo_conic_device_connect (TnyMaemoConicDevice *self, const gchar* iap_id)
+void 
+tny_maemo_conic_device_connect_async (TnyMaemoConicDevice *self, 
+				      const gchar* iap_id, 
+				      gboolean user_requested,
+				      TnyMaemoConicDeviceConnectCallback callback, 
+				      gpointer user_data)
 {
-#ifdef MAEMO_CONIC_DUMMY 
-	return dummy_con_ic_connection_connect_by_id (self, iap_id);
-#else
 	TnyMaemoConicDevicePriv *priv = NULL;
 	gboolean request_failed = FALSE;
+	ConnectInfo *info;
+	GError *err = NULL;
+	ConIcConnectFlags flags;
+
+	g_return_if_fail (self && TNY_IS_MAEMO_CONIC_DEVICE(self));
 
-	g_return_val_if_fail (TNY_IS_DEVICE(self), FALSE);
 	priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (self);
 
-	g_return_val_if_fail (priv->cnx, FALSE);
+	info = g_slice_new (ConnectInfo);
+	info->self = (TnyMaemoConicDevice *) g_object_ref (self);
+	info->callback = callback;
+	info->user_data = user_data;
+	info->iap_id = iap_id ? g_strdup (iap_id) : NULL; /* iap_id can be NULL */
 
-#ifdef DEBUG
-	g_message (__FUNCTION__);
-	g_message ("connecting to %s", iap_id ? iap_id : "<any>");
-#endif
+	priv->connect_slot = info;
 
-	priv->loop = g_main_loop_new(NULL, FALSE /* not running immediately. */);
+	/* Set the flags */
+	if (user_requested)
+		flags = CON_IC_CONNECT_FLAG_NONE;
+	else
+		flags = CON_IC_CONNECT_FLAG_AUTOMATICALLY_TRIGGERED;
 
 	if (iap_id) {
-		if (!con_ic_connection_connect_by_id (priv->cnx, iap_id, CON_IC_CONNECT_FLAG_NONE)) {
-			g_warning ("could not send connect_by_id dbus message");
+		if (!con_ic_connection_connect_by_id (priv->cnx, iap_id, flags)) {
+			g_set_error (&err, TNY_SERVICE_ERROR, TNY_SERVICE_ERROR_UNKNOWN,
+				"Could not send connect_by_id dbus message");
 			request_failed = TRUE;
 		}
 	} else {
-		if (!con_ic_connection_connect (priv->cnx, CON_IC_CONNECT_FLAG_NONE)) {
-			g_warning ("could not send connect dbus message");
+		if (!con_ic_connection_connect (priv->cnx, flags)) {
+			g_set_error (&err, TNY_SERVICE_ERROR, TNY_SERVICE_ERROR_UNKNOWN,
+				"Could not send connect dbus message");
 			request_failed = TRUE;
 		}
 	}
 
 	if (request_failed) {
-		g_object_unref (priv->loop);
-		priv->loop = NULL;
+		priv->connect_slot = NULL;
+		if (info->callback)
+			info->callback (info->self, iap_id, FALSE, err, info->user_data);
+		g_free (info->iap_id);
+		g_object_unref (info->self);
+		g_slice_free (ConnectInfo, info);
 	}
-	
-	/* Wait for the CON_IC_STATUS_CONNECTED (succeeded) or 
-	 * CON_IC_STATUS_DISCONNECTED event: */
-	 
-	/* This is based on code found in gtk_dialog_run(): */
-	GDK_THREADS_LEAVE();
-	/* Run until g_main_loop_quit() is called by our signal handler. */
-	g_main_loop_run (priv->loop);
-	GDK_THREADS_ENTER();
-
-	g_main_loop_unref (priv->loop);
-	priv->loop = NULL;
-
-	return priv->is_online;
-#endif /* MAEMO_CONIC_DUMMY */
 }
 
 
@@ -406,103 +408,28 @@ tny_maemo_conic_device_connect (TnyMaemoConicDevice *self, const gchar* iap_id)
 gboolean
 tny_maemo_conic_device_disconnect (TnyMaemoConicDevice *self, const gchar* iap_id)
 {
-/* don't try to disconnect if we're in dummy mode, as we're not "really"
- * connected in that case either
- */
-#ifndef MAEMO_CONIC_DUMMY 
 	TnyMaemoConicDevicePriv *priv = NULL;
 
-	g_return_val_if_fail (TNY_IS_MAEMO_CONIC_DEVICE(self), FALSE);
+	g_return_val_if_fail (self && TNY_IS_MAEMO_CONIC_DEVICE(self), FALSE);
 
 	priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (self);
 	g_return_val_if_fail (priv->cnx, FALSE);
 
-#ifdef DEBUG
-	g_message (__FUNCTION__);
-	g_message ("disconnecting from %s", iap_id ? iap_id : "<any>");
-#endif
-
 	if (iap_id) {
 		if (!con_ic_connection_disconnect_by_id (priv->cnx, iap_id)) {
-			g_warning ("could not send disconnect_by_id dbus message");
+			g_warning ("Could not send disconnect_by_id dbus message");
 			return FALSE;
 		}
 	} else {
 		/* don't try to disconnect if iap_id==NULL, or conic will crash... */
-		g_warning ("could not send disconnect dbus message");
+		g_warning ("Could not send disconnect dbus message");
 		return FALSE;
 	}
-#endif /* MAEMO_CONIC_DUMMY*/
 
 	return TRUE;
 }
 
 
-#ifdef MAEMO_CONIC_DUMMY
-static gboolean
-on_dummy_connection_check (gpointer user_data)
-{
-	TnyMaemoConicDevice *self = NULL;
-	TnyMaemoConicDevicePriv *priv = NULL;
-	gchar *filename = NULL;
-	gchar *contents = NULL;
-	GError* error = NULL;
-	gboolean test = FALSE;
-		
-	self = TNY_MAEMO_CONIC_DEVICE (user_data);
-
-	priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (self);
-	
-	/* Check whether the enviroment variable has changed, 
-	 * so we can fake a connection change: */
-	filename = get_dummy_filename ();
-	
-	test = g_file_get_contents (filename, &contents, NULL, &error);
-
-	if(error) {
-		/* g_debug("%s: error from g_file_get_contents(): %s\n", __FUNCTION__, error->message); */
-		g_error_free (error);
-		error = NULL;
-	}
-	
-	if (!test || !contents) {
-		/* g_debug ("DEBUG1: %s: priv->iap = %s\n", priv->iap); */
-		/* Default to the first debug connection: */
-		contents = g_strdup ("debug id0");
-	}
-	
-	if (contents)
-		g_strstrip(contents);
-
-	if ((priv->iap == NULL) || (strcmp (contents, priv->iap) != 0)) {
-		if (priv->iap) {
-			g_free (priv->iap);
-			priv->iap = NULL;
-		}
-			
-		/* We store even the special "none" text, so we can detect changes. */
-		priv->iap = g_strdup (contents);
-		
-		if (strcmp (priv->iap, MAEMO_CONIC_DUMMY_IAP_ID_NONE) == 0) {
-			priv->is_online = FALSE;
-			g_debug ("DEBUG: TnyMaemoConicDevice: %s:\n  Dummy connection changed to no connection.\n", __FUNCTION__);
-		} else {
-			priv->is_online = TRUE;
-			g_debug ("DEBUG: TnyMaemoConicDevice: %s:\n  Dummy connection changed to '%s\n", __FUNCTION__, priv->iap);
-		}
-		
-		g_debug ("DEBUG1: %s: emitting is_online=%d\n", __FUNCTION__, priv->is_online);
-
-		conic_emit_status (TNY_DEVICE (self), priv->is_online);
-
-	}
-	
-	g_free (contents);
-	g_free (filename);
-	
-	return TRUE;
-}
-#endif /* MAEMO_CONIC_DUMMY */
 
 /**
  * tny_maemo_conic_device_get_current_iap_id:
@@ -523,13 +450,6 @@ tny_maemo_conic_device_get_current_iap_id (TnyMaemoConicDevice *self)
 
 	priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (self);
 
-#ifdef MAEMO_CONIC_DUMMY
-	on_dummy_connection_check (self);
-	/* Handle the special "none" text: */
-	if (priv->iap && (strcmp (priv->iap, MAEMO_CONIC_DUMMY_IAP_ID_NONE) == 0))
-		return NULL;
-#endif
-
 	return priv->iap;
 }
 
@@ -550,33 +470,19 @@ tny_maemo_conic_device_get_current_iap_id (TnyMaemoConicDevice *self)
 ConIcIap*
 tny_maemo_conic_device_get_iap (TnyMaemoConicDevice *self, const gchar *iap_id)
 {
-#ifdef MAEMO_CONIC_DUMMY
-	ConIcIap *iap = NULL;
-#else
-	TnyMaemoConicDevicePriv *priv = NULL;
-#endif
-
+	TnyMaemoConicDevicePriv *priv;
 	g_return_val_if_fail (TNY_IS_MAEMO_CONIC_DEVICE(self), NULL);
 	g_return_val_if_fail (iap_id, NULL);
 
-#ifdef MAEMO_CONIC_DUMMY 
-	/* Note that we have re-declared the private struct so that we 
-	 * can do this, which is very bad and fragile: */
-	iap = g_object_new (CON_IC_TYPE_IAP, NULL);
-	iap->id = g_strdup(iap_id);
-	iap->name = g_strdup_printf("%s name", iap->id);
-	return iap;
-#else
 	priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (self);
 	g_return_val_if_fail (priv->cnx, NULL);
 
-	/* Note that it is very unusual to return a reference from a get_() function, 
-	 * but we must do so because that mistake has already been made in 
-	 * con_ic_connection_get_iap().
-	 * If we just unref immediately then libconic might destroy the object.
-	 */
+	/* Note that it is very unusual to return a reference from a get_() 
+	 * function, but we must do so because that mistake has already been 
+	 * made in con_ic_connection_get_iap (). If we just unref immediately 
+	 * then libconic might destroy the object. */
+
 	return con_ic_connection_get_iap (priv->cnx, iap_id);
-#endif
 }
 
 
@@ -593,93 +499,52 @@ tny_maemo_conic_device_get_iap (TnyMaemoConicDevice *self, const gchar *iap_id)
 GSList*
 tny_maemo_conic_device_get_iap_list (TnyMaemoConicDevice *self)
 {
-#ifdef MAEMO_CONIC_DUMMY
-	GSList* result = NULL;
-	int i = 0;
-#else
-	TnyMaemoConicDevicePriv *priv = NULL;
-#endif /* MAEMO_CONIC_DUMMY */
+	TnyMaemoConicDevicePriv *priv;
 	
 	g_return_val_if_fail (TNY_IS_MAEMO_CONIC_DEVICE(self), NULL);
 
-#ifdef MAEMO_CONIC_DUMMY
-
-	/* libconic does not return a list of connections when running 
-	 * in scratchbox, though it might do this in future when 
-	 * "ethernet support" is implemented.
-	 * So we return a fake list so we can exercise funcationality 
-	 * that uses connections: */
-	for (i = 0; i < 10; ++i) {
-		/* con_ic_iap_new (id) would checkc for a gconf dir and 
-		 * fails, though _new() functions should just call g_object_new().
-		 *
-		gchar *id = g_strdup_printf("debug id%d", i);
-		ConIcIap *iap = con_ic_iap_new (id);
-		g_free (id);
-		*/
-		
-		/* Note that we have re-declared the private struct so that we 
-		 * can do this, which is very bad and fragile: */
-		ConIcIap *iap = g_object_new (CON_IC_TYPE_IAP, NULL);
-		iap->id = g_strdup_printf("debug id%d", i);
-		iap->name = g_strdup_printf("%s name", iap->id);
-
-		result = g_slist_append (result, iap);	
-	}
-	
-	return result;
-#else
 	priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (self);
 	g_return_val_if_fail (priv->cnx, NULL);
 
 	return con_ic_connection_get_all_iaps (priv->cnx);
-#endif
 }
 
 
 /**
  * tny_maemo_conic_device_free_iap_list:
  * @self: a #TnyDevice object
- *  
+ * @cnx_list: a list of IAP objects
+ *
  * free a  list of IAP objects retrieved from tny_maemo_conic_device_get_iap_list
- *  
  **/
 void
 tny_maemo_conic_device_free_iap_list (TnyMaemoConicDevice *self, GSList* cnx_list)
 {
-	GSList *cur = cnx_list;
-	while (cur) {
-		g_object_unref (G_OBJECT(cur->data));
-		cur = g_slist_next (cur);
-	}
+	g_slist_foreach (cnx_list, (GFunc)g_object_unref, NULL);
 	g_slist_free (cnx_list);
 }
 
 
-
 static void 
 tny_maemo_conic_device_force_online (TnyDevice *device)
 {
 	TnyMaemoConicDevice *self;
 	TnyMaemoConicDevicePriv *priv;
 	gboolean already_online = FALSE;
+
 	g_return_if_fail (TNY_IS_DEVICE(device));
+
 	self = TNY_MAEMO_CONIC_DEVICE (device);
 	priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (self);
 	
 	already_online = tny_maemo_conic_device_is_online (device);
 
-#ifdef DEBUG
-	g_message (__FUNCTION__);
-	g_message ("force online, current status is: %s", already_online ? "online" : "offline");
-#endif
-
 	priv->forced = TRUE;
+	priv->is_online = TRUE;
 
 	/* Signal if it changed: */
 	if (!already_online)
-		g_signal_emit (device, tny_device_signals [TNY_DEVICE_CONNECTION_CHANGED],
-		       0, TRUE);
+		g_signal_emit (device, tny_device_signals [TNY_DEVICE_CONNECTION_CHANGED], 0, TRUE);
 }
 
 
@@ -688,24 +553,20 @@ tny_maemo_conic_device_force_offline (TnyDevice *device)
 {
 	TnyMaemoConicDevice *self;
 	TnyMaemoConicDevicePriv *priv;
-#ifndef MAEMO_CONIC_DUMMY
 	gboolean already_offline = FALSE;
-#endif
 
 	g_return_if_fail (TNY_IS_DEVICE(device));
+
 	self = TNY_MAEMO_CONIC_DEVICE (device);
 	priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (self);
 
-#ifdef MAEMO_CONIC_DUMMY
-	return;
-#else
 	already_offline = !tny_maemo_conic_device_is_online (device);
 	priv->forced = TRUE;
+	priv->is_online = FALSE;
+
 	/* Signal if it changed: */
 	if (!already_offline)
 		conic_emit_status (device, FALSE);
-	return;
-#endif
 }
 
 static gboolean
@@ -713,10 +574,6 @@ tny_maemo_conic_device_is_online (TnyDevice *self)
 {
 	g_return_val_if_fail (TNY_IS_DEVICE(self), FALSE);
 
-#ifdef MAEMO_CONIC_DUMMY
-	on_dummy_connection_check (self);
-#endif /* MAEMO_CONIC_DUMMY */
-
 	return TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (self)->is_online;
 }
 
@@ -728,45 +585,32 @@ tny_maemo_conic_device_instance_init (GTypeInstance *instance, gpointer g_class)
 	TnyMaemoConicDevicePriv *priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (self);
 
 	/* We should not have a real is_online, based on what libconic has told us: */
+
 	priv->forced = FALSE;
 	priv->iap = NULL;
 	priv->is_online = dnsmasq_has_resolv ();
-
-
-#ifndef MAEMO_CONIC_DUMMY
+	priv->connect_slot = NULL;
+	
 	priv->cnx = con_ic_connection_new ();
-	if (!priv->cnx) {
+
+	if (!priv->cnx)
 		g_warning ("con_ic_connection_new failed. The TnyMaemoConicDevice will be useless.");
-	}
 
 	/* This might be necessary to make the connection object actually emit 
 	 * the signal, though the documentation says that they should be sent 
 	 * even when this is not set, when we explicitly try to connect. The 
 	 * signal still does not seem to be emitted. */
-
 	g_object_set (priv->cnx, "automatic-connection-events", TRUE, NULL);
 
-	g_signal_connect (priv->cnx, "connection-event",
+	priv->signal1 = (gint) g_signal_connect (priv->cnx, "connection-event",
 			  G_CALLBACK(on_connection_event), self);
-	
+
 	/* This will get us in connected state only if there is already a connection.
 	 * thus, this will setup our state correctly when we receive the signals. */
-
 	if (!con_ic_connection_connect (priv->cnx, CON_IC_CONNECT_FLAG_AUTOMATICALLY_TRIGGERED))
 		g_warning ("could not send connect dbus message");
-		
-#endif /* MAEMO_CONIC_DUMMY */
 
-
-#ifdef MAEMO_CONIC_DUMMY
-
-	/* Allow debuggers to fake a connection change by setting an environment 
-	 * variable, which we check ever 1 second. This should match one of the 
-	 * fake iap IDs that we created in tny_maemo_conic_device_get_iap_list().*/
-
-	priv->dummy_env_check_timeout = 
-		g_timeout_add (1000, on_dummy_connection_check, self);
-#endif /* MAEMO_CONIC_DUMMY */
+	return;
 }
 
 
@@ -778,34 +622,29 @@ tny_maemo_conic_device_instance_init (GTypeInstance *instance, gpointer g_class)
 TnyDevice*
 tny_maemo_conic_device_new (void)
 {
-	TnyMaemoConicDevice *self = g_object_new (TNY_TYPE_MAEMO_CONIC_DEVICE, NULL);
-	return TNY_DEVICE (self);
+	return TNY_DEVICE(g_object_new (TNY_TYPE_MAEMO_CONIC_DEVICE, NULL));
 }
 
 static void
 tny_maemo_conic_device_finalize (GObject *obj)
 {
-	TnyMaemoConicDevicePriv *priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (obj);
+	TnyMaemoConicDevicePriv *priv;
+
+	g_return_if_fail (obj && G_IS_OBJECT(obj));
+	
+	priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (obj);
+	g_signal_handler_disconnect (obj, priv->signal1);
 
-#ifndef MAEMO_CONIC_DUMMY
 	if (priv->cnx && CON_IC_IS_CONNECTION(priv->cnx)) {
-		if (!tny_maemo_conic_device_disconnect (TNY_MAEMO_CONIC_DEVICE(obj),priv->iap))
-			g_warning ("failed to disconnect dbus message");
+		tny_maemo_conic_device_disconnect (TNY_MAEMO_CONIC_DEVICE(obj),priv->iap);
 		g_object_unref (priv->cnx);
 		priv->cnx = NULL;
 	}
-#else
-	if (priv->dummy_env_check_timeout) {
-		g_source_remove (priv->dummy_env_check_timeout);
-		priv->dummy_env_check_timeout = 0;
-	}
-#endif /* MAEMO_CONIC_DUMMY */
 
 	if (priv->iap) {
 		g_free (priv->iap);
 		priv->iap = NULL;
 	}
-	
 
 	(*parent_class->finalize) (obj);
 }
@@ -816,14 +655,13 @@ tny_device_init (gpointer g, gpointer iface_data)
 {
 	TnyDeviceIface *klass = (TnyDeviceIface *)g;
 
-	klass->is_online_func     = tny_maemo_conic_device_is_online;
-	klass->reset_func         = tny_maemo_conic_device_reset;
-	klass->force_offline_func = tny_maemo_conic_device_force_offline;
-	klass->force_online_func  = tny_maemo_conic_device_force_online;
+	klass->is_online     = tny_maemo_conic_device_is_online;
+	klass->reset         = tny_maemo_conic_device_reset;
+	klass->force_offline = tny_maemo_conic_device_force_offline;
+	klass->force_online  = tny_maemo_conic_device_force_online;
 }
 
 
-
 static void 
 tny_maemo_conic_device_class_init (TnyMaemoConicDeviceClass *class)
 {
@@ -874,3 +712,80 @@ tny_maemo_conic_device_get_type (void)
 	}
 	return type;
 }
+
+static void 
+stop_loop (TnyMaemoConicDevice *self)
+{
+	TnyMaemoConicDevicePriv *priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (self);
+	if (priv->loop)
+		g_main_loop_quit (priv->loop);
+	return;
+}
+
+
+/**
+ * tny_maemo_conic_device_connect:
+ * @self: a #TnyDevice object
+ * @iap_id: the id of the Internet Access Point (IAP), or NULL for 'any;
+ * 
+ * Try to connect to a specific IAP, or to any if @iap_id == NULL
+ * this calls con_ic_connection_connect(_by_id).
+ * This may show a dialog to allow the user to select a connection, or 
+ * may otherwise take a significant amount of time. This function blocks until 
+ * the connection has either succeeded or failed.
+ * 
+ * Returns TRUE if a connection was made, FALSE otherwise.
+ **/
+gboolean
+tny_maemo_conic_device_connect (TnyMaemoConicDevice *self, 
+				const gchar* iap_id,
+				gboolean user_requested)
+{
+	TnyMaemoConicDevicePriv *priv = NULL;
+	gboolean request_failed = FALSE;
+	ConIcConnectFlags flags;
+
+	g_return_val_if_fail (TNY_IS_DEVICE(self), FALSE);
+	priv = TNY_MAEMO_CONIC_DEVICE_GET_PRIVATE (self);
+
+	g_return_val_if_fail (priv->cnx, FALSE);
+	priv->loop = g_main_loop_new(NULL, FALSE /* not running immediately. */);
+
+	/* Set the flags */
+	if (user_requested)
+		flags = CON_IC_CONNECT_FLAG_NONE;
+	else
+		flags = CON_IC_CONNECT_FLAG_AUTOMATICALLY_TRIGGERED;
+
+	if (iap_id) {
+		if (!con_ic_connection_connect_by_id (priv->cnx, iap_id, flags)) {
+			g_warning ("could not send connect_by_id dbus message");
+			request_failed = TRUE;
+		}
+	} else {
+		if (!con_ic_connection_connect (priv->cnx, flags)) {
+			g_warning ("could not send connect dbus message");
+			request_failed = TRUE;
+		}
+	}
+
+	if (request_failed) {
+		g_object_unref (priv->loop);
+		priv->loop = NULL;
+	}
+	
+	/* Wait for the CON_IC_STATUS_CONNECTED (succeeded) or 
+	 * CON_IC_STATUS_DISCONNECTED event: */
+	 
+	/* This is based on code found in gtk_dialog_run(): */
+	GDK_THREADS_LEAVE();
+	/* Run until g_main_loop_quit() is called by our signal handler. */
+	if (priv->loop)
+		g_main_loop_run (priv->loop);
+	GDK_THREADS_ENTER();
+
+	g_main_loop_unref (priv->loop);
+	priv->loop = NULL;
+
+	return priv->is_online;
+}
