@@ -13,8 +13,16 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with self library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ */
+
+/**
+ * TnyCombinedAccount:
+ * 
+ * A combined account
+ *
+ * free-function: g_object_unref
  */
 
 #include <config.h>
@@ -299,11 +307,11 @@ tny_combined_account_get_folders (TnyFolderStore *self, TnyList *list, TnyFolder
 }
 
 static void
-tny_combined_account_get_folders_async (TnyFolderStore *self, TnyList *list, TnyGetFoldersCallback callback, TnyFolderStoreQuery *query, TnyStatusCallback status_callback, gpointer user_data)
+tny_combined_account_get_folders_async (TnyFolderStore *self, TnyList *list, TnyFolderStoreQuery *query, TnyGetFoldersCallback callback, TnyStatusCallback status_callback, gpointer user_data)
 {
 	TnyCombinedAccountPriv *priv = TNY_COMBINED_ACCOUNT_GET_PRIVATE (self);
 
-	tny_folder_store_get_folders_async (TNY_FOLDER_STORE (priv->store_account), list, callback, query, status_callback, user_data);
+	tny_folder_store_get_folders_async (TNY_FOLDER_STORE (priv->store_account), list, query, callback, status_callback, user_data);
 }
 
 static void
@@ -369,19 +377,22 @@ on_subscription_changed_signal (TnyStoreAccount *sa, TnyFolder *folder, gpointer
 
 /**
  * tny_combined_account_new:
- * @ta: A #TnyTransportAccount instance
- * @sa: a #TnyStoreAccount instance
+ * @ta: A #TnyTransportAccount
+ * @sa: a #TnyStoreAccount
  * 
  * Create a decorator for @ta, in case the tny_transport_account_send is used,
- * and for @sa in case any other method of either TnyFolderStore, TnyAccount
- * or TnyStoreAccount is used.
+ * and for @sa in case a method of either #TnyFolderStore, #TnyAccount or 
+ * #TnyStoreAccount is used.
  *
  * Note though that you must not use instances created by this constructor for
  * either setting or getting members of the #TnyAccount type. You must get the
- * actual instances to read this from using either 
- * tny_combined_account_get_transport_account or tny_combined_account_get_store_account.
+ * actual instances to read these properties.
  *
- * Return value: A new account instance that decorates both @ta and @sa
+ * returns: (caller-owns): A new account instance that decorates both @ta and @sa
+ *
+ * since: 1.0
+ * complexity: high
+ * audience: application-developer
  **/
 TnyAccount *
 tny_combined_account_new (TnyTransportAccount *ta, TnyStoreAccount *sa)
@@ -401,12 +412,15 @@ tny_combined_account_new (TnyTransportAccount *ta, TnyStoreAccount *sa)
 
 /**
  * tny_combined_account_get_transport_account:
- * @self: a #TnyCombinedAccount instance
+ * @self: a #TnyCombinedAccount
  * 
  * Get the transport account that is being decorated by @self. You must unreference
  * the returned value if not needed anymore.
  *
- * Return value: the transport account being decorated
+ * returns: (caller-owns): the transport account in @self
+ * since: 1.0
+ * complexity: low
+ * audience: application-developer
  **/
 TnyTransportAccount* 
 tny_combined_account_get_transport_account (TnyCombinedAccount *self)
@@ -418,12 +432,15 @@ tny_combined_account_get_transport_account (TnyCombinedAccount *self)
 
 /**
  * tny_combined_account_get_store_account:
- * @self: a #TnyCombinedAccount instance
+ * @self: a #TnyCombinedAccount
  * 
  * Get the store account that is being decorated by @self. You must unreference
  * the returned value if not needed anymore.
  *
- * Return value: the store account being decorated
+ * returns: (caller-owns): the store account in @self
+ * since: 1.0
+ * complexity: low
+ * audience: application-developer
  **/
 TnyStoreAccount* 
 tny_combined_account_get_store_account (TnyCombinedAccount *self)
@@ -451,7 +468,7 @@ tny_combined_account_finalize (GObject *object)
 {
 	TnyCombinedAccountPriv *priv = TNY_COMBINED_ACCOUNT_GET_PRIVATE (object);
 
-	g_signal_handler_disconnect (G_OBJECT (priv->store_account),
+	g_signal_handler_disconnect (priv->store_account,
 			priv->subs_changed_signal);
 
 	g_object_unref (priv->store_account);
@@ -468,61 +485,61 @@ tny_combined_account_finalize (GObject *object)
 static void
 tny_store_account_init (TnyStoreAccountIface *klass)
 {
-	klass->subscribe_func = tny_combined_account_subscribe;
-	klass->unsubscribe_func = tny_combined_account_unsubscribe;
-	klass->find_folder_func = tny_combined_account_find_folder;
+	klass->subscribe= tny_combined_account_subscribe;
+	klass->unsubscribe= tny_combined_account_unsubscribe;
+	klass->find_folder= tny_combined_account_find_folder;
 }
 
 
 static void
 tny_transport_account_init (TnyTransportAccountIface *klass)
 {
-	klass->send_func = tny_combined_account_send;
+	klass->send= tny_combined_account_send;
 }
 
 
 static void
 tny_account_init (TnyAccountIface *klass)
 {
-	klass->get_connection_status_func = tny_combined_account_get_connection_status;
-	klass->set_id_func = tny_combined_account_set_id;
-	klass->set_name_func = tny_combined_account_set_name;
-	klass->set_secure_auth_mech_func = tny_combined_account_set_secure_auth_mech;
-	klass->set_proto_func = tny_combined_account_set_proto;
-	klass->set_user_func = tny_combined_account_set_user;
-	klass->set_hostname_func = tny_combined_account_set_hostname;
-	klass->set_port_func = tny_combined_account_set_port;
-	klass->set_url_string_func = tny_combined_account_set_url_string;
-	klass->set_pass_func_func = tny_combined_account_set_pass_func;
-	klass->set_forget_pass_func_func = tny_combined_account_set_forget_pass_func;
-	klass->get_pass_func_func = tny_combined_account_get_pass_func;
-	klass->get_forget_pass_func_func = tny_combined_account_get_forget_pass_func;
-	klass->get_id_func = tny_combined_account_get_id;
-	klass->get_name_func = tny_combined_account_get_name;
-	klass->get_secure_auth_mech_func = tny_combined_account_get_secure_auth_mech;
-	klass->get_proto_func = tny_combined_account_get_proto;
-	klass->get_user_func = tny_combined_account_get_user;
-	klass->get_hostname_func = tny_combined_account_get_hostname;
-	klass->get_port_func = tny_combined_account_get_port;
-	klass->get_url_string_func = tny_combined_account_get_url_string;
-	klass->get_account_type_func = tny_combined_account_get_account_type;
-	klass->cancel_func = tny_combined_account_cancel;
-	klass->matches_url_string_func = tny_combined_account_matches_url_string;
-	klass->start_operation_func = tny_combined_account_start_operation;
-	klass->stop_operation_func = tny_combined_account_stop_operation;
-	klass->is_ready_func = tny_combined_account_is_ready;
+	klass->get_connection_status= tny_combined_account_get_connection_status;
+	klass->set_id= tny_combined_account_set_id;
+	klass->set_name= tny_combined_account_set_name;
+	klass->set_secure_auth_mech= tny_combined_account_set_secure_auth_mech;
+	klass->set_proto= tny_combined_account_set_proto;
+	klass->set_user= tny_combined_account_set_user;
+	klass->set_hostname= tny_combined_account_set_hostname;
+	klass->set_port= tny_combined_account_set_port;
+	klass->set_url_string= tny_combined_account_set_url_string;
+	klass->set_pass_func= tny_combined_account_set_pass_func;
+	klass->set_forget_pass_func= tny_combined_account_set_forget_pass_func;
+	klass->get_pass_func= tny_combined_account_get_pass_func;
+	klass->get_forget_pass_func= tny_combined_account_get_forget_pass_func;
+	klass->get_id= tny_combined_account_get_id;
+	klass->get_name= tny_combined_account_get_name;
+	klass->get_secure_auth_mech= tny_combined_account_get_secure_auth_mech;
+	klass->get_proto= tny_combined_account_get_proto;
+	klass->get_user= tny_combined_account_get_user;
+	klass->get_hostname= tny_combined_account_get_hostname;
+	klass->get_port= tny_combined_account_get_port;
+	klass->get_url_string= tny_combined_account_get_url_string;
+	klass->get_account_type= tny_combined_account_get_account_type;
+	klass->cancel= tny_combined_account_cancel;
+	klass->matches_url_string= tny_combined_account_matches_url_string;
+	klass->start_operation= tny_combined_account_start_operation;
+	klass->stop_operation= tny_combined_account_stop_operation;
+	klass->is_ready= tny_combined_account_is_ready;
 }
 
 
 static void
 tny_folder_store_init (TnyFolderStoreIface *klass)
 {
-	klass->remove_folder_func = tny_combined_account_remove_folder;
-	klass->create_folder_func = tny_combined_account_create_folder;
-	klass->get_folders_func = tny_combined_account_get_folders;
-	klass->get_folders_async_func = tny_combined_account_get_folders_async;
-	klass->add_observer_func = tny_combined_account_add_observer;
-	klass->remove_observer_func = tny_combined_account_remove_observer;
+	klass->remove_folder= tny_combined_account_remove_folder;
+	klass->create_folder= tny_combined_account_create_folder;
+	klass->get_folders= tny_combined_account_get_folders;
+	klass->get_folders_async= tny_combined_account_get_folders_async;
+	klass->add_observer= tny_combined_account_add_observer;
+	klass->remove_observer= tny_combined_account_remove_observer;
 }
 
 static void
