@@ -13,11 +13,20 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with self library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
+/**
+ * TnyMergeFolder:
+ *
+ * A #TnyFolder that merges other folders together 
+ *
+ * free-function: g_object_unref
+ **/
+
 #include <config.h>
+
 #include <string.h>
 #include <glib.h>
 #include <glib/gi18n-lib.h>
@@ -26,6 +35,7 @@
 #include <tny-error.h>
 #include <tny-simple-list.h>
 #include <tny-folder-observer.h>
+#include <tny-noop-lockable.h>
 
 static GObjectClass *parent_class = NULL;
 
@@ -34,9 +44,11 @@ typedef struct _TnyMergeFolderPriv TnyMergeFolderPriv;
 struct _TnyMergeFolderPriv
 {
 	gchar *id, *name;
-	TnyList *mothers, *observers;
+	TnyList *mothers;
 	GStaticRecMutex *lock;
 	TnyFolderType folder_type;
+	GList *obs;
+	TnyLockable *ui_locker;
 };
 
 #define TNY_MERGE_FOLDER_GET_PRIVATE(o) \
@@ -48,33 +60,37 @@ static void
 notify_folder_observers_about (TnyFolder *self, TnyFolderChange *change)
 {
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
-	TnyList *copy = NULL;
-	TnyIterator *iter;
+	GList *list, *list_iter;
 
 	g_static_rec_mutex_lock (priv->lock);
-	if (!priv->observers) {
+	if (!priv->obs) {
 		g_static_rec_mutex_unlock (priv->lock);
 		return;
 	}
-	copy = tny_list_copy (priv->observers);
+	list = g_list_copy (priv->obs);
+	list_iter = list;
 	g_static_rec_mutex_unlock (priv->lock);
 
-	iter = tny_list_create_iterator (copy);
-	while (!tny_iterator_is_done (iter))
+
+	while (list_iter)
 	{
-		TnyFolderObserver *observer = TNY_FOLDER_OBSERVER (tny_iterator_get_current (iter));
-		/* TNY TODO: tny_lockable_lock (ui_lock); */
+		TnyFolderObserver *observer = TNY_FOLDER_OBSERVER (list_iter->data);
+
+		/* We don't need to hold the ui-lock here, because this is called
+		 * from the update function, which is guaranteed to happen in the
+		 * UI context already (the TnyFolder layer takes care of this) */
+
 		tny_folder_observer_update (observer, change);
-		/* TNY TODO: tny_lockable_unlock (ui_lock); */
-		g_object_unref (G_OBJECT (observer));
-		tny_iterator_next (iter);
+		list_iter = g_list_next (list_iter);
 	}
 
-	g_object_unref (iter);
-	g_object_unref (copy);
+	g_list_free (list);
+
+	return;
 }
 
 
+
 static void
 tny_merge_folder_remove_msg (TnyFolder *self, TnyHeader *header, GError **err)
 {
@@ -86,6 +102,19 @@ tny_merge_folder_remove_msg (TnyFolder *self, TnyHeader *header, GError **err)
 	return;
 }
 
+static void
+tny_merge_folder_remove_msgs (TnyFolder *self, TnyList *headers, GError **err)
+{
+	TnyIterator *iter = tny_list_create_iterator (headers);
+	while (!tny_iterator_is_done (iter)) {
+		TnyHeader *cur = (TnyHeader *) tny_iterator_get_current (iter);
+		tny_merge_folder_remove_msg (self, cur, err);
+		/* TODO: check for err */
+		tny_iterator_next (iter);
+	}
+	g_object_unref (iter);
+}
+
 static void
 tny_merge_folder_add_msg_async (TnyFolder *self, TnyMsg *msg, TnyFolderCallback callback, TnyStatusCallback status_callback, gpointer user_data)
 {
@@ -94,8 +123,8 @@ tny_merge_folder_add_msg_async (TnyFolder *self, TnyMsg *msg, TnyFolderCallback
 
 	if (callback) {
 		GError *err = NULL;
-		g_set_error (&err, TNY_FOLDER_ERROR, 
-			TNY_FOLDER_ERROR_ADD_MSG,
+		g_set_error (&err, TNY_SERVICE_ERROR, 
+			TNY_SERVICE_ERROR_UNSUPPORTED,
 			"tny_merge_folder_add_msg not implemented: add it to the mother "
 			"folder instead. This problem indicates a bug in the software.");
 		callback (self, TRUE, err, user_data);
@@ -110,8 +139,8 @@ tny_merge_folder_add_msg (TnyFolder *self, TnyMsg *msg, GError **err)
 	g_warning ("tny_merge_folder_add_msg not implemented: "
 		   "add it to the mother folder instead\n");
 
-	g_set_error (err, TNY_FOLDER_ERROR, 
-		TNY_FOLDER_ERROR_ADD_MSG,
+	g_set_error (err, TNY_SERVICE_ERROR, 
+		TNY_SERVICE_ERROR_UNSUPPORTED,
 		"tny_merge_folder_add_msg not implemented: add it to the mother "
 		"folder instead. This problem indicates a bug in the software.");
 }
@@ -121,10 +150,13 @@ tny_merge_folder_sync (TnyFolder *self, gboolean expunge, GError **err)
 {
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 	TnyIterator *iter;
+	TnyList *copy;
 
 	g_static_rec_mutex_lock (priv->lock);
+	copy = tny_list_copy (priv->mothers);
+	g_static_rec_mutex_unlock (priv->lock);
 
-	iter = tny_list_create_iterator (priv->mothers);
+	iter = tny_list_create_iterator (copy);
 	while (!tny_iterator_is_done (iter))
 	{
 		GError *new_err = NULL;
@@ -142,8 +174,8 @@ tny_merge_folder_sync (TnyFolder *self, gboolean expunge, GError **err)
 		tny_iterator_next (iter);
 	}
 	g_object_unref (iter);
+	g_object_unref (copy);
 
-	g_static_rec_mutex_unlock (priv->lock);
 
 	return;
 }
@@ -171,7 +203,7 @@ sync_async_destroyer (gpointer thr_user_data)
 	TnyFolder *self = info->self;
 
 	/* thread reference */
-	g_object_unref (G_OBJECT (self));
+	g_object_unref (self);
 	if (info->err)
 		g_error_free (info->err);
 
@@ -184,11 +216,12 @@ static gboolean
 sync_async_callback (gpointer thr_user_data)
 {
 	SyncFolderInfo *info = thr_user_data;
+	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (info->self);
 
 	if (info->callback) {
-		/* TNY TODO: tny_lockable_lock (ui_lock); */
+		tny_lockable_lock (priv->ui_locker);
 		info->callback (info->self, info->cancelled, info->err, info->user_data);
-		/* TNY TODO: tny_lockable_unlock (ui_lock); */
+		tny_lockable_unlock (priv->ui_locker);
 	}
 
 	return FALSE;
@@ -203,12 +236,15 @@ sync_async_thread (gpointer thr_user_data)
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 	TnyIterator *iter;
 	GError *err = NULL;
+	TnyList *copy;
 
 	g_static_rec_mutex_lock (priv->lock);
+	copy = tny_list_copy (priv->mothers);
+	g_static_rec_mutex_unlock (priv->lock);
 
 	info->cancelled = FALSE;
 
-	iter = tny_list_create_iterator (priv->mothers);
+	iter = tny_list_create_iterator (copy);
 	while (!tny_iterator_is_done (iter))
 	{
 		TnyFolder *cur = TNY_FOLDER (tny_iterator_get_current (iter));
@@ -225,12 +261,12 @@ sync_async_thread (gpointer thr_user_data)
 		g_object_unref (cur);
 		tny_iterator_next (iter);
 	}
+
 	g_object_unref (iter);
+	g_object_unref (copy);
 
 	info->err = NULL;
 
-	g_static_rec_mutex_unlock (priv->lock);
-
 	if (info->callback)
 	{
 		if (info->depth > 0)
@@ -243,7 +279,7 @@ sync_async_thread (gpointer thr_user_data)
 			sync_async_destroyer (info);
 		}
 	} else /* Thread reference */
-		g_object_unref (G_OBJECT (self));
+		g_object_unref (self);
 
 	g_thread_exit (NULL);
 
@@ -267,7 +303,7 @@ tny_merge_folder_sync_async (TnyFolder *self, gboolean expunge, TnyFolderCallbac
 	info->depth = g_main_depth ();
 
 	/* thread reference */
-	g_object_ref (G_OBJECT (self));
+	g_object_ref (self);
 
 	thread = g_thread_create (sync_async_thread, info, FALSE, NULL);
 
@@ -320,6 +356,7 @@ tny_merge_folder_get_msg (TnyFolder *self, TnyHeader *header, GError **err)
 {
 	TnyFolder *fol = tny_header_get_folder (header);
 	TnyMsg *retval = tny_folder_get_msg (fol, header, err);
+
 	g_object_unref (fol);
 
 	return retval;
@@ -332,10 +369,13 @@ tny_merge_folder_find_msg (TnyFolder *self, const gchar *url_string, GError **er
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 	TnyIterator *iter;
 	TnyMsg *retval = NULL;
+	TnyList *copy;
 
 	g_static_rec_mutex_lock (priv->lock);
+	copy = tny_list_copy (priv->mothers);
+	g_static_rec_mutex_unlock (priv->lock);
 
-	iter = tny_list_create_iterator (priv->mothers);
+	iter = tny_list_create_iterator (copy);
 	while (!tny_iterator_is_done (iter) && !retval)
 	{
 		TnyFolder *cur = TNY_FOLDER (tny_iterator_get_current (iter));
@@ -345,12 +385,12 @@ tny_merge_folder_find_msg (TnyFolder *self, const gchar *url_string, GError **er
 	}
 
 	g_object_unref (iter);
+	g_object_unref (copy);
 
-	g_static_rec_mutex_unlock (priv->lock);
 
 	if (!retval)
-		g_set_error (err, TNY_FOLDER_ERROR, 
-				TNY_FOLDER_ERROR_GET_MSG,
+		g_set_error (err, TNY_SERVICE_ERROR, 
+				TNY_SERVICE_ERROR_NO_SUCH_MESSAGE,
 				"Message not found");
 
 	return retval;
@@ -375,9 +415,10 @@ get_msg_async_destroyer (gpointer thr_user_data)
 	GetMsgInfo *info = (GetMsgInfo *) thr_user_data;
 
 	/* thread reference */
-	g_object_unref (G_OBJECT (info->self));
+	g_object_unref (info->self);
+
 	if (info->msg)
-		g_object_unref (G_OBJECT (info->msg));
+		g_object_unref (info->msg);
 
 	if (info->err)
 		g_error_free (info->err);
@@ -389,13 +430,13 @@ static gboolean
 get_msg_async_callback (gpointer thr_user_data)
 {
 	GetMsgInfo *info = (GetMsgInfo *) thr_user_data;
+	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (info->self);
 
 	if (info->callback) { 
 		/* TNY TODO: the cancelled field */
-
-		/* TNY TODO: tny_lockable_lock (ui_lock); */
+		tny_lockable_lock (priv->ui_locker);
 		info->callback (info->self, FALSE, info->msg, info->err, info->user_data);
-		/* TNY TODO: tny_lockable_unlock (ui_lock); */
+		tny_lockable_unlock (priv->ui_locker);
 	}
 
 	return FALSE;
@@ -412,11 +453,11 @@ get_msg_async_thread (gpointer thr_user_data)
 	if (info->err != NULL)
 	{
 		if (info->msg && G_IS_OBJECT (info->msg))
-			g_object_unref (G_OBJECT (info->msg));
+			g_object_unref (info->msg);
 		info->msg = NULL;
 	}
 
-	g_object_unref (G_OBJECT (info->header));
+	g_object_unref (info->header);
 
 	if (info->callback)
 	{
@@ -430,7 +471,7 @@ get_msg_async_thread (gpointer thr_user_data)
 			get_msg_async_destroyer (info);
 		}
 	} else /* thread reference */
-		g_object_unref (G_OBJECT (info->self));
+		g_object_unref (info->self);
 
 	g_thread_exit (NULL);
 
@@ -498,11 +539,12 @@ static gboolean
 get_headers_async_callback (gpointer thr_user_data)
 {
 	GetHeadersFolderInfo *info = thr_user_data;
+	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (info->self);
 
 	if (info->callback) {
-		/* TNY TODO: tny_lockable_lock (ui_lock); */
+		tny_lockable_lock (priv->ui_locker);
 		info->callback (info->self, info->cancelled, info->headers, info->err, info->user_data);
-		/* TNY TODO: tny_lockable_unlock (ui_lock); */
+		tny_lockable_unlock (priv->ui_locker);
 	}
 
 	return FALSE;
@@ -517,12 +559,15 @@ get_headers_async_thread (gpointer thr_user_data)
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 	TnyIterator *iter;
 	GError *err = NULL;
+	TnyList *copy;
 
 	g_static_rec_mutex_lock (priv->lock);
+	copy = tny_list_copy (priv->mothers);
+	g_static_rec_mutex_unlock (priv->lock);
 
 	info->cancelled = FALSE;
 
-	iter = tny_list_create_iterator (priv->mothers);
+	iter = tny_list_create_iterator (copy);
 	while (!tny_iterator_is_done (iter))
 	{
 		TnyFolder *cur = TNY_FOLDER (tny_iterator_get_current (iter));
@@ -539,12 +584,12 @@ get_headers_async_thread (gpointer thr_user_data)
 		g_object_unref (cur);
 		tny_iterator_next (iter);
 	}
+
 	g_object_unref (iter);
+	g_object_unref (copy);
 
 	info->err = NULL;
 
-	g_static_rec_mutex_unlock (priv->lock);
-
 	if (info->callback)
 	{
 		if (info->depth > 0)
@@ -597,10 +642,13 @@ tny_merge_folder_get_headers (TnyFolder *self, TnyList *headers, gboolean refres
 {
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 	TnyIterator *iter;
+	TnyList *copy;
 
 	g_static_rec_mutex_lock (priv->lock);
+	copy = tny_list_copy (priv->mothers);
+	g_static_rec_mutex_unlock (priv->lock);
 
-	iter = tny_list_create_iterator (priv->mothers);
+	iter = tny_list_create_iterator (copy);
 
 	while (!tny_iterator_is_done (iter))
 	{
@@ -619,8 +667,7 @@ tny_merge_folder_get_headers (TnyFolder *self, TnyList *headers, gboolean refres
 	}
 
 	g_object_unref (iter);
-
-	g_static_rec_mutex_unlock (priv->lock);
+	g_object_unref (copy);
 
 }
 
@@ -642,16 +689,24 @@ tny_merge_folder_get_id (TnyFolder *self)
 		TnyIterator *iter;
 		GString *ids = g_string_new ("");
 		gboolean first = TRUE;
+		TnyList *copy;
 
 		g_static_rec_mutex_lock (priv->lock);
+		copy = tny_list_copy (priv->mothers);
+		g_static_rec_mutex_unlock (priv->lock);
 
-		iter = tny_list_create_iterator (priv->mothers);
+		iter = tny_list_create_iterator (copy);
 
 		while (!tny_iterator_is_done (iter))
 		{
 			TnyFolder *cur = TNY_FOLDER (tny_iterator_get_current (iter));
+			TnyAccount *account = tny_folder_get_account (cur);
 			if (!first)
 				g_string_append_c (ids, '&');
+			if (TNY_IS_ACCOUNT (account)) {
+				g_string_append (ids, tny_account_get_id (account));
+				g_string_append_c (ids, '+');
+			}
 			g_string_append (ids, tny_folder_get_id (cur));
 			g_object_unref (cur);
 			first = FALSE;
@@ -660,17 +715,17 @@ tny_merge_folder_get_id (TnyFolder *self)
 
 		priv->id = ids->str;
 		g_string_free (ids, FALSE);
-		g_object_unref (iter);
 
-		g_static_rec_mutex_unlock (priv->lock);
+		g_object_unref (iter);
+		g_object_unref (copy);
 
 	}
 
 	/* The get_id_func() DBC contract does not allow this to be NULL or "": */
 	if ( (priv->id == NULL) || (strlen (priv->id) == 0)) {
-		priv->id = g_strdup ("unknown_mergefolder");	
+		priv->id = g_strdup ("unknown_mergefolder");
 	}
-	
+
 	return priv->id;
 }
 
@@ -686,15 +741,16 @@ tny_merge_folder_get_account (TnyFolder *self)
 
 /**
  * tny_merge_folder_set_folder_type
- * @self: a #TnyMergeFolder object
- * @folder_type: the new folder type
+ * @self: a #TnyMergeFolder
+ * @folder_type: the new folder
  * 
- * Set the folder type of the TnyMergeFolder. The default is TNY_FOLDER_TYPE_MERGE
+ * Set the folder type of the #TnyMergeFolder. The default is TNY_FOLDER_TYPE_MERGE
  * but you can change this to any folder type. It will not affect anything except
- * that get_folder_type() will return the new type now.
+ * that tny_folder_get_folder_type() will return the new type.
  *
+ * since: 1.0
+ * audience: application-developer
  **/
-
 void
 tny_merge_folder_set_folder_type (TnyMergeFolder* self, TnyFolderType folder_type)
 {
@@ -717,10 +773,13 @@ tny_merge_folder_get_all_count (TnyFolder *self)
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 	TnyIterator *iter;
 	guint total = 0;
+	TnyList *copy;
 
 	g_static_rec_mutex_lock (priv->lock);
+	copy = tny_list_copy (priv->mothers);
+	g_static_rec_mutex_unlock (priv->lock);
 
-	iter = tny_list_create_iterator (priv->mothers);
+	iter = tny_list_create_iterator (copy);
 
 	while (!tny_iterator_is_done (iter))
 	{
@@ -731,8 +790,8 @@ tny_merge_folder_get_all_count (TnyFolder *self)
 	}
 
 	g_object_unref (iter);
+	g_object_unref (copy);
 
-	g_static_rec_mutex_unlock (priv->lock);
 
 	return total;
 }
@@ -743,10 +802,13 @@ tny_merge_folder_get_unread_count (TnyFolder *self)
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 	TnyIterator *iter;
 	guint total = 0;
+	TnyList *copy;
 
 	g_static_rec_mutex_lock (priv->lock);
+	copy = tny_list_copy (priv->mothers);
+	g_static_rec_mutex_unlock (priv->lock);
 
-	iter = tny_list_create_iterator (priv->mothers);
+	iter = tny_list_create_iterator (copy);
 
 	while (!tny_iterator_is_done (iter))
 	{
@@ -757,8 +819,7 @@ tny_merge_folder_get_unread_count (TnyFolder *self)
 	}
 
 	g_object_unref (iter);
-
-	g_static_rec_mutex_unlock (priv->lock);
+	g_object_unref (copy);
 
 	return total;
 }
@@ -770,10 +831,13 @@ tny_merge_folder_get_local_size (TnyFolder *self)
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 	TnyIterator *iter;
 	guint total = 0;
+	TnyList *copy;
 
 	g_static_rec_mutex_lock (priv->lock);
+	copy = tny_list_copy (priv->mothers);
+	g_static_rec_mutex_unlock (priv->lock);
 
-	iter = tny_list_create_iterator (priv->mothers);
+	iter = tny_list_create_iterator (copy);
 
 	while (!tny_iterator_is_done (iter))
 	{
@@ -784,8 +848,7 @@ tny_merge_folder_get_local_size (TnyFolder *self)
 	}
 
 	g_object_unref (iter);
-
-	g_static_rec_mutex_unlock (priv->lock);
+	g_object_unref (copy);
 
 	return total;
 }
@@ -802,10 +865,13 @@ tny_merge_folder_refresh (TnyFolder *self, GError **err)
 {
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 	TnyIterator *iter;
+	TnyList *copy;
 
 	g_static_rec_mutex_lock (priv->lock);
+	copy = tny_list_copy (priv->mothers);
+	g_static_rec_mutex_unlock (priv->lock);
 
-	iter = tny_list_create_iterator (priv->mothers);
+	iter = tny_list_create_iterator (copy);
 	while (!tny_iterator_is_done (iter))
 	{
 		GError *new_err = NULL;
@@ -823,8 +889,7 @@ tny_merge_folder_refresh (TnyFolder *self, GError **err)
 	}
 
 	g_object_unref (iter);
-
-	g_static_rec_mutex_unlock (priv->lock);
+	g_object_unref (copy);
 
 	return;
 }
@@ -849,7 +914,7 @@ refresh_async_destroyer (gpointer thr_user_data)
 	TnyFolder *self = info->self;
 
 	/* thread reference */
-	g_object_unref (G_OBJECT (self));
+	g_object_unref (self);
 	if (info->err)
 		g_error_free (info->err);
 
@@ -862,11 +927,12 @@ static gboolean
 refresh_async_callback (gpointer thr_user_data)
 {
 	RefreshFolderInfo *info = thr_user_data;
+	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (info->self);
 
 	if (info->callback) {
-		/* TNY TODO: tny_lockable_lock (ui_lock); */
+		tny_lockable_lock (priv->ui_locker);
 		info->callback (info->self, info->cancelled, info->err, info->user_data);
-		/* TNY TODO: tny_lockable_unlock (ui_lock); */
+		tny_lockable_unlock (priv->ui_locker);
 	}
 
 	/* TNY TODO: trigger this change notification
@@ -894,12 +960,15 @@ refresh_async_thread (gpointer thr_user_data)
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 	TnyIterator *iter;
 	GError *err = NULL;
+	TnyList *copy;
 
 	g_static_rec_mutex_lock (priv->lock);
+	copy = tny_list_copy (priv->mothers);
+	g_static_rec_mutex_unlock (priv->lock);
 
 	info->cancelled = FALSE;
 
-	iter = tny_list_create_iterator (priv->mothers);
+	iter = tny_list_create_iterator (copy);
 	while (!tny_iterator_is_done (iter))
 	{
 		TnyFolder *cur = TNY_FOLDER (tny_iterator_get_current (iter));
@@ -916,11 +985,12 @@ refresh_async_thread (gpointer thr_user_data)
 		g_object_unref (cur);
 		tny_iterator_next (iter);
 	}
+
 	g_object_unref (iter);
+	g_object_unref (copy);
 
 	info->err = NULL;
 
-	g_static_rec_mutex_unlock (priv->lock);
 
 	if (info->callback)
 	{
@@ -934,7 +1004,7 @@ refresh_async_thread (gpointer thr_user_data)
 			refresh_async_destroyer (info);
 		}
 	} else /* Thread reference */
-		g_object_unref (G_OBJECT (self));
+		g_object_unref (self);
 
 	g_thread_exit (NULL);
 
@@ -957,7 +1027,7 @@ tny_merge_folder_refresh_async (TnyFolder *self, TnyFolderCallback callback, Tny
 	info->depth = g_main_depth ();
 
 	/* thread reference */
-	g_object_ref (G_OBJECT (self));
+	g_object_ref (self);
 
 	thread = g_thread_create (refresh_async_thread, info, FALSE, NULL);
 
@@ -968,11 +1038,8 @@ tny_merge_folder_refresh_async (TnyFolder *self, TnyFolderCallback callback, Tny
 static void
 tny_merge_folder_transfer_msgs (TnyFolder *self, TnyList *header_list, TnyFolder *folder_dst, gboolean delete_originals, GError **err)
 {
-	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 	TnyIterator *iter;
 
-	g_static_rec_mutex_lock (priv->lock);
-
 	iter = tny_list_create_iterator (header_list);
 
 	while (!tny_iterator_is_done (iter))
@@ -1000,8 +1067,6 @@ tny_merge_folder_transfer_msgs (TnyFolder *self, TnyList *header_list, TnyFolder
 
 	g_object_unref (iter);
 
-	g_static_rec_mutex_unlock (priv->lock);
-
 	return;
 }
 
@@ -1024,9 +1089,9 @@ transfer_msgs_async_destroyer (gpointer thr_user_data)
 	TransferMsgsInfo *info = thr_user_data;
 
 	/* thread reference */
-	g_object_unref (G_OBJECT (info->self));
-	g_object_unref (G_OBJECT (info->folder_dst));
-	g_object_unref (G_OBJECT (info->header_list));
+	g_object_unref (info->self);
+	g_object_unref (info->folder_dst);
+	g_object_unref (info->header_list);
 
 	if (info->err)
 		g_error_free (info->err);
@@ -1040,13 +1105,13 @@ static gboolean
 transfer_msgs_async_callback (gpointer thr_user_data)
 {
 	TransferMsgsInfo *info = thr_user_data;
+	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (info->self);
 
 	if (info->callback) {
 		/* TNY TODO: the cancelled field */
-
-		/* TNY TODO: tny_lockable_lock (ui_lock); */
+		tny_lockable_lock (priv->ui_locker);
 		info->callback (info->self, FALSE, info->err, info->user_data);
-		/* TNY TODO: tny_lockable_unlock (ui_lock); */
+		tny_lockable_unlock (priv->ui_locker);
 	}
 
 	return FALSE;
@@ -1057,12 +1122,8 @@ static gpointer
 transfer_msgs_async_thread (gpointer thr_user_data)
 {
 	TransferMsgsInfo *info = thr_user_data;
-	TnyFolder *self = info->self;
-	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 
-	g_static_rec_mutex_lock (priv->lock);
 	tny_merge_folder_transfer_msgs (info->self, info->header_list, info->folder_dst, info->delete_originals, &info->err);
-	g_static_rec_mutex_unlock (priv->lock);
 
 	if (info->callback)
 	{
@@ -1076,9 +1137,9 @@ transfer_msgs_async_thread (gpointer thr_user_data)
 			transfer_msgs_async_destroyer (info);
 		}
 	} else  { /* Thread reference */
-		g_object_unref (G_OBJECT (info->self));
-		g_object_unref (G_OBJECT (info->folder_dst));
-		g_object_unref (G_OBJECT (info->header_list));
+		g_object_unref (info->self);
+		g_object_unref (info->folder_dst);
+		g_object_unref (info->header_list);
 	}
 
 	g_thread_exit (NULL);
@@ -1104,9 +1165,9 @@ tny_merge_folder_transfer_msgs_async (TnyFolder *self, TnyList *header_list, Tny
 	info->err = NULL;
 
 	/* thread reference */
-	g_object_ref (G_OBJECT (self));
-	g_object_ref (G_OBJECT (folder_dst));
-	g_object_ref (G_OBJECT (header_list));
+	g_object_ref (self);
+	g_object_ref (folder_dst);
+	g_object_ref (header_list);
 
 	thread = g_thread_create (transfer_msgs_async_thread, info, FALSE, NULL);
 
@@ -1121,10 +1182,13 @@ tny_merge_folder_copy (TnyFolder *self, TnyFolderStore *into, const gchar *new_n
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 	TnyIterator *iter;
 	TnyFolder *nfol = NULL;
+	TnyList *copy;
 
 	g_static_rec_mutex_lock (priv->lock);
+	copy = tny_list_copy (priv->mothers);
+	g_static_rec_mutex_unlock (priv->lock);
 
-	iter = tny_list_create_iterator (priv->mothers);
+	iter = tny_list_create_iterator (copy);
 	while (!tny_iterator_is_done (iter))
 	{
 		TnyFolder *folder = TNY_FOLDER (tny_iterator_get_current (iter));
@@ -1165,8 +1229,7 @@ tny_merge_folder_copy (TnyFolder *self, TnyFolderStore *into, const gchar *new_n
 	}
 
 	g_object_unref (iter);
-
-	g_static_rec_mutex_unlock (priv->lock);
+	g_object_unref (copy);
 
 	return nfol;
 }
@@ -1176,10 +1239,13 @@ tny_merge_folder_poke_status (TnyFolder *self)
 {
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 	TnyIterator *iter;
+	TnyList *copy;
 
 	g_static_rec_mutex_lock (priv->lock);
+	copy = tny_list_copy (priv->mothers);
+	g_static_rec_mutex_unlock (priv->lock);
 
-	iter = tny_list_create_iterator (priv->mothers);
+	iter = tny_list_create_iterator (copy);
 	while (!tny_iterator_is_done (iter))
 	{
 		TnyFolder *cur = TNY_FOLDER (tny_iterator_get_current (iter));
@@ -1189,24 +1255,31 @@ tny_merge_folder_poke_status (TnyFolder *self)
 	}
 
 	g_object_unref (iter);
-
-	g_static_rec_mutex_unlock (priv->lock);
+	g_object_unref (copy);
 
 	return;
 }
 
+
+static void 
+notify_observer_del (gpointer user_data, GObject *observer)
+{
+	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (user_data);
+	g_static_rec_mutex_lock (priv->lock);
+	priv->obs = g_list_remove (priv->obs, observer);
+	g_static_rec_mutex_unlock (priv->lock);
+}
+
 static void
 tny_merge_folder_add_observer (TnyFolder *self, TnyFolderObserver *observer)
 {
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 
 	g_static_rec_mutex_lock (priv->lock);
-
-	if (!priv->observers)
-		priv->observers = tny_simple_list_new ();
-
-	tny_list_prepend (priv->observers, G_OBJECT (observer));
-
+	if (!g_list_find (priv->obs, observer)) {
+		priv->obs = g_list_prepend (priv->obs, observer);
+		g_object_weak_ref (G_OBJECT (observer), notify_observer_del, self);
+	}
 	g_static_rec_mutex_unlock (priv->lock);
 
 	return;
@@ -1216,11 +1289,23 @@ static void
 tny_merge_folder_remove_observer (TnyFolder *self, TnyFolderObserver *observer)
 {
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
+	GList *found = NULL;
+
+	g_assert (TNY_IS_FOLDER_OBSERVER (observer));
 
 	g_static_rec_mutex_lock (priv->lock);
 
-	if (priv->observers)
-		tny_list_remove (priv->observers, G_OBJECT (observer));
+	if (!priv->obs) {
+		g_static_rec_mutex_unlock (priv->lock);
+		return;
+	}
+
+	found = g_list_find (priv->obs, observer);
+	if (found) {
+		priv->obs = g_list_remove_link (priv->obs, found);
+		g_object_weak_unref (found->data, notify_observer_del, self);
+		g_list_free (found);
+	}
 
 	g_static_rec_mutex_unlock (priv->lock);
 
@@ -1243,10 +1328,13 @@ tny_merge_folder_get_stats (TnyFolder *self)
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 	TnyIterator *iter;
 	gint total_size = 0;
+	TnyList *copy;
 
 	g_static_rec_mutex_lock (priv->lock);
+	copy = tny_list_copy (priv->mothers);
+	g_static_rec_mutex_unlock (priv->lock);
 
-	iter = tny_list_create_iterator (priv->mothers);
+	iter = tny_list_create_iterator (copy);
 	while (!tny_iterator_is_done (iter))
 	{
 		TnyFolder *cur = TNY_FOLDER (tny_iterator_get_current (iter));
@@ -1258,8 +1346,7 @@ tny_merge_folder_get_stats (TnyFolder *self)
 	}
 
 	g_object_unref (iter);
-
-	g_static_rec_mutex_unlock (priv->lock);
+	g_object_unref (copy);
 
 	/* TNY TODO: update unread, all_count and local_size here ! */
 
@@ -1271,10 +1358,7 @@ tny_merge_folder_get_stats (TnyFolder *self)
 static gchar*
 tny_merge_folder_get_url_string (TnyFolder *self)
 {
-	g_warning ("tny_merge_folder_get_url_string not reliable. "
-		   "Please don't use this functionality\n");
-
-	return "not://implemented";
+	return g_strdup_printf ("merge://%s", tny_folder_get_id (self));
 }
 
 static TnyFolderCaps
@@ -1297,15 +1381,33 @@ tny_merge_folder_update (TnyFolderObserver *self, TnyFolderChange *change)
 	 * in the actual underlaying folder she should rather observe that
 	 * particular folder). We also do not propagate folder renames
 	 * because these do not rename the merge folder. */
+
+	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 	TnyFolderChange* new_change = tny_folder_change_new (TNY_FOLDER (self));
 	TnyList *list;
 	TnyIterator *iter;
+	gint total = 0, unread = 0;
+	TnyList *copy;
 
-	if (tny_folder_change_get_changed (change) & TNY_FOLDER_CHANGE_CHANGED_ALL_COUNT)
-		tny_folder_change_set_new_all_count (new_change, tny_folder_change_get_new_all_count (change));
+	g_static_rec_mutex_lock (priv->lock);
+	copy = tny_list_copy (priv->mothers);
+	g_static_rec_mutex_unlock (priv->lock);
 
-	if (tny_folder_change_get_changed (change) & TNY_FOLDER_CHANGE_CHANGED_UNREAD_COUNT)
-		tny_folder_change_set_new_unread_count (new_change, tny_folder_change_get_new_unread_count (change));
+	iter = tny_list_create_iterator (copy);
+	while (!tny_iterator_is_done (iter))
+	{
+		TnyFolder *cur = TNY_FOLDER (tny_iterator_get_current (iter));
+		total += tny_folder_get_all_count (cur);
+		unread += tny_folder_get_unread_count (cur);
+		g_object_unref (cur);
+		tny_iterator_next (iter);
+	}
+
+	g_object_unref (iter);
+	g_object_unref (copy);
+
+	tny_folder_change_set_new_all_count (new_change, total);
+	tny_folder_change_set_new_unread_count (new_change, unread);
 
 	if (tny_folder_change_get_changed (change) & TNY_FOLDER_CHANGE_CHANGED_ADDED_HEADERS)
 	{
@@ -1352,10 +1454,13 @@ tny_merge_folder_update (TnyFolderObserver *self, TnyFolderChange *change)
 
 /**
  * tny_merge_folder_add_folder:
- * @self: a #TnyMergeFolder object
- * @folder: a #TnyFolder object 
+ * @self: a #TnyMergeFolder
+ * @folder: a #TnyFolder 
  *
- * Add @folder to the list of folders that will be merged by @self.
+ * Add @folder to the list of folders that are merged by @self.
+ *
+ * since: 1.0
+ * audience: application-developer
  **/
 void 
 tny_merge_folder_add_folder (TnyMergeFolder *self, TnyFolder *folder)
@@ -1364,11 +1469,10 @@ tny_merge_folder_add_folder (TnyMergeFolder *self, TnyFolder *folder)
 
 	/* TODO: register @self as observer of @folder and proxy the
 	 * events through to observers of @self. */
-	
+
 	g_static_rec_mutex_lock (priv->lock);
 
 	tny_list_prepend (priv->mothers, G_OBJECT (folder));
-
 	tny_folder_add_observer (folder, TNY_FOLDER_OBSERVER (self));
 
 	g_static_rec_mutex_unlock (priv->lock);
@@ -1378,11 +1482,14 @@ tny_merge_folder_add_folder (TnyMergeFolder *self, TnyFolder *folder)
 
 /**
  * tny_merge_folder_remove_folder:
- * @self: a #TnyMergeFolder object
- * @folder: a #TnyFolder object 
+ * @self: a #TnyMergeFolder
+ * @folder: a #TnyFolder 
  *
- * Removes @folder from the list of folders that will be merged by
+ * Removes @folder from the list of folders that are merged by
  * @self.
+ *
+ * since: 1.0
+ * audience: application-developer
  **/
 void 
 tny_merge_folder_remove_folder (TnyMergeFolder *self, TnyFolder *folder)
@@ -1393,11 +1500,10 @@ tny_merge_folder_remove_folder (TnyMergeFolder *self, TnyFolder *folder)
 	g_return_if_fail (TNY_IS_FOLDER (folder));
 
 	priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
-	
+
 	g_static_rec_mutex_lock (priv->lock);
 
 	tny_folder_remove_observer (folder, TNY_FOLDER_OBSERVER (self));
-
 	tny_list_remove (priv->mothers, G_OBJECT (folder));
 
 	g_static_rec_mutex_unlock (priv->lock);
@@ -1407,25 +1513,31 @@ tny_merge_folder_remove_folder (TnyMergeFolder *self, TnyFolder *folder)
 
 /**
  * tny_merge_folder_get_folders:
- * @self: a #TnyMergeFolder object
- * @list: a #TnyList to fillup
+ * @self: a #TnyMergeFolder
+ * @list: a #TnyList to which the folders will be prepended
  *
- * Returns the folders that are merged in the @self merge folder.
+ * Get the folders that are merged in the merge folder @self.
+ *
+ * since: 1.0
+ * audience: application-developer
  **/
 void 
 tny_merge_folder_get_folders (TnyMergeFolder *self, TnyList *list)
 {
 	TnyMergeFolderPriv *priv;
 	TnyIterator *iter;
+	TnyList *copy;
 
 	g_return_if_fail (TNY_IS_MERGE_FOLDER (self));
 	g_return_if_fail (TNY_IS_LIST (list));
 
 	priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
-	
-	g_static_rec_mutex_lock (priv->lock);
 
-	iter = tny_list_create_iterator (priv->mothers);
+	g_static_rec_mutex_lock (priv->lock);
+	copy = tny_list_copy (priv->mothers);
+	g_static_rec_mutex_unlock (priv->lock);
+
+	iter = tny_list_create_iterator (copy);
 	while (!tny_iterator_is_done (iter)) {
 		GObject *folder;
 
@@ -1435,9 +1547,10 @@ tny_merge_folder_get_folders (TnyMergeFolder *self, TnyList *list)
 
 		tny_iterator_next (iter);
 	}
-	g_object_unref (iter);
 
-	g_static_rec_mutex_unlock (priv->lock);
+	g_object_unref (iter);
+	g_object_unref (copy);
+
 }
 
 /**
@@ -1447,7 +1560,18 @@ tny_merge_folder_get_folders (TnyMergeFolder *self, TnyList *list)
  * Creates a a new TnyMergeFolder instance that can merge multiple #TnyFolder 
  * instances together (partly read only, though).
  *
- * Return value: a new #TnyMergeFolder instance
+ * Important consideration: if you use this type within a Gtk+ application,
+ * you probably want to use tny_merge_folder_new_with_ui_locker in stead. If
+ * your UI toolkid isn't thread safe (most aren't), but so-called thread aware
+ * because it has a UI context lock (like Gtk+ has), then you must create this
+ * type with a TnyLockable that aquires and releases this lock correctly.
+ * For Gtk+ there's a default implementation called #TnyGtkLockable available
+ * in libtinymailui-gtk. Please use this, else you can have threading related
+ * problems in your user interface software.
+ *
+ * returns: (caller-owns): a new #TnyMergeFolder instance
+ * since: 1.0
+ * audience: application-developer
  **/
 TnyFolder*
 tny_merge_folder_new (const gchar *folder_name)
@@ -1461,20 +1585,61 @@ tny_merge_folder_new (const gchar *folder_name)
 }
 
 
+/**
+ * tny_merge_folder_new_with_ui_locker:
+ * @folder_name: the name of the merged folder
+ * @ui_locker: a #TnyLockable for locking your ui
+ *
+ * Creates a a new TnyMergeFolder instance that can merge multiple #TnyFolder 
+ * instances together (partly read only, though). Upon construction it 
+ * instantly sets the ui locker. For Gtk+ you should use a #TnyGtkLockable here.
+ *
+ * returns: (caller-owns): a new #TnyMergeFolder instance
+ * since: 1.0
+ * audience: application-developer
+ **/
+TnyFolder*
+tny_merge_folder_new_with_ui_locker (const gchar *folder_name, TnyLockable *ui_locker)
+{
+	TnyMergeFolder *self = TNY_MERGE_FOLDER (tny_merge_folder_new (folder_name));
+	tny_merge_folder_set_ui_locker (self, ui_locker);
+	return TNY_FOLDER (self);
+}
+
+/**
+ * tny_merge_folder_set_ui_locker:
+ * @self: a #TnyMergeFolder
+ * @ui_locker: a #TnyLockable locker for locking your ui
+ *
+ * Sets the ui locker. For Gtk+ you should use a #TnyGtkLockable here.
+ *
+ * since: 1.0
+ * audience: application-developer
+ **/
+void 
+tny_merge_folder_set_ui_locker (TnyMergeFolder *self, TnyLockable *ui_locker)
+{
+	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
+	if (priv->ui_locker)
+		g_object_unref (priv->ui_locker);
+	priv->ui_locker = TNY_LOCKABLE (g_object_ref (ui_locker));
+	return;
+}
+
+
 static void
 tny_merge_folder_instance_init (GTypeInstance *instance, gpointer g_class)
 {
 	TnyMergeFolder *self = (TnyMergeFolder *) instance;
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 
-	/* The get_id_func() DBC contract does not allow this to be NULL or "": */
-	priv->id = g_strdup ("unknown_mergefolder");
-	
+	priv->id = NULL;
 	priv->mothers = tny_simple_list_new ();
 	priv->lock = g_new0 (GStaticRecMutex, 1);
 	g_static_rec_mutex_init (priv->lock);
-	priv->observers = NULL;
+	priv->obs = NULL;
 	priv->folder_type = TNY_FOLDER_TYPE_MERGE;
+	priv->ui_locker = tny_noop_lockable_new ();
 
 	return;
 }
@@ -1485,10 +1650,13 @@ tny_merge_folder_finalize (GObject *object)
 	TnyMergeFolder *self = (TnyMergeFolder *) object;
 	TnyMergeFolderPriv *priv = TNY_MERGE_FOLDER_GET_PRIVATE (self);
 	TnyIterator *iter;
+	TnyList *copy;
 
 	g_static_rec_mutex_lock (priv->lock);
+	copy = tny_list_copy (priv->mothers);
+	g_static_rec_mutex_unlock (priv->lock);
 
-	iter = tny_list_create_iterator (priv->mothers);
+	iter = tny_list_create_iterator (copy);
 	while (!tny_iterator_is_done (iter))
 	{
 		TnyFolder *cur = TNY_FOLDER (tny_iterator_get_current (iter));
@@ -1496,12 +1664,23 @@ tny_merge_folder_finalize (GObject *object)
 		g_object_unref (cur);
 		tny_iterator_next (iter);
 	}
-	g_object_unref (iter);
 
-	if (priv->observers) 
-		g_object_unref (priv->observers);
+	g_object_unref (iter);
+	g_object_unref (copy);
+
+
+	if (priv->obs) {
+		GList *copy = priv->obs;
+		while (copy) {
+			g_object_weak_unref ((GObject *) copy->data, notify_observer_del, self);
+			copy = g_list_next (copy);
+		}
+		g_list_free (priv->obs);
+		priv->obs = NULL;
+	}
 
 	g_object_unref (priv->mothers);
+	g_object_unref (priv->ui_locker);
 
 	if (priv->id)
 		g_free (priv->id);
@@ -1509,9 +1688,8 @@ tny_merge_folder_finalize (GObject *object)
 	if (priv->name)
 		g_free (priv->name);
 
-	g_static_rec_mutex_unlock (priv->lock);
-
-	g_static_rec_mutex_free (priv->lock);
+	/* g_static_rec_mutex_free (priv->lock); */
+	g_free (priv->lock);
 	priv->lock = NULL;
 
 	parent_class->finalize (object);
@@ -1520,46 +1698,47 @@ tny_merge_folder_finalize (GObject *object)
 static void
 tny_folder_observer_init (TnyFolderObserverIface *klass)
 {
-	klass->update_func = tny_merge_folder_update;
+	klass->update= tny_merge_folder_update;
 }
 
 static void
 tny_folder_init (TnyFolderIface *klass)
 {
-	klass->remove_msg_func = tny_merge_folder_remove_msg;
-	klass->add_msg_func = tny_merge_folder_add_msg;
-	klass->add_msg_async_func = tny_merge_folder_add_msg_async;
-	klass->sync_func = tny_merge_folder_sync;
-	klass->get_msg_remove_strategy_func = tny_merge_folder_get_msg_remove_strategy;
-	klass->set_msg_remove_strategy_func = tny_merge_folder_set_msg_remove_strategy;
-	klass->get_msg_receive_strategy_func = tny_merge_folder_get_msg_receive_strategy;
-	klass->set_msg_receive_strategy_func = tny_merge_folder_set_msg_receive_strategy;
-	klass->get_msg_func = tny_merge_folder_get_msg;
-	klass->find_msg_func = tny_merge_folder_find_msg;
-	klass->get_msg_async_func = tny_merge_folder_get_msg_async;
-	klass->get_headers_func = tny_merge_folder_get_headers;
-	klass->get_headers_async_func = tny_merge_folder_get_headers_async;
-	klass->get_name_func = tny_merge_folder_get_name;
-	klass->get_id_func = tny_merge_folder_get_id;
-	klass->get_account_func = tny_merge_folder_get_account;
-	klass->get_folder_type_func = tny_merge_folder_get_folder_type;
-	klass->get_all_count_func = tny_merge_folder_get_all_count;
-	klass->get_unread_count_func = tny_merge_folder_get_unread_count;
-	klass->get_local_size_func = tny_merge_folder_get_local_size;
-	klass->is_subscribed_func = tny_merge_folder_is_subscribed;
-	klass->sync_async_func = tny_merge_folder_sync_async;
-	klass->refresh_async_func = tny_merge_folder_refresh_async;
-	klass->refresh_func = tny_merge_folder_refresh;
-	klass->transfer_msgs_func = tny_merge_folder_transfer_msgs;
-	klass->transfer_msgs_async_func = tny_merge_folder_transfer_msgs_async;
-	klass->copy_func = tny_merge_folder_copy;
-	klass->poke_status_func = tny_merge_folder_poke_status;
-	klass->add_observer_func = tny_merge_folder_add_observer;
-	klass->remove_observer_func = tny_merge_folder_remove_observer;
-	klass->get_folder_store_func = tny_merge_folder_get_folder_store;
-	klass->get_stats_func = tny_merge_folder_get_stats;
-	klass->get_url_string_func = tny_merge_folder_get_url_string;
-	klass->get_caps_func = tny_merge_folder_get_caps;
+	klass->remove_msg= tny_merge_folder_remove_msg;
+	klass->add_msg= tny_merge_folder_add_msg;
+	klass->add_msg_async= tny_merge_folder_add_msg_async;
+	klass->sync= tny_merge_folder_sync;
+	klass->get_msg_remove_strategy= tny_merge_folder_get_msg_remove_strategy;
+	klass->set_msg_remove_strategy= tny_merge_folder_set_msg_remove_strategy;
+	klass->get_msg_receive_strategy= tny_merge_folder_get_msg_receive_strategy;
+	klass->set_msg_receive_strategy= tny_merge_folder_set_msg_receive_strategy;
+	klass->get_msg= tny_merge_folder_get_msg;
+	klass->find_msg= tny_merge_folder_find_msg;
+	klass->get_msg_async= tny_merge_folder_get_msg_async;
+	klass->get_headers= tny_merge_folder_get_headers;
+	klass->get_headers_async= tny_merge_folder_get_headers_async;
+	klass->get_name= tny_merge_folder_get_name;
+	klass->get_id= tny_merge_folder_get_id;
+	klass->get_account= tny_merge_folder_get_account;
+	klass->get_folder_type= tny_merge_folder_get_folder_type;
+	klass->get_all_count= tny_merge_folder_get_all_count;
+	klass->get_unread_count= tny_merge_folder_get_unread_count;
+	klass->get_local_size= tny_merge_folder_get_local_size;
+	klass->is_subscribed= tny_merge_folder_is_subscribed;
+	klass->sync_async= tny_merge_folder_sync_async;
+	klass->refresh_async= tny_merge_folder_refresh_async;
+	klass->refresh= tny_merge_folder_refresh;
+	klass->transfer_msgs= tny_merge_folder_transfer_msgs;
+	klass->transfer_msgs_async= tny_merge_folder_transfer_msgs_async;
+	klass->copy= tny_merge_folder_copy;
+	klass->poke_status= tny_merge_folder_poke_status;
+	klass->add_observer= tny_merge_folder_add_observer;
+	klass->remove_observer= tny_merge_folder_remove_observer;
+	klass->get_folder_store= tny_merge_folder_get_folder_store;
+	klass->get_stats= tny_merge_folder_get_stats;
+	klass->get_url_string= tny_merge_folder_get_url_string;
+	klass->get_caps= tny_merge_folder_get_caps;
+	klass->remove_msgs= tny_merge_folder_remove_msgs;
 }
 
 static void
