@@ -13,11 +13,36 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with self library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
+
+/**
+ * TnyGtkFolderStoreTreeModel:
+ *
+ * A #GtkTreeModel for #TnyFolderStore instances that'll become #TnyFolderStoreObserver
+ * and #TnyFolderObserver (in case the #TnyFolderStore instance is a #TnyFolder
+ * too) of each added #TnyFolderStore and each of its children en grandchildren
+ * (recursively).
+ *
+ * It will detect changes in the instance's tree structure this way and it will
+ * adapt itself to a new situation automatically. It also contains columns that
+ * contain certain popular numbers (like the unread and the total counts of
+ * #TnyFolder instances).
+ *
+ * Note that a #TnyGtkFolderStoreTreeModel is a #TnyList too. You can use the
+ * #TnyList API on instances of this type too.
+ *
+ * Note that you must make sure that you unreference #TnyFolderStore instances
+ * that you get out of the instance column of this type using the #GtkTreeModel
+ * API gtk_tree_model_get().
+ *
+ * free-function: g_object_unref
+ **/
+
 #include <config.h>
+#include <string.h>
 
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -28,6 +53,7 @@
 #include <tny-folder.h>
 #include <tny-folder-store.h>
 #include <tny-simple-list.h>
+#include <tny-merge-folder.h>
 
 #include <tny-store-account.h>
 #include <tny-folder-store-change.h>
@@ -43,12 +69,50 @@ static GObjectClass *parent_class = NULL;
 
 typedef void (*treeaddfunc) (GtkTreeStore *tree_store, GtkTreeIter *iter, GtkTreeIter *parent);
 
+
+static void 
+add_folder_observer_weak (TnyGtkFolderStoreTreeModel *self, TnyFolder *folder)
+{
+	if (TNY_IS_FOLDER (folder)) {
+		tny_folder_add_observer (folder, TNY_FOLDER_OBSERVER (self));
+		self->fol_obs = g_list_prepend (self->fol_obs, folder);
+	}
+}
+
+static void 
+add_folder_store_observer_weak (TnyGtkFolderStoreTreeModel *self, TnyFolderStore *store)
+{
+	if (TNY_IS_FOLDER_STORE (store)) {
+		tny_folder_store_add_observer (store, TNY_FOLDER_STORE_OBSERVER (self));
+		self->store_obs = g_list_prepend (self->store_obs, store);
+	}
+}
+
+static void 
+remove_folder_observer_weak (TnyGtkFolderStoreTreeModel *self, TnyFolder *folder, gboolean final)
+{
+	if (TNY_IS_FOLDER (folder)) {
+		if (!final)
+			self->fol_obs = g_list_remove (self->fol_obs, folder);
+		tny_folder_remove_observer (folder, (TnyFolderObserver *) self);
+	}
+}
+
+static void 
+remove_folder_store_observer_weak (TnyGtkFolderStoreTreeModel *self, TnyFolderStore *store, gboolean final)
+{
+	if (TNY_IS_FOLDER_STORE (store)) {
+		if (!final)
+			self->store_obs = g_list_remove (self->store_obs, store);
+		tny_folder_store_remove_observer (store, (TnyFolderStoreObserver *) self);
+	}
+}
+
 static void
 recurse_folders_sync (TnyGtkFolderStoreTreeModel *self, TnyFolderStore *store, GtkTreeIter *parent_tree_iter)
 {
 	TnyIterator *iter;
 	TnyList *folders = tny_simple_list_new ();
-	TnyGtkFolderStoreTreeModel *me = (TnyGtkFolderStoreTreeModel*) self;
 
 	/* TODO add error checking and reporting here */
 	tny_folder_store_get_folders (store, folders, self->query, NULL);
@@ -66,11 +130,11 @@ recurse_folders_sync (TnyGtkFolderStoreTreeModel *self, TnyFolderStore *store, G
 		gboolean found = FALSE;
 		GObject *mark_for_removal = NULL;
 
-		if (TNY_IS_FOLDER (instance))
+		if (instance && (TNY_IS_FOLDER (instance) || TNY_IS_MERGE_FOLDER (instance)))
 			folder = TNY_FOLDER (instance);
 
-		if (TNY_IS_FOLDER_STORE (folder))
-			folder_store =  TNY_FOLDER_STORE (instance);
+		if (instance && (TNY_IS_FOLDER_STORE (instance)))
+			folder_store = TNY_FOLDER_STORE (instance);
 
 		/* We check whether we have this folder already in the tree, or 
 		 * whether it's a brand new one. If it's a new one, we'll add
@@ -89,10 +153,10 @@ recurse_folders_sync (TnyGtkFolderStoreTreeModel *self, TnyFolderStore *store, G
 			if (citem == instance)
 			{
 				found = TRUE;
+				if (citem)
+					g_object_unref (citem);
 				break;
 			}
-			if (citem)
-				g_object_unref (citem);
 
 			/* We search whether this folder that we have in the 
 			 * model, still exists in the actual list. Because if
@@ -103,13 +167,19 @@ recurse_folders_sync (TnyGtkFolderStoreTreeModel *self, TnyFolderStore *store, G
 			while (!tny_iterator_is_done (niter))
 			{
 				TnyFolder *ifound = TNY_FOLDER (tny_iterator_get_current (niter));
-				if (citem == (GObject *) ifound)
+				if (citem == (GObject *) ifound) {
+					if (mark_for_removal)
+						g_object_unref (mark_for_removal);
 					mark_for_removal = g_object_ref (ifound);
+				}
 				g_object_unref (ifound);
 				tny_iterator_next (niter);
 			}
 			g_object_unref (niter);
 
+			if (citem)
+				g_object_unref (citem);
+
 		  } while (gtk_tree_model_iter_next (mmodel, &miter));
 
 		/* It was not found, so let's start adding it to the model! */
@@ -125,26 +195,13 @@ recurse_folders_sync (TnyGtkFolderStoreTreeModel *self, TnyFolderStore *store, G
 
 			if (folder)
 			{
-				tny_folder_add_observer (folder, TNY_FOLDER_OBSERVER (self));
-				me->folder_observables = g_list_prepend (me->folder_observables, folder);
-			}
+				/* This adds a reference count to folder too. When it gets removed, that
+				   reference count is decreased automatically by the gtktreestore infra-
+				   structure. */
 
-			if (folder_store)
-			{
-				tny_folder_store_add_observer (folder_store, TNY_FOLDER_STORE_OBSERVER (self));
-				me->store_observables = g_list_prepend (me->store_observables, folder);
-			}
+				add_folder_observer_weak (self, folder);
 
-
-			/* This adds a reference count to folder too. When it gets removed, that
-			   reference count is decreased automatically by the gtktreestore infra-
-			   structure. */
-
-			if (folder)
-			{
-				TnyFolder *folder = TNY_FOLDER (instance);
-				
-				gtk_tree_store_set (model, &tree_iter,
+				gtk_tree_store_set  (model, &tree_iter,
 					TNY_GTK_FOLDER_STORE_TREE_MODEL_NAME_COLUMN, 
 					tny_folder_get_name (TNY_FOLDER (folder)),
 					TNY_GTK_FOLDER_STORE_TREE_MODEL_UNREAD_COLUMN, 
@@ -155,11 +212,14 @@ recurse_folders_sync (TnyGtkFolderStoreTreeModel *self, TnyFolderStore *store, G
 					tny_folder_get_folder_type (TNY_FOLDER (folder)),
 					TNY_GTK_FOLDER_STORE_TREE_MODEL_INSTANCE_COLUMN,
 					folder, -1);
+
 			}
 
 			/* it's a store by itself, so keep on recursing */
-			if (folder_store)
+			if (folder_store) {
+				add_folder_store_observer_weak (self, folder_store);
 				recurse_folders_sync (self, folder_store, &tree_iter);
+			}
 
 			/* We're a folder, we'll request a status, since we've
 			 * set self to be a folder observers of folder, we'll 
@@ -177,11 +237,16 @@ recurse_folders_sync (TnyGtkFolderStoreTreeModel *self, TnyFolderStore *store, G
 
 			if (folder)
 				tny_folder_poke_status (TNY_FOLDER (folder));
+
+			if (mark_for_removal) {
+				g_object_unref (mark_for_removal);
+				mark_for_removal = NULL;
+			}
+
 		} else {
-			if (mark_for_removal)
-			{
-				printf ("We need to remove %s\n", 
-					tny_folder_get_id (TNY_FOLDER (mark_for_removal)));
+			if (mark_for_removal) {
+				g_object_unref (mark_for_removal);
+				mark_for_removal = NULL;
 			}
 		}
 
@@ -234,13 +299,17 @@ get_folders_cb (TnyFolderStore *fstore, gboolean cancelled, TnyList *list, GErro
 		gtk_tree_model_get (model, &iter, 
 			TNY_GTK_FOLDER_STORE_TREE_MODEL_INSTANCE_COLUMN, 
 			&citem, -1);
+
 		if (citem == (GObject *) fstore)
 		{
 			name_iter = iter;
 			found = TRUE;
+			if (citem)
+				g_object_unref (citem);
 			break;
 		}
-		g_object_unref (G_OBJECT (citem));
+
+		g_object_unref (citem);
 
 	  } while (gtk_tree_model_iter_next (model, &iter));
 
@@ -261,6 +330,9 @@ tny_gtk_folder_store_tree_model_on_constatus_changed (TnyAccount *account, TnyCo
 {
 	TnyList *list = NULL;
 
+	if (!self || !TNY_IS_GTK_FOLDER_STORE_TREE_MODEL (self))
+		return;
+
 	/* This callback handler deals with connection status changes. In case
 	 * we got connected, we can expect that, at least sometimes, new folders
 	 * might have arrived. We'll need to scan for those, so we'll recursively
@@ -271,7 +343,7 @@ tny_gtk_folder_store_tree_model_on_constatus_changed (TnyAccount *account, TnyCo
 
 	list = tny_simple_list_new ();
 	tny_folder_store_get_folders_async (TNY_FOLDER_STORE (account), 
-		list, get_folders_cb, NULL, NULL, g_object_ref (self));
+		list, self->query, get_folders_cb, NULL, g_object_ref (self));
 
 	return;
 }
@@ -289,18 +361,23 @@ tny_gtk_folder_store_tree_model_on_changed (TnyAccount *account, TnyGtkFolderSto
 	if (gtk_tree_model_get_iter_first (model, &iter))
 	  do 
 	  {
-		GObject *citem;
+		GObject *citem = NULL;
 
 		gtk_tree_model_get (model, &iter, 
 			TNY_GTK_FOLDER_STORE_TREE_MODEL_INSTANCE_COLUMN, 
 			&citem, -1);
+
 		if (citem == (GObject *) account)
 		{
 			name_iter = iter;
 			found = TRUE;
+			if (citem)
+				g_object_unref (citem);
 			break;
 		}
-		g_object_unref (G_OBJECT (citem));
+
+		if (citem)
+			g_object_unref (citem);
 
 	  } while (gtk_tree_model_iter_next (model, &iter));
 
@@ -311,12 +388,31 @@ tny_gtk_folder_store_tree_model_on_changed (TnyAccount *account, TnyGtkFolderSto
 	}
 }
 
+typedef struct
+{
+	GObject *instance;
+	guint handler_id;
+} SignalSlot;
+
 typedef struct
 {
 	TnyGtkFolderStoreTreeModel *self;
 	TnyAccount *account;
 } AccNotYetReadyInfo;
 
+static void 
+notify_signal_slots (gpointer user_data, GObject *instance)
+{
+	TnyGtkFolderStoreTreeModel *self = (TnyGtkFolderStoreTreeModel *) user_data;
+	int i=0;
+
+	for (i=0; i < self->signals->len; i++) {
+		SignalSlot *slot = (SignalSlot *) self->signals->pdata[i];
+		if (slot->instance == instance)
+			slot->instance = NULL;
+	}
+}
+
 static gboolean
 account_was_not_yet_ready_idle (gpointer user_data)
 {
@@ -325,11 +421,21 @@ account_was_not_yet_ready_idle (gpointer user_data)
 
 	if (tny_account_is_ready (info->account))
 	{
-		g_signal_connect (info->account, "connection-status-changed",
-			G_CALLBACK (tny_gtk_folder_store_tree_model_on_constatus_changed), info->self);
+		SignalSlot *slot;
 
-		g_signal_connect (info->account, "changed",
+		slot = g_slice_new (SignalSlot);
+		slot->instance = (GObject *) info->account;
+		slot->handler_id = g_signal_connect (info->account, "connection-status-changed",
+			G_CALLBACK (tny_gtk_folder_store_tree_model_on_constatus_changed), info->self);
+		g_object_weak_ref (G_OBJECT (info->account), notify_signal_slots, info->self);
+		g_ptr_array_add (info->self->signals, slot);
+
+		slot = g_slice_new (SignalSlot);
+		slot->instance = (GObject *) info->account;
+		slot->handler_id = g_signal_connect (info->account, "changed",
 			G_CALLBACK (tny_gtk_folder_store_tree_model_on_changed), info->self);
+		g_object_weak_ref (G_OBJECT (info->account), notify_signal_slots, info->self);
+		g_ptr_array_add (info->self->signals, slot);
 
 		tny_gtk_folder_store_tree_model_on_constatus_changed (info->account, 
 			tny_account_get_connection_status (info->account), info->self);
@@ -354,6 +460,7 @@ account_was_not_yet_ready_destroy (gpointer user_data)
 	g_slice_free (AccNotYetReadyInfo, user_data);
 }
 
+
 static void
 tny_gtk_folder_store_tree_model_add_i (TnyGtkFolderStoreTreeModel *self, TnyFolderStore *folder_store, treeaddfunc func, const gchar *root_name)
 {
@@ -389,10 +496,22 @@ tny_gtk_folder_store_tree_model_add_i (TnyGtkFolderStoreTreeModel *self, TnyFold
 				info, account_was_not_yet_ready_destroy);
 
 		} else {
-			g_signal_connect (folder_store, "connection-status-changed",
+			SignalSlot *slot;
+
+			slot = g_slice_new (SignalSlot);
+
+			slot->instance = (GObject *)folder_store; 
+			slot->handler_id = g_signal_connect (folder_store, "connection-status-changed",
 				G_CALLBACK (tny_gtk_folder_store_tree_model_on_constatus_changed), self);
-			g_signal_connect (folder_store, "changed",
+			g_object_weak_ref (G_OBJECT (folder_store), notify_signal_slots, self);
+			g_ptr_array_add (self->signals, slot);
+
+			slot = g_slice_new (SignalSlot);
+			slot->instance = (GObject *)folder_store; 
+			slot->handler_id = g_signal_connect (folder_store, "changed",
 				G_CALLBACK (tny_gtk_folder_store_tree_model_on_changed), self);
+			g_object_weak_ref (G_OBJECT (folder_store), notify_signal_slots, self);
+			g_ptr_array_add (self->signals, slot);
 		}
 	}
 
@@ -403,7 +522,7 @@ tny_gtk_folder_store_tree_model_add_i (TnyGtkFolderStoreTreeModel *self, TnyFold
 	 * before this one) */
 
 	tny_folder_store_get_folders_async (TNY_FOLDER_STORE (folder_store), 
-		folders, get_folders_cb, NULL, NULL, g_object_ref (self));
+		folders, self->query, get_folders_cb, NULL, g_object_ref (self));
 
 	/* recurse_folders_sync (self, TNY_FOLDER_STORE (folder_store), &name_iter);  */
 
@@ -411,8 +530,7 @@ tny_gtk_folder_store_tree_model_add_i (TnyGtkFolderStoreTreeModel *self, TnyFold
 	/* Add an observer for the root folder store, so that we can observe 
 	 * the actual account too. */
 
-	tny_folder_store_add_observer (folder_store, TNY_FOLDER_STORE_OBSERVER (self));
-	self->store_observables = g_list_prepend (self->store_observables, folder_store);
+	add_folder_store_observer_weak (self, folder_store);
 
 	/* g_object_unref (G_OBJECT (folders)); */
 
@@ -422,13 +540,13 @@ tny_gtk_folder_store_tree_model_add_i (TnyGtkFolderStoreTreeModel *self, TnyFold
 
 /**
  * tny_gtk_folder_store_tree_model_new:
- * @query: the #TnyFolderStoreQuery that will be used to retrieve the folders of each folder_store
+ * @query: the #TnyFolderStoreQuery that will be used to retrieve the child folders of each #TnyFolderStore
  *
- * Create a new #GtkTreeModel instance suitable for showing  
- * #TnyFolderStore instances
+ * Create a new #GtkTreeModel for showing #TnyFolderStore instances
  * 
- * Return value: a new #GtkTreeModel instance suitable for showing  
- * #TnyFolderStore instances
+ * returns: (caller-owns): a new #GtkTreeModel for #TnyFolderStore instances
+ * since: 1.0
+ * audience: application-developer
  **/
 GtkTreeModel*
 tny_gtk_folder_store_tree_model_new (TnyFolderStoreQuery *query)
@@ -436,57 +554,63 @@ tny_gtk_folder_store_tree_model_new (TnyFolderStoreQuery *query)
 	TnyGtkFolderStoreTreeModel *self = g_object_new (TNY_TYPE_GTK_FOLDER_STORE_TREE_MODEL, NULL);
 
 	if (query) 
-		self->query = g_object_ref (G_OBJECT (query));
+		self->query = g_object_ref (query);
 
 	return GTK_TREE_MODEL (self);
 }
 
-static void 
-destroy_folder_stores (gpointer item, gpointer user_data)
-{
-	if (item && G_IS_OBJECT (item))
-		g_object_unref (G_OBJECT (item));
-	return;
-}
 
-static void 
-unregister_folder_observerable (gpointer item, gpointer user_data)
-{
-	TnyFolder *f = (TnyFolder *) item;
-	tny_folder_remove_observer (f, TNY_FOLDER_OBSERVER (user_data));
-}
-
-static void 
-unregister_store_observerable (gpointer item, gpointer user_data)
-{
-	TnyFolderStore *fstore = (TnyFolderStore *) item;
-	tny_folder_store_remove_observer (fstore, TNY_FOLDER_STORE_OBSERVER (user_data));
-}
 
 static void
 tny_gtk_folder_store_tree_model_finalize (GObject *object)
 {
 	TnyGtkFolderStoreTreeModel *me = (TnyGtkFolderStoreTreeModel*) object;
+	int i = 0;
+
+	for (i = 0; i < me->signals->len; i++) {
+		SignalSlot *slot = (SignalSlot *) me->signals->pdata [i];
+		if (slot->instance) {
+			g_signal_handler_disconnect (slot->instance, slot->handler_id);
+			g_object_weak_unref (G_OBJECT (slot->instance), notify_signal_slots, object);
+		}
+		g_slice_free (SignalSlot, slot);
+	}
+
+	g_ptr_array_free (me->signals, TRUE);
+	me->signals = NULL;
+
+/* Experimentally removed this. With weak referencing this ain't needed ...
+
+	while (copy) {
+		remove_folder_observer_weak (me, (TnyFolder *) copy->data, TRUE);
+		copy = g_list_next (copy);
+	}
+
+	copy = me->store_obs;
+	while (copy) {
+		remove_folder_store_observer_weak (me, (TnyFolderStore *) copy->data, TRUE);
+		copy = g_list_next (copy);
+	}
+*/
+
+	if (me->fol_obs)
+		g_list_free (me->fol_obs);
+	me->fol_obs = NULL;
+
+	if (me->store_obs)
+		g_list_free (me->store_obs);
+	me->store_obs = NULL;
 
 	g_mutex_lock (me->iterator_lock);
-	if (me->first)
-	{
-		g_list_foreach (me->first, destroy_folder_stores, NULL);
-		g_list_free (me->first); me->first = NULL;
+	if (me->first) {
+		if (me->first_needs_unref)
+			g_list_foreach (me->first, (GFunc)g_object_unref, NULL);
+		me->first_needs_unref = FALSE;
+		g_list_free (me->first); 
 	}
+	me->first = NULL;
 	g_mutex_unlock (me->iterator_lock);
 
-	if (me->folder_observables)
-	{
-		g_list_foreach (me->folder_observables, unregister_folder_observerable, me);
-		g_list_free (me->store_observables);
-	}
-
-	if (me->store_observables)
-	{
-		g_list_foreach (me->store_observables, unregister_store_observerable, me);
-		g_list_free (me->store_observables);
-	}
 
 	g_mutex_free (me->iterator_lock);
 	me->iterator_lock = NULL;
@@ -517,9 +641,11 @@ tny_gtk_folder_store_tree_model_instance_init (GTypeInstance *instance, gpointer
 	TnyGtkFolderStoreTreeModel *me = (TnyGtkFolderStoreTreeModel*) instance;
 	static GType types[] = { G_TYPE_STRING, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_INT, G_TYPE_OBJECT };
 
+	me->signals = g_ptr_array_new ();
+	me->fol_obs = NULL;
+	me->store_obs = NULL;
 	me->iterator_lock = g_mutex_new ();
-	me->folder_observables = NULL;
-	me->store_observables = NULL;
+	me->first_needs_unref = FALSE;
 
 	gtk_tree_store_set_column_types (store, 
 		TNY_GTK_FOLDER_STORE_TREE_MODEL_N_COLUMNS, types);
@@ -542,11 +668,14 @@ tny_gtk_folder_store_tree_model_create_iterator (TnyList *self)
 
 /**
  * tny_gtk_folder_store_tree_model_prepend:
- * @self: A #TnyGtkFolderStoreTreeModel instance
- * @item: A #TnyFolderStore instance to add
+ * @self: a #TnyGtkFolderStoreTreeModel
+ * @item: a #TnyFolderStore to add
  * @root_name: The node's root name 
  *
  * Prepends an item to the model
+ *
+ * since: 1.0
+ * audience: application-developer
  **/
 void
 tny_gtk_folder_store_tree_model_prepend (TnyGtkFolderStoreTreeModel *self, TnyFolderStore* item, const gchar *root_name)
@@ -556,7 +685,6 @@ tny_gtk_folder_store_tree_model_prepend (TnyGtkFolderStoreTreeModel *self, TnyFo
 	g_mutex_lock (me->iterator_lock);
 
 	/* Prepend something to the list */
-	g_object_ref (item);
 	me->first = g_list_prepend (me->first, item);
 
 	tny_gtk_folder_store_tree_model_add_i (me, TNY_FOLDER_STORE (item), 
@@ -574,7 +702,6 @@ tny_gtk_folder_store_tree_model_prepend_i (TnyList *self, GObject* item)
 	g_mutex_lock (me->iterator_lock);
 
 	/* Prepend something to the list */
-	g_object_ref (item);
 	me->first = g_list_prepend (me->first, item);
 
 	tny_gtk_folder_store_tree_model_add_i (me, TNY_FOLDER_STORE (item), 
@@ -585,11 +712,14 @@ tny_gtk_folder_store_tree_model_prepend_i (TnyList *self, GObject* item)
 
 /**
  * tny_gtk_folder_store_tree_model_append:
- * @self: A #TnyGtkFolderStoreTreeModel instance
- * @item: A #TnyFolderStore instance to add
+ * @self: a #TnyGtkFolderStoreTreeModel
+ * @item: a #TnyFolderStore to add
  * @root_name: The node's root name 
  *
  * Appends an item to the model
+ *
+ * since: 1.0
+ * audience: application-developer
  **/
 void
 tny_gtk_folder_store_tree_model_append (TnyGtkFolderStoreTreeModel *self, TnyFolderStore* item, const gchar *root_name)
@@ -599,7 +729,6 @@ tny_gtk_folder_store_tree_model_append (TnyGtkFolderStoreTreeModel *self, TnyFol
 	g_mutex_lock (me->iterator_lock);
 
 	/* Append something to the list */
-	g_object_ref (item);
 	me->first = g_list_append (me->first, item);
 
 	tny_gtk_folder_store_tree_model_add_i (me, TNY_FOLDER_STORE (item), 
@@ -616,7 +745,6 @@ tny_gtk_folder_store_tree_model_append_i (TnyList *self, GObject* item)
 	g_mutex_lock (me->iterator_lock);
 
 	/* Append something to the list */
-	g_object_ref (item);
 	me->first = g_list_append (me->first, item);
 
 	tny_gtk_folder_store_tree_model_add_i (me, TNY_FOLDER_STORE (item), 
@@ -670,11 +798,12 @@ tny_gtk_folder_store_tree_model_remove (TnyList *self, GObject* item)
 		{
 			/* This removes a reference count */
 			gtk_tree_store_remove (GTK_TREE_STORE (me), &iter);
-			g_object_unref (G_OBJECT (item));
+			if (citem)
+				g_object_unref (citem);
 			break;
 		}
 		if (citem)
-			g_object_unref (G_OBJECT (citem));
+			g_object_unref (citem);
 
 	  } while (gtk_tree_model_iter_next (model, &iter));
 
@@ -687,6 +816,7 @@ tny_gtk_folder_store_tree_model_copy_the_list (TnyList *self)
 {
 	TnyGtkFolderStoreTreeModel *me = (TnyGtkFolderStoreTreeModel*)self;
 	TnyGtkFolderStoreTreeModel *copy = g_object_new (TNY_TYPE_GTK_FOLDER_STORE_TREE_MODEL, NULL);
+	GList *list_copy = NULL;
 
 	/* This only copies the TnyList pieces. The result is not a correct or good
 	   TnyHeaderListModel. But it will be a correct TnyList instance. It's the 
@@ -696,8 +826,9 @@ tny_gtk_folder_store_tree_model_copy_the_list (TnyList *self)
 	   copy of the list-nodes of course. */
 
 	g_mutex_lock (me->iterator_lock);
-	GList *list_copy = g_list_copy (me->first);
+	list_copy = g_list_copy (me->first);
 	g_list_foreach (list_copy, (GFunc)g_object_ref, NULL);
+	copy->first_needs_unref = TRUE;
 	copy->first = list_copy;
 	g_mutex_unlock (me->iterator_lock);
 
@@ -731,6 +862,25 @@ updater (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer use
 	 * the changed_folder pointer, and if there's a match it will update
 	 * the values of the model with the values from the folder. */
 
+/*
+	For your debugging advertures:
+
+	print ((TnyCamelFolderPriv *) g_type_instance_get_private \
+		(tny_iterator_get_current (tny_list_create_iterator \
+		(((TnyMergeFolderPriv *) g_type_instance_get_private \
+		(folder, tny_merge_folder_get_type()))->mothers)), \
+		tny_camel_folder_get_type()))->folder->summary->messages->len
+
+	print * (CamelLocalFolder *) ((TnyCamelFolderPriv *) \
+		g_type_instance_get_private (tny_iterator_get_current \
+		(tny_list_create_iterator (((TnyMergeFolderPriv *) \
+		g_type_instance_get_private (folder, \
+		tny_merge_folder_get_type()))->mothers)), \
+		tny_camel_folder_get_type()))->folder
+
+*/
+
+
 	gtk_tree_model_get (model, iter, 
 		TNY_GTK_FOLDER_STORE_TREE_MODEL_TYPE_COLUMN, 
 		&type, -1);
@@ -777,13 +927,13 @@ updater (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer use
 	return FALSE;
 }
 
-
 static gboolean 
 deleter (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer user_data1)
 {
 	gboolean retval = FALSE;
 	TnyFolderType type = TNY_FOLDER_TYPE_UNKNOWN;
 	GObject *folder = user_data1;
+	TnyGtkFolderStoreTreeModel *me = (TnyGtkFolderStoreTreeModel*) model;
 
 	/* The deleter will compare all folders in the model with the deleted 
 	 * folder @folder, and if there's a match it will delete the folder's
@@ -802,6 +952,10 @@ deleter (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer use
 			&fol, -1);
 
 		if (fol == folder) {
+
+			remove_folder_observer_weak (me, TNY_FOLDER (folder), FALSE);
+			remove_folder_store_observer_weak (me, TNY_FOLDER_STORE (folder), FALSE);
+
 			gtk_tree_store_remove (GTK_TREE_STORE (model), iter);
 			retval = TRUE;
 		}
@@ -813,47 +967,115 @@ deleter (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer use
 	return retval;
 }
 
-
 static gboolean
-find_store_iter (GtkTreeModel *model, GtkTreeIter *iter, GtkTreeIter *f, gpointer user_data)
+creater (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *in_iter, gpointer user_data)
 {
-	/* Finding the iter for a given folder */
+	TnyGtkFolderStoreTreeModel *self = (TnyGtkFolderStoreTreeModel*) model;
+	TnyFolderStore *fol = NULL;
+	gboolean found = FALSE;
+	const gchar *mid = NULL, *sid = NULL;
+	TnyFolderStoreChange *change = (TnyFolderStoreChange *) user_data;
+	TnyFolderStore *parent_store;
+	GtkTreeIter iter;
 
-	do
+	/* This creater will get the store out of the model, compare with 
+	 * the change's store pointer, and if there's a match it will add the
+	 * the created folders to the model at that location. */
+
+	if (!gtk_tree_model_get_iter (model, &iter, path)) {
+		g_warning ("Internal state of the TnyGtkFolderStoreTreeModel is corrupted\n");
+		return FALSE;
+	}
+
+	parent_store = tny_folder_store_change_get_folder_store (change);
+	gtk_tree_model_get (model, &iter, 
+		TNY_GTK_FOLDER_STORE_TREE_MODEL_INSTANCE_COLUMN, 
+		&fol, -1);
+
+	if (fol == parent_store)
+		found = TRUE;
+
+	if (!found) {
+		TnyAccount *maccount = NULL, *saccount = NULL;
+		const gchar *ma_id = NULL, *sa_id = NULL;
+
+		if (fol && TNY_IS_FOLDER (fol)) {
+			mid = tny_folder_get_id (TNY_FOLDER (fol));
+			if (!TNY_IS_MERGE_FOLDER (fol))
+				maccount = tny_folder_get_account (TNY_FOLDER (fol));
+			if (maccount)
+				ma_id = tny_account_get_id (maccount);
+		}
+		if (parent_store && TNY_IS_FOLDER (parent_store)) {
+			sid = tny_folder_get_id (TNY_FOLDER (parent_store));
+			if (!TNY_IS_MERGE_FOLDER (parent_store))
+				saccount = tny_folder_get_account (TNY_FOLDER (parent_store));
+			if (saccount)
+				sa_id = tny_account_get_id (saccount);
+		}
+		if (sid && mid && !strcmp (sid, mid)) {
+			if (ma_id && sa_id) {
+				if (!strcmp (ma_id, sa_id))
+					found = TRUE;
+			} else {
+				found = TRUE;
+			}
+		}
+		if (maccount)
+			g_object_unref (maccount);
+		if (saccount)
+			g_object_unref (saccount);
+	}
+
+	if (found) 
 	{
-		GtkTreeIter child;
-		TnyFolderType type = TNY_FOLDER_TYPE_UNKNOWN;
-		TnyFolderStore *fol;
-		gboolean found = FALSE;
+		TnyList *created = tny_simple_list_new ();
+		TnyIterator *miter;
 
-		gtk_tree_model_get (model, iter, 
-			TNY_GTK_FOLDER_STORE_TREE_MODEL_TYPE_COLUMN, 
-			&type, -1);
+		tny_folder_store_change_get_created_folders (change, created);
+		miter = tny_list_create_iterator (created);
 
-		gtk_tree_model_get (model, iter, 
-			TNY_GTK_FOLDER_STORE_TREE_MODEL_INSTANCE_COLUMN, 
-			&fol, -1);
-		
-		if (fol == user_data)
-			found = TRUE;
-		
-		if (fol)
-			g_object_unref (fol);
-
-		if (found) {
-			*f = *iter;
-			return TRUE;
-		}
-
-		if (gtk_tree_model_iter_children (model, &child, iter))
+		while (!tny_iterator_is_done (miter))
 		{
-			if (find_store_iter (model, &child, f, user_data))
-				return TRUE;
+			GtkTreeIter newiter;
+			TnyFolder *folder = TNY_FOLDER (tny_iterator_get_current (miter));
+
+			add_folder_observer_weak (self, folder);
+			add_folder_store_observer_weak (self, TNY_FOLDER_STORE (folder));
+
+			/* This adds a reference count to folder_store too. When it gets 
+			   removed, that reference count is decreased automatically by 
+			   the gtktreestore infrastructure. */
+
+
+			gtk_tree_store_prepend (GTK_TREE_STORE (model), &newiter, in_iter);
+
+			gtk_tree_store_set (GTK_TREE_STORE (model), &newiter,
+				TNY_GTK_FOLDER_STORE_TREE_MODEL_NAME_COLUMN, 
+				tny_folder_get_name (TNY_FOLDER (folder)),
+				TNY_GTK_FOLDER_STORE_TREE_MODEL_UNREAD_COLUMN, 
+				tny_folder_get_unread_count (TNY_FOLDER (folder)),
+				TNY_GTK_FOLDER_STORE_TREE_MODEL_ALL_COLUMN, 
+				tny_folder_get_all_count (TNY_FOLDER (folder)),
+				TNY_GTK_FOLDER_STORE_TREE_MODEL_TYPE_COLUMN,
+				tny_folder_get_folder_type (TNY_FOLDER (folder)),
+				TNY_GTK_FOLDER_STORE_TREE_MODEL_INSTANCE_COLUMN,
+				folder, -1);
+
+			g_object_unref (folder);
+			tny_iterator_next (miter);
 		}
+		g_object_unref (miter);
+		g_object_unref (created);
+	}
 
-	} while (gtk_tree_model_iter_next (model, iter));
+	if (fol)
+		g_object_unref (fol);
 
-	return FALSE;
+	if (parent_store)
+		g_object_unref (parent_store);
+
+	return found;
 }
 
 
@@ -874,91 +1096,11 @@ tny_gtk_folder_store_tree_model_folder_obsr_update (TnyFolderObserver *self, Tny
 	return;
 }
 
-
-static void
-tny_gtk_folder_store_tree_model_store_obsr_update (TnyFolderStoreObserver *self, TnyFolderStoreChange *change)
+static void 
+delete_these_folders (GtkTreeModel *model, TnyList *list)
 {
-	TnyFolderStoreChangeChanged changed = tny_folder_store_change_get_changed (change);
-	GtkTreeModel *model = GTK_TREE_MODEL (self);
-	TnyGtkFolderStoreTreeModel *me = (TnyGtkFolderStoreTreeModel*) self;
-
-	if (changed & TNY_FOLDER_STORE_CHANGE_CHANGED_CREATED_FOLDERS)
-	{
-		TnyList *created = tny_simple_list_new ();
 		TnyIterator *miter;
-
-		tny_folder_store_change_get_created_folders (change, created);
-		miter = tny_list_create_iterator (created);
-
-		while (!tny_iterator_is_done (miter))
-		{
-			TnyFolder *folder = TNY_FOLDER (tny_iterator_get_current (miter));
-			tny_folder_add_observer (TNY_FOLDER (folder), TNY_FOLDER_OBSERVER (self));
-			tny_folder_store_add_observer (TNY_FOLDER_STORE (folder), TNY_FOLDER_STORE_OBSERVER (self));
-			g_object_unref (folder);
-			tny_iterator_next (miter);
-		}
-		g_object_unref (miter);
-		g_object_unref (created);
-	}
-
-	if (changed & TNY_FOLDER_STORE_CHANGE_CHANGED_CREATED_FOLDERS)
-	{
-		TnyFolderStore *parentstore = tny_folder_store_change_get_folder_store (change);
-		GtkTreeIter first, iter;
-
-		if (gtk_tree_model_get_iter_first (model, &first) && 
-			find_store_iter (model, &first, &iter, parentstore))
-		{
-			TnyList *created = tny_simple_list_new ();
-			TnyIterator *miter;
-
-			tny_folder_store_change_get_created_folders (change, created);
-			miter = tny_list_create_iterator (created);
-
-			while (!tny_iterator_is_done (miter))
-			{
-				GtkTreeIter newiter;
-				TnyFolder *folder = TNY_FOLDER (tny_iterator_get_current (miter));
-
-				me->folder_observables = g_list_prepend (me->folder_observables, folder);
-				me->store_observables = g_list_prepend (me->store_observables, folder);
-
-				gtk_tree_store_append (GTK_TREE_STORE (model), &newiter, &iter);
-
-				/* This adds a reference count to folder_store too. When it gets 
-				   removed, that reference count is decreased automatically by 
-				   the gtktreestore infrastructure. */
-
-				gtk_tree_store_set (GTK_TREE_STORE (model), &newiter,
-					TNY_GTK_FOLDER_STORE_TREE_MODEL_NAME_COLUMN, 
-					tny_folder_get_name (TNY_FOLDER (folder)),
-					TNY_GTK_FOLDER_STORE_TREE_MODEL_UNREAD_COLUMN, 
-					tny_folder_get_unread_count (TNY_FOLDER (folder)),
-					TNY_GTK_FOLDER_STORE_TREE_MODEL_ALL_COLUMN, 
-					tny_folder_get_all_count (TNY_FOLDER (folder)),
-					TNY_GTK_FOLDER_STORE_TREE_MODEL_TYPE_COLUMN,
-					tny_folder_get_folder_type (TNY_FOLDER (folder)),
-					TNY_GTK_FOLDER_STORE_TREE_MODEL_INSTANCE_COLUMN,
-					folder, -1);
-
-				g_object_unref (folder);
-				tny_iterator_next (miter);
-			}
-			g_object_unref (miter);
-			g_object_unref (created);
-		}
-		g_object_unref (G_OBJECT (parentstore));
-	}
-
-	if (changed & TNY_FOLDER_STORE_CHANGE_CHANGED_REMOVED_FOLDERS)
-	{
-		TnyList *removed = tny_simple_list_new ();
-		TnyIterator *miter;
-
-		tny_folder_store_change_get_removed_folders (change, removed);
-		miter = tny_list_create_iterator (removed);
-
+		miter = tny_list_create_iterator (list);
 		while (!tny_iterator_is_done (miter))
 		{
 			TnyFolder *folder = TNY_FOLDER (tny_iterator_get_current (miter));
@@ -967,9 +1109,29 @@ tny_gtk_folder_store_tree_model_store_obsr_update (TnyFolderStoreObserver *self,
 			tny_iterator_next (miter);
 		}
 		g_object_unref (miter);
-		g_object_unref (removed);
+}
+
+static void
+tny_gtk_folder_store_tree_model_store_obsr_update (TnyFolderStoreObserver *self, TnyFolderStoreChange *change)
+{
+	TnyFolderStoreChangeChanged changed = tny_folder_store_change_get_changed (change);
+	GtkTreeModel *model = GTK_TREE_MODEL (self);
+
+	if (changed & TNY_FOLDER_STORE_CHANGE_CHANGED_CREATED_FOLDERS) {
+		/* TnyList *created = tny_simple_list_new ();
+		 * tny_folder_store_change_get_created_folders (change, created);
+		 * delete_these_folders (model, created);
+		 * g_object_unref (created); */
+		gtk_tree_model_foreach (model, creater, change);
 	}
 
+	if (changed & TNY_FOLDER_STORE_CHANGE_CHANGED_REMOVED_FOLDERS)
+	{
+		TnyList *removed = tny_simple_list_new ();
+		tny_folder_store_change_get_removed_folders (change, removed);
+		delete_these_folders (model, removed);
+		g_object_unref (removed);
+	}
 
 	return;
 }
@@ -977,25 +1139,25 @@ tny_gtk_folder_store_tree_model_store_obsr_update (TnyFolderStoreObserver *self,
 static void
 tny_folder_store_observer_init (TnyFolderStoreObserverIface *klass)
 {
-	klass->update_func = tny_gtk_folder_store_tree_model_store_obsr_update;
+	klass->update= tny_gtk_folder_store_tree_model_store_obsr_update;
 }
 
 static void
 tny_folder_observer_init (TnyFolderObserverIface *klass)
 {
-	klass->update_func = tny_gtk_folder_store_tree_model_folder_obsr_update;
+	klass->update= tny_gtk_folder_store_tree_model_folder_obsr_update;
 }
 
 static void
 tny_list_init (TnyListIface *klass)
 {
-	klass->get_length_func = tny_gtk_folder_store_tree_model_get_length;
-	klass->prepend_func = tny_gtk_folder_store_tree_model_prepend_i;
-	klass->append_func = tny_gtk_folder_store_tree_model_append_i;
-	klass->remove_func = tny_gtk_folder_store_tree_model_remove;
-	klass->create_iterator_func = tny_gtk_folder_store_tree_model_create_iterator;
-	klass->copy_func = tny_gtk_folder_store_tree_model_copy_the_list;
-	klass->foreach_func = tny_gtk_folder_store_tree_model_foreach_in_the_list;
+	klass->get_length= tny_gtk_folder_store_tree_model_get_length;
+	klass->prepend= tny_gtk_folder_store_tree_model_prepend_i;
+	klass->append= tny_gtk_folder_store_tree_model_append_i;
+	klass->remove= tny_gtk_folder_store_tree_model_remove;
+	klass->create_iterator= tny_gtk_folder_store_tree_model_create_iterator;
+	klass->copy= tny_gtk_folder_store_tree_model_copy_the_list;
+	klass->foreach= tny_gtk_folder_store_tree_model_foreach_in_the_list;
 
 	return;
 }
@@ -1005,7 +1167,7 @@ tny_list_init (TnyListIface *klass)
  *
  * GType system helper function
  *
- * Return value: a GType
+ * returns: a #GType
  **/
 GType
 tny_gtk_folder_store_tree_model_get_type (void)
@@ -1027,9 +1189,6 @@ tny_gtk_folder_store_tree_model_get_type (void)
 		  tny_gtk_folder_store_tree_model_instance_init    /* instance_init */
 		};
 
-		type = g_type_register_static (GTK_TYPE_TREE_STORE, "TnyGtkFolderStoreTreeModel",
-					    &info, 0);
-
 		static const GInterfaceInfo tny_list_info = {
 			(GInterfaceInitFunc) tny_list_init,
 			NULL,
@@ -1048,6 +1207,9 @@ tny_gtk_folder_store_tree_model_get_type (void)
 			NULL
 		};
 
+		type = g_type_register_static (GTK_TYPE_TREE_STORE, "TnyGtkFolderStoreTreeModel",
+					    &info, 0);
+
 		g_type_add_interface_static (type, TNY_TYPE_LIST,
 					     &tny_list_info);
 		g_type_add_interface_static (type, TNY_TYPE_FOLDER_STORE_OBSERVER,
@@ -1066,7 +1228,7 @@ tny_gtk_folder_store_tree_model_get_type (void)
  *
  * GType system helper function
  *
- * Return value: a GType
+ * returns: a #GType
  **/
 GType
 tny_gtk_folder_store_tree_model_column_get_type (void)
