@@ -13,15 +13,34 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with self library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
+/**
+ * TnyGtkFolderStoreTreeModel:
+ *
+ * A #GtkTreeModel for #TnyHeader instances. Just like with any other #TnyList,
+ * you can use it together with a #TnyFolderMonitor. You will typically fill
+ * instances of this type using tny_folder_get_headers().
+ *
+ * Note that a #TnyGtkFolderStoreTreeModel is a #TnyList too. You can use the
+ * #TnyList API on instances of this type too. This is what #TnyFolderMonitor
+ * does too.
+ *
+ * Note that you must make sure that you unreference #TnyHeader instances that 
+ * you get out of the instance column of this type using the #GtkTreeModel API
+ * gtk_tree_model_get().
+ *
+ * free-function: g_object_unref
+ **/
+
 #include <config.h>
 
 #include <glib.h>
 #include <glib/gi18n-lib.h>
 #include <gdk/gdk.h>
+#include <gtk/gtk.h>
 
 #include <tny-gtk-header-list-model.h>
 #include <tny-gtk-header-list-iterator-priv.h>
@@ -39,6 +58,10 @@ static GObjectClass *parent_class;
 
 #include "tny-gtk-header-list-iterator-priv.h"
 
+#define TINYMAIL_ENABLE_PRIVATE_API
+#include "tny-common-priv.h"
+#undef TINYMAIL_ENABLE_PRIVATE_API
+
 
 static gint 
 add_del_timeout (TnyGtkHeaderListModel *me, guint num)
@@ -215,14 +238,16 @@ _get_readable_date (time_t file_time_raw)
 
 /**
  * tny_gtk_header_list_model_received_date_sort_func:
- * @model: The GtkTreeModel the comparison is within
- * @a : A GtkTreeIter in model
- * @b : Another GtkTreeIter in model
- * @user_data: Data passed
+ * @model: a #GtkTreeModel the comparison is within
+ * @a: a #GtkTreeIter in @model
+ * @b: another #GtkTreeIter in @model
+ * @user_data: (null-ok): user data passed
  *
- * A GtkTreeIterCompareFunc that sorts using the received date
+ * A #GtkTreeIterCompareFunc that sorts using the received date
  *
- * Return value: a negative integer, zero, or a positive integer 
+ * returns: a negative integer, zero, or a positive integer 
+ * since: 1.0
+ * audience: application-developer, tinymail-developer
  **/
 gint 
 tny_gtk_header_list_model_received_date_sort_func (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer user_data)
@@ -247,14 +272,16 @@ tny_gtk_header_list_model_received_date_sort_func (GtkTreeModel *model, GtkTreeI
 
 /**
  * tny_gtk_header_list_model_sent_date_sort_func:
- * @model: The GtkTreeModel the comparison is within
- * @a : A GtkTreeIter in model
- * @b : Another GtkTreeIter in model
- * @user_data: Data passed
+ * @model: the #GtkTreeModel the comparison is within
+ * @a: a GtkTreeIter in @model
+ * @b: another GtkTreeIter in @model
+ * @user_data: (null-ok): user data passed
  *
- * A GtkTreeIterCompareFunc that sorts using the sent date
+ * A @GtkTreeIterCompareFunc that sorts using the sent date
  *
- * Return value: a negative integer, zero, or a positive integer 
+ * returns: a negative integer, zero, or a positive integer 
+ * since: 1.0
+ * audience: application-developer, tinymail-developer
  **/
 gint  
 tny_gtk_header_list_model_sent_date_sort_func (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer user_data)
@@ -277,6 +304,43 @@ tny_gtk_header_list_model_sent_date_sort_func (GtkTreeModel *model, GtkTreeIter
 	return (recv_a - recv_b);
 }
 
+
+static void 
+set_dummy (gint column, GValue *value)
+{
+	switch (column) 
+	{
+		case TNY_GTK_HEADER_LIST_MODEL_TO_COLUMN:
+		case TNY_GTK_HEADER_LIST_MODEL_SUBJECT_COLUMN:
+		case TNY_GTK_HEADER_LIST_MODEL_FROM_COLUMN:
+		case TNY_GTK_HEADER_LIST_MODEL_DATE_RECEIVED_COLUMN:
+		case TNY_GTK_HEADER_LIST_MODEL_DATE_SENT_COLUMN:
+		case TNY_GTK_HEADER_LIST_MODEL_CC_COLUMN:
+			g_value_init (value, G_TYPE_STRING);
+			g_value_set_string (value, "Expunged");
+			break;
+		case TNY_GTK_HEADER_LIST_MODEL_DATE_SENT_TIME_T_COLUMN:
+		case TNY_GTK_HEADER_LIST_MODEL_DATE_RECEIVED_TIME_T_COLUMN:
+		case TNY_GTK_HEADER_LIST_MODEL_MESSAGE_SIZE_COLUMN:
+			g_value_init (value, G_TYPE_INT);
+			g_value_set_int (value, -1);
+			break;
+		case TNY_GTK_HEADER_LIST_MODEL_INSTANCE_COLUMN: {
+			static TnyHeader *header = NULL;
+			if (!header) 
+				header = tny_expunged_header_new ();
+			g_value_init (value, G_TYPE_OBJECT);
+			g_value_set_object (value, header);
+			} break;
+		case TNY_GTK_HEADER_LIST_MODEL_FLAGS_COLUMN:
+			g_value_init (value, G_TYPE_INT);
+			g_value_set_int (value, TNY_HEADER_FLAG_EXPUNGED);
+			break;
+		default:
+			break;
+	}
+}
+
 static void
 tny_gtk_header_list_model_get_value (GtkTreeModel *self, GtkTreeIter *iter, gint column, GValue *value)
 {
@@ -286,8 +350,11 @@ tny_gtk_header_list_model_get_value (GtkTreeModel *self, GtkTreeIter *iter, gint
 	gchar *rdate = NULL;
 	gint i;
 
-	if (iter->stamp != priv->stamp)
+	if (iter->stamp != priv->stamp) {
+		g_warning ("GtkTreeModel in invalid state\n");
+		set_dummy (column, value);
 		return;
+	}
 
 	g_static_rec_mutex_lock (priv->iterator_lock);
 
@@ -302,13 +369,25 @@ tny_gtk_header_list_model_get_value (GtkTreeModel *self, GtkTreeIter *iter, gint
 	 * length, although in 99.999999% of the cases it's the same) */
 	if (i < 0 || i >= priv->items->len)
 	{
+		set_dummy (column, value);
 		g_warning ("GtkTreeModel in invalid state\n");
 		g_static_rec_mutex_unlock (priv->iterator_lock);
 		return;
 	}
 
-	if (priv->items->pdata[i] == NULL)
+	if (priv->items->pdata[i] == NULL) {
+		g_warning ("GtkTreeModel in invalid state\n");
+		set_dummy (column, value);
+		g_static_rec_mutex_unlock (priv->iterator_lock);
 		return;
+	}
+
+	if (!TNY_IS_HEADER (priv->items->pdata[i])) {
+		g_warning ("GtkTreeModel in invalid state\n");
+		set_dummy (column, value);
+		g_static_rec_mutex_unlock (priv->iterator_lock);
+		return;
+	}
 
 	switch (column) 
 	{
@@ -479,11 +558,11 @@ tny_gtk_header_list_model_iter_nth_child (GtkTreeModel *self, GtkTreeIter *iter,
 	return retval;
 }
 
-static gboolean                                                                                                                                                             
+static gboolean
 tny_gtk_header_list_model_iter_children (GtkTreeModel *tree_model,
 					 GtkTreeIter  *iter,
 					 GtkTreeIter  *parent) 
-{                                                                                                                                                                           
+{
 	gboolean retval = FALSE;
 
 	retval = tny_gtk_header_list_model_iter_nth_child (tree_model, iter, parent, 0);
@@ -521,13 +600,56 @@ notify_views_add_destroy (gpointer data)
 	g_mutex_unlock (priv->ra_lock);
 
 	if (priv->timeout_span < 5000)
-		priv->timeout_span += 300;
+		priv->timeout_span += 500;
 	priv->add_timeout = 0;
 	g_object_unref (data);
 
 	return;
 }
 
+#if GTK_CHECK_VERSION (2,16,0)
+
+static inline GtkTreePath *
+gtk_tree_path_new_internal (gint index)
+{
+	GtkTreePath *path = gtk_tree_path_new ();
+	gtk_tree_path_append_index (path, index);
+	return path;
+}
+static inline void
+gtk_tree_path_free_internal (GtkTreePath *path_in)
+{
+	gtk_tree_path_free (path_in);
+}
+
+#else
+
+typedef struct 
+{
+  gint depth;
+  gint *indices;
+}GtkTreePathInternal;
+
+static inline GtkTreePath *
+gtk_tree_path_new_internal (gint index)
+{
+  GtkTreePathInternal *retval = g_slice_new (GtkTreePathInternal);
+  retval->depth = 1;
+  retval->indices = g_slice_alloc (sizeof(gint));
+  retval->indices[0] = index;
+  return (GtkTreePath *) retval;
+}
+
+static inline void
+gtk_tree_path_free_internal (GtkTreePath *path_in)
+{
+  GtkTreePathInternal *path = (GtkTreePathInternal *) path_in;
+  g_slice_free1 (sizeof(gint), path->indices);
+  g_slice_free (GtkTreePathInternal, path);
+}
+
+#endif
+
 static gboolean
 notify_views_add (gpointer data)
 {
@@ -547,9 +669,9 @@ notify_views_add (gpointer data)
 
 	already_registered = priv->registered;
 
-	if (priv->items->len - already_registered > 1000) 
+	if (priv->items->len - already_registered > 3000) 
 	{
-		going_tb_registered = already_registered + 1000;
+		going_tb_registered = already_registered + 3000;
 		needmore = TRUE;
 	} else
 		going_tb_registered = priv->items->len;
@@ -567,14 +689,10 @@ notify_views_add (gpointer data)
 	{
 		iter.stamp = priv->stamp;
 		iter.user_data = (gpointer) i;
-		path = gtk_tree_path_new ();
-		gtk_tree_path_append_index (path, i);
-		if (G_LIKELY (path))
-		{
-			priv->cur_len = i+1;
-			gtk_tree_model_row_inserted ((GtkTreeModel *) data, path, &iter);
-			gtk_tree_path_free (path);
-		}
+		path = gtk_tree_path_new_internal (i);
+		priv->cur_len = i+1;
+		gtk_tree_model_row_inserted ((GtkTreeModel *) data, path, &iter);
+		gtk_tree_path_free_internal (path);
 	}
 	gdk_threads_leave();
 
@@ -644,6 +762,14 @@ typedef struct
 	gint src;
 } notify_views_data_t;
 
+typedef struct
+{
+	TnyGtkHeaderListModel *self;
+	GList *items;
+	/* GMainLoop *loop; */
+	gint src;
+} notify_views_data_list_t;
+
 
 static void 
 notify_views_delete_destroy (gpointer data)
@@ -736,6 +862,114 @@ tny_gtk_header_list_model_remove (TnyList *self, GObject* item)
 	return;
 }
 
+
+static void 
+notify_views_delete_destroy_list (gpointer data)
+{
+	notify_views_data_list_t *stuff = data;
+	TnyGtkHeaderListModelPriv *priv = TNY_GTK_HEADER_LIST_MODEL_GET_PRIVATE (stuff->self);
+
+	g_mutex_lock (priv->to_lock);
+	if (stuff->src != -1 && stuff->src < priv->del_timeouts->len)
+		g_array_index (priv->del_timeouts, guint, (guint) stuff->src) = 0;
+	g_mutex_unlock (priv->to_lock);
+
+	while (stuff->items) {
+		g_object_unref (stuff->items->data);
+		stuff->items = g_list_next (stuff->items);
+	}
+
+	if (stuff->items)
+		g_list_free (stuff->items);
+
+	g_object_unref (stuff->self);
+	/* g_main_loop_unref (stuff->loop); */
+
+	g_slice_free (notify_views_data_list_t, data);
+	return;
+}
+
+
+static gboolean
+notify_views_delete_list (gpointer data)
+{
+	notify_views_data_list_t *stuff = data;
+	TnyGtkHeaderListModelPriv *priv = TNY_GTK_HEADER_LIST_MODEL_GET_PRIVATE (stuff->self);
+	GList *copy = stuff->items;
+
+	g_static_rec_mutex_lock (priv->iterator_lock);
+
+	while (copy) 
+	{
+		gint i; gboolean found = FALSE;
+		GtkTreePath *path;
+		GtkTreeIter iter;
+		GObject *mitem, *item = copy->data;
+
+		for (i=0; i < priv->items->len; i++)
+			if (priv->items->pdata[i] == item)
+			{
+				found = TRUE;
+				break;
+			}
+
+		if (found)
+		{
+			iter.stamp = priv->stamp;
+			iter.user_data = (gpointer) i;
+			path = gtk_tree_path_new ();
+			gtk_tree_path_append_index (path, i);
+			gtk_tree_model_row_deleted ((GtkTreeModel *) stuff->self, path);
+			priv->stamp++;
+			g_mutex_lock (priv->ra_lock);
+			priv->cur_len--;
+			priv->registered--;
+			g_mutex_unlock (priv->ra_lock);
+			gtk_tree_path_free (path);
+
+			mitem = g_ptr_array_remove_index (priv->items, i);
+			if (mitem)
+				g_object_unref (mitem);
+		}
+
+		copy = g_list_next (copy);
+	}
+
+	g_static_rec_mutex_unlock (priv->iterator_lock);
+
+
+	return FALSE;
+}
+
+
+static void
+tny_gtk_header_list_model_remove_matches (TnyList *self, TnyListMatcher matcher, gpointer match_data)
+{
+	TnyGtkHeaderListModelPriv *priv = TNY_GTK_HEADER_LIST_MODEL_GET_PRIVATE (self);
+	int i; GList *items = NULL;
+
+	for (i=0; i < priv->items->len; i++) {
+		if (matcher (self, priv->items->pdata[i], match_data))
+			items = g_list_append (items, g_object_ref (priv->items->pdata[i]));
+	}
+
+
+	if (items) 
+	{
+		notify_views_data_list_t *stuff;
+		guint src;
+
+		stuff = g_slice_new (notify_views_data_list_t);
+		stuff->src = -1;
+		stuff->self = g_object_ref (self);
+		stuff->items = items;
+		src = g_timeout_add_full (G_PRIORITY_HIGH_IDLE, 0,
+			notify_views_delete_list, stuff, notify_views_delete_destroy_list);
+		stuff->src = (gint) add_del_timeout ((TnyGtkHeaderListModel *) self, src);
+	}
+}
+
+
 static guint
 tny_gtk_header_list_model_get_length (TnyList *self)
 {
@@ -769,6 +1003,7 @@ tny_gtk_header_list_model_copy_the_list (TnyList *self)
 	TnyGtkHeaderListModelPriv *priv = TNY_GTK_HEADER_LIST_MODEL_GET_PRIVATE (self);
 	TnyGtkHeaderListModel *copy = g_object_new (TNY_TYPE_GTK_HEADER_LIST_MODEL, NULL);
 	TnyGtkHeaderListModelPriv *cpriv = TNY_GTK_HEADER_LIST_MODEL_GET_PRIVATE (copy);
+	GPtrArray *items_copy = NULL;
 
 	/* This only copies the TnyList pieces. The result is not a
 	   correct or good TnyGtkHeaderListModel. But it will be a correct
@@ -779,8 +1014,7 @@ tny_gtk_header_list_model_copy_the_list (TnyList *self)
 	   only a copy of the list-nodes of course. */
 
 	g_static_rec_mutex_lock (priv->iterator_lock);
-
-	GPtrArray *items_copy = g_ptr_array_sized_new (priv->items->len);
+	items_copy = g_ptr_array_sized_new (priv->items->len);
 	g_ptr_array_foreach (priv->items, copy_it, items_copy);
 	cpriv->items = items_copy;
 	g_static_rec_mutex_unlock (priv->iterator_lock);
@@ -803,13 +1037,14 @@ tny_gtk_header_list_model_foreach_in_the_list (TnyList *self, GFunc func, gpoint
 static void
 tny_list_init (TnyListIface *klass)
 {
-	klass->get_length_func = tny_gtk_header_list_model_get_length;
-	klass->prepend_func = tny_gtk_header_list_model_prepend;
-	klass->append_func = tny_gtk_header_list_model_append;
-	klass->remove_func = tny_gtk_header_list_model_remove;
-	klass->create_iterator_func = tny_gtk_header_list_model_create_iterator;
-	klass->copy_func = tny_gtk_header_list_model_copy_the_list;
-	klass->foreach_func = tny_gtk_header_list_model_foreach_in_the_list;
+	klass->get_length= tny_gtk_header_list_model_get_length;
+	klass->prepend= tny_gtk_header_list_model_prepend;
+	klass->append= tny_gtk_header_list_model_append;
+	klass->remove= tny_gtk_header_list_model_remove;
+	klass->create_iterator= tny_gtk_header_list_model_create_iterator;
+	klass->copy= tny_gtk_header_list_model_copy_the_list;
+	klass->foreach= tny_gtk_header_list_model_foreach_in_the_list;
+	klass->remove_matches= tny_gtk_header_list_model_remove_matches;
 
 	return;
 }
@@ -857,7 +1092,8 @@ tny_gtk_header_list_model_finalize (GObject *object)
 
 	g_static_rec_mutex_unlock (priv->iterator_lock);
 
-	g_static_rec_mutex_free (priv->iterator_lock);
+	/* g_static_rec_mutex_free (priv->iterator_lock); */
+	g_free (priv->iterator_lock);
 	priv->iterator_lock = NULL;
 
 	g_mutex_free (priv->ra_lock);
@@ -916,15 +1152,17 @@ tny_gtk_header_list_model_init (TnyGtkHeaderListModel *self)
 
 /**
  * tny_gtk_header_list_model_set_folder:
- * @self: A #TnyGtkHeaderListModel instance
- * @folder: a #TnyFolder instance
+ * @self: a #TnyGtkHeaderListModel
+ * @folder: a #TnyFolder
  * @refresh: refresh first
- * @callback: a #TnyGetHeadersCallback
- * @status_callback: a #TnyStatusCallback
- * @user_data: user data for the callbacks
+ * @callback: (null-ok): a #TnyGetHeadersCallback or NULL
+ * @status_callback: (null-ok): a #TnyStatusCallback or NULL
+ * @user_data: (null-ok): user data for the callbacks
  *
- * Set the folder where the #TnyHeader instances are located
+ * Set the @folder of @self where #TnyHeader instances are located
  * 
+ * since: 1.0
+ * audience: application-developer
  **/
 void
 tny_gtk_header_list_model_set_folder (TnyGtkHeaderListModel *self, TnyFolder *folder, gboolean refresh, TnyGetHeadersCallback callback, TnyStatusCallback status_callback, gpointer user_data)
@@ -984,11 +1222,11 @@ tny_gtk_header_list_model_set_folder (TnyGtkHeaderListModel *self, TnyFolder *fo
 /**
  * tny_gtk_header_list_model_new:
  *
- * Create a new #GtkTreeModel instance suitable for showing lots of 
- * #TnyHeader instances
+ * Create a new #GtkTreeModel for #TnyHeader instances
  *
- * Return value: a new #GtkTreeModel instance suitable for showing lots of 
- * #TnyHeader instances
+ * returns: (caller-owns): a new #GtkTreeModel for #TnyHeader instances
+ * since: 1.0
+ * audience: application-developer
  **/
 GtkTreeModel*
 tny_gtk_header_list_model_new (void)
@@ -1003,7 +1241,7 @@ tny_gtk_header_list_model_new (void)
  *
  * GType system helper function
  *
- * Return value: a GType
+ * returns: a #GType
  **/
 GType
 tny_gtk_header_list_model_get_type (void)
@@ -1058,7 +1296,7 @@ tny_gtk_header_list_model_get_type (void)
  *
  * GType system helper function
  *
- * Return value: a GType
+ * returns: a #GType
  **/
 GType
 tny_gtk_header_list_model_column_get_type (void)
