@@ -13,10 +13,20 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with self library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
+/**
+ * TnyGtkMsgView:
+ *
+ * a #TnyMsgView for showing a message in Gtk+. It's recommended to wrap instances
+ * of this type into a #GtkScrolledWindow.
+ *
+ * free-function: g_object_unref
+ **/
+
+
 #include <config.h>
 
 #include <glib/gi18n-lib.h>
@@ -40,8 +50,10 @@
 #include <tny-gtk-attach-list-model.h>
 #include <tny-header-view.h>
 #include <tny-gtk-header-view.h>
+#include <tny-gtk-image-mime-part-view.h>
 #include <tny-gtk-text-mime-part-view.h>
 #include <tny-gtk-attachment-mime-part-view.h>
+#include <tny-gtk-expander-mime-part-view.h>
 #include <tny-mime-part-saver.h>
 
 #ifdef GNOME
@@ -70,7 +82,9 @@ struct _TnyGtkMsgViewPriv
 	gboolean display_attachments;
 	gboolean display_rfc822;
 	gboolean first_attachment;
-	GtkBox *kid; gboolean in_expander;
+	GtkBox *kid; gboolean in_expander, parented;
+	TnyStatusCallback status_callback;
+	gpointer status_user_data;
 };
 
 typedef struct
@@ -90,11 +104,13 @@ static gboolean tny_gtk_msg_view_display_part (TnyMsgView *self, TnyMimePart *pa
 
 /**
  * tny_gtk_msg_view_get_display_html:
- * @self: A #TnyGtkMsgView instance
+ * @self: a #TnyGtkMsgView
  *
  * Get whether or not to display text/html mime parts
  * 
- * Return value: whether or not to display text/html mime parts
+ * returns: whether or not to display text/html mime parts
+ * since: 1.0
+ * audience: application-developer
  **/
 gboolean 
 tny_gtk_msg_view_get_display_html (TnyGtkMsgView *self)
@@ -105,11 +121,13 @@ tny_gtk_msg_view_get_display_html (TnyGtkMsgView *self)
 
 /**
  * tny_gtk_msg_view_get_display_rfc822:
- * @self: A #TnyGtkMsgView instance
+ * @self: a #TnyGtkMsgView
  *
  * Get whether or not to display message/rfc822 mime parts
  *
- * Return value: whether or not to display message/rfc822 mime parts
+ * returns: whether or not to display message/rfc822 mime parts
+ * since: 1.0
+ * audience: application-developer
  **/
 gboolean 
 tny_gtk_msg_view_get_display_rfc822 (TnyGtkMsgView *self)
@@ -120,11 +138,13 @@ tny_gtk_msg_view_get_display_rfc822 (TnyGtkMsgView *self)
 
 /**
  * tny_gtk_msg_view_get_display_attachments:
- * @self: A #TnyGtkMsgView instance
+ * @self: a #TnyGtkMsgView
  *
  * Get whether or not to display attachments
  *
- * Return value: whether or not to display attachments
+ * returns: whether or not to display attachments
+ * since: 1.0
+ * audience: application-developer
  **/
 gboolean 
 tny_gtk_msg_view_get_display_attachments (TnyGtkMsgView *self)
@@ -135,11 +155,13 @@ tny_gtk_msg_view_get_display_attachments (TnyGtkMsgView *self)
 
 /**
  * tny_gtk_msg_view_get_display_plain:
- * @self: A #TnyGtkMsgView instance
+ * @self: a #TnyGtkMsgView
  *
  * Get whether or not to display text/plain mime parts
  * 
- * Return value: whether or not to display text/plain mime parts
+ * returns: whether or not to display text/plain mime parts
+ * since: 1.0
+ * audience: application-developer
  **/
 gboolean 
 tny_gtk_msg_view_get_display_plain (TnyGtkMsgView *self)
@@ -151,20 +173,25 @@ tny_gtk_msg_view_get_display_plain (TnyGtkMsgView *self)
 
 /**
  * tny_gtk_msg_view_set_display_html:
- * @self: A #TnyGtkMsgView instance
+ * @self: a #TnyGtkMsgView
  * @setting: whether or not to display text/html mime parts
  *
  * With this setting will the default implementation of #TnyGtkMsgView display
  * the HTML source code of text/html mime parts. Default is FALSE.
  *
  * Note that these settings only affect the instance in case an overridden
- * implementation of tny_msg_view_create_mime_part_view_for doesn't handle
+ * implementation of tny_msg_view_create_mime_part_view_for() doesn't handle
  * creating a viewer for a mime part.
  * 
  * So for example in case a more advanced implementation that inherits this
  * type implements viewing a text/html mime part, and will therefore not call
- * this types original tny_msg_view_create_mime_part_view_for method for the
- * mime part anymore, the setting isn't used.
+ * this types original tny_msg_view_create_mime_part_view_for() method for
+ * the mime part anymore, the setting isn't used.
+ *
+ * The effect, by default, of this setting is showing the HTML source code.
+ *
+ * since: 1.0
+ * audience: application-developer
  **/
 void 
 tny_gtk_msg_view_set_display_html (TnyGtkMsgView *self, gboolean setting)
@@ -176,22 +203,23 @@ tny_gtk_msg_view_set_display_html (TnyGtkMsgView *self, gboolean setting)
 
 /**
  * tny_gtk_msg_view_set_display_rfc822:
- * @self: A #TnyGtkMsgView instance
+ * @self: a #TnyGtkMsgView
  * @setting: whether or not to display message/rfc822 mime parts
  *
  * With this setting will the default implementation of #TnyGtkMsgView display
  * RFC822 inline message mime parts (forwards). Default is FALSE.
  *
- * (Note that this support is unimplemented at this moment)
- *
  * Note that these settings only affect the instance in case an overridden
- * implementation of tny_msg_view_create_mime_part_view_for doesn't handle
+ * implementation of tny_msg_view_create_mime_part_view_for() doesn't handle
  * creating a viewer for a mime part.
  * 
  * So for example in case a more advanced implementation that inherits this
  * type implements viewing a text/html mime part, and will therefore not call
- * this types original tny_msg_view_create_mime_part_view_for method for the
- * mime part anymore, the setting isn't used.
+ * this types original tny_msg_view_create_mime_part_view_for() method for
+ * the mime part anymore, the setting isn't used.
+ *
+ * since: 1.0
+ * audience: application-developer
  **/
 void 
 tny_gtk_msg_view_set_display_rfc822 (TnyGtkMsgView *self, gboolean setting)
@@ -204,7 +232,7 @@ tny_gtk_msg_view_set_display_rfc822 (TnyGtkMsgView *self, gboolean setting)
 
 /**
  * tny_gtk_msg_view_set_display_attachments:
- * @self: A #TnyGtkMsgView instance
+ * @self: a #TnyGtkMsgView
  * @setting: whether or not to display attachment mime parts
  *
  * With this setting will the default implementation of #TnyGtkMsgView display
@@ -212,13 +240,16 @@ tny_gtk_msg_view_set_display_rfc822 (TnyGtkMsgView *self, gboolean setting)
  * of the #TnyGtkMsgView's scrollwindow. Default is TRUE.
  *
  * Note that these settings only affect the instance in case an overridden
- * implementation of tny_msg_view_create_mime_part_view_for doesn't handle
+ * implementation of tny_msg_view_create_mime_part_view_for() doesn't handle
  * creating a viewer for a mime part.
  * 
  * So for example in case a more advanced implementation that inherits this
  * type implements viewing a text/html mime part, and will therefore not call
- * this types original tny_msg_view_create_mime_part_view_for method for the
- * mime part anymore, the setting isn't used.
+ * this types original tny_msg_view_create_mime_part_view_for() method for
+ * the mime part anymore, the setting isn't used.
+ *
+ * since: 1.0
+ * audience: application-developer
  **/
 void 
 tny_gtk_msg_view_set_display_attachments (TnyGtkMsgView *self, gboolean setting)
@@ -237,13 +268,16 @@ tny_gtk_msg_view_set_display_attachments (TnyGtkMsgView *self, gboolean setting)
  * text/plain mime parts. Default is TRUE.
  *
  * Note that these settings only affect the instance in case an overridden
- * implementation of tny_msg_view_create_mime_part_view_for doesn't handle
+ * implementation of tny_msg_view_create_mime_part_view_for() doesn't handle
  * creating a viewer for a mime part.
  * 
  * So for example in case a more advanced implementation that inherits this
  * type implements viewing a text/html mime part, and will therefore not call
- * this types original tny_msg_view_create_mime_part_view_for method for the
- * mime part anymore, the setting isn't used.
+ * this types original tny_msg_view_create_mime_part_view_for() method for
+ * the mime part anymore, the setting isn't used.
+ *
+ * since: 1.0
+ * audience: application-developer
  **/
 void 
 tny_gtk_msg_view_set_display_plain (TnyGtkMsgView *self, gboolean setting)
@@ -258,7 +292,7 @@ tny_gtk_msg_view_set_display_plain (TnyGtkMsgView *self, gboolean setting)
 static void
 tny_gtk_msg_view_set_unavailable (TnyMsgView *self)
 {
-	TNY_GTK_MSG_VIEW_GET_CLASS (self)->set_unavailable_func (self);
+	TNY_GTK_MSG_VIEW_GET_CLASS (self)->set_unavailable(self);
 }
 
 static void
@@ -278,7 +312,7 @@ tny_gtk_msg_view_set_unavailable_default (TnyMsgView *self)
 static TnyMsg*
 tny_gtk_msg_view_get_msg (TnyMsgView *self)
 {
-	return TNY_GTK_MSG_VIEW_GET_CLASS (self)->get_msg_func (self);
+	return TNY_GTK_MSG_VIEW_GET_CLASS (self)->get_msg(self);
 }
 
 static TnyMsg* 
@@ -293,26 +327,92 @@ tny_gtk_msg_view_get_msg_default (TnyMsgView *self)
 static TnyMsgView*
 tny_gtk_msg_view_create_new_inline_viewer (TnyMsgView *self)
 {
-	return TNY_GTK_MSG_VIEW_GET_CLASS (self)->create_new_inline_viewer_func (self);
+	return TNY_GTK_MSG_VIEW_GET_CLASS (self)->create_new_inline_viewer(self);
+}
+
+/**
+ * tny_gtk_msg_view_set_parented:
+ * @self: a #TnyGtkMsgView
+ * @parented: parented or not
+ *
+ * Set @self as parented. Usually internally used.
+ *
+ * since: 1.0
+ * audience: type-implementer, tinymail-developer
+ **/
+void 
+tny_gtk_msg_view_set_parented (TnyGtkMsgView *self, gboolean parented)
+{
+	TnyGtkMsgViewPriv *priv = TNY_GTK_MSG_VIEW_GET_PRIVATE (self);
+	priv->parented = TRUE;
+	return;
+}
+
+/**
+ * tny_gtk_msg_view_set_status_callback:
+ * @self: a #TnyGtkMsgView
+ * @status_callback: (null-ok): a #TnyStatusCallback or NULL
+ * @status_user_data: (null-ok): user data for @status_callback
+ *
+ * Set the status callback info. This callback can be NULL and will be called
+ * when status information happens. You can for example set a progress bar's
+ * position here (for for example when downloading of a message takes place).
+ *
+ * since: 1.0
+ * audience: application-developer, type-implementer, tinymail-developer
+ **/
+void 
+tny_gtk_msg_view_set_status_callback (TnyGtkMsgView *self, TnyStatusCallback status_callback, gpointer status_user_data)
+{
+	TnyGtkMsgViewPriv *priv = TNY_GTK_MSG_VIEW_GET_PRIVATE (self);
+	priv->status_callback = status_callback;
+	priv->status_user_data = status_user_data;
+	return;
+}
+
+/**
+ * tny_gtk_msg_view_get_status_callback:
+ * @self: a #TnyGtkMsgView
+ * @status_callback: (out): byref a #TnyStatusCallback 
+ * @status_user_data: (out): byref user data for @status_callback
+ *
+ * Get the status callback info. Usually internally used.
+ *
+ * since: 1.0
+ * audience: type-implementer, tinymail-developer
+ **/
+void 
+tny_gtk_msg_view_get_status_callback (TnyGtkMsgView *self, TnyStatusCallback *status_callback, gpointer *status_user_data)
+{
+	TnyGtkMsgViewPriv *priv = TNY_GTK_MSG_VIEW_GET_PRIVATE (self);
+	*status_callback = priv->status_callback;
+	*status_user_data = priv->status_user_data;
 }
 
 static TnyMsgView*
 tny_gtk_msg_view_create_new_inline_viewer_default (TnyMsgView *self)
 {
 	TnyMsgView *retval = tny_gtk_msg_view_new ();
+	TnyGtkMsgViewPriv *priv = TNY_GTK_MSG_VIEW_GET_PRIVATE (self);
+
+	tny_gtk_msg_view_set_parented (TNY_GTK_MSG_VIEW (retval), TRUE);
+	tny_gtk_msg_view_set_status_callback (TNY_GTK_MSG_VIEW (retval), 
+		priv->status_callback, priv->status_user_data);
+
 	return retval;
 }
 
 static TnyMimePartView*
 tny_gtk_msg_view_create_mime_part_view_for (TnyMsgView *self, TnyMimePart *part)
 {
-	return TNY_GTK_MSG_VIEW_GET_CLASS (self)->create_mime_part_view_for_func (self, part);
+	return TNY_GTK_MSG_VIEW_GET_CLASS (self)->create_mime_part_view_for(self, part);
 }
 
+
 /**
  * tny_gtk_msg_view_create_mime_part_view_for_default:
- * @self: a #TnyGtkMsgView instance
- * @part: a #TnyMimePart instance
+ * @self: a #TnyGtkMsgView
+ * @part: a #TnyMimePart
  *
  * This is non-public API documentation
  *
@@ -343,22 +443,30 @@ tny_gtk_msg_view_create_mime_part_view_for_default (TnyMsgView *self, TnyMimePar
 	/* PLAIN mime part */
 	if (priv->display_plain && tny_mime_part_content_type_is (part, "text/plain"))
 	{
-		retval = tny_gtk_text_mime_part_view_new ();
+		retval = tny_gtk_text_mime_part_view_new (priv->status_callback, priv->status_user_data);
 
 	/* HTML mime part (shows HTML source code) (should be overridden in case there's
 	   support for a HTML TnyMsgView (like the TnyMozEmbedMsgView) */
 	} else if (priv->display_html && tny_mime_part_content_type_is (part, "text/html"))
 	{
-		retval = tny_gtk_text_mime_part_view_new ();
+		retval = tny_gtk_text_mime_part_view_new (priv->status_callback, priv->status_user_data);
 
 	/* Inline message RFC822 */
-	} else if (priv->display_rfc822 && (tny_mime_part_content_type_is (part, "message/rfc822")||
-		tny_mime_part_content_type_is (part, "multipart/*")))
+	} else if (tny_mime_part_content_type_is (part, "image/*"))
+	{
+		TnyMimePartView *image_view = tny_gtk_image_mime_part_view_new (priv->status_callback, priv->status_user_data);
+		const gchar *desc = tny_mime_part_get_description (part);
+		retval = tny_gtk_expander_mime_part_view_new (image_view);
+		if (!desc)
+			desc = _("Attached image");
+		gtk_expander_set_label (GTK_EXPANDER (retval), desc);
+	} else if (tny_mime_part_content_type_is (part, "multipart/*"))
 	{
 		retval = TNY_MIME_PART_VIEW (tny_msg_view_create_new_inline_viewer (self));
 
 	/* Attachments */
-	} else if (priv->display_attachments && tny_mime_part_is_attachment (part))
+	} else if ((priv->display_attachments && tny_mime_part_is_attachment (part)) || 
+		(priv->display_rfc822 && (tny_mime_part_content_type_is (part, "message/rfc822"))))
 	{
 		GtkTreeModel *model;
 
@@ -402,8 +510,8 @@ on_mpview_realize (GtkWidget *widget, gpointer user_data)
 
 /**
  * tny_gtk_msg_view_display_part:
- * @self: a #TnyGtkMsgView instance
- * @part: a #TnyMimePart instance
+ * @self: a #TnyGtkMsgView
+ * @part: a #TnyMimePart
  *
  * This is non-public API documentation
  * 
@@ -434,14 +542,15 @@ tny_gtk_msg_view_display_part (TnyMsgView *self, TnyMimePart *part, const gchar
 			if (TNY_IS_GTK_MSG_VIEW (mpview) && !GTK_IS_WINDOW (mpview) && !priv->in_expander)
 			{
 				TnyGtkMsgViewPriv *mppriv = TNY_GTK_MSG_VIEW_GET_PRIVATE (mpview);
-				const gchar *label = desc;
+				const gchar *label = tny_mime_part_get_description (part);
 				GtkWidget *expander;
 
 				mppriv->in_expander = TRUE;
 				if (label == NULL || strlen (label) <= 0)
 					label = _("Email message attachment");
 				expander = gtk_expander_new (label);
-				gtk_expander_set_expanded (GTK_EXPANDER (expander), FALSE);
+				gtk_expander_set_expanded (GTK_EXPANDER (expander), 
+					tny_mime_part_content_type_is (part, "text/*"));
 				gtk_expander_set_spacing (GTK_EXPANDER (expander), 7);
 				gtk_container_add (GTK_CONTAINER (expander), GTK_WIDGET (mpview));
 				gtk_widget_show (GTK_WIDGET (expander));
@@ -485,9 +594,10 @@ tny_gtk_msg_view_display_part (TnyMsgView *self, TnyMimePart *part, const gchar
 }
 
 
+
 /**
  * tny_gtk_msg_view_display_parts:
- * @self: a #TnyGtkMsgView instance
+ * @self: a #TnyGtkMsgView
  * @parts: a #TnyList instance containing #TnyMimePart instances
  * @desc: description of the parent part (if any)
  *
@@ -517,14 +627,14 @@ tny_gtk_msg_view_display_parts (TnyMsgView *self, TnyList *parts, gboolean alter
 		tny_iterator_next (iterator);
 	}
 
-	g_object_unref (G_OBJECT (iterator));
+	g_object_unref (iterator);
 
 }
 
 static void
 tny_gtk_msg_view_set_msg (TnyMsgView *self, TnyMsg *msg)
 {
-	TNY_GTK_MSG_VIEW_GET_CLASS (self)->set_msg_func (self, msg);
+	TNY_GTK_MSG_VIEW_GET_CLASS (self)->set_msg(self, msg);
 }
 
 static void 
@@ -546,7 +656,7 @@ remove_mime_part_viewer (TnyMimePartView *mpview, GtkContainer *mpviewers)
 static void
 tny_gtk_msg_view_clear (TnyMsgView *self)
 {
-	TNY_GTK_MSG_VIEW_GET_CLASS (self)->clear_func (self);
+	TNY_GTK_MSG_VIEW_GET_CLASS (self)->clear(self);
 }
 
 static void
@@ -586,14 +696,14 @@ tny_gtk_msg_view_clear_default (TnyMsgView *self)
 static void 
 tny_gtk_msg_view_mp_set_part (TnyMimePartView *self, TnyMimePart *part)
 {
-	TNY_GTK_MSG_VIEW_GET_CLASS (self)->set_part_func (self, part);
+	TNY_GTK_MSG_VIEW_GET_CLASS (self)->set_part(self, part);
 	return;
 }
 
 /**
  * tny_gtk_msg_view_mp_set_part_default:
- * @self: a #TnyGtkMsgView instance
- * @part: a #TnyMimePart instance
+ * @self: a #TnyGtkMsgView
+ * @part: a #TnyMimePart
  *
  * This is non-public API documentation
  *
@@ -625,14 +735,11 @@ tny_gtk_msg_view_mp_set_part_default (TnyMimePartView *self, TnyMimePart *part)
 
 		priv->part = g_object_ref (G_OBJECT (part));
 
-		if (!tny_mime_part_content_type_is (part, "multipart/*") && 
-			!tny_mime_part_content_type_is (part, "message/*"))
-		{
+		if (!tny_mime_part_content_type_is (part, "multipart/*")) {
 
 			tny_gtk_msg_view_display_part (TNY_MSG_VIEW (self), part, NULL);
 
-		} else
-		{
+		} else {
 			TnyList *list;
 			list = tny_simple_list_new ();
 
@@ -651,7 +758,7 @@ tny_gtk_msg_view_mp_set_part_default (TnyMimePartView *self, TnyMimePart *part)
 static TnyMimePart* 
 tny_gtk_msg_view_mp_get_part (TnyMimePartView *self)
 {
-	return TNY_GTK_MSG_VIEW_GET_CLASS (self)->get_part_func (self);
+	return TNY_GTK_MSG_VIEW_GET_CLASS (self)->get_part(self);
 }
 
 
@@ -672,9 +779,11 @@ tny_gtk_msg_view_mp_clear (TnyMimePartView *self)
 /**
  * tny_gtk_msg_view_new:
  *
- * Create a new #TnyMsgView instance implemented for Gtk+
+ * Create a new #TnyMsgView
  *
- * Return value: a new #TnyMsgView instance implemented for Gtk+
+ * returns: (caller-owns): a new #TnyMsgView 
+ * since: 1.0
+ * audience: application-developer
  **/
 TnyMsgView*
 tny_gtk_msg_view_new (void)
@@ -684,6 +793,12 @@ tny_gtk_msg_view_new (void)
 }
 
 
+static TnyHeaderView * 
+tny_gtk_msg_view_create_header_view_default (TnyGtkMsgView *self)
+{
+	return tny_gtk_header_view_new ();
+}
+
 static void
 tny_gtk_msg_view_instance_init (GTypeInstance *instance, gpointer g_class)
 {
@@ -694,6 +809,7 @@ tny_gtk_msg_view_instance_init (GTypeInstance *instance, gpointer g_class)
 	priv->kid = GTK_BOX (gtk_vbox_new (FALSE, 0));
 	vbox = priv->kid;
 
+	priv->parented = FALSE;
 	priv->in_expander = FALSE;
 
 	/* Defaults */
@@ -705,7 +821,9 @@ tny_gtk_msg_view_instance_init (GTypeInstance *instance, gpointer g_class)
 	priv->unattached_views = NULL;
 	priv->part = NULL;
 
-	priv->headerview = tny_gtk_header_view_new ();
+
+	priv->headerview = TNY_GTK_MSG_VIEW_GET_CLASS (self)->create_header_view(self);
+
 	gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (priv->headerview), FALSE, FALSE, 0);
 
 	TNY_GTK_MSG_VIEW (self)->viewers = GTK_CONTAINER (gtk_vbox_new (FALSE, 0));
@@ -779,12 +897,12 @@ tny_msg_view_init (gpointer g, gpointer iface_data)
 {
 	TnyMsgViewIface *klass = (TnyMsgViewIface *)g;
 
-	klass->get_msg_func = tny_gtk_msg_view_get_msg;
-	klass->set_msg_func = tny_gtk_msg_view_set_msg;
-	klass->set_unavailable_func = tny_gtk_msg_view_set_unavailable;
-	klass->clear_func = tny_gtk_msg_view_clear;
-	klass->create_mime_part_view_for_func = tny_gtk_msg_view_create_mime_part_view_for;
-	klass->create_new_inline_viewer_func = tny_gtk_msg_view_create_new_inline_viewer;
+	klass->get_msg= tny_gtk_msg_view_get_msg;
+	klass->set_msg= tny_gtk_msg_view_set_msg;
+	klass->set_unavailable= tny_gtk_msg_view_set_unavailable;
+	klass->clear= tny_gtk_msg_view_clear;
+	klass->create_mime_part_view_for= tny_gtk_msg_view_create_mime_part_view_for;
+	klass->create_new_inline_viewer= tny_gtk_msg_view_create_new_inline_viewer;
 
 	return;
 }
@@ -795,9 +913,9 @@ tny_mime_part_view_init (gpointer g, gpointer iface_data)
 {
 	TnyMimePartViewIface *klass = (TnyMimePartViewIface *)g;
 
-	klass->get_part_func = tny_gtk_msg_view_mp_get_part;
-	klass->set_part_func = tny_gtk_msg_view_mp_set_part;
-	klass->clear_func = tny_gtk_msg_view_mp_clear;
+	klass->get_part= tny_gtk_msg_view_mp_get_part;
+	klass->set_part= tny_gtk_msg_view_mp_set_part;
+	klass->clear= tny_gtk_msg_view_mp_clear;
 
 	return;
 }
@@ -818,14 +936,16 @@ tny_gtk_msg_view_class_init (TnyGtkMsgViewClass *class)
 
 	object_class->finalize = tny_gtk_msg_view_finalize;
 
-	class->get_part_func = tny_gtk_msg_view_mp_get_part_default;
-	class->set_part_func = tny_gtk_msg_view_mp_set_part_default;
-	class->get_msg_func = tny_gtk_msg_view_get_msg_default;
-	class->set_msg_func = tny_gtk_msg_view_set_msg_default;
-	class->set_unavailable_func = tny_gtk_msg_view_set_unavailable_default;
-	class->clear_func = tny_gtk_msg_view_clear_default;
-	class->create_mime_part_view_for_func = tny_gtk_msg_view_create_mime_part_view_for_default;
-	class->create_new_inline_viewer_func = tny_gtk_msg_view_create_new_inline_viewer_default;
+	class->get_part= tny_gtk_msg_view_mp_get_part_default;
+	class->set_part= tny_gtk_msg_view_mp_set_part_default;
+	class->get_msg= tny_gtk_msg_view_get_msg_default;
+	class->set_msg= tny_gtk_msg_view_set_msg_default;
+	class->set_unavailable= tny_gtk_msg_view_set_unavailable_default;
+	class->clear= tny_gtk_msg_view_clear_default;
+	class->create_mime_part_view_for= tny_gtk_msg_view_create_mime_part_view_for_default;
+	class->create_new_inline_viewer= tny_gtk_msg_view_create_new_inline_viewer_default;
+
+	class->create_header_view= tny_gtk_msg_view_create_header_view_default;
 
 	g_type_class_add_private (object_class, sizeof (TnyGtkMsgViewPriv));
 
@@ -838,7 +958,7 @@ tny_gtk_msg_view_class_init (TnyGtkMsgViewClass *class)
  *
  * GType system helper function
  *
- * Return value: a GType
+ * returns: a #GType
  **/
 GType 
 tny_gtk_msg_view_get_type (void)
