@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with self library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -29,6 +29,8 @@
 #include <tny-iterator.h>
 #include <tny-platform-factory.h>
 
+#include <tny-gtk-msg-view.h>
+
 #if PLATFORM==1
 #include <tny-gnome-platform-factory.h>
 #include <tny-gnome-account-store.h>
@@ -69,6 +71,7 @@
 
 #include <tny-camel-send-queue.h>
 
+
 #define GO_ONLINE_TXT _("Go online")
 #define GO_OFFLINE_TXT _("Go offline")
 
@@ -79,12 +82,6 @@ static GObjectClass *parent_class = NULL;
 #include <tny-folder-monitor.h>
 
 
-#ifdef ASYNCWORKER
-#include <tny-async-worker-queue.h>
-#include <tny-get-msg-queue.h>
-#endif
-
-
 typedef struct _TnyDemouiSummaryViewPriv TnyDemouiSummaryViewPriv;
 
 struct _TnyDemouiSummaryViewPriv
@@ -116,7 +113,7 @@ cleanup_statusbar (gpointer data)
 {
 	TnyDemouiSummaryViewPriv *priv = data;
 
-	gtk_widget_hide (GTK_WIDGET (priv->progress));
+	//gtk_widget_hide (GTK_WIDGET (priv->progress));
 	gtk_statusbar_pop (GTK_STATUSBAR (priv->status), priv->status_id);
 
 	return FALSE;
@@ -176,7 +173,7 @@ clear_header_view (TnyDemouiSummaryViewPriv *priv)
 		if (priv->monitor)
 		{
 			tny_folder_monitor_stop (priv->monitor);
-			g_object_unref (G_OBJECT (priv->monitor));
+			g_object_unref (priv->monitor);
 		}
 		priv->monitor = NULL;
 	}
@@ -282,6 +279,8 @@ load_accounts (TnySummaryView *self, gboolean first_time)
 	TnyAccountStore *account_store = priv->account_store;
 	GtkTreeModel *sortable, *maccounts, *mailbox_model;
 	TnyFolderStoreQuery *query;
+	TnyList *accounts;
+	TnyList *saccounts;
 
 	/* Show only subscribed folders */
 	query = tny_folder_store_query_new ();
@@ -292,10 +291,10 @@ load_accounts (TnySummaryView *self, gboolean first_time)
 	   TnyList and the GtkTreeModel interfaces) */
 	mailbox_model = tny_gtk_folder_store_tree_model_new (NULL);
 
-	g_object_unref (G_OBJECT (query));
+	g_object_unref (query);
 
-	TnyList *accounts = TNY_LIST (mailbox_model);
-	TnyList *saccounts = tny_simple_list_new ();
+	accounts = TNY_LIST (mailbox_model);
+	saccounts = tny_simple_list_new ();
 
 	maccounts = tny_gtk_account_list_model_new ();
 
@@ -449,20 +448,21 @@ killacc_button_toggled (GtkToggleButton *togglebutton, gpointer user_data)
 	TnyDemouiSummaryViewPriv *priv = TNY_DEMOUI_SUMMARY_VIEW_GET_PRIVATE (self);
 	TnyList *accounts = tny_simple_list_new ();
 	TnyIterator *a_iter;
+	GObject *a_cur;
 
 	tny_account_store_get_accounts (priv->account_store, accounts, 
 		TNY_ACCOUNT_STORE_STORE_ACCOUNTS);
 
 	a_iter = tny_list_create_iterator (accounts);
 
-	GObject *a_cur = tny_iterator_get_current (a_iter);
+	a_cur = tny_iterator_get_current (a_iter);
 
 	g_object_unref (a_iter);
 	g_object_unref (accounts);
 
 	printf ("%d\n", a_cur->ref_count);
 
-	/* This is indeed incorrect, don't use this */
+	/* This is indeed incorrect, don't do this (it's a test) */
 	g_object_unref (a_cur);
 	g_object_unref (a_cur);
 	g_object_unref (a_cur);
@@ -746,7 +746,7 @@ on_get_msg (TnyFolder *folder, gboolean cancelled, TnyMsg *msg, GError *merr, gp
 	if (msg) {
 		TnyHeaderFlags flags = tny_header_get_flags (header);
 		if (!(flags & TNY_HEADER_FLAG_SEEN))
-			tny_header_set_flags (header, TNY_HEADER_FLAG_SEEN);
+			tny_header_set_flag (header, TNY_HEADER_FLAG_SEEN);
 		tny_msg_view_set_msg (info->msg_view, msg);
 	} else 
 		tny_msg_view_set_unavailable (info->msg_view);
@@ -856,6 +856,23 @@ refresh_current_folder (TnyFolder *folder, gboolean cancelled, GError *err, gpoi
 }
 
 
+static void 
+set_folder_cb (TnyFolder *self, gboolean cancelled, TnyList *headers, GError *err, gpointer user_data)
+{
+	if (err) {
+		TnySummaryView *self = user_data;
+		GtkWidget *edialog;
+		edialog = gtk_message_dialog_new (
+				  GTK_WINDOW (gtk_widget_get_parent (GTK_WIDGET (self))),
+				  GTK_DIALOG_DESTROY_WITH_PARENT,
+				  GTK_MESSAGE_ERROR,
+				  GTK_BUTTONS_CLOSE,
+				  err->message);
+		g_signal_connect_swapped (edialog, "response",
+			G_CALLBACK (gtk_widget_destroy), edialog);
+		gtk_widget_show_all (edialog);
+	}
+}
 
 static void
 on_mailbox_view_tree_selection_changed (GtkTreeSelection *selection, 
@@ -909,13 +926,13 @@ on_mailbox_view_tree_selection_changed (GtkTreeSelection *selection,
 
 			hmodel = tny_gtk_header_list_model_new ();
 			tny_gtk_header_list_model_set_folder (TNY_GTK_HEADER_LIST_MODEL (hmodel), 
-				folder, FALSE, NULL, status_update, self);
+				folder, FALSE, set_folder_cb, status_update, self);
 
 			g_mutex_lock (priv->monitor_lock);
 			{
 				if (priv->monitor) {
 					tny_folder_monitor_stop (priv->monitor);
-					g_object_unref (G_OBJECT (priv->monitor));
+					g_object_unref (priv->monitor);
 				}
 				priv->monitor = TNY_FOLDER_MONITOR (tny_folder_monitor_new (folder));
 				tny_folder_monitor_add_list (priv->monitor, TNY_LIST (hmodel));
@@ -937,11 +954,15 @@ on_mailbox_view_tree_selection_changed (GtkTreeSelection *selection,
 
 			set_header_view_model (header_view, sortable);
 
+#ifndef NONASYNC_TEST
 			tny_folder_refresh_async (folder, 
 				refresh_current_folder, 
 				status_update, self);
-
-			g_object_unref (G_OBJECT (folder));
+#else
+			tny_folder_refresh (folder, NULL);
+			refresh_current_folder (folder, FALSE, NULL, self);
+#endif
+			g_object_unref (folder);
 		}
 	  }
 	} else {
@@ -980,7 +1001,7 @@ on_mailbox_view_tree_selection_changed (GtkTreeSelection *selection,
 		{
 			if (priv->monitor) {
 				tny_folder_monitor_stop (priv->monitor);
-				g_object_unref (G_OBJECT (priv->monitor));
+				g_object_unref (priv->monitor);
 			}
 			priv->monitor = TNY_FOLDER_MONITOR (tny_folder_monitor_new (merge));
 			tny_folder_monitor_add_list (priv->monitor, TNY_LIST (hmodel));
@@ -1060,6 +1081,10 @@ on_header_view_tree_row_activated (GtkTreeView *treeview, GtkTreePath *path,
 			{
 				OnGetMsgInfo *info = g_slice_new (OnGetMsgInfo);
 				info->msg_view = TNY_MSG_VIEW (tny_gtk_msg_window_new (tny_platform_factory_new_msg_view (platfact)));
+
+				if (TNY_IS_GTK_MSG_VIEW (info->msg_view))
+					tny_gtk_msg_view_set_status_callback (TNY_GTK_MSG_VIEW (info->msg_view), status_update, self);
+
 				g_object_ref (info->msg_view);
 				gtk_widget_show (GTK_WIDGET (info->msg_view));
 				info->self = TNY_DEMOUI_SUMMARY_VIEW (g_object_ref (self));
@@ -1074,55 +1099,6 @@ on_header_view_tree_row_activated (GtkTreeView *treeview, GtkTreePath *path,
 	}
 }
 
-#ifdef ASYNCWORKER
-static TnyQueue *fullqueue = NULL;
-static TnyQueue *real_queue = NULL;
-
-static TnyQueue*
-get_queue (void)
-{
-	if (!real_queue)
-		real_queue = tny_async_worker_queue_new_from_scratch ();
-	return real_queue;
-}
-
-static void 
-on_full_download_folder_activate (GtkMenuItem *mitem, gpointer user_data)
-{
-	TnyDemouiSummaryView *self = user_data;
-	TnyDemouiSummaryViewPriv *priv = TNY_DEMOUI_SUMMARY_VIEW_GET_PRIVATE (self);
-	GtkTreeIter iter;
-	GtkTreeModel *model;
-
-	if (gtk_tree_selection_get_selected (priv->mailbox_select, &model, &iter))
-	{
-		gint type;
-
-		gtk_tree_model_get (model, &iter, 
-			TNY_GTK_FOLDER_STORE_TREE_MODEL_TYPE_COLUMN, 
-			&type, -1);
-
-		if (type != TNY_FOLDER_TYPE_ROOT) 
-		{ 
-			TnyFolder *folder;
-
-			gtk_tree_model_get (model, &iter, 
-				TNY_GTK_FOLDER_STORE_TREE_MODEL_INSTANCE_COLUMN, 
-				&folder, -1);
-
-
-			if (!fullqueue)
-				fullqueue = tny_get_msg_queue_new (get_queue ());
-
-			tny_get_msg_queue_full_msg_retrieval (TNY_GET_MSG_QUEUE (fullqueue), 
-				folder, NULL, NULL, status_update, self);
-
-			g_object_unref (G_OBJECT (folder));
-
-		}
-	}
-}
-#endif
 
 static void 
 on_rename_folder_activate (GtkMenuItem *mitem, gpointer user_data)
@@ -1413,6 +1389,51 @@ sync_button_clicked (GtkWidget *button, gpointer user_data)
 }
 
 
+
+static void
+cancel_button_clicked (GtkWidget *button, gpointer user_data)
+{
+	TnyDemouiSummaryView *self = user_data;
+	TnyDemouiSummaryViewPriv *priv = TNY_DEMOUI_SUMMARY_VIEW_GET_PRIVATE (self);
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+
+	if (gtk_tree_selection_get_selected (priv->mailbox_select, &model, &iter))
+	{
+		gint type;
+		TnyAccount *account = NULL;
+
+		gtk_tree_model_get (model, &iter, 
+			TNY_GTK_FOLDER_STORE_TREE_MODEL_TYPE_COLUMN, 
+			&type, -1);
+
+		if (type != TNY_FOLDER_TYPE_ROOT) 
+		{ 
+			TnyFolder *folder;
+
+			gtk_tree_model_get (model, &iter, 
+				TNY_GTK_FOLDER_STORE_TREE_MODEL_INSTANCE_COLUMN, 
+				&folder, -1);
+
+			if (TNY_IS_FOLDER (folder))
+				account = tny_folder_get_account (folder);
+
+			g_object_unref (folder);
+
+		} else {
+			gtk_tree_model_get (model, &iter, 
+				TNY_GTK_FOLDER_STORE_TREE_MODEL_INSTANCE_COLUMN, 
+				&account, -1);
+		}
+
+		if (account) {
+			tny_account_cancel (account);
+			g_object_unref (account);
+		}
+	}
+}
+
+
 static void 
 on_create_folder_activate (GtkMenuItem *mitem, gpointer user_data)
 {
@@ -1517,16 +1538,104 @@ on_merge_view_activate (GtkMenuItem *mitem, gpointer user_data)
 
 }
 
+
+typedef struct {
+	TnyHeader *header;
+	TnyFolder *to_folder;
+} OnMoveToFolderInfo;
+
+static void 
+on_move_to_folder_activate (GtkMenuItem *mitem, gpointer user_data)
+{
+	OnMoveToFolderInfo *info = user_data;
+	TnyFolder *folder = tny_header_get_folder (info->header);
+
+	if (folder) 
+	{
+		TnyList *headers = tny_simple_list_new ();
+
+		g_print ("Transfering: %s to %s\n",
+			tny_header_get_subject (info->header), 
+			tny_folder_get_name (info->to_folder));
+
+		tny_list_prepend (headers, (GObject *) info->header);
+		tny_folder_transfer_msgs_async (folder, headers, info->to_folder, 
+			FALSE, NULL, NULL, user_data);
+		g_object_unref (headers);
+	}
+
+	return;
+}
+
+static void
+recursive_all_folders (GtkWidget *my_widget, GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data, GtkWidget *menu)
+{
+	TnyDemouiSummaryView *self = user_data;
+	TnyDemouiSummaryViewPriv *priv = TNY_DEMOUI_SUMMARY_VIEW_GET_PRIVATE (self);
+
+	do {
+		GtkTreeIter niter;
+		GObject *citem;
+		gchar *line;
+
+		gtk_tree_model_get (model, iter, 
+			TNY_GTK_FOLDER_STORE_TREE_MODEL_INSTANCE_COLUMN, 
+			&citem, TNY_GTK_FOLDER_STORE_TREE_MODEL_NAME_COLUMN, &line, -1);
+
+		if (TNY_IS_FOLDER (citem))
+		{
+			GtkTreeIter hiter;
+			GtkTreeModel *hmodel = gtk_tree_view_get_model (priv->header_view);
+			GtkTreeSelection *sel = gtk_tree_view_get_selection (priv->header_view);
+
+			if (gtk_tree_selection_get_selected (sel, &hmodel, &hiter))
+			{
+				/* Small leak, indeed */
+				OnMoveToFolderInfo *info = g_new0 (OnMoveToFolderInfo, 1);
+				GtkWidget *menuitem = NULL;
+
+				info->to_folder = (TnyFolder *) citem;
+				gtk_tree_model_get (hmodel, &hiter, 
+					TNY_GTK_HEADER_LIST_MODEL_INSTANCE_COLUMN, 
+					&info->header, -1);
+
+				menuitem = gtk_menu_item_new_with_label (tny_folder_get_name ((TnyFolder *)citem));
+
+				g_signal_connect (G_OBJECT (menuitem), "activate",
+					G_CALLBACK (on_move_to_folder_activate), info);
+
+				gtk_widget_show (menuitem);
+				gtk_menu_prepend (menu, menuitem);
+			}
+		}
+
+		g_object_unref (citem);
+
+		if (gtk_tree_model_iter_children (model, &niter, iter)) {
+			recursive_all_folders (my_widget, model, &niter, user_data, menu);
+		}
+
+	} while (gtk_tree_model_iter_next (model, iter));
+}
+
 static void
 header_view_do_popup_menu (GtkWidget *my_widget, GdkEventButton *event, gpointer user_data)
 {
+	TnyDemouiSummaryView *self = user_data;
+	TnyDemouiSummaryViewPriv *priv = TNY_DEMOUI_SUMMARY_VIEW_GET_PRIVATE (self);
+
 	GtkWidget *menu;
 	int button, event_time;
+	GtkTreeView *view = priv->mailbox_view;
+	GtkTreeModel *model = gtk_tree_view_get_model (view);
 
 	menu = gtk_menu_new ();
-	g_signal_connect (menu, "deactivate", G_CALLBACK (gtk_widget_destroy), NULL);
 
-	/* ... add menu items ... */
+	if (model) {
+		GtkTreeIter iter;
+		gtk_tree_model_get_iter_first (model, &iter);
+		recursive_all_folders (my_widget, model, &iter, user_data, menu);
+	}
 
 	if (event)
 	{
@@ -1573,9 +1682,6 @@ mailbox_view_do_popup_menu (GtkWidget *my_widget, GdkEventButton *event, gpointe
 	int button, event_time;
 	GtkSelectionMode mode;
 	GtkWidget *mrename, *mdelete, *mcreate, *mmerge, *muncache;
-#ifdef ASYNCWORKER
-	GtkWidget *fdown;
-#endif
 
 	menu = gtk_menu_new ();
 
@@ -1584,10 +1690,6 @@ mailbox_view_do_popup_menu (GtkWidget *my_widget, GdkEventButton *event, gpointe
 	mdelete = gtk_menu_item_new_with_label (_("Delete folder"));
 	muncache = gtk_menu_item_new_with_label (_("Uncache account of this folder"));
 
-#ifdef ASYNCWORKER
-	fdown = gtk_menu_item_new_with_label (_("Download entire folder"));
-#endif
-
 	mode = gtk_tree_selection_get_mode (priv->mailbox_select);
 
 	if (mode == GTK_SELECTION_SINGLE)
@@ -1605,21 +1707,12 @@ mailbox_view_do_popup_menu (GtkWidget *my_widget, GdkEventButton *event, gpointe
 		G_CALLBACK (on_delete_folder_activate), user_data);
 	g_signal_connect (G_OBJECT (mmerge), "activate",
 		G_CALLBACK (on_merge_view_activate), user_data);
-#ifdef ASYNCWORKER
-	g_signal_connect (G_OBJECT (fdown), "activate",
-		G_CALLBACK (on_full_download_folder_activate), user_data);
-#endif
 
 	gtk_menu_prepend (menu, mrename);
 	gtk_menu_prepend (menu, mcreate);
 	gtk_menu_prepend (menu, mdelete);
 	gtk_menu_prepend (menu, muncache);
 	gtk_menu_prepend (menu, mmerge);
-#ifdef ASYNCWORKER
-	gtk_menu_prepend (menu, fdown);
-
-	gtk_widget_show (fdown);
-#endif
 	gtk_widget_show (mrename);
 	gtk_widget_show (mcreate);
 	gtk_widget_show (mdelete);
@@ -1682,7 +1775,7 @@ tny_demoui_summary_view_instance_init (GTypeInstance *instance, gpointer g_class
 	TnyDemouiSummaryViewPriv *priv = TNY_DEMOUI_SUMMARY_VIEW_GET_PRIVATE (self);
 	TnyPlatformFactory *platfact;
 	GtkVBox *vbox = GTK_VBOX (self);
-	GtkWidget *mailbox_sw, *widget;
+	GtkWidget *mailbox_sw, *widget, *cancel_button;
 	GtkWidget *header_sw;
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
@@ -1704,6 +1797,9 @@ tny_demoui_summary_view_instance_init (GTypeInstance *instance, gpointer g_class
 	priv->sync_button = gtk_button_new_with_label ("Sync");
 	priv->killacc_button = gtk_button_new_with_label ("Kill account");
 
+	cancel_button = gtk_button_new_with_label ("Cancel");
+
+
 	priv->current_accounts = NULL;
 
 	priv->online_button_signal = g_signal_connect (G_OBJECT (priv->online_button), "toggled", 
@@ -1712,6 +1808,9 @@ tny_demoui_summary_view_instance_init (GTypeInstance *instance, gpointer g_class
 	g_signal_connect (G_OBJECT (priv->poke_button), "clicked", 
 		G_CALLBACK (poke_button_toggled), self);
 
+	g_signal_connect (G_OBJECT (cancel_button), "clicked", 
+		G_CALLBACK (cancel_button_clicked), self);
+
 	g_signal_connect (G_OBJECT (priv->sync_button), "clicked", 
 		G_CALLBACK (sync_button_clicked), self);
 
@@ -1747,7 +1846,9 @@ tny_demoui_summary_view_instance_init (GTypeInstance *instance, gpointer g_class
 	gtk_box_pack_start (GTK_BOX (priv->status), priv->poke_button, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (priv->status), priv->sync_button, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (priv->status), priv->killacc_button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (priv->status), cancel_button, FALSE, FALSE, 0);
 
+	gtk_widget_show (cancel_button);
 	gtk_widget_show (priv->online_button);
 	gtk_widget_show (priv->poke_button);
 	gtk_widget_show (priv->killacc_button);
@@ -1760,9 +1861,11 @@ tny_demoui_summary_view_instance_init (GTypeInstance *instance, gpointer g_class
 
 	vpaned1 = gtk_vpaned_new ();
 	gtk_widget_show (vpaned1);
-	
+
 	priv->msg_view = tny_platform_factory_new_msg_view (platfact);
 
+	tny_gtk_msg_view_set_status_callback (TNY_GTK_MSG_VIEW (priv->msg_view), status_update, self);
+
 	gtk_widget_show (GTK_WIDGET (priv->msg_view));
 
 	widget = gtk_scrolled_window_new (NULL, NULL);
@@ -1852,7 +1955,7 @@ tny_demoui_summary_view_instance_init (GTypeInstance *instance, gpointer g_class
 		"text", TNY_GTK_HEADER_LIST_MODEL_FROM_COLUMN, NULL);
 	gtk_tree_view_column_set_sort_column_id (column, TNY_GTK_HEADER_LIST_MODEL_FROM_COLUMN);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
-	gtk_tree_view_column_set_fixed_width (column, 100);
+	gtk_tree_view_column_set_fixed_width (column, 230);
 	gtk_tree_view_append_column (GTK_TREE_VIEW(priv->header_view), column);
 
 	if (G_UNLIKELY (FALSE))
@@ -1951,7 +2054,8 @@ tny_demoui_summary_view_finalize (GObject *object)
 	if (priv->monitor)
 	{
 		tny_folder_monitor_stop (priv->monitor);
-		g_object_unref (G_OBJECT (priv->monitor));
+		g_object_unref (priv->monitor);
+		priv->monitor = NULL;
 	}
 	g_mutex_unlock (priv->monitor_lock);
 
@@ -1973,7 +2077,7 @@ tny_account_store_view_init (gpointer g, gpointer iface_data)
 {
 	TnyAccountStoreViewIface *klass = (TnyAccountStoreViewIface *)g;
 
-	klass->set_account_store_func = tny_demoui_summary_view_set_account_store;
+	klass->set_account_store= tny_demoui_summary_view_set_account_store;
 
 	return;
 }
